package cn.com.qmth.stmms.ms.accesscontrol;

import cn.com.qmth.stmms.ms.accesscontrol.config.LoginConfig;
import cn.com.qmth.stmms.ms.commons.utils.EncrypAES;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.domain.user.AbstractUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLDecoder;
import java.util.Objects;

public class LoginInterceptor extends HandlerInterceptorAdapter {

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private LoginConfig loginConfig;

    @Autowired
    private CacheService cacheService;

    @Override
    public boolean preHandle(HttpServletRequest request,
                             HttpServletResponse response, Object handler) throws Exception {
        String authorization = request.getHeader("Authorization");
        if (StringUtils.isEmpty(authorization)) {
            authorization = request.getParameter("Authorization");
        }
        String userId = request.getHeader("userId");
        if (StringUtils.isEmpty(userId)) {
            userId = request.getParameter("userId");
        }
        String token = cacheService.queryTokenCache(userId);
        if (StringUtils.isEmpty(authorization) || StringUtils.isEmpty(userId) || StringUtils.isEmpty(token)) {
            response.sendError(HttpStatus.UNAUTHORIZED.value(), "身份验证失效，请重新登录!");
            return false;
        }

        authorization = URLDecoder.decode(authorization, "UTF-8");
        token = URLDecoder.decode(token, "UTF-8");

        String tokenDec = EncrypAES.decrypt(token);
        String[] strs = tokenDec.split("#");
        String loginName = strs[1];
        if (loginConfig.adminLoginConfig().getLoginName().equals(loginName)) {
            return true;
        }
        AbstractUser user = markUserRepo.findOne(Long.valueOf(userId));
        // 管理员用户，不在数据库表中
        if (user != null) {
            if (Objects.equals(Role.COLLECTOR, ((MarkUser) user).getRole())) {
                response.sendError(HttpStatus.UNAUTHORIZED.value(), "该账户只能在采集端登录");
                return false;
            }

            if (!Objects.equals(Role.QC, ((MarkUser) user).getRole()) && !authorization.equals(token)) {
                response.sendError(HttpStatus.UNAUTHORIZED.value(), "该账号在其它地方登录");
                return false;
            }

            String sessionId = request.getSession().getId();
            if (user.getSessionId().equalsIgnoreCase(sessionId)) {
                return true;
            }
        }
        return true;
    }

    public void postHandle(HttpServletRequest request,
                           HttpServletResponse response) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request,
                                HttpServletResponse response, Object handler, Exception ex)
            throws Exception {

    }
}
