package cn.com.qmth.stmms.ms.accesscontrol.api;

import cn.com.qmth.stmms.ms.accesscontrol.config.LoginConfig;
import cn.com.qmth.stmms.ms.commons.utils.EncrypAES;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.net.URLEncoder;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * Created by zhengmin on 2016/11/1.
 */
@RestController
@RequestMapping("/api")
public class AuthApi {

    @Autowired
    private LoginConfig loginConfig;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private CacheService cacheService;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public MarkUser login(MarkUser user, HttpServletRequest request) throws Exception {
        MarkUser domain = null;
        String loginName = loginConfig.adminLoginConfig().getLoginName();
        List<String> loginNames = Arrays.asList(loginName.split(","));
        if (loginNames.contains(user.getLoginName()) &&
                loginConfig.adminLoginConfig().getPassword().equals(user.getPassword())) {
            domain = new MarkUser(user.getLoginName(), loginConfig.adminLoginConfig().getPassword(), null, null, "系统管理员", Role.ADMIN, null);
            domain.setId(Long.valueOf(loginNames.indexOf(user.getLoginName())));
        } else {
            domain = markUserRepo.findByLoginName(user.getLoginName());
            if (domain == null) {
                if ((loginConfig.adminLoginConfig().getLoginName().equals(user.getLoginName()) && !loginConfig.adminLoginConfig().getPassword().equals(user.getPassword()))
                ) {
                    throw new RuntimeException("用户或密码错误");
                } else {
                    throw new RuntimeException("用户不存在");
                }
            }
            if (Objects.equals(Role.COLLECTOR, domain.getRole())) {
                throw new RuntimeException("该账户只能在采集端登录");
            }
            if (!user.getPassword().equals(domain.getPassword())) {
                throw new RuntimeException("用户或密码错误");
            }
            if (!domain.isEnabled()) {
                throw new RuntimeException("用户被禁用");
            }
            domain.setSessionId(request.getSession().getId());
            markUserRepo.save(domain);
        }

        if (Objects.equals(Role.MARKER, domain.getRole())) {
            MarkSubject markSubject = markSubjectRepo.findOne(domain.getWorkId() + "-" + domain.getSubject().name());
            int count = 0;
            if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                List<MarkTaskRoughLevel> markTaskRoughLevelList = markTaskRoughLevelRepo.findByWorkIdAndSubjectAndMarkerIdAndStageLimit(domain.getWorkId(), domain.getSubject().name(), domain.getId(), MarkStage.ROUGH_LEVEL.ordinal());
                count = markTaskRoughLevelList.size();
            } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                List<MarkTaskLevel> markTaskLevelList = markTaskLevelRepo.findByWorkIdAndSubjectAndMarkerIdAndStageLimit(domain.getWorkId(), domain.getSubject().name(), domain.getId(), MarkStage.LEVEL.ordinal());
                List<MarkTaskRoughLevel> markTaskRoughLevelList = markTaskRoughLevelRepo.findByWorkIdAndSubjectAndMarkerIdAndStageLimit(domain.getWorkId(), domain.getSubject().name(), domain.getId(), MarkStage.ROUGH_LEVEL.ordinal());
                count = markTaskLevelList.size() + markTaskRoughLevelList.size();
            } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
                List<MarkTaskScore> markTaskScoreList = markTaskScoreRepo.findByWorkIdAndSubjectAndMarkerIdAndStageLimit(domain.getWorkId(), domain.getSubject().name(), domain.getId(), MarkStage.SCORE.ordinal());
                List<MarkTaskLevel> markTaskLevelList = markTaskLevelRepo.findByWorkIdAndSubjectAndMarkerIdAndStageLimit(domain.getWorkId(), domain.getSubject().name(), domain.getId(), MarkStage.LEVEL.ordinal());
                count = markTaskScoreList.size() + markTaskLevelList.size();
            }
            if (count == 0) {
                throw new RuntimeException("没有评卷任务");
            }
        }

        String token = System.currentTimeMillis() + "#" + domain.getLoginName() + "#" + domain.getId();
        String tokenAes = EncrypAES.encrypt(token);
        token = URLEncoder.encode(tokenAes, "UTF-8");
        cacheService.saveTokenCache(domain.getId().toString(), token);
        domain.setToken(token);
        return domain;
    }

    @RequestMapping(value = "/logout", method = RequestMethod.DELETE)
    public void logout(HttpServletRequest request) {
        HttpSession session = request.getSession();
        session.setAttribute("token", null);
    }

    @RequestMapping(value = "/{domain}/password", method = RequestMethod.PATCH)
    public void pwd(@PathVariable MarkUser domain, @RequestBody MarkUser markUser) {
        if (markUser.getPassword() == null || markUser.getPassword().length() < 6) {
            throw new RuntimeException("密码长度不能小于6位");
        }
        domain.setPassword(markUser.getPassword());
        domain.setPwChangedCount(markUser.getPwChangedCount() + 1);
        markUserRepo.save(domain);
    }
}
