package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Message;
import cn.com.qmth.stmms.ms.core.domain.MessageReceive;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.service.MessageService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 消息管理
 */
@RestController
@RequestMapping("api/message")
public class MessageApi {

    @Resource
    MessageService messageService;

    @PostMapping("/send")
    public Message save(@RequestBody String data) {
        return messageService.sendMessage(data);
    }

    /**
     * 评卷员查询消息
     *
     * @param subject       科目
     * @param stage         阶段
     * @param receiveUserId 评卷员ID
     * @param lastId        上次最后一次查询的消息ID最大值
     * @param pageable      分页
     */
    @GetMapping("/page_receive")
    public Page<MessageReceive> listByMarker(@RequestParam Subject subject,
                                             @RequestParam MarkStage stage,
                                             @RequestParam Long receiveUserId,
                                             @RequestParam Long lastId,
                                             Pageable pageable) {
        return messageService.listByReceiveUserIdAll(subject, stage, receiveUserId, lastId, pageable);
    }

    /**
     * 评卷员查询消息
     *
     * @param subject       科目
     * @param stage         阶段
     * @param receiveUserId 收件人ID
     */
    @GetMapping("/list_receive_unread")
    public List<MessageReceive> listByMarker(@RequestParam Subject subject,
                                             @RequestParam MarkStage stage,
                                             @RequestParam Long receiveUserId) {
        return messageService.listByReceiveUserIdRead(subject, stage, receiveUserId);
    }

    /**
     * @param subject    科目
     * @param stage      阶段
     * @param sendUserId 发件人ID
     * @param pageable   分页
     */
    @GetMapping("/page_send")
    public Page<Message> listBySendUserId(@RequestParam Subject subject,
                                          @RequestParam MarkStage stage,
                                          @RequestParam Long sendUserId,
                                          Pageable pageable) {
        return messageService.listBySendUserIdAll(subject, stage, sendUserId, pageable);
    }

    /**
     * 标记已读
     *
     * @param object 消息ID
     */
    @PostMapping("/read")
    public void listByMarkerLeader(@RequestBody JSONObject object) {
        List<Long> ids = JSON.parseArray(object.getString("ids"), Long.class);
        messageService.markRead(ids);
    }

}
