package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.ParamSetting;
import cn.com.qmth.stmms.ms.core.domain.enums.ParamSettingTypeEnum;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.CollectConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.LevelConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.RoughLevelConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.ScoreConfig;
import cn.com.qmth.stmms.ms.core.repository.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * 全局参数
 */
@RestController
@RequestMapping("/api/param")
public class ParamApi {

    @Resource
    private ParamSettingRepo paramSettingRepo;

    @Resource
    private PaperRepo paperRepo;

    @Resource
    private MarkSubjectRepo markSubjectRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    /**
     * 查询
     *
     * @param workId 工作Id
     */
    @GetMapping("/getParam")
    public ParamSetting getOne(@RequestParam Long workId) {
        return paramSettingRepo.findByWorkId(workId);
    }

    /**
     * 更新采集参数
     *
     * @param data data
     */
    @PostMapping("/collect")
    public ResponseEntity updateCollectParam(@RequestBody String data) {
        JSONObject object = JSON.parseObject(data);
        Integer packageScan = object.getInteger("packageScan");
        Integer imageEncrypt = object.getInteger("imageEncrypt");
        Integer nameRule = object.getInteger("nameRule");
        Integer paperStage = object.getInteger("paperStage");

        Long workId = ServletUtil.getWordId();
        //保存采集参数
        ParamSetting paramSetting = paramSettingRepo.findByWorkId(workId);
        if (paramSetting == null) {
            paramSetting = ParamSetting.init(workId);
        }

        // 是否全部为采集阶段
        List<MarkSubject> workIdAndStageNot = markSubjectRepo.findAllByWorkIdAndStageNot(workId, MarkStage.INIT);

        CollectConfig oldCollectConfig = JSON.parseObject(paramSetting.getCollectConfig(), CollectConfig.class);
        if (workIdAndStageNot != null && !workIdAndStageNot.isEmpty() && !Objects.equals(packageScan, oldCollectConfig.getPackageScan())) {
            throw new RuntimeException("该评卷工作当前不是采集阶段，不能修改是【否整包扫描】参数");
        }
        //有采集数据，不能修改采集参数
        List<Paper> papers = paperRepo.findByWorkId(workId);
        if (papers != null && !papers.isEmpty()) {
            if (!Objects.equals(imageEncrypt, oldCollectConfig.getImageEncrypt())) {
                throw new RuntimeException("该评卷工作已有采集数据，不能修改【图片是否加密】参数");
            }
            if (!Objects.equals(nameRule, oldCollectConfig.getNameRule())) {
                throw new RuntimeException("该评卷工作已有采集数据，不能修改【图片命名规则】参数");
            }
            if (!Objects.equals(paperStage, oldCollectConfig.getPaperStage())) {
                throw new RuntimeException("该评卷工作已有采集数据，不能修改【试卷档位】参数");
            }
        }

        oldCollectConfig.setPackageScan(packageScan);
        oldCollectConfig.setImageEncrypt(imageEncrypt);
        oldCollectConfig.setNameRule(nameRule);
        oldCollectConfig.setPaperStage(paperStage);
        paramSetting.setCollectConfig(JSON.toJSONString(oldCollectConfig));
        paramSettingRepo.saveAndFlush(paramSetting);
        ParamCache.cacheParam(ParamSettingTypeEnum.COLLECT, workId, paramSetting.getCollectConfig());

        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 粗分档规则参数
     *
     * @param data data
     */
    @PostMapping("/rough_level")
    public ResponseEntity updateFirstLevelParam(@RequestBody String data) {
        Long workId = ServletUtil.getWordId();
        JSONObject object = JSON.parseObject(data);
        Integer deviation = object.getInteger("deviation");
        Integer autoCallback = object.getInteger("autoCallback");
        Integer majority = object.getInteger("majority");
        Integer autoCallbackShowDeviation = object.getInteger("autoCallbackShowDeviation");
        Integer cumulativeError = object.getInteger("cumulativeError");
        Integer takeBest = object.getInteger("takeBest");
        Integer clearData = object.getInteger("clearData");
        Integer levelShowAllPaper = object.getInteger("levelShowAllPaper");
        Integer propDenominator = object.getInteger("propDenominator");
        Integer showStandardPaperManage = object.getInteger("showStandardPaperManage");
        Integer removeHighAndLow = object.getInteger("removeHighAndLow");

        if (ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 0) {
            throw new RuntimeException("【是否增加粗档位环节】为【否】，不能修改粗分档参数");
        }

        List<MarkStage> markStageList = Arrays.asList(new MarkStage[]{MarkStage.ROUGH_LEVEL, MarkStage.LEVEL, MarkStage.SCORE});
        List<MarkSubject> markSubjects = markSubjectRepo.findByWorkIdAndTestAndStageIn(workId, TrialEnum.DEFAULT.ordinal(), markStageList);

        int countMarkTasks = markTaskRoughLevelRepo.countByWorkId(workId);
        //保存分档参数
        ParamSetting paramSetting = paramSettingRepo.findByWorkId(workId);
        if (paramSetting == null) {
            paramSetting = ParamSetting.init(workId);
        }

        RoughLevelConfig oldRoughLevelConfig = JSON.parseObject(paramSetting.getRoughLevelConfig(), RoughLevelConfig.class);
        if (markSubjects != null && !markSubjects.isEmpty() && countMarkTasks > 0) {
            if (!Objects.equals(deviation, oldRoughLevelConfig.getDeviation())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【仲裁档位差】参数");
            }
            if (!Objects.equals(autoCallback, oldRoughLevelConfig.getAutoCallback())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【系统自动打回】参数");
            }
            if (!Objects.equals(cumulativeError, oldRoughLevelConfig.getCumulativeError())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【打回累计误差】参数");
            }
            if (!Objects.equals(majority, oldRoughLevelConfig.getMajority())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【是否过半定档】参数");
            }
            if (!Objects.equals(takeBest, oldRoughLevelConfig.getTakeBest())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【是否启用取优原则】参数");
            }
            if (!Objects.equals(removeHighAndLow, oldRoughLevelConfig.getRemoveHighAndLow())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【是否开启去高去低再加权评卷】参数");
            }
        }

        // 开启自动打回，打回累计误差必填
        if (autoCallback == 1 && cumulativeError == null) {
            throw new RuntimeException("系统自动打回已开启，【打回累计误差】必填");
        }

        oldRoughLevelConfig.setDeviation(deviation);
        oldRoughLevelConfig.setAutoCallback(autoCallback);
        oldRoughLevelConfig.setCumulativeError(cumulativeError);
        oldRoughLevelConfig.setAutoCallbackShowDeviation(autoCallbackShowDeviation);
        oldRoughLevelConfig.setMajority(majority);
        oldRoughLevelConfig.setTakeBest(takeBest);
        oldRoughLevelConfig.setLevelShowAllPaper(levelShowAllPaper);
        oldRoughLevelConfig.setPropDenominator(propDenominator);
        oldRoughLevelConfig.setShowStandardPaperManage(showStandardPaperManage);
        oldRoughLevelConfig.setClearData(clearData);
        oldRoughLevelConfig.setRemoveHighAndLow(removeHighAndLow);

        paramSetting.setRoughLevelConfig(JSON.toJSONString(oldRoughLevelConfig));
        paramSettingRepo.saveAndFlush(paramSetting);
        ParamCache.cacheParam(ParamSettingTypeEnum.ROUGH_LEVEL, workId, paramSetting.getRoughLevelConfig());

        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 细分档规则参数
     *
     * @param data data
     */
    @PostMapping("/level")
    public ResponseEntity updateLevelParam(@RequestBody String data) {
        Long workId = ServletUtil.getWordId();
        JSONObject object = JSON.parseObject(data);
        Integer deviation = object.getInteger("deviation");
        Integer autoCallback = object.getInteger("autoCallback");
        Integer majority = object.getInteger("majority");
        Integer autoCallbackShowDeviation = object.getInteger("autoCallbackShowDeviation");
        Integer cumulativeError = object.getInteger("cumulativeError");
        Integer takeBest = object.getInteger("takeBest");
        Integer clearData = object.getInteger("clearData");
        Integer levelShowAllPaper = object.getInteger("levelShowAllPaper");
        Integer propDenominator = object.getInteger("propDenominator");
        Integer showStandardPaperManage = object.getInteger("showStandardPaperManage");
        Integer roughLevelEnable = object.getInteger("roughLevelEnable");
        Integer removeHighAndLow = object.getInteger("removeHighAndLow");

        List<MarkStage> markStageList = Arrays.asList(new MarkStage[]{MarkStage.LEVEL, MarkStage.SCORE});
        List<MarkSubject> markSubjects = markSubjectRepo.findByWorkIdAndTestAndStageIn(workId, TrialEnum.DEFAULT.ordinal(), markStageList);
        int countMarkTasks = markTaskLevelRepo.countByWorkId(workId);
        //保存分档参数
        ParamSetting paramSetting = paramSettingRepo.findByWorkId(workId);
        if (paramSetting == null) {
            paramSetting = ParamSetting.init(workId);
        }

        LevelConfig oldLevelConfig = JSON.parseObject(paramSetting.getLevelConfig(), LevelConfig.class);
        if (!markSubjects.isEmpty() && countMarkTasks > 0) {
            if (!Objects.equals(deviation, oldLevelConfig.getDeviation())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【仲裁档位差】参数");
            }
            if (!Objects.equals(autoCallback, oldLevelConfig.getAutoCallback())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【系统自动打回】参数");
            }
            if (!Objects.equals(cumulativeError, oldLevelConfig.getCumulativeError())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【打回累计误差】参数");
            }
            if (!Objects.equals(majority, oldLevelConfig.getMajority())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【是否过半定档】参数");
            }
            if (!Objects.equals(takeBest, oldLevelConfig.getTakeBest())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【是否启用取优原则】参数");
            }
            if (!Objects.equals(removeHighAndLow, oldLevelConfig.getRemoveHighAndLow())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【是否开启去高去低再加权评卷】参数");
            }
        }

        // 开启自动打回，打回累计误差必填
        if (autoCallback == 1 && cumulativeError == null) {
            throw new RuntimeException("系统自动打回已开启，【打回累计误差】必填");
        }

        if (oldLevelConfig.getRoughLevelEnable() == 1 && roughLevelEnable == 0) {
            List<MarkStage> markStageList1 = Arrays.asList(new MarkStage[]{MarkStage.ROUGH_LEVEL});
            List<MarkSubject> subjectList = markSubjectRepo.findByWorkIdAndTestAndStageIn(workId, TrialEnum.DEFAULT.getId(), markStageList1);
            int roughLevelCount = markTaskRoughLevelRepo.countByWorkId(workId);
            if (!subjectList.isEmpty() || roughLevelCount > 0) {
                throw new RuntimeException("评卷工作有科目已进入粗分档阶段或者粗分档阶段已有正评数据，不能修改【是否增加粗档位环节】参数为【否】");
            }
        } else if (oldLevelConfig.getRoughLevelEnable() == 0 && roughLevelEnable == 1) {
            List<MarkStage> markStageList1 = Arrays.asList(new MarkStage[]{MarkStage.LEVEL});
            List<MarkSubject> subjectList = markSubjectRepo.findByWorkIdAndTestAndStageIn(workId, TrialEnum.DEFAULT.getId(), markStageList1);
            if (!subjectList.isEmpty() || countMarkTasks > 0) {
                throw new RuntimeException("评卷工作有科目已进入细分档阶段或者细分档阶段已有正评数据，不能修改【是否增加粗档位环节】参数为【是】");
            }
        }

        oldLevelConfig.setDeviation(deviation);
        oldLevelConfig.setAutoCallback(autoCallback);
        oldLevelConfig.setCumulativeError(cumulativeError);
        oldLevelConfig.setAutoCallbackShowDeviation(autoCallbackShowDeviation);
        oldLevelConfig.setMajority(majority);
        oldLevelConfig.setTakeBest(takeBest);
        oldLevelConfig.setLevelShowAllPaper(levelShowAllPaper);
        oldLevelConfig.setPropDenominator(propDenominator);
        oldLevelConfig.setShowStandardPaperManage(showStandardPaperManage);
        oldLevelConfig.setClearData(clearData);
        oldLevelConfig.setRemoveHighAndLow(removeHighAndLow);
        oldLevelConfig.setRoughLevelEnable(roughLevelEnable);

        paramSetting.setLevelConfig(JSON.toJSONString(oldLevelConfig));
        paramSettingRepo.saveAndFlush(paramSetting);
        ParamCache.cacheParam(ParamSettingTypeEnum.LEVEL, workId, paramSetting.getLevelConfig());

        return new ResponseEntity(HttpStatus.OK);
    }


    /**
     * 打分改档参数
     *
     * @param data data
     */
    @PostMapping("/score")
    public ResponseEntity updateScoreParam(@RequestBody String data) {
        Long workId = ServletUtil.getWordId();
        JSONObject object = JSON.parseObject(data);
        Integer roundUp = object.getInteger("roundUp");
        Integer changeStage = object.getInteger("changeStage");
        Integer scoreShowAllPaper = object.getInteger("scoreShowAllPaper");
        Integer removeHighAndLow = object.getInteger("removeHighAndLow");

        //保存打分参数
        ParamSetting paramSetting = paramSettingRepo.findByWorkId(workId);
        if (paramSetting == null) {
            paramSetting = ParamSetting.init(workId);
        }

        ScoreConfig oldScoreConfig = JSON.parseObject(paramSetting.getScoreConfig(), ScoreConfig.class);
        List<MarkStage> markStageList = Arrays.asList(new MarkStage[]{MarkStage.SCORE});
        List<MarkSubject> markSubjects = markSubjectRepo.findByWorkIdAndTestAndStageIn(workId, TrialEnum.DEFAULT.ordinal(), markStageList);

        int countMarkTasks = markTaskScoreRepo.countByWorkId(workId);
        if (markSubjects != null && !markSubjects.isEmpty() && countMarkTasks > 0) {
            if (!Objects.equals(roundUp, oldScoreConfig.getRoundUp())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【分数处理方式】参数");
            }
            if (!Objects.equals(removeHighAndLow, oldScoreConfig.getRemoveHighAndLow())) {
                throw new RuntimeException("评卷工作已有正评数据，不能修改【分数计算方式】参数");
            }
        }

        oldScoreConfig.setRoundUp(roundUp);
        oldScoreConfig.setChangeStage(changeStage);
        oldScoreConfig.setScoreShowAllPaper(scoreShowAllPaper);
        oldScoreConfig.setRemoveHighAndLow(removeHighAndLow);

        paramSetting.setScoreConfig(JSON.toJSONString(oldScoreConfig));
        paramSettingRepo.saveAndFlush(paramSetting);
        ParamCache.cacheParam(ParamSettingTypeEnum.SCORE, workId, paramSetting.getScoreConfig());

        return new ResponseEntity(HttpStatus.OK);
    }

}
