package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.commons.utils.DigitalToChineseUtil;
import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkLogOperType;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.RoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskLevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRoughLevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.service.DetermineRoughLevelService;
import cn.com.qmth.stmms.ms.marking.service.MarkerGroupLeaderService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 质量分析报表controller
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/11/8
 */
@RestController
@RequestMapping("api/quality")
public class QualityAnalyseController {

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private SqlUtil sqlUtil;

    @Autowired
    MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Autowired
    MarkTaskLevelRepo markTaskLevelRepo;

    @Autowired
    MarkUserRepo markUserRepo;

    @Autowired
    MarkerGroupLeaderService markerGroupLeaderService;

    /**
     * 档位占比报表
     *
     * @param workId    工作ID
     * @param subject   科目
     * @param areaCode  考区
     * @param startTime 查询时间段（开始时间）
     * @param endTime   查询时间段（结束时间）
     */
    @RequestMapping(value = "levelsPropReport", method = RequestMethod.GET)
    public Object levelsPropReport(
            @RequestParam Long workId,
            @RequestParam(required = false) MarkStage stage,
            @RequestParam(required = false) Subject subject,
            @RequestParam(required = false) String areaCode,
            @RequestParam(required = false) Long groupId,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime) {
        stage = Objects.isNull(stage) ? MarkStage.LEVEL : stage;
        List fList = new ArrayList();
        List<Level> levels = levelRepo.findByWorkId(workId);
        Map<String, String> codesMap = levels.stream().collect(Collectors.toMap(Level::getCode, Level::getCode));
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            List<RoughLevel> roughLevels = DetermineRoughLevelService.parseRoughLevels(levels);
            codesMap = roughLevels.stream().collect(Collectors.toMap(RoughLevel::getRoughCode, m-> DigitalToChineseUtil.int2chineseNum(Integer.parseInt(m.getRoughCode()))));
        }

        List<Long> markerIds = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndGroupId(workId, subject, stage, groupId);
        String markerId = null;
        if (!CollectionUtils.isEmpty(markerIds)) {
            markerId = markerIds.stream().map(String::valueOf).collect(Collectors.joining(","));
        }
        String finalMarkerId = markerId;
        MarkStage finalStage = stage;
        for (Map.Entry<String, String> entry : codesMap.entrySet()) {
            StringBuffer sql = new StringBuffer();
            sql.append("  SELECT                                                        ");
            sql.append("      mu.id markerId,                                           ");
            sql.append("      mu.login_name markerName,                                       ");
            sql.append("      SUM(IF(mt.result = '" + entry.getKey() + "', 1, 0)) levelCount,  ");
            sql.append("      COUNT(1) total                                            ");
            sql.append("  FROM                                                          ");
            sql.append("      mark_user mu                                              ");
            sql.append("           JOIN                                             ");
            sql.append("      (SELECT                                                   ");
            sql.append("          m.*                                                     ");
            sql.append("      FROM                                                      ");
            if (MarkStage.ROUGH_LEVEL.equals(finalStage)) {
                sql.append("mark_task_rough_level");
            } else if (MarkStage.LEVEL.equals(finalStage)) {
                sql.append("mark_task_level");
            }
            sql.append("        m  left join paper p on m.paper_id = p.id                ");
            sql.append("      WHERE                                                     ");
            sql.append("          m.stage = " + finalStage.ordinal() + "             ");
            if (!Objects.isNull(areaCode)) {
                sql.append(" and p.area_code = " + areaCode);
            }
            if (StringUtils.isNotBlank(finalMarkerId)) {
                sql.append(" and m.marker_id in (" + finalMarkerId + ")");
            }
            if (!Objects.isNull(startTime)) {
                sql.append(" and m.updated_on >= '" + startTime + "'");
            }
            if (!Objects.isNull(endTime)) {
                sql.append(" and m.updated_on <= '" + endTime + "'");
            }
            sql.append("          ) mt ON mu.work_id = mt.work_id                       ");
            sql.append("          AND mu.subject = mt.subject                           ");
            sql.append("          AND mu.id = mt.marker_id                              ");
            sql.append("  WHERE                                                         ");
            sql.append("      mu.work_id = " + workId);
            if (Objects.nonNull(subject)) {
                sql.append(" AND mu.subject = '" + subject + "' ");
            }
            sql.append("          AND mu.role = 'MARKER'                                ");
            sql.append("  GROUP BY mu.id , mu.name                                      ");
            sql.append("  ORDER BY mu.id                                                ");

            List list = sqlUtil.execSqlForMap(sql.toString());
            Map map = new HashMap();
            List<Map> data = new ArrayList();
            map.put("name", entry.getValue());
            for (Object o : list) {
                Map m = (Map) o;
                //该档任务数
                BigDecimal levelCount = BigDecimal.valueOf(Long.parseLong(m.get("levelCount").toString()));
                //总任务数
                BigDecimal total = BigDecimal.valueOf(Long.parseLong(m.get("total").toString()));
                //占比
                BigDecimal propBig = BigDecimal.ZERO.compareTo(total) == 0 ? BigDecimal.ZERO : levelCount.divide(total, 4, BigDecimal.ROUND_HALF_UP);
                double prop = propBig.doubleValue();
                Map temp = new HashMap();
                temp.put("markerId", m.get("markerId"));
                temp.put("markerName", m.get("markerName"));
                temp.put("prop", prop * 100);
                data.add(temp);
            }
            //排序
            Collections.sort(data, Comparator.comparing(o -> o.get("markerId").toString()));
            map.put("data", data);
            fList.add(map);
        }
        return fList;
    }

    /**
     * 打回次数报表
     *
     * @param workId    工作ID
     * @param subject   科目
     * @param areaCode  考区
     * @param startTime 查询时间段（开始时间）
     * @param endTime   查询时间段（结束时间）
     */
    @RequestMapping(value = "callbackReport", method = RequestMethod.GET)
    public List callbackReport(
            @RequestParam Long workId,
            @RequestParam(required = false) MarkStage stage,
            @RequestParam(required = false) Subject subject,
            @RequestParam(required = false) String areaCode,
            @RequestParam(required = false) Long groupId,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime) {
        stage = Objects.isNull(stage) ? MarkStage.LEVEL : stage;
        List list = new ArrayList();
        List<Long> markerIds = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndGroupId(workId, subject, stage, groupId);
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            Specification<MarkTaskRoughLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (Objects.nonNull(subject)) {
                    predicates.add(builder.equal(root.get("subject"), subject));
                }
                if (Objects.nonNull(areaCode)) {
                    predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
                }
                if (!CollectionUtils.isEmpty(markerIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("markerId"));
                    for (Long id : markerIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification);

            if (Objects.nonNull(markTasks) && markTasks.size() > 0) {
                Map<Long, String> markerMap = markTasks.stream().collect(Collectors.toMap(MarkTaskRoughLevel::getMarkerId, MarkTaskRoughLevel::getMarkerName, (dto1, dto2) -> dto1));
                List<Long> markerIdList = markerMap.entrySet().stream().map(Map.Entry::getKey).collect(Collectors.toList());
                Object o = StringUtils.join(markerIdList.toArray(), ",");
                String sql = new StringBuffer("select mu.login_name as userName, count(t.create_user_id) as sumCount,t.create_user_id as userId from mark_log t left join mark_user mu on t.create_user_id = mu.id where t.stage = ").append(stage.ordinal()).append(" and t.oper_type in (").append(MarkLogOperType.CALLBACK_LEVEl.getId()).append(",").append(MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getId()).append(") and t.create_user_id in (").append(o).append(")").toString();
                if (Objects.nonNull(startTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{startTime}, new String[]{">="});
                }
                if (Objects.nonNull(endTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{endTime}, new String[]{"<="});
                }
                sql = sqlUtil.sqlGroupBy(sql, "t.create_user_id");
                list = sqlUtil.execSqlForMap(sql);
                list = distinctCommon(list, markerIdList, markerMap);
            }
        } else if (MarkStage.LEVEL.equals(stage)) {
            Specification<MarkTaskLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (Objects.nonNull(subject)) {
                    predicates.add(builder.equal(root.get("subject"), subject));
                }
                if (Objects.nonNull(areaCode)) {
                    predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
                }
                if (!CollectionUtils.isEmpty(markerIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("markerId"));
                    for (Long id : markerIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            List<MarkTaskLevel> markTasks = markTaskLevelRepo.findAll(specification);

            if (Objects.nonNull(markTasks) && markTasks.size() > 0) {
                Map<Long, String> markerMap = markTasks.stream().collect(Collectors.toMap(MarkTaskLevel::getMarkerId, MarkTaskLevel::getMarkerName, (dto1, dto2) -> dto1));
                List<Long> markerIdList = markerMap.entrySet().stream().map(Map.Entry::getKey).collect(Collectors.toList());
                Object o = StringUtils.join(markerIdList.toArray(), ",");
                String sql = new StringBuffer("select mu.login_name as userName, count(t.create_user_id) as sumCount,t.create_user_id as userId from mark_log t left join mark_user mu on t.create_user_id = mu.id where t.stage = ").append(stage.ordinal()).append(" and  t.oper_type in (").append(MarkLogOperType.CALLBACK_LEVEl.getId()).append(",").append(MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getId()).append(") and t.create_user_id in (").append(o).append(")").toString();
                if (Objects.nonNull(startTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{startTime}, new String[]{">="});
                }
                if (Objects.nonNull(endTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{endTime}, new String[]{"<="});
                }
                sql = sqlUtil.sqlGroupBy(sql, "t.create_user_id");
                list = sqlUtil.execSqlForMap(sql);
                list = distinctCommon(list, markerIdList, markerMap);
            }
        }
        return list;
    }

    /**
     * 差值和报表
     *
     * @param workId    工作ID
     * @param subject   科目
     * @param areaCode  考区
     * @param startTime 查询时间段（开始时间）
     * @param endTime   查询时间段（结束时间）
     */
    @RequestMapping(value = "distanceReport", method = RequestMethod.GET)
    public List distanceReport(
            @RequestParam Long workId,
            @RequestParam(required = false) MarkStage stage,
            @RequestParam(required = false) Subject subject,
            @RequestParam(required = false) String areaCode,
            @RequestParam(required = false) Long groupId,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime) {
        stage = Objects.isNull(stage) ? MarkStage.LEVEL : stage;
        List list = new ArrayList();
        List<Long> markerIds = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndGroupId(workId, subject, stage, groupId);
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            Specification<MarkTaskRoughLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (Objects.nonNull(subject)) {
                    predicates.add(builder.equal(root.get("subject"), subject));
                }
                if (Objects.nonNull(areaCode)) {
                    predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
                }
                if (!CollectionUtils.isEmpty(markerIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("markerId"));
                    for (Long id : markerIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification);
            if (Objects.nonNull(markTasks) && !markTasks.isEmpty()) {
                Map<Long, String> markerMap = markTasks.stream().collect(Collectors.toMap(MarkTaskRoughLevel::getMarkerId, MarkTaskRoughLevel::getMarkerName, (dto1, dto2) -> dto1));
                List<Long> markerIdList = markerMap.entrySet().stream().map(Map.Entry::getKey).collect(Collectors.toList());
                Object o = StringUtils.join(markerIdList.toArray(), ",");
                String sql = new StringBuffer("select mu.login_name as userName, sum(t.oper_data_after) as sumCount,t.create_user_id as userId from mark_log t left join mark_user mu on t.create_user_id = mu.id where t.stage = ").append(stage.ordinal()).append(" and  t.oper_type = ").append(MarkLogOperType.LEVEl_DIFFERENCE.getId()).append(" and t.create_user_id in (").append(o).append(")").toString();
                if (Objects.nonNull(startTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{startTime}, new String[]{">="});
                }
                if (Objects.nonNull(endTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{endTime}, new String[]{"<="});
                }
                sql = sqlUtil.sqlGroupBy(sql, "t.create_user_id");
                list = sqlUtil.execSqlForMap(sql);
                list = distinctCommon(list, markerIdList, markerMap);
            }
        } else if (MarkStage.LEVEL.equals(stage)) {
            Specification<MarkTaskLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (Objects.nonNull(subject)) {
                    predicates.add(builder.equal(root.get("subject"), subject));
                }
                if (Objects.nonNull(areaCode)) {
                    predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
                }
                if (!CollectionUtils.isEmpty(markerIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("markerId"));
                    for (Long id : markerIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            List<MarkTaskLevel> markTasks = markTaskLevelRepo.findAll(specification);
            if (Objects.nonNull(markTasks) && !markTasks.isEmpty()) {
                Map<Long, String> markerMap = markTasks.stream().collect(Collectors.toMap(MarkTaskLevel::getMarkerId, MarkTaskLevel::getMarkerName, (dto1, dto2) -> dto1));
                List<Long> markerIdList = markerMap.entrySet().stream().map(Map.Entry::getKey).collect(Collectors.toList());
                Object o = StringUtils.join(markerIdList.toArray(), ",");
                String sql = new StringBuffer("select mu.login_name as userName, sum(t.oper_data_after) as sumCount,t.create_user_id as userId from mark_log t left join mark_user mu on t.create_user_id = mu.id where t.stage = ").append(stage.ordinal()).append(" and  t.oper_type = ").append(MarkLogOperType.LEVEl_DIFFERENCE.getId()).append(" and t.create_user_id in (").append(o).append(")").toString();
                if (Objects.nonNull(startTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{startTime}, new String[]{">="});
                }
                if (Objects.nonNull(endTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{endTime}, new String[]{"<="});
                }
                sql = sqlUtil.sqlGroupBy(sql, "t.create_user_id");
                list = sqlUtil.execSqlForMap(sql);
                list = distinctCommon(list, markerIdList, markerMap);
            }
        }
        return list;
    }

    /**
     * 累计误差报表
     *
     * @param workId    工作ID
     * @param subject   科目
     * @param areaCode  考区
     * @param startTime 查询时间段（开始时间）
     * @param endTime   查询时间段（结束时间）
     */
    @RequestMapping(value = "deviationReport", method = RequestMethod.GET)
    public List cumulativeReport(
            @RequestParam Long workId,
            @RequestParam(required = false) MarkStage stage,
            @RequestParam(required = false) Subject subject,
            @RequestParam(required = false) String areaCode,
            @RequestParam(required = false) Long groupId,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime) {
        stage = Objects.isNull(stage) ? MarkStage.LEVEL : stage;
        List list = new ArrayList();
        List<Long> markerIds = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndGroupId(workId, subject, stage, groupId);
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            Specification<MarkTaskRoughLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (Objects.nonNull(subject)) {
                    predicates.add(builder.equal(root.get("subject"), subject));
                }
                if (Objects.nonNull(areaCode)) {
                    predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
                }
                if (!CollectionUtils.isEmpty(markerIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("markerId"));
                    for (Long id : markerIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findAll(specification);
            if (Objects.nonNull(markTasks) && markTasks.size() > 0) {
                Map<Long, String> markerMap = markTasks.stream().collect(Collectors.toMap(MarkTaskRoughLevel::getMarkerId, MarkTaskRoughLevel::getMarkerName, (dto1, dto2) -> dto1));
                List<Long> markerIdList = markerMap.entrySet().stream().map(Map.Entry::getKey).collect(Collectors.toList());
                Object o = StringUtils.join(markerIdList.toArray(), ",");
                String sql = new StringBuffer("select mu.login_name as userName, sum(t.oper_data_after) as sumCount,t.create_user_id as userId from mark_log t left join mark_user mu on t.create_user_id = mu.id where t.stage = ").append(stage.ordinal()).append(" and  t.oper_type = ").append(MarkLogOperType.LEVEl_DEVIATION.getId()).append(" and t.create_user_id in (").append(o).append(")").toString();
                if (Objects.nonNull(startTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{startTime}, new String[]{">="});
                }
                if (Objects.nonNull(endTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{endTime}, new String[]{"<="});
                }
                sql = sqlUtil.sqlGroupBy(sql, "t.create_user_id");
                list = sqlUtil.execSqlForMap(sql);
                list = distinctCommon(list, markerIdList, markerMap);
            }
        } else if (MarkStage.LEVEL.equals(stage)) {
            Specification<MarkTaskLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (Objects.nonNull(subject)) {
                    predicates.add(builder.equal(root.get("subject"), subject));
                }
                if (Objects.nonNull(areaCode)) {
                    predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
                }
                if (!CollectionUtils.isEmpty(markerIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("markerId"));
                    for (Long id : markerIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            List<MarkTaskLevel> markTasks = markTaskLevelRepo.findAll(specification);
            if (Objects.nonNull(markTasks) && markTasks.size() > 0) {
                Map<Long, String> markerMap = markTasks.stream().collect(Collectors.toMap(MarkTaskLevel::getMarkerId, MarkTaskLevel::getMarkerName, (dto1, dto2) -> dto1));
                List<Long> markerIdList = markerMap.entrySet().stream().map(Map.Entry::getKey).collect(Collectors.toList());
                Object o = StringUtils.join(markerIdList.toArray(), ",");
                String sql = new StringBuffer("select mu.login_name as userName, sum(t.oper_data_after) as sumCount,t.create_user_id as userId from mark_log t left join mark_user mu on t.create_user_id = mu.id where t.stage = ").append(stage.ordinal()).append(" and  t.oper_type = ").append(MarkLogOperType.LEVEl_DEVIATION.getId()).append(" and t.create_user_id in (").append(o).append(")").toString();
                if (Objects.nonNull(startTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{startTime}, new String[]{">="});
                }
                if (Objects.nonNull(endTime)) {
                    sql = sqlUtil.sqlConditionAnd(sql, new String[]{"t.create_time"}, new String[]{endTime}, new String[]{"<="});
                }
                sql = sqlUtil.sqlGroupBy(sql, "t.create_user_id");
                list = sqlUtil.execSqlForMap(sql);
                list = distinctCommon(list, markerIdList, markerMap);
            }
        }
        return list;
    }

    /**
     * 求差值公用
     *
     * @param list
     * @param markerIdList
     * @param markerMap
     * @return
     */
    public List distinctCommon(List list, List<Long> markerIdList, Map<Long, String> markerMap) {
        if (Objects.nonNull(list) && list.size() > 0) {
            List<Long> dataIdList = new ArrayList<>();
            for (int i = 0; i < list.size(); i++) {
                Map map = (Map) list.get(i);
                dataIdList.add(Long.parseLong(String.valueOf(map.get("userId"))));
            }
            //求差值id集合
            List<Long> distinctIdList = markerIdList.stream().filter(item -> !dataIdList.contains(item)).collect(Collectors.toList());
            if (distinctIdList.size() > 0) {
                for (Long l : distinctIdList) {
                    Map map = new HashMap();
                    map.put("userName", markerMap.get(l));
                    map.put("sumCount", 0);
                    map.put("userId", l);
                    list.add(map);
                }
            }
        }
        //排序
        Collections.sort(list, (Comparator<Map<String, Object>>) (o1, o2) -> {
            String o1Value = String.valueOf(o1.get("userName"));
            String o2Value = String.valueOf(o2.get("userName"));
            return o1Value.compareTo(o2Value);
        });
        return list;
    }
}
