package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.admin.assembler.WorkOverviewAssembler;
import cn.com.qmth.stmms.ms.admin.dto.WorkOverview;
import cn.com.qmth.stmms.ms.admin.service.WorkService;
import cn.com.qmth.stmms.ms.marking.utils.RandomUtil;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskLevelRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 评卷工作api
 * Created by zhengmin on 2016/10/14.
 */
@RestController
@RequestMapping("api/admin/works")
public class WorkApi {

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private WorkService workService;

    @Autowired
    private WorkOverviewAssembler workOverviewAssembler;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Autowired
    RandomUtil randomUtil;

    /**
     * 列表
     *
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public List<Work> list() {
        Sort sort = new Sort(Sort.Direction.DESC, "id");
        return workRepo.findAll(sort);
    }

    /**
     * 单个信息
     *
     * @param work
     * @return
     */
    @RequestMapping(value = "{work}", method = RequestMethod.GET)
    public Work getOne(@PathVariable Work work) {
        if (work.isActive()) {
            randomUtil.initRandom(work.getId());
        }
        int count = markTaskLevelRepo.countByWorkIdAndStageAndResultNotNull(work.getId(), MarkStage.LEVEL);
        work.setModifyOtherVal(count == 0);
        return work;
    }

    /**
     * 新增工作
     *
     * @param work work对象
     */
    @PostMapping()
    public void create(@RequestBody Work work) {
        workService.save(work);
        //生成随机数
        randomUtil.initRandom(work.getId());
    }

    /**
     * 修改
     *
     * @param workId
     * @param work
     * @return
     */
    @Transactional
    @RequestMapping(value = "{workId}", method = RequestMethod.PUT)
    public Work update(@PathVariable Long workId, @RequestBody Work work) {
        List<MarkSubject> subjects = markSubjectRepo.findAllByWorkId(workId);
        long count = subjects.stream().filter(m -> m.getStage().ordinal() != 0).count();
        List<Level> levels = work.getLevels();
        if (count > 0) {
            List<Level> levelList = levelRepo.findByWorkId(workId);
            Map<String, Level> levelMap = levelList.stream().collect(Collectors.toMap(Level::getCode, Function.identity()));
            if (levels.size() != levelList.size()) {
                throw new RuntimeException("开始分档后，不允许增加或删除档位");
            }
            for (Level level : levels) {
                Level oldLevel = levelMap.get(level.getCode());
                if (oldLevel == null) {
                    throw new RuntimeException("开始分档后，不允许增加或删除档位");
                }
                if (level.getMinScore().intValue() != oldLevel.getMinScore().intValue()
                        || level.getMaxScore().intValue() != oldLevel.getMaxScore().intValue()
                        || level.getIntervalScore() != oldLevel.getIntervalScore()
                        || level.getWeight().intValue() != oldLevel.getWeight().intValue()
                        || !level.getLevelType().equals(oldLevel.getLevelType())) {
                    throw new RuntimeException("开始分档后，只允许修改阈值");
                }
            }
        }

        levelRepo.deleteByWorkId(work.getId());

        //根据A~Z排序后，重新设置levelValue

        AtomicInteger ai = new AtomicInteger(0);
        levels.stream().sorted(Comparator.comparing(Level::getCode)).map(l -> {
            l.setId(null);
            l.setLevelValue(ai.getAndIncrement());
            return l;
        }).collect(Collectors.toList());
        work.setLevels(levels);
        work.setCreatedOn(new Date());
        return workRepo.save(work);
    }

    /**
     * 激活为当前，采集和评卷用户登录影响
     *
     * @param workId
     */
    @RequestMapping(value = "{workId}", method = RequestMethod.PATCH)
    public void active(@PathVariable Long workId) {
        List<Work> works = workRepo.findAll();
        for (Work w : works) {
            w.setActive(false);
            if (w.getId().equals(workId)) {
                w.setActive(true);
                randomUtil.initRandom(workId);
            }
        }
        workRepo.save(works);
    }

    /**
     * 删除
     *
     * @param workId
     */
    @RequestMapping(value = "{workId}", method = RequestMethod.DELETE)
    public void remove(@PathVariable Long workId) {
        workService.delete(workId);
    }

    //////////////////////    levels

    /**
     * 档位设定列表
     *
     * @param workId
     * @return
     */
    @RequestMapping(value = "{workId}/levels", method = RequestMethod.GET)
    public List<Level> list(@PathVariable Long workId) {
        return levelRepo.findByWorkId(workId);
    }

    /**
     * 删除档位
     *
     * @param workId
     * @param levelId
     */
    @RequestMapping(value = "{workId}/levels/{levelId}", method = RequestMethod.DELETE)
    public void deleteLevel(@PathVariable Long workId, @PathVariable Long levelId) {
        levelRepo.delete(levelId);
    }
    //////////////////////   overview

    /**
     * 总览，各统计数据
     *
     * @param work
     */
    @GetMapping("{work}/overview")
    public WorkOverview overview(@PathVariable Work work) {
        return workOverviewAssembler.toDTO(work);
    }

}
