package cn.com.qmth.stmms.ms.admin.exporter;

import cn.com.qmth.stmms.ms.admin.dto.MarkLogCollectDTO;
import cn.com.qmth.stmms.ms.admin.dto.MarkLogOperDTO;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExportUtils;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.MarkLog;
import cn.com.qmth.stmms.ms.core.domain.MarkLogOperType;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.repository.MarkLogRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@RestController
@RequestMapping("api/export/log")
public class MarkLogExporter {

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Autowired
    MarkLogRepo markLogRepo;

    @Resource
    MarkSubjectRepo markSubjectRepo;

    /**
     * 导出操作日志
     *
     * @param workId
     * @param subject
     * @param response
     */
    @GetMapping("/oper")
    public void oper(@RequestParam(required = false) Long workId,
                     @RequestParam(required = false) String subject,
                     @RequestParam(required = false) String examNumber,
                     @RequestParam(required = false) String studentName,
                     @RequestParam(required = false) Integer operType,
                     @RequestParam(required = false) String startTime,
                     @RequestParam(required = false) String endTime,
                     HttpServletResponse response) {
        Specification<MarkLog> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(subject)) {
                predicates.add(cb.equal(root.get("subject").as(String.class), subject));
            }
            if (Objects.nonNull(examNumber) && !examNumber.equals("")) {
                predicates.add(cb.equal(root.get("examNumber").as(String.class), examNumber));
            }
            if (Objects.nonNull(studentName) && !studentName.equals("")) {
                predicates.add(cb.like(root.get("studentName").as(String.class), "%" + studentName + "%"));
            }
            if (Objects.nonNull(operType)) {
                predicates.add(cb.equal(root.get("operType").as(Integer.class), operType));
            }
            if (Objects.nonNull(startTime)) {
                //大于或等于传入时间
                predicates.add(cb.greaterThanOrEqualTo(root.get("createTime").as(String.class), startTime));
            }
            if (Objects.nonNull(endTime)) {
                //小于或等于传入时间
                predicates.add(cb.lessThanOrEqualTo(root.get("createTime").as(String.class), endTime));
            }
            if (Objects.nonNull(workId)) {
                predicates.add(cb.equal(root.get("workId").as(Long.class), workId));
            }

            List<Map> mapList = MarkLogOperType.listTypes();
            if (!mapList.isEmpty()) {
                CriteriaBuilder.In<Object> in = cb.in(root.get("operType"));
                for (Map map : mapList) {
                    in.value(map.get("id"));
                }
                predicates.add(cb.and(cb.and(in)));
            }
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        List<MarkLog> result = markLogRepo.findAll(specification);
        List<MarkLogOperDTO> markLogOperDTOS = new ArrayList<>();
        boolean roughLevelEnable = ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1;
        List<MarkSubject> subjectList = markSubjectRepo.findAllByWorkId(workId);
        Map<String, String> subjectMap = subjectList.stream().collect(Collectors.toMap(x -> x.getSubject().name(), y -> y.getName()));
        //科目名称
        for (MarkLog markLog : result) {
            MarkLogOperDTO markLogOperDTO = new MarkLogOperDTO();
            markLogOperDTO.setWorkName(markLog.getWorkName());
            markLogOperDTO.setCreateUserName(markLog.getCreateUserName());
            MarkStage stage = markLog.getStage();
            if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                markLogOperDTO.setStageName("粗分档阶段");
            } else if (MarkStage.LEVEL.equals(stage)) {
                markLogOperDTO.setStageName(roughLevelEnable ? "细分档阶段" : "分档阶段");
            } else if (MarkStage.SCORE.equals(stage)) {
                markLogOperDTO.setStageName("打分阶段");
            } else {
                markLogOperDTO.setStageName("无");
            }
            markLogOperDTO.setCreateRole(markLog.getCreateRole());
            String subjectName = subjectMap.containsKey(markLog.getSubject()) ? subjectMap.get(markLog.getSubject()) : markLog.getSubject();
            markLogOperDTO.setSubject(subjectName);
            markLogOperDTO.setExamNumber(markLog.getExamNumber());
            markLogOperDTO.setStudentName(markLog.getStudentName());
            markLogOperDTO.setOperType(markLog.getOperType());
            markLogOperDTO.setOperDataBefore(Objects.isNull(markLog.getOperDataBefore()) ? "" : markLog.getOperDataBefore());
            markLogOperDTO.setOperDataAfter(Objects.isNull(markLog.getOperDataAfter()) ? "" : markLog.getOperDataAfter());
            markLogOperDTO.setCreateTime(sdf.format(markLog.getCreateTime()));
            markLogOperDTOS.add(markLogOperDTO);
        }
        ExportUtils.exportEXCEL("操作日志导出", MarkLogOperDTO.class, markLogOperDTOS, response);
    }


    /**
     * 导出采集日志
     *
     * @param workId
     * @param subject
     * @param response
     */
    @GetMapping("collect")
    public void collect(@RequestParam(required = false) Long workId,
                        @RequestParam(required = false) String subject,
                        @RequestParam(required = false) String examNumber,
                        @RequestParam(required = false) String studentName,
                        @RequestParam(required = false) String createUserId,
                        @RequestParam(required = false) String startTime,
                        @RequestParam(required = false) String endTime,
                        HttpServletResponse response) {
        Specification<MarkLog> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(subject)) {
                predicates.add(cb.equal(root.get("subject").as(String.class), subject));
            }
            if (Objects.nonNull(examNumber) && !examNumber.equals("")) {
                predicates.add(cb.equal(root.get("examNumber").as(String.class), examNumber));
            }
            if (Objects.nonNull(studentName) && !studentName.equals("")) {
                predicates.add(cb.like(root.get("studentName").as(String.class), "%" + studentName + "%"));
            }
            if (Objects.nonNull(createUserId)) {
                predicates.add(cb.equal(root.get("createUserId").as(String.class), createUserId));
            }
            if (Objects.nonNull(startTime)) {
                //大于或等于传入时间
                predicates.add(cb.greaterThanOrEqualTo(root.get("createTime").as(String.class), startTime));
            }
            if (Objects.nonNull(endTime)) {
                //小于或等于传入时间
                predicates.add(cb.lessThanOrEqualTo(root.get("createTime").as(String.class), endTime));
            }
            if (Objects.nonNull(workId)) {
                predicates.add(cb.equal(root.get("workId").as(Long.class), workId));
            }
            //采集端
            predicates.add(cb.equal(root.get("operType").as(Integer.class), MarkLogOperType.COLLECT.getId()));
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        List<MarkLog> result = markLogRepo.findAll(specification);
        List<MarkLogCollectDTO> markLogCollectDTOS = new ArrayList<>();
        //科目名称
        for (MarkLog markLog : result) {
            MarkLogCollectDTO markLogCollectDTO = new MarkLogCollectDTO();
            markLogCollectDTO.setWorkName(markLog.getWorkName());
            markLogCollectDTO.setSubject(markLog.getSubject());
            markLogCollectDTO.setStudentName(markLog.getStudentName());
            markLogCollectDTO.setExamNumber(markLog.getExamNumber());
            markLogCollectDTO.setCreateUserName(markLog.getCreateUserName());
            markLogCollectDTO.setCreateTime(sdf.format(markLog.getCreateTime()));
            markLogCollectDTO.setRemark(Objects.isNull(markLog.getRemark()) ? "" : markLog.getRemark());
            markLogCollectDTO.setLoginTime(sdf.format(markLog.getLoginTime()));
            markLogCollectDTOS.add(markLogCollectDTO);
        }
        ExportUtils.exportEXCEL("采集日志导出", MarkLogCollectDTO.class, markLogCollectDTOS, response);
    }
}
