package cn.com.qmth.stmms.ms.admin.importer;

import cn.com.qmth.stmms.ms.admin.dto.StudentDTO;
import cn.com.qmth.stmms.ms.admin.service.DataUploadService;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelError;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExportUtils;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * 考生相关文件导入Controller
 */
@RestController
@RequestMapping("api/import/students")
public class StudentImporter {

    @Resource
    private DataUploadService dataUploadService;

    /**
     * 考生导入
     *
     * @param workId   工作Id
     * @param isAbsent 是否缺考
     * @param file     考生文件
     */
    @PostMapping
    public List<ExcelError> batchAll(@RequestParam Long workId,
                                     @RequestParam(required = false, defaultValue = "false") boolean isAbsent,
                                     @RequestParam MultipartFile file) throws Exception {
        return dataUploadService.uploadStudents(workId, isAbsent, file.getInputStream());
    }

    /**
     * 试评考生导入
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param file    试评文件
     */
    @PostMapping("/batchAllForTrial")
    public List<ExcelError> batchAllForTrial(
            @RequestParam Long workId,
            @RequestParam Subject subject,
            @RequestParam MultipartFile file)
            throws Exception {
        return dataUploadService.uploadStudentsForTrial(workId, subject, file.getInputStream());
    }

    @GetMapping
    public void download(HttpServletResponse response) {
        List<StudentDTO> studentDTOs = new ArrayList<>();
        StudentDTO dto = new StudentDTO();

        dto.setExamNumber("5360001");
        dto.setName("张三");
        dto.setAreaName("四川音乐学院");
        dto.setAreaCode("1");
        dto.setExamRoom("第1考场");
        dto.setSourceName("四川");

        studentDTOs.add(dto);
        ExportUtils.exportEXCEL("考生导入模板", StudentDTO.class, studentDTOs, response);
    }

    /**
     * 导入考生关联信息
     *
     * @param workId 工作ID
     * @param file   关联考生文件
     */
    @PostMapping("/relateStudent")
    public List<ExcelError> relateStudent(@RequestParam Long workId,
                                          @RequestParam MultipartFile file) throws Exception {
        return dataUploadService.uploadStudentRelate(workId, file.getInputStream());
    }

    /**
     * 导入分组学生名单
     *
     * @param subject 科目
     * @param stage   阶段
     * @param groupId 分组
     * @param file    考生名单文件
     */
    @PostMapping("/import_group_student")
    public List<ExcelError> groupStudent(@RequestParam Subject subject,
                                         @RequestParam MarkStage stage,
                                         @RequestParam Long groupId,
                                         @RequestParam MultipartFile file) throws Exception {
        return dataUploadService.uploadGroupStudent(subject, stage, groupId, file.getInputStream());
    }
}
