package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.admin.dto.StudentDTO;
import cn.com.qmth.stmms.ms.admin.dto.StudentRelateDTO;
import cn.com.qmth.stmms.ms.marking.utils.RandomUtil;
import cn.com.qmth.stmms.ms.commons.config.ImageCompressionConfig;
import cn.com.qmth.stmms.ms.commons.config.ImageConfig;
import cn.com.qmth.stmms.ms.commons.config.ImageServerConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.constant.SystemConstant;
import cn.com.qmth.stmms.ms.commons.utils.MD5Util;
import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelError;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelReader;
import cn.com.qmth.stmms.ms.commons.utils.excel.ExcelReaderHandle;
import cn.com.qmth.stmms.ms.commons.utils.image.ImageCompression;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.*;
import java.lang.reflect.Field;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 数据上传服务 Created by zhengmin on 2016/11/18.
 */
@Service
public class DataUploadService {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(DataUploadService.class);

    @Autowired
    private StudentRepo studentRepo;

    @Autowired
    private ExamQuestionRepo examQuestionRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    private ImageCompressionConfig compressionConfig;

    @Autowired
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Autowired
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private LevelRepo levelRepo;

    @Resource
    private MarkUserRepo markUserRepo;

    @Resource
    private MarkerGroupRepo markerGroupRepo;

    @Autowired
    ImageConfig imageConfig;

    @Autowired
    MarkSubjectRepo markSubjectRepo;

    @Resource
    MarkerGroupStudentRepo markerGroupStudentRepo;

    @Autowired
    WorkRepo workRepo;

    @Autowired
    RandomUtil randomUtil;

    @Autowired
    ImageServerConfig imageServerConfig;

    /**
     * 上传考生试卷数据
     *
     * @param studentId 考生id
     * @param subject   科目
     * @param in        图片文件输入流
     * @param isManual  是否手工绑定
     * @throws IOException
     */
    @Transactional
    public void uploadPaper(Long studentId, Subject subject, InputStream in, boolean isManual) throws Exception {
        long start = System.currentTimeMillis();
        LOGGER.info("准备生成裁切原图和缩略图:{}", start);
        File imageFileTemp = null;
        try {
            FileInputStream sheetIn = null;
            FileInputStream slicein = null;
            Student student = studentRepo.findOne(studentId);
            if (imageConfig.isCustomSubject()) {
                subject = Subject.CUSTOM;
            }
            if (ParamCache.collectConfigMap.get(String.valueOf(student.getWorkId())).getImageEncrypt() == 1) {
                //保存裁切原图+文件名加密
                String imageDir = systemConfig.getImageDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode();
                File out = new File(imageDir);
                if (!out.exists()) {
                    out.mkdirs();
                }

                String imageMd5 = MD5Util.getImageRuleMd5(student.getWorkId(), subject.ordinal(), student.getAreaCode(), student.getExamNumber(), student.getId());
                File imageFile = new File(imageDir + File.separator + imageMd5 + ".jpg");
                imageFileTemp = new File(imageDir + File.separator + imageMd5 + "temp.jpg");
                LOGGER.info("image存放目录:{}", imageFile.getPath());
                OutputStream outputStream = new FileOutputStream(imageFile);
                OutputStream outputStreamTemp = new FileOutputStream(imageFileTemp);
                int index = 0;
                byte[] bytes = new byte[1024];
                byte[] bytesEnc = new byte[1024];
                while ((index = in.read(bytes)) != -1) {
                    //将字节数组的数据全部写入到输出流中
                    for (int i = 0; i < index; i++) {
                        //通过异或运算加密
                        bytesEnc[i] = (byte) (bytes[i] ^ SystemConstant.SECRET_KEY);
                    }
                    outputStream.write(bytesEnc, 0, index);
                    outputStreamTemp.write(bytes, 0, index);
                }
                in.close();
                outputStream.flush();
                outputStream.close();
                outputStreamTemp.flush();
                outputStreamTemp.close();

                //生成缩略图
                String thumbDir = systemConfig.getThumbDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode();
                File thumb = new File(thumbDir);
                if (!thumb.exists()) {
                    thumb.mkdirs();
                }

                BufferedImage bufferedImage = ImageCompression.compress(imageFileTemp, compressionConfig);
                File thumbFile = new File(thumbDir + File.separator + imageMd5 + ".jpg");
                LOGGER.info("thumb存放目录:{}", thumbFile.getPath());
                OutputStream outputStream1 = new FileOutputStream(thumbFile);
                ByteArrayOutputStream os = new ByteArrayOutputStream();
                ImageIO.write(bufferedImage, "jpg", os);
                InputStream in1 = new ByteArrayInputStream(os.toByteArray());

                SystemConstant.writeStream(in1, outputStream1);
                long end = System.currentTimeMillis();
                LOGGER.info("生成原图和缩略图耗时：{}", (end - start) / 1000 + "s");

                in1.close();
                outputStream1.flush();
                outputStream1.close();
                sheetIn = new FileInputStream(imageFile.getPath());
                slicein = new FileInputStream(thumbFile.getPath());
            } else {
                String savePath = systemConfig.getImageDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode();
                File out = new File(savePath);
                if (!out.exists()) {
                    out.mkdirs();
                }
                File outFile = new File(savePath + File.separator + student.getExamNumber() + ".jpg");
                if (!outFile.exists()) {
                    outFile.createNewFile();
                }
                OutputStream output = new FileOutputStream(outFile);
                byte[] buffer = new byte[in.available()];
                in.read(buffer);
                output.write(buffer);
                output.close();

                // 生成缩略图
                String thumbDir = systemConfig.getThumbDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode();
                File thumb = new File(thumbDir);
                if (!thumb.exists()) {
                    thumb.mkdirs();
                }
                BufferedImage bufferedImage = ImageCompression.compress(outFile, compressionConfig);
                String thumbFileName = thumbDir + File.separator + student.getExamNumber() + ".jpg";
                ImageIO.write(bufferedImage, "jpg", new File(thumbFileName));

                sheetIn = new FileInputStream(savePath + File.separator + student.getExamNumber() + ".jpg");
                slicein = new FileInputStream(thumbFileName);
            }

            List<Paper> allPapers = paperRepo.findByWorkIdAndSubjectAndTest(student.getWorkId(), subject, TrialEnum.DEFAULT.getId());

            Paper exist = allPapers.stream().filter(m -> student.getExamNumber().equals(m.getExamNumber())).collect(Collectors.toList()).get(0);
            String sheetMD5 = DigestUtils.md5Hex(sheetIn);
            String sliceMD5 = DigestUtils.md5Hex(slicein);
            sheetIn.close();
            slicein.close();
            if (exist != null) {
                exist.setManual(isManual);
                exist.setUploadedOn(new Date());
                exist.setSheetMD5(sheetMD5);
                exist.setSliceMD5(sliceMD5);
                paperRepo.save(exist);
            } else {
                ExamQuestion examQuestion = examQuestionRepo.findByWorkIdAndSubjectAndAreaCode(student.getWorkId(), subject, student.getAreaCode());
                Long random = randomUtil.randomPaper(student.getWorkId());
                Paper paper = new Paper(student.getWorkId(), subject, examQuestion, student, false, random);
                paper.setSheetMD5(sheetMD5);
                paper.setSliceMD5(sliceMD5);
                paper.setExamRoom(student.getExamRoom());
                paperRepo.save(paper);
                // 更新科目上传状态
                /**
                 * Long count =
                 * paperRepo.countByWorkIdAndExamNumber(student.getWorkId(),student.
                 * getExamNumber()); if(count == Subject.values().length){
                 * student.setUpload(true); studentRepo.save(student); }
                 */
                synchronized (this) {
                    String uploadStatus = student.getUploadStatus();
                    String replace = subject.toString() + ":1";
                    String regx = subject.toString() + ":[0-1]";
                    String newStatus = uploadStatus.replaceFirst(regx, replace);
                    student.setUploadStatus(newStatus);
                    student.setAbsent(false);
                    studentRepo.save(student);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            //原图删除
            if (Objects.nonNull(imageFileTemp) && imageFileTemp.exists()) {
                imageFileTemp.delete();
            }
        }
    }

    @Transactional
    public List<ExcelError> uploadStudents(Long workId, final boolean isAbsent, InputStream inputStream) throws Exception {
        //有采集数据，不能导入
        List<Paper> papers = paperRepo.findByWorkId(workId);
        if (papers != null && papers.size() > 0) {
            throw new Exception("已有采集数据，不能导入考生数据");
        }

        boolean roughLevelEnable = ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1;

        List<Student> studentList = new ArrayList<>();

        ExcelReader excelReader = new ExcelReader(StudentDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {

            @Override
            public ExcelError handle(Object obj) {
                try {
                    StudentDTO dto = (StudentDTO) obj;
                    //校验excel文件中数据是否完整
                    if (!checkObjFieldIsNotNull(dto)) {
                        throw new RuntimeException("考生信息缺失");
                    }

                    String examNumber = dto.getExamNumber();
                    //如果没有设置区域代码，用准考证前2位作为区域代码
                    String areaCode = null;
                    if (dto.getAreaCode() == null || dto.getAreaCode().isEmpty()) {
                        areaCode = examNumber.substring(0, 2);
                        dto.setAreaCode(areaCode);
                    }

                    long isExist = examQuestionRepo.countByWorkIdAndAreaCode(workId, dto.getAreaCode());
                    if (isExist == 0) {
                        examQuestionRepo.deleteByWorkIdAndAreaCode(workId, dto.getAreaCode());
                        if (!imageConfig.isCustomSubject()) {
                            for (Subject subject : Subject.values()) {
                                ExamQuestion question = new ExamQuestion(dto.getAreaCode(), subject, workId, dto.getAreaName());
                                examQuestionRepo.save(question);
                            }
                        } else {
                            MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + Subject.CUSTOM.name());
                            ExamQuestion question = new ExamQuestion(dto.getAreaCode(), Subject.CUSTOM, markSubject.getName(), workId, dto.getAreaName());
                            examQuestionRepo.save(question);
                        }
                    }
                    Student student = studentRepo.findByWorkIdAndExamNumber(workId, dto.getExamNumber());
                    if (student == null && !isAbsent) {
                        student = new Student(dto.getName(), dto.getExamNumber(), dto.getAreaName(), dto.getAreaCode(),
                                workId, dto.getExamRoom(), dto.getSourceName(), dto.getSchool());
                        if (imageConfig.isCustomSubject()) {
                            student.setUploadStatus("CUSTOM:0");
                        }
//                        studentRepo.save(student);
                        studentList.add(student);
                    } else if (student != null) {
                        if (isAbsent) {
                            student.setAbsent(isAbsent);
                            if (!imageConfig.isCustomSubject()) {
                                student.setUploadStatus("SX:0,SC:0,SM:0");
                            } else {
                                student.setUploadStatus("CUSTOM:0");
                            }

                            /**
                             * 查询该考生所有paper,若paper无任务则删除，有任务就保留
                             */
                            List<Paper> papers = paperRepo.findByWorkIdAndExamNumber(workId, student.getExamNumber());
                            if (!papers.isEmpty()) {
                                for (Paper paper : papers) {
                                    long count;
                                    if (roughLevelEnable) {
                                        count = markTaskRoughLevelRepo.countByPaperId(paper.getId());
                                    } else {
                                        count = markTaskLevelRepo.countByPaperId(paper.getId());
                                    }
                                    if (count == 0) {
                                        paperRepo.delete(paper);
                                    }
                                }
                            }
                        } else {
                            student.setName(dto.getName());
                            student.setAreaCode(dto.getAreaCode());
                            student.setAreaName(dto.getAreaName());
                            student.setExamRoom(dto.getExamRoom());
                            student.setSourceName(dto.getSourceName());
                            student.setSchool(dto.getSchool());
                        }
//                        studentRepo.save(student);
                        studentList.add(student);
                    }
                    return null;
                } catch (RuntimeException e) {
                    //手动回滚
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    ExcelError excelError = new ExcelError();
                    excelError.setExcelErrorType(e.getMessage());
                    return excelError;
                }
            }
        });
        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new Exception(errors);
        }

        if (!CollectionUtils.isEmpty(studentList)) {
            List<Student> data = new ArrayList<>();
            for (Student stu : studentList) {
                if (data.size() == 2000) {
                    studentRepo.save(data);
                    data.clear();
                }
                data.add(stu);
            }
            //将剩下的数据也导入
            if (!data.isEmpty()) {
                studentRepo.save(data);
            }
        }
        return excelErrors;
    }

    /**
     * 试评考生导入
     *
     * @param workId      工作ID
     * @param subject     科目
     * @param inputStream 文件输入流
     */
    @Transactional
    public List<ExcelError> uploadStudentsForTrial(Long workId, Subject subject, InputStream inputStream) {
        Work work = workRepo.findOne(workId);
        if (Objects.isNull(work)) {
            throw new RuntimeException("没有此工作区");
        }
        long count = paperRepo.countByWorkIdAndSubject(workId, subject);
        if (count == 0) {
            throw new RuntimeException("没有采集数据，不能导入试评数据");
        }
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        if (markSubject.getTest() == TrialEnum.START_TRIAL.getId()) {
            throw new RuntimeException("已经开始试评，请先结束试评再重新导入数据");
        }

        List<Level> levels = levelRepo.findByWorkId(workId);
        if (levels == null || levels.isEmpty()) {
            throw new RuntimeException("没有设定档位");
        }

        List<MarkUser> markers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        List<MarkUser> levelMarkers = markers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_ALL || m.getMarkRight() == MarkRight.ALLOW_LEVELING).collect(Collectors.toList());
        if (levelMarkers.isEmpty()) {
            throw new RuntimeException("没有设定评卷员");
        }

        List<MarkUser> markerLeader = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARK_LEADER);
        if (markerLeader == null || markerLeader.isEmpty()) {
            throw new RuntimeException("没有设定科组长");
        }

        MarkStage stage = markSubject.getStage();
        if (MarkStage.INIT.equals(markSubject.getStage())) {
            if (ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1) {
                stage = MarkStage.ROUGH_LEVEL;
            } else {
                stage = MarkStage.LEVEL;
            }
        }

        List<MarkerGroup> markerGroups = markerGroupRepo.findByWorkIdAndSubjectAndStage(workId, subject, stage);
        if (markerGroups == null || markerGroups.isEmpty()) {
            throw new RuntimeException("评卷员没有进行分组");
        }

        //更新科目
        markSubject.setTest(TrialEnum.INIT.getId());
        markSubjectRepo.save(markSubject);
        // 更新试卷试评状态(把导入考生数据状态更新为默认状态,等待重新导入数据)
        paperRepo.updateTestByWorkId(workId, TrialEnum.INIT.getId(), TrialEnum.DEFAULT.getId());

        List<Student> students = studentRepo.findByWorkId(workId);
        Map<String, Student> studentMap = students.stream().collect(Collectors.toMap(Student::getExamNumber, Function.identity()));

        List<Paper> papers = paperRepo.findByWorkId(workId);
        Map<String, Paper> paperMap = papers.stream().collect(Collectors.toMap(m -> m.getExamNumber() + "#" + m.getSubject() + "#" + m.getAreaCode(), Function.identity()));

        ExcelReader excelReader = new ExcelReader(StudentDTO.class);
        List<Paper> paperList = new ArrayList<>();
        List<ExcelError> excelErrors = excelReader.reader(inputStream, obj -> {
            try {
                StudentDTO dto = (StudentDTO) obj;
                //校验excel文件中数据是否完整
                if (!checkObjFieldIsNotNull(dto)) {
                    throw new RuntimeException("考生信息缺失");
                }
                //试评学生
                Student student = studentMap.get(dto.getExamNumber());
                if (student == null) {
                    return null;
                }
                if (!student.getUploadStatus().contains(subject + ":1")) {
                    return null;
                }

                Paper paper = paperMap.get(student.getExamNumber() + "#" + subject + "#" + dto.getAreaCode());
                if (Objects.isNull(paper) || paper.getIsMissing()) {
                    return null;
                }
                paperList.add(paper);
            } catch (Exception e) {
                ExcelError excelError = new ExcelError();
                excelError.setExcelErrorType(e.getMessage());
                return excelError;
            }
            return null;
        });
        if (!CollectionUtils.isEmpty(paperList)) {
            List<Paper> data = new ArrayList<>();
            //500条提交一次
            for (Paper paper : paperList) {
                paper.setTest(TrialEnum.INIT.getId());
                if (data.size() == 1000) {
                    paperRepo.save(data);
                    data.clear();
                }
                data.add(paper);
            }
            //将剩下的数据也导入
            if (!data.isEmpty()) {
                paperRepo.save(data);
            }
        }
        if (excelErrors != null && !excelErrors.isEmpty()) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new RuntimeException(errors);
        }
        return excelErrors;
    }

    /**
     * 清空考生数据 有上传数据无法清空
     *
     * @param workId
     */
    @Transactional
    public void clearStudents(Long workId) {
        Long isExist = paperRepo.countByWorkId(workId);
        if (isExist > 0) {
            throw new RuntimeException("有考生已上传，无法清空");
        }
        studentRepo.deleteByWorkId(workId);
        examQuestionRepo.deleteByWorkId(workId);
    }

    @Transactional
    public Paper savePaper(Student student, Subject subject, boolean isManual, String level, Long scanUserId) throws Exception {
        ExamQuestion examQuestion = examQuestionRepo.findByWorkIdAndSubjectAndAreaCode(student.getWorkId(), subject, student.getAreaCode());

        List<Paper> allPapers = paperRepo.findByWorkIdAndSubjectAndTest(student.getWorkId(), subject, TrialEnum.DEFAULT.getId());
        List<Paper> filterList = allPapers.stream().filter(m -> student.getExamNumber().equals(m.getExamNumber())).collect(Collectors.toList());
        Paper paper = filterList.isEmpty() ? null : filterList.get(0);
        if (paper == null) {
            Long random = randomUtil.randomPaper(student.getWorkId());
            paper = new Paper(student.getWorkId(), subject, examQuestion, student, isManual, random);
        }
        paper.setUploadedOn(new Date());
        paper.setManual(isManual);

        // 采集员ID
        paper.setScanUserId(scanUserId);
        paper.setSortNum(0);

        if (Objects.nonNull(level) && !"".equals(level)) {
            paper.setLevel(level);
            //默认给1，与实际分档批次区分
            paper.setBatchNo(1L);
        }

        if (!imageServerConfig.isAliyunOss()) {
            String sheetPath = null;
            String slicePath = null;
            String fileName = student.getExamNumber();
            //0：随机码 1：考号
            if (ParamCache.collectConfigMap.get(String.valueOf(student.getWorkId())).getNameRule() == 1) {
                fileName = MD5Util.getImageRuleMd5(student.getWorkId(), subject.ordinal(), student.getAreaCode(), student.getExamNumber(), student.getId());
            }
            //图片是否加密
            Integer imageEnc = Optional.ofNullable(ParamCache.collectConfigMap.get(String.valueOf(student.getWorkId())).getImageEncrypt()).orElse(0);
            if (imageEnc == 1) {
                String imageMd5 = MD5Util.getImageRuleMd5(student.getWorkId(), subject.ordinal(), student.getAreaCode(), student.getExamNumber(), student.getId());
                sheetPath = systemConfig.getSheetDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode() + File.separator + imageMd5 + ".jpg";
                slicePath = systemConfig.getImageDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode() + File.separator + imageMd5 + ".jpg";
            } else {
                sheetPath = systemConfig.getSheetDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode() + File.separator + fileName + ".jpg";
                slicePath = systemConfig.getImageDir() + File.separator + student.getWorkId() + File.separator + subject
                        + File.separator + student.getAreaCode() + File.separator + fileName + ".jpg";
            }
            FileInputStream sheetIn = new FileInputStream(sheetPath);
            FileInputStream slicein = new FileInputStream(slicePath);
            String sheetMD5 = DigestUtils.md5Hex(sheetIn);
            String sliceMD5 = DigestUtils.md5Hex(slicein);
            paper.setSheetMD5(sheetMD5);
            paper.setSliceMD5(sliceMD5);
            sheetIn.close();
            slicein.close();
        }
        paperRepo.save(paper);
        synchronized (this) {
            String uploadStatus = student.getUploadStatus();
            String replace = subject.toString() + ":1";
            String regx = subject.toString() + ":[0-1]";
            String newStatus = uploadStatus.replaceFirst(regx, replace);
            student.setUploadStatus(newStatus);
            studentRepo.save(student);
        }
        return paper;
    }

    @Transactional
    public List<ExcelError> uploadStudentRelate(Long workId, InputStream inputStream) throws Exception {
        //已进入分档，不能导入
        List<MarkSubject> list = markSubjectRepo.findAllByWorkIdAndStageNot(workId, MarkStage.INIT);
        if (list != null && list.size() > 0) {
            throw new Exception("已进入分档或者打分阶段，不能导入关联考生数据");
        }

        ExcelReader excelReader = new ExcelReader(StudentRelateDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, obj -> {
            try {
                StudentRelateDTO dto = (StudentRelateDTO) obj;
                //校验excel文件中数据是否完整
                if (!checkObjFieldIsNotNull(dto)) {
                    throw new RuntimeException("考生信息缺失");
                }

                String examNumber = dto.getExamNumber();
                //如果没有设置区域代码，用准考证前2位作为区域代码
                String areaCode = null;
                if (dto.getAreaCode() == null || dto.getAreaCode().isEmpty()) {
                    areaCode = examNumber.substring(0, 2);
                    dto.setAreaCode(areaCode);
                }

                Student student = studentRepo.findByWorkIdAndExamNumber(workId, dto.getExamNumber());
                if (student == null) {
                    throw new RuntimeException("考生不存在");
                }
                //检验其它信息是否一致
                if (!Objects.equals(student.getName(), dto.getName())
                        || !Objects.equals(student.getAreaName(), dto.getAreaName())
                        || !Objects.equals(student.getExamRoom(), dto.getExamRoom())
                        || !Objects.equals(student.getSourceName(), dto.getSourceName())) {
                    throw new RuntimeException("考生信息与系统数据不匹配");
                }
                //设置关联考号
                student.setRelateExamNumber(dto.getRelateExamNumber());
                studentRepo.save(student);
                return null;
            } catch (RuntimeException e) {
                //手动回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                ExcelError excelError = new ExcelError();
                excelError.setExcelErrorType(e.getMessage());
                return excelError;
            }
        });
        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new Exception(errors);
        }
        return excelErrors;
    }

    /**
     * 校验学生导入所有属性
     *
     * @param obj
     * @return
     */
    private boolean checkObjFieldIsNotNull(Object obj) {
        for (Field field : obj.getClass().getDeclaredFields()) {
            if (Objects.equals("id", field.getName()) || Objects.equals("test", field.getName()) || Objects.equals("workId", field.getName())) {
                continue;
            }
            field.setAccessible(true);
            try {
                if (field.get(obj) == null) {
                    return false;
                }
            } catch (IllegalAccessException e) {
                return false;
            }
        }
        return true;
    }

    @Transactional
    public List<ExcelError> uploadGroupStudent(Subject subject, MarkStage stage, Long groupId, InputStream inputStream) throws Exception {
        Long workId = ServletUtil.getWordId();

        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        if (stage.equals(MarkStage.SCORE)) {
            throw new RuntimeException("当前阶段不能导入考生数据");
        }

        MarkerGroup markerGroup = markerGroupRepo.findOne(groupId);
        // 删除
        markerGroupStudentRepo.deleteByGroupIdAndUsed(groupId, false);

        // 查询考生
        List<Student> studentList = studentRepo.findByWorkId(workId);
        if (studentList == null || studentList.isEmpty()) {
            throw new RuntimeException("没有考生信息，请先导入");
        }
        Map<String, Student> studentMap = studentList.stream().collect(Collectors.toMap(Student::getExamNumber, Function.identity()));

        // 查询所有试卷
        List<Paper> allPapers = paperRepo.findByWorkIdAndSubjectAndIsMissingAndTest(workId, subject, false, markSubject.getTest());
        if (allPapers == null || allPapers.isEmpty()) {
            throw new RuntimeException("没有可用试卷，请先采集");
        }
        Map<String, Paper> allPapersMap = allPapers.stream().collect(Collectors.toMap(Paper::getExamNumber, Function.identity()));

        // 查询已发布任务的所有试卷
        List<Paper> activePaperList = null;
        if (MarkStage.ROUGH_LEVEL.equals(markerGroup.getStage())) {
            activePaperList = allPapers.stream().filter(m -> m.getRoughBatchNo() != null).collect(Collectors.toList());
        } else if (MarkStage.LEVEL.equals(markerGroup.getStage())) {
            activePaperList = allPapers.stream().filter(m -> m.getBatchNo() != null).collect(Collectors.toList());
        }
        Map<String, Paper> activePaperMap = null;
        if (activePaperList != null && !activePaperList.isEmpty()) {
            activePaperMap = activePaperList.stream().collect(Collectors.toMap(Paper::getExamNumber, Function.identity()));
        }
        final Map<String, Paper> finalActivePaperMap = activePaperMap;
        // 查询【工作+科目+阶段】下所有数据
        List<MarkerGroupStudent> markerGroupStudents = markerGroupStudentRepo.findByWorkIdAndSubjectAndStageAndUsed(workId, subject, markerGroup.getStage(), false);
        Map<String, MarkerGroupStudent> markerGroupStudentMap = null;
        if (markerGroupStudents != null && !markerGroupStudents.isEmpty()) {
            // 查询其它分组导入数据
            markerGroupStudentMap = markerGroupStudents.stream().filter(m -> !m.getGroupId().equals(groupId)).collect(Collectors.toMap(MarkerGroupStudent::getExamNumber, Function.identity()));
        }
        final Map<String, MarkerGroupStudent> finalMarkerGroupStudentMap = markerGroupStudentMap;

        List<MarkerGroupStudent> markerGroupStudentList = new ArrayList<>();
        long batchNo = System.currentTimeMillis();
        ExcelReader excelReader = new ExcelReader(StudentDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, obj -> {
            try {
                StudentDTO dto = (StudentDTO) obj;
                String examNumber = dto.getExamNumber();
                if (StringUtils.isBlank(examNumber)) {
                    throw new RuntimeException("准考证号不能为空");
                }

                if (studentMap.get(examNumber) == null) {
                    throw new RuntimeException(String.format("考生管理中未查到[%s]考生", examNumber));
                }

                Paper paper = allPapersMap.get(examNumber);
                if (paper == null) {
                    throw new RuntimeException(String.format("考生[%s]试卷未采集或已被标记缺考", examNumber));
                }

                if (finalActivePaperMap != null && finalActivePaperMap.get(examNumber) != null) {
                    throw new RuntimeException(String.format("考生[%s]已经发布任务，不能重复导入", examNumber));
                }

                if (finalMarkerGroupStudentMap != null && finalMarkerGroupStudentMap.get(examNumber) != null) {
                    throw new RuntimeException(String.format("考生[%s]已经导入其它分组，不能重复导入", examNumber));
                }

                MarkerGroupStudent markerGroupStudent = new MarkerGroupStudent(workId, markerGroup.getStage(), subject, groupId, paper.getId(), paper.getQuestionId(), dto.getExamNumber(), dto.getName(), dto.getAreaCode(), dto.getAreaName(), dto.getExamRoom(), dto.getSchool(), dto.getSourceName(), batchNo);
                markerGroupStudentList.add(markerGroupStudent);
                return null;
            } catch (RuntimeException e) {
                //手动回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                ExcelError excelError = new ExcelError();
                excelError.setExcelErrorType(e.getMessage());
                return excelError;
            }
        });
        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new RuntimeException(errors);
        }

        if (!CollectionUtils.isEmpty(markerGroupStudentList)) {
            List<MarkerGroupStudent> data = new ArrayList<>();
            for (MarkerGroupStudent stu : markerGroupStudentList) {
                if (data.size() == 2000) {
                    markerGroupStudentRepo.save(data);
                    data.clear();
                }
                data.add(stu);
            }
            //将剩下的数据也导入
            if (!data.isEmpty()) {
                markerGroupStudentRepo.save(data);
            }
        }
        return excelErrors;
    }

    /**
     * 拼接导入异常信息
     *
     * @param excelErrors
     * @return
     */
    private String errorsString(List<ExcelError> excelErrors) {
        StringJoiner sj = new StringJoiner(";");
        if (excelErrors != null && !excelErrors.isEmpty()) {
            int forint = excelErrors.size() < 10 ? excelErrors.size() : 9;
            for (int i = 0; i < forint; i++) {
                ExcelError excelError = excelErrors.get(i);
                StringBuffer sb = new StringBuffer();
                sb.append("第").append(excelError.getRow()).append("行,").append(excelError.getExcelErrorType());
                sj.add(sb.toString());
            }
        }
        return sj.toString();
    }
}
