package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.admin.dto.SysAdjustLevelDTO;
import cn.com.qmth.stmms.ms.commons.utils.DbBackupUtils;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ToolService {

    @Resource
    private LevelRepo levelRepo;

    @Resource
    private WorkRepo workRepo;

    @Resource
    private PaperRepo paperRepo;

    @Resource
    private MarkLogRepo markLogRepo;

    @Resource
    private MarkSubjectRepo markSubjectRepo;

    @Resource
    DbBackupUtils dbBackupUtils;

    public List<SysAdjustLevelDTO> listAdjustLevels(Long workId, Subject subject) {
        Work work = workRepo.findOne(workId);
        if (work == null) {
            throw new RuntimeException("不存在此工作");
        }

        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject);
        if (markSubject == null || !MarkStage.LEVEL.equals(markSubject.getStage())) {
            throw new RuntimeException("不是分档阶段");
        }

        List<Level> levels = levelRepo.findByWorkId(workId);

        List<Paper> papers = paperRepo.findByWorkIdAndSubject(workId, subject);
        if (papers.isEmpty()) {
            throw new RuntimeException("不是分档阶段");
        }
        long nullLevel = papers.stream().filter(m -> StringUtils.isBlank(m.getLevel())).count();
        if (nullLevel > 0) {
            throw new RuntimeException("分档未结束");
        }

        Map<String, Long> countMap = papers.stream().collect(Collectors.groupingBy(Paper::getLevel, Collectors.counting()));
        List<SysAdjustLevelDTO> list = new ArrayList<>();
        for (Level level : levels) {
            SysAdjustLevelDTO sysAdjustLevelDTO = new SysAdjustLevelDTO();
            sysAdjustLevelDTO.setSubject(subject.name());
            sysAdjustLevelDTO.setLevel(level.getCode());
            int actualCount = 0;
            double actualRate = 0;
            if (countMap.containsKey(level.getCode())) {
                actualCount = Math.toIntExact(countMap.get(level.getCode()));
                actualRate = new BigDecimal(actualCount * 100).divide(new BigDecimal(papers.size()), 2, RoundingMode.UP).doubleValue();
            }
            sysAdjustLevelDTO.setActualCount(actualCount);
            sysAdjustLevelDTO.setActualRate(actualRate);
            sysAdjustLevelDTO.setStudentCount(papers.size());
            list.add(sysAdjustLevelDTO);
        }

        return list;
    }

    /**
     * 调档操作
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param list    list内容
     */
    @Transactional
    public void deal(Long workId, Subject subject, List<SysAdjustLevelDTO> list) {
        Work work = workRepo.findOne(workId);

        // 备份数据库
        dbBackupUtils.startBackup(workId, "系统调档");

        // 处理调档,记录日志
        List<Paper> paperList = paperRepo.findByWorkIdAndSubjectOrderByLevelAscSortNumDesc(workId, subject);
        int sum = list.stream().mapToInt(SysAdjustLevelDTO::getExpectCount).sum();
        if (sum != paperList.size()) {
            throw new RuntimeException("总数量不一致");
        }

        List<Paper> finalPapers = new ArrayList<>();
        List<MarkLog> markLogs = new ArrayList<>();
        int total = 0;
        for (SysAdjustLevelDTO levelDTO : list) {
            int expectCount = levelDTO.getExpectCount();
            if (expectCount == 0) {
                continue;
            }
            int lastIndex = total + expectCount;
            // 取试卷
            List<Paper> papers = paperList.subList(total, lastIndex);
            String level = levelDTO.getLevel();
            for (Paper paper : papers) {
                if (level.equals(paper.getLevel())) {
                    continue;
                }

                // 记录日志
                MarkLog markLog = new MarkLog(workId, work.getName(), paper.getId(), subject, MarkStage.LEVEL, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), 0L, "admin", Role.ADMIN, MarkLogOperType.SYSTEM_ADJUST_LEVEL.getId(),MarkLogOperType.SYSTEM_ADJUST_LEVEL.getName(), paper.getLevel(), level, "系统批量调档", new Date());
                markLogs.add(markLog);

                // 有档位变动的试卷
                paper.setLevel(level);
                finalPapers.add(paper);
            }
            total = lastIndex;
        }

        if (!CollectionUtils.isEmpty(finalPapers)) {
            List<Paper> data = new ArrayList<>();
            //500条提交一次
            for (Paper paper : finalPapers) {
                if (data.size() == 1000) {
                    paperRepo.save(data);
                    data.clear();
                }
                data.add(paper);
            }
            //将剩下的数据也导入
            if (!data.isEmpty()) {
                paperRepo.save(data);
            }
        }
        if (!CollectionUtils.isEmpty(markLogs)) {
            List<MarkLog> data = new ArrayList<>();
            //500条提交一次
            for (MarkLog markLog : markLogs) {
                if (data.size() == 1000) {
                    markLogRepo.save(markLog);
                    data.clear();
                }
                data.add(markLog);
            }
            //将剩下的数据也导入
            if (!data.isEmpty()) {
                markLogRepo.save(data);
            }
        }
    }
}
