package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.admin.utils.BuiltInParamUtils;
import cn.com.qmth.stmms.ms.commons.config.ImageConfig;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.ParamSetting;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.enums.ParamSettingTypeEnum;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

/**
 * 评卷工作服务
 */
@Service
public class WorkService {

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private ParamSettingRepo paramSettingRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Resource
    ImageConfig imageConfig;

    @Autowired
    BuiltInParamUtils builtInParamUtils;

    /**
     * 删除评卷工作
     *
     * @param workId
     */
    @Transactional
    public void delete(Long workId) {
        long paperCount = paperRepo.countByWorkId(workId);
        if (paperCount > 0) {
            throw new RuntimeException("该评卷工作已有数据，不能删除");
        }
        workRepo.delete(workId);

        // 删除参数
        paramSettingRepo.deleteByWorkId(workId);
    }

    /**
     * 创建评卷工作
     *
     * @param work
     */
    @Transactional
    public void save(Work work) {
        Work workTemp = workRepo.findByName(work.getName());
        if(workTemp != null){
            throw new RuntimeException("工作名称不能重复");
        }
        workRepo.save(work);
        Subject[] subjects = Subject.values();
        if (!imageConfig.isCustomSubject()) {
            for (int i = 0; i < subjects.length - 1; i++) {
                MarkSubject markSubject = new MarkSubject(subjects[i], work.getId());
                markSubject.setWorkId(work.getId());
                markSubject.setEnable(true);
                markSubjectRepo.save(markSubject);
            }
        } else {
            MarkSubject markSubject = new MarkSubject(work.getName(), Subject.CUSTOM, work.getId());
            markSubject.setWorkId(work.getId());
            markSubject.setEnable(true);
            markSubjectRepo.save(markSubject);
        }

        //新增工作的同时，初始化该工作的全局参数
        ParamSetting paramSetting = ParamSetting.init(work.getId());
        paramSettingRepo.save(paramSetting);

        ParamCache.cacheParam(ParamSettingTypeEnum.COLLECT, work.getId(), paramSetting.getCollectConfig());
        ParamCache.cacheParam(ParamSettingTypeEnum.ROUGH_LEVEL, work.getId(), paramSetting.getRoughLevelConfig());
        ParamCache.cacheParam(ParamSettingTypeEnum.LEVEL, work.getId(), paramSetting.getLevelConfig());
        ParamCache.cacheParam(ParamSettingTypeEnum.SCORE, work.getId(), paramSetting.getScoreConfig());

        // 设置默认档位
        builtInParamUtils.createLevels(work.getId());

        //生成内置账号
        builtInParamUtils.createAccount(work.getId());
    }

}
