package cn.com.qmth.stmms.ms.admin.utils;

import cn.com.qmth.stmms.ms.commons.constant.SystemConstant;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.enums.BuiltInAccountEnum;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import com.alibaba.fastjson.JSONObject;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;

@Component
public class BuiltInParamUtils {

    @Resource
    MarkUserRepo markUserRepo;

    @Resource
    LevelRepo levelRepo;

    /**
     * 添加工作下的默认档位
     *
     * @param workId 工作ID
     */
    public void createLevels(Long workId) {
        //默认增加档位
        Level aLevel = new Level(workId, "A", 0, 100, 90, Level.LevelType.ADMITED, 95, "1", 93);
        Level bLevel = new Level(workId, "B", 1, 89, 85, Level.LevelType.ADMITED, 87, "1", 93);
        Level cLevel = new Level(workId, "C", 2, 84, 80, Level.LevelType.ADMITED, 82, "2", 80);
        Level dLevel = new Level(workId, "D", 3, 79, 75, Level.LevelType.ADMITED, 77, "2", 80);
        Level eLevel = new Level(workId, "E", 4, 74, 70, Level.LevelType.ADMITED, 72, "3", 68);
        Level fLevel = new Level(workId, "F", 5, 69, 65, Level.LevelType.ADMITED, 67, "3", 68);
        Level gLevel = new Level(workId, "G", 6, 64, 60, Level.LevelType.ADMITED, 62, "3", 68);
        Level hLevel = new Level(workId, "H", 7, 59, 55, Level.LevelType.ADMITED, 57, "4", 55);
        Level iLevel = new Level(workId, "I", 8, 54, 50, Level.LevelType.ADMITED, 52, "4", 55);
        Level jLevel = new Level(workId, "J", 9, 49, 45, Level.LevelType.ADMITED, 47, "5", 35);
        Level kLevel = new Level(workId, "K", 10, 44, 0, Level.LevelType.ADMITED, 40, "5", 35);
        List<Level> levels = Arrays.asList(aLevel, bLevel, cLevel, dLevel, eLevel, fLevel, gLevel, hLevel, iLevel, jLevel, kLevel);
        levelRepo.save(levels);
    }

    /**
     * 添加工作下的默认用户
     *
     * @param workId 工作ID
     */
    public void createAccount(Long workId) {
        List<MarkUser> userList = new ArrayList<>();
        List<JSONObject> mapList = BuiltInAccountEnum.listAccounts();
        if (mapList.size() > 0) {
            for (JSONObject map : mapList) {
                int count = map.getIntValue("count");
                if (count == 0) {
                    continue;
                }
                String loginName = map.getString("code").toLowerCase();
                if (count == 1) {
                    MarkUser markUser = markUserRepo.findByLoginName(loginName);
                    if (markUser != null) {
                        continue;
                    }
                    markUser = new MarkUser(loginName, SystemConstant.INIT_PASSWORD, 0L, null, loginName, Role.findByName((String) map.get("name")), null);
                    userList.add(markUser);
                } else {
                    for (int i = 1; i <= count; i++) {
                        String loginNameTemp = workId + "-" + loginName + lpad(2, i);
                        MarkUser markUser = markUserRepo.findByLoginName(loginNameTemp);
                        if (markUser != null) {
                            continue;
                        }
                        //默认密码123456
                        //随机密码randomPassword()
                        markUser = new MarkUser(loginNameTemp, SystemConstant.INIT_PASSWORD, workId, null, loginNameTemp, Role.findByName((String) map.get("name")), null);
                        //默认为禁用
                        markUser.setEnabled(false);
                        userList.add(markUser);
                    }
                }
            }
            markUserRepo.save(userList);
        }
    }

    private String randomPassword() {
        String sources = "0123456789";
        String password = "123456";
        for (int i = 0; i <= 100; i++) {
            Random rand = new Random();
            StringBuffer flag = new StringBuffer();
            for (int j = 0; j < 6; j++) {
                flag.append(sources.charAt(rand.nextInt(10)) + "");
            }
            password = flag.toString();
        }
        return password;
    }

    /**
     * 补齐不足长度
     *
     * @param length 长度
     * @param number 数字
     * @return String
     */
    private static String lpad(int length, int number) {
        String f = "%0" + length + "d";
        return String.format(f, number);
    }

}
