package cn.com.qmth.stmms.ms.commons.utils;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.itextpdf.awt.AsianFontMapper;
import com.itextpdf.text.*;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfPCell;
import com.itextpdf.text.pdf.PdfPTable;
import com.itextpdf.text.pdf.PdfWriter;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.List;

public class PdfUtils {

    // 每页20条
    private static final int PAGE_SIZE = 20;

    public static void exportPdf(String title, List<JSONObject> datas, HttpServletResponse response) throws Exception {

        // 1:建立Document对象实例
        Document document = new Document();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        try {
            // 2:建立一个PDF 写入器与document对象关联通过书写器(Writer)可以将文档写入到磁盘中
            PdfWriter.getInstance(document, baos);
//            PdfWriter.getInstance(document, new FileOutputStream("D:\\" + System.currentTimeMillis() + ".pdf"));
            // 3:打开文档
            document.open();

            //解决中文不显示问题
            BaseFont bfChinese = BaseFont.createFont(AsianFontMapper.ChineseSimplifiedFont, AsianFontMapper.ChineseSimplifiedEncoding_H, BaseFont.NOT_EMBEDDED);
            Font fontChina18 = new Font(bfChinese, 18);
            Font fontChina12 = new Font(bfChinese, 12);

            // 4:向文档添加内容
            if (datas.isEmpty()) {
                document.newPage();

                // 标题
                Paragraph titleParagraph = new Paragraph("没有改档数据", fontChina18);
                titleParagraph.setAlignment(Element.ALIGN_CENTER);// 居中
                document.add(titleParagraph);
            } else {
                // 计算页数
                int divide = datas.size() / PAGE_SIZE;
                // 总页数
                int totalPage = datas.size() % PAGE_SIZE == 0 ? divide : divide + 1;

                for (int i = 0; i < totalPage; i++) {
                    // 若大于2页，需要增加页面
                    if (i > 0) {
                        document.newPage();
                    }
                    // 标题
                    Paragraph titleParagraph = new Paragraph(title, fontChina18);
                    titleParagraph.setAlignment(Element.ALIGN_CENTER);// 居中
                    document.add(titleParagraph);

                    // 空格
                    Paragraph blank1 = new Paragraph(" ");
                    document.add(blank1);

                    // 编号
                    Chunk c1 = new Chunk("共" + totalPage + "页  ", fontChina12);
                    Chunk c2 = new Chunk("第" + (i + 1) + "页", fontChina12);
                    Paragraph snoParagraph = new Paragraph();
                    snoParagraph.add(c1);
                    snoParagraph.add(c2);
                    snoParagraph.setAlignment(Element.ALIGN_RIGHT);
                    document.add(snoParagraph);

                    // 空格
                    document.add(blank1);

                    // 表格处理
                    PdfPTable table = new PdfPTable(4);// 四列
                    table.setWidthPercentage(100);// 表格宽度为100%

                    // 表头
                    // 序号
                    fillCell(table, "序号", fontChina12, BaseColor.GRAY);
                    // 试卷密号
                    fillCell(table, "试卷密号", fontChina12, BaseColor.GRAY);
                    // 原档位
                    fillCell(table, "原档位", fontChina12, BaseColor.GRAY);
                    // 改后档位
                    fillCell(table, "改后档位", fontChina12, BaseColor.GRAY);

                    List<JSONObject> subList = CollUtil.sub(datas, i * 20, (i + 1) * 20);
                    for (int j = 0; j < subList.size(); j++) {
                        JSONObject object = subList.get(j);
                        // 序号填充
                        fillCell(table, i * 20 + j + 1, fontChina12, null);
                        // 试卷密号
                        fillCell(table, object.getString("secretNumber"), fontChina12, null);
                        // 原档位
                        fillCell(table, object.getString("oldLevel"), fontChina12, null);
                        // 改后档位
                        fillCell(table, object.getString("newLevel"), fontChina12, null);
                    }

                    // 不足20个，自动补齐
                    for (int k = 0; k < PAGE_SIZE - subList.size(); k++) {
                        // 序号填充
                        fillCell(table, i * 20 + subList.size() + k + 1, fontChina12, null);
                        // 试卷密号
                        fillCell(table, null, fontChina12, null);
                        // 原档位
                        fillCell(table, null, fontChina12, null);
                        // 改后档位
                        fillCell(table, null, fontChina12, null);
                    }

                    Paragraph pDate = new Paragraph(" 年        月        日", fontChina12);
                    pDate.setAlignment(Element.ALIGN_RIGHT);
                    pDate.setIndentationRight(20);

                    // 备注
                    PdfPCell cell34 = new PdfPCell();
                    cell34.setBorderWidth(0.5F);
                    cell34.setMinimumHeight(40);
                    cell34.setPhrase(new Paragraph("签字栏", fontChina12));
                    cell34.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
                    cell34.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
                    table.addCell(cell34);
                    PdfPCell cell35 = new PdfPCell();
                    cell35.setBorderWidth(0.5F);
                    cell35.setColspan(3);
                    cell35.addElement(blank1);
                    cell35.addElement(blank1);
                    cell35.addElement(blank1);
                    cell35.addElement(pDate);
                    cell35.addElement(blank1);
                    table.addCell(cell35);

                    document.add(table);
                }
            }
            // 5:关闭文档
            document.close();
        } catch (DocumentException | IOException e) {
            e.printStackTrace();
        }

        String fileName = title + ".pdf";
        outputFile(response, baos, fileName);
    }

    /**
     * 填充table
     *
     * @param table       表格对象
     * @param content     填充值
     * @param fontChina12 字体
     * @param baseColor   背景色
     */
    private static void fillCell(PdfPTable table, Object content, Font fontChina12, BaseColor baseColor) {
        PdfPCell cell = new PdfPCell();
        cell.setBorderWidth(0.5F);
        cell.setPhrase(new Paragraph(content == null ? "  " : String.valueOf(content), fontChina12));
        cell.setVerticalAlignment(PdfPCell.ALIGN_MIDDLE);
        cell.setHorizontalAlignment(PdfPCell.ALIGN_CENTER);
        cell.setExtraParagraphSpace(10);
        if (baseColor != null) {
            cell.setBackgroundColor(baseColor);
        }
        table.addCell(cell);
    }

    public static void outputFile(HttpServletResponse response, ByteArrayOutputStream baos, String fileName) throws Exception {
        try {
            String fName = URLEncoder.encode(fileName, "UTF-8");

            response.reset();
            response.setContentType("application/x-msdownload");
            response.setHeader("Content-Disposition", "attachment; filename=" + fName);

            OutputStream outStream = response.getOutputStream();
            baos.writeTo(outStream);
            outStream.close();
        } catch (IOException e) {
            throw new Exception("下载失败");
        }
    }
}
