package cn.com.qmth.stmms.ms.core.cache;

import cn.com.qmth.stmms.ms.core.domain.ParamSetting;
import cn.com.qmth.stmms.ms.core.domain.enums.ParamSettingTypeEnum;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.CollectConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.RoughLevelConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.LevelConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.ScoreConfig;
import cn.com.qmth.stmms.ms.core.repository.ParamSettingRepo;
import com.alibaba.fastjson.JSON;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class ParamCache {
    private static final Logger LOGGER = LoggerFactory.getLogger(ParamCache.class);

    public static Map<String, CollectConfig> collectConfigMap = new HashMap<>();
    public static Map<String, RoughLevelConfig> roughLevelConfigMap = new HashMap<>();
    public static Map<String, LevelConfig> levelConfigMap = new HashMap<>();
    public static Map<String, ScoreConfig> scoreConfigMap = new HashMap<>();

    @Resource
    private ParamSettingRepo paramSettingRepo;

    @PostConstruct
    public void init() {
        //系统启动中。。。加载codeMap
        List<ParamSetting> paramSettings = paramSettingRepo.findAll();
        for (ParamSetting param : paramSettings) {
            // 采集
            cacheParam(ParamSettingTypeEnum.COLLECT, param.getWorkId(), param.getCollectConfig());
            // 粗分档
            cacheParam(ParamSettingTypeEnum.ROUGH_LEVEL, param.getWorkId(), param.getRoughLevelConfig());
            // 细分档
            cacheParam(ParamSettingTypeEnum.LEVEL, param.getWorkId(), param.getLevelConfig());
            // 打分
            cacheParam(ParamSettingTypeEnum.SCORE, param.getWorkId(), param.getScoreConfig());
        }
        LOGGER.info("加载采集参数成功：{}", JSON.toJSONString(collectConfigMap));
        LOGGER.info("加载粗分档参数成功：{}", JSON.toJSONString(roughLevelConfigMap));
        LOGGER.info("加载细分档参数成功：{}", JSON.toJSONString(levelConfigMap));
        LOGGER.info("加载打分参数成功：{}", JSON.toJSONString(scoreConfigMap));
    }

    @PreDestroy
    public void destroy() {
        //系统运行结束
        collectConfigMap.clear();
        roughLevelConfigMap.clear();
        levelConfigMap.clear();
        scoreConfigMap.clear();
    }

    /**
     * 缓存值
     *
     * @param typeEnum 类型
     * @param workId   工作ID
     * @param value    参数值
     */
    public static void cacheParam(ParamSettingTypeEnum typeEnum, Long workId, String value) {
        String workIdStr = String.valueOf(workId);
        // 采集
        if (ParamSettingTypeEnum.COLLECT.equals(typeEnum)) {
            CollectConfig collectConfig = JSON.parseObject(value, CollectConfig.class);
            collectConfigMap.remove(workIdStr);
            collectConfigMap.put(workIdStr, collectConfig);
        }
        // 粗分档
        if (ParamSettingTypeEnum.ROUGH_LEVEL.equals(typeEnum)) {
            RoughLevelConfig roughLevelConfig = JSON.parseObject(value, RoughLevelConfig.class);
            roughLevelConfigMap.remove(workIdStr);
            roughLevelConfigMap.put(workIdStr, roughLevelConfig);
        }
        // 细分档
        if (ParamSettingTypeEnum.LEVEL.equals(typeEnum)) {
            LevelConfig levelConfig = JSON.parseObject(value, LevelConfig.class);
            levelConfigMap.remove(workIdStr);
            levelConfigMap.put(workIdStr, levelConfig);
        }
        // 打分
        if (ParamSettingTypeEnum.SCORE.equals(typeEnum)) {
            ScoreConfig scoreConfig = JSON.parseObject(value, ScoreConfig.class);
            scoreConfigMap.remove(workIdStr);
            scoreConfigMap.put(workIdStr, scoreConfig);
        }
    }
}
