package cn.com.qmth.stmms.ms.core.domain;

import cn.com.qmth.stmms.ms.core.domain.paramsetting.CollectConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.LevelConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.RoughLevelConfig;
import cn.com.qmth.stmms.ms.core.domain.paramsetting.ScoreConfig;
import com.alibaba.fastjson.JSON;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * 全局参数设置
 */
@Entity
@Table(name = "param_setting")
public class ParamSetting implements Serializable {

    @Id
    @GeneratedValue
    private Long id;

    @NotNull
    private Long workId;

    // 采集参数
    private String collectConfig;

    // 粗分档参数
    private String roughLevelConfig;

    // 细分档参数
    private String levelConfig;

    // 打分参数
    private String scoreConfig;


    public static ParamSetting init(Long workId) {
        ParamSetting paramSetting = new ParamSetting();
        paramSetting.setWorkId(workId);
        // 粗分档参数
        CollectConfig collectConfig = new CollectConfig();
        collectConfig.setPackageScan(0); //是否整包扫描
        collectConfig.setImageEncrypt(0); //图片是否加密
        collectConfig.setNameRule(0); //图片命名规则
        collectConfig.setPaperStage(0);//试卷档位
        paramSetting.setCollectConfig(JSON.toJSONString(collectConfig));

        // 细分档参数
        LevelConfig levelConfig = new LevelConfig();
        levelConfig.setDeviation(4);//仲裁档位差
        levelConfig.setAutoCallback(0);//是否自动打回
        levelConfig.setMajority(1);//是否过半定档
        levelConfig.setAutoCallbackShowDeviation(1);//自动打回时显示偏差
//        levelConfig.setCumulativeError(4);//打回累计误差（默认为空）
        levelConfig.setTakeBest(1);//取优原则
        levelConfig.setLevelShowAllPaper(0);//阅卷员是否显示所有试卷
        levelConfig.setPropDenominator(1);//档位百分比分母
        levelConfig.setShowStandardPaperManage(1);//是否显示标准卷管理
        levelConfig.setClearData(0);//是否显示清除当前阅卷数据
        levelConfig.setRoughLevelEnable(0);//是否开启粗分档
        levelConfig.setRemoveHighAndLow(0);//是否开启去高去低再加权评卷
        paramSetting.setLevelConfig(JSON.toJSONString(levelConfig));

        // 粗分档参数
        RoughLevelConfig roughLevelConfig = new RoughLevelConfig();
        roughLevelConfig.setAutoCallback(0);//是否自动打回
        roughLevelConfig.setDeviation(4);//仲裁档位差
        roughLevelConfig.setMajority(1);//是否过半定档
        roughLevelConfig.setAutoCallbackShowDeviation(1);//自动打回时显示偏差
//        roughLevelConfig.setCumulativeError(4);//打回累计误差（默认为空）
        roughLevelConfig.setTakeBest(1);//取优原则
        roughLevelConfig.setLevelShowAllPaper(0);//阅卷员是否显示所有试卷
        roughLevelConfig.setPropDenominator(1);//档位百分比分母
        roughLevelConfig.setClearData(0);//是否显示清除当前阅卷数据
        roughLevelConfig.setShowStandardPaperManage(1);//是否显示标准卷管理
        roughLevelConfig.setRemoveHighAndLow(0);//是否开启去高去低再加权评卷
        paramSetting.setRoughLevelConfig(JSON.toJSONString(roughLevelConfig));

        // 打分参数
        ScoreConfig scoreConfig = new ScoreConfig();
        scoreConfig.setRoundUp(1);//分数处理方式
        scoreConfig.setChangeStage(0);//阅卷员是否显示改档及改档打分
        scoreConfig.setScoreShowAllPaper(0); //阅卷员是否显示所有试卷
        scoreConfig.setRemoveHighAndLow(0);// 分数计算方式（1：去高去低加权平均，0：全部加权平均）
        paramSetting.setScoreConfig(JSON.toJSONString(scoreConfig));

        // 取优原则默认开启
        return paramSetting;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getWorkId() {
        return workId;
    }

    public void setWorkId(Long workId) {
        this.workId = workId;
    }

    public String getCollectConfig() {
        return collectConfig;
    }

    public void setCollectConfig(String collectConfig) {
        this.collectConfig = collectConfig;
    }

    public String getRoughLevelConfig() {
        return roughLevelConfig;
    }

    public void setRoughLevelConfig(String roughLevelConfig) {
        this.roughLevelConfig = roughLevelConfig;
    }

    public String getLevelConfig() {
        return levelConfig;
    }

    public void setLevelConfig(String levelConfig) {
        this.levelConfig = levelConfig;
    }

    public String getScoreConfig() {
        return scoreConfig;
    }

    public void setScoreConfig(String scoreConfig) {
        this.scoreConfig = scoreConfig;
    }
}
