package cn.com.qmth.stmms.ms.core.repository;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;
import java.util.Set;

/**
 * Created by zhengmin on 2016/9/23.
 */
public interface MarkTaskLevelRepo extends JpaRepository<MarkTaskLevel, Long>, JpaSpecificationExecutor {

    int countByMarkerIdAndPaperIdAndWorkIdAndRandomSeqNew(Long markerId, Long paperId, Long randomSeqNew, Long workId);

    List<MarkTaskLevel> findByWorkIdAndPaperIdAndStage(Long workId, Long paperId, MarkStage stage);

    /**
     * 查询指定阶段试卷的评卷任务
     *
     * @param paperId 试卷id
     * @param stage   评卷阶段
     */
    List<MarkTaskLevel> findByPaperIdAndStage(Long paperId, MarkStage stage);

    @Query(value = "select new MarkTaskLevel(m.id, m.markerId , m.markerName, m.result, m.secretNumber, m.stage) from MarkTaskLevel m where m.paper.id = ?1 and m.stage = ?2")
    List<MarkTaskLevel> findByPaperIdAndStageNative(Long id, MarkStage level);

    /**
     * 评卷员任务数量
     *
     * @param markerId 评卷员ID
     */
    long countByMarkerId(Long markerId);

    @Query(value = "select count(1) from mark_task_level m where m.question_id = ?1 and m.marker_id = ?3 and m.batch_no = ?2", nativeQuery = true)
    long countByQuestionIdAndBatchNo(Long questionId, Long batchNo, Long markerId);

    @Query(value = "select count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.batch_no = ?5 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result = ?4 and m.is_rejected = false", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndResult(Long questionId, Long markerId, int stage, String result, Long batchNo);

    @Query(value = "select count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.batch_no is not null where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result = ?4 and m.is_rejected = false", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndResultAll(Long questionId, Long markerId, int stage, String result);

    /**
     * 查询评卷员的评卷任务
     *
     * @param markerId 评卷员ID
     * @param stage    阶段
     */
    @Query(value = "select mt from MarkTaskLevel mt left outer join fetch mt.paper where mt.markerId = ?1 and mt.stage = ?2")
    List<MarkTaskLevel> findByMarkerIdAndStage(Long markerId, MarkStage stage);

    /**
     * 删除评卷员所有任务
     *
     * @param subject 科目
     * @param stage   阶段
     */
    Long deleteByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage);

    /**
     * 统计评卷员各试题已评数及总数（分档阶段）
     *
     * @param markerId 评卷员ID
     * @param stageId  阶段
     * @param test     试评状态
     */
    @Query(value = "SELECT q.`id`,q.`name`,SUM(IF(t.`result` IS NULL,1,0)),COUNT(t.question_id), q.area_code \n" +
            "FROM mark_task_level t LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.`marker_id` = ? and t.stage = ? and p.is_missing = false and p.is_test = ?  GROUP BY t.`question_id` ORDER BY q.`id`", nativeQuery = true)
    List<Object[]> countGroupByQuestion(Long markerId, Integer stageId, int test);

    /**
     * 统计各评卷员各试题已评数及总数（分档阶段）
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param stageId 阶段
     * @param test    试评状态
     * @param markers 评卷员ID集合
     */
    @Query(value = "SELECT q.`id`,t.`marker_id` markerId,mu.login_name loginName,t.`marker_name` markerName,q.`name`,SUM(IF(t.`result` IS NULL,1,0)) leftCount,COUNT(t.question_id) totalCount, SUM(t.is_rejected) rejectedCount, q.area_code areaCode " +
            "FROM mark_task_level t LEFT JOIN mark_user mu on t.marker_id = mu.id LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.work_id = :workId and t.subject = :subject and t.stage = :stageId and p.is_missing = false and p.is_test = :test and p.batch_no is not null and t.marker_id in (:markers) GROUP BY t.marker_id,mu.login_name, t.`marker_name` ORDER BY q.`id`,t.marker_id", nativeQuery = true)
    List<Object[]> listGroupByQuestionAndMarkerId(@Param("workId") Long workId, @Param("subject") String subject, @Param("stageId") Integer stageId, @Param("test") int test, @Param("markers") List<Long> markers);

    /**
     * 统计各评卷员各试题已评数及总数（分档阶段）
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param stageId 阶段
     * @param test    试评状态
     * @param batchNo 批次号
     * @param markers 评卷员ID集合
     */
    @Query(value = "SELECT q.`id`,t.`marker_id` markerId,mu.login_name loginName,t.`marker_name` markerName,q.`name`,SUM(IF(t.`result` IS NULL,1,0)) leftCount,COUNT(t.question_id) totalCount, SUM(t.is_rejected) rejectedCount, q.area_code areaCode " +
            "FROM mark_task_level t LEFT JOIN mark_user mu on t.marker_id = mu.id LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.work_id = :workId and t.subject = :subject and t.stage = :stageId and p.is_missing = false and p.is_test = :test and p.batch_no = :batchNo and t.marker_id in (:markers) GROUP BY t.marker_id,mu.login_name, t.`marker_name` ORDER BY q.`id`,t.marker_id", nativeQuery = true)
    List<Object[]> listGroupByQuestionAndBatchNoAndMarkerId(@Param("workId") Long workId, @Param("subject") String subject, @Param("stageId") Integer stageId, @Param("test") int test, @Param("batchNo") Long batchNo, @Param("markers") List<Long> markers);

    /**
     * 考区已评数及总数（分档阶段）
     *
     * @param workId 工作ID
     * @param name   科目
     * @param test   试评状态
     */
    @Query(value = "SELECT q.area_code areaCode, q.area_name areaName, q.`name`,SUM(IF(p.`level` IS NULL or (p.level is not null and p.batch_no is null),1,0)) leftCount,COUNT(p.question_id) totalCount " +
            "FROM exam_question q LEFT  JOIN paper p ON q.id = p.question_id AND q.subject = p.subject WHERE " +
            "p.work_id = ? and p.subject = ? and p.is_missing = false and p.is_test = ?  GROUP BY q.area_code, q.area_name ORDER BY q.area_code", nativeQuery = true)
    List<Object[]> listGroupByAreaName(Long workId, String name, int test);

    /**
     * 统计指定试卷的所有任务数量
     *
     * @param id 试卷ID
     */
    Long countByPaperId(Long id);

    List<MarkTaskLevel> findByWorkIdAndSecretNumber(Long workId, String secretNumber);

    List<MarkTaskLevel> findByWorkIdAndRandomSeqNew(Long workId, Long randomSeqNew);

    @Query(value = "SELECT count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.is_shift = 0 and p.is_shift_score = 0 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null and m.batch_no = ?4 and m.is_rejected = false", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndResultNull(Long questionId, Long markerId, int stage, Long batchNo);

    @Query(value = "SELECT count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.is_shift = 0 and p.is_shift_score = 0 and p.score_batch_no = ?4 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null", nativeQuery = true)
    int countScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(Long questionId, Long markerId, int stage, Long batchNo);

    List<MarkTaskLevel> findByWorkId(Long workId);

    @Query(value = "SELECT count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.is_missing = ?4 and p.is_shift = 1 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null", nativeQuery = true)
    int countShiftByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(Long questionId, Long id, int ordinal, boolean b);

    @Query(value = "SELECT count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.is_shift = 0 and p.is_shift_score = 1 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null", nativeQuery = true)
    int countShiftScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(Long questionId, Long id, int ordinal);

    @Query(value = "select count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.batch_no is not null where m.question_id = ?1 and m.marker_id = ?2", nativeQuery = true)
    long countByQuestionIdAll(Long questionId, Long markId);

    @Query(value = "SELECT count(1) from mark_task_level m inner join paper p on m.paper_id = p.id and p.is_shift = 0 and p.is_shift_score = 0 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null and m.batch_no is not null and m.is_rejected = false", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndResultIsNullAll(Long questionId, Long markerId, int stage);

    @Query(value = "SELECT count(1) from mark_task_level m where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and  m.batch_no is not null and m.is_rejected = true", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndIsRejectedTrue(Long questionId, Long markerId, int ordinal);

    @Query(value = "select * from mark_task_level where work_id = ?1 and subject = ?2 and marker_id = ?3 and stage = ?4 limit 1", nativeQuery = true)
    List<MarkTaskLevel> findByWorkIdAndSubjectAndMarkerIdAndStageLimit(Long workId, String subject, Long valueOf, int stage);

    int countByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage);

    int countByWorkIdAndStageAndResultNotNull(Long id, MarkStage stage);

    @Query(value = "select count(1) from mark_task_level m inner join paper p on m.paper_id = p.id where m.work_id = ?1 and m.subject = ?2 and m.marker_id = ?3 and m.stage = ?4 and m.result is null and p.is_shift = true", nativeQuery = true)
    int countByWorkIdAndSubjectAndMarkerIdAndStageAndShiftAndResult(Long workId, String subject, Long valueOf, int stage);

    @Query(value = "select count(1) from mark_task_level m inner join paper p on m.paper_id = p.id where m.work_id = ?1 and m.subject = ?2 and m.marker_id = ?3 and m.stage = ?4 and m.result is null and p.is_shift_score = 1", nativeQuery = true)
    int countByWorkIdAndSubjectAndMarkerIdAndStageAndShiftScoreAndResult(Long workId, String subject, Long valueOf, int stage);

    @Query(value = "select count(1) from mark_task_level m where m.work_id = ?1 and m.subject = ?2 and m.marker_id = ?3 and m.stage = ?4 and m.is_rejected = 1", nativeQuery = true)
    int findByWorkIdAndSubjectAndMarkerIdAndStageReject(Long workId, String subject, Long valueOf, int stage);

    @Query(value = "select count(1) from mark_task_level m where m.work_id = ?1", nativeQuery = true)
    int countByWorkId(Long workId);

    @Query(value = "select id, random_seq_new randomSeqNew from mark_task_level m where m.work_id = ?1", nativeQuery = true)
    List<Object[]> findAllByWorkId(Long workId);

    @Modifying
    @Query(value = "update mark_task_level m set m.level = ?1 where m.paper_id = ?2", nativeQuery = true)
    void updateLevelByPaperId(String level, Long id);

    @Query("select s.paper from MarkTaskLevel s where s.workId = :workId and s.subject = :subject and s.stage = :stage and s.markerId in (:markers)")
    List<Paper> findByWorkIdAndSubjectAndStageAndMarkerIdIn(@Param("workId") Long workId, @Param("subject") Subject subject, @Param("stage") MarkStage stage, @Param("markers") List<Long> markers);

    @Query(value = "select s.paper_id, s.marker_id from mark_task_level s where s.work_id = :workId and s.subject = :subject and s.stage = :stage", nativeQuery = true)
    List<Object[]> findByWorkIdAndSubjectAndStage(@Param("workId") Long workId, @Param("subject") String subject, @Param("stage") int stage);

    List<MarkTaskLevel> findByWorkIdAndSubjectAndSecretNumberAndTest(Long workId, Subject subject, String secretNumber, int id);

    @Modifying
    @Query("update MarkTaskLevel m set m.isMark = ?2 where m.id = ?1")
    void updateMarkTaskMarkById(Long markTaskId, Boolean isMark);

    void deleteByWorkIdAndSubject(Long workId, Subject subject);

    List<MarkTaskLevel> findByPaperId(Long paperId);

    @Query(value = "select max(s.randomSeq) from MarkTaskLevel s where s.workId = ?1 and s.subject = ?2 and s.markerId = ?3 and s.questionId = ?4")
    Integer findRandomSeqByWorkIdAndSubjectAndMarkerIdAndQuestionId(Long workId, Subject subject, Long markerId, Long questionId);

    @Query("select m.paper.id from MarkTaskLevel m where m.workId = ?1 and m.questionId = ?2 and m.markerId = ?3 and m.stage = ?4 and m.isMark = ?5")
    List<Long> findByWorkIdAndQuestionIdAndMarkerIdAndStageAndIsMark(Long workId, Long questionId, Long id, MarkStage stage, boolean b);

    @Query("select m.paper.id from MarkTaskLevel m where m.workId = ?1 and m.questionId = ?2 and m.markerId = ?3 and m.stage = ?4 and m.batchNo = ?5 and m.isMark = true")
    List<Long> findByWorkIdAndQuestionIdAndMarkerIdAndStageAndBatchNoAndPaperIsMark(Long workId, Long questionId, Long id, MarkStage stage, Long batchNo);

    int countByWorkIdAndSubjectAndStageAndTest(Long workId, Subject subject, MarkStage stage, int id);

    MarkTaskLevel findByPaperIdAndMarkerId(Long markerId, Long userId);

    @Query("select s.randomSeqNew from MarkTaskLevel s where s.workId = ?1")
    Set<Long> listRandomByWorkId(Long workId);
}
