package cn.com.qmth.stmms.ms.core.repository;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;
import java.util.Set;

/**
 * Created by zhengmin on 2016/9/23.
 */
public interface MarkTaskScoreRepo extends JpaRepository<MarkTaskScore, Long>, JpaSpecificationExecutor {

    int countByMarkerIdAndPaperIdAndWorkIdAndRandomSeqNew(Long markerId, Long paperId, Long randomSeqNew, Long workId);

    /**
     * 查询指定阶段试卷的评卷任务
     *
     * @param paperId 试卷id
     * @param stage   评卷阶段
     */
    List<MarkTaskScore> findByPaperIdAndStage(Long paperId, MarkStage stage);

    @Query(value = "select new MarkTaskScore(m.id, m.markerId , m.markerName, m.result, m.secretNumber, m.stage) from MarkTaskScore m where m.paper.id = ?1 and m.stage = ?2")
    List<MarkTaskScore> findByPaperIdAndStageNative(Long id, MarkStage level);

    @Query(value = "select count(1) from mark_task_score m inner join paper p on m.paper_id = p.id and p.score_batch_no = ?5 and p.level = ?4 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is not null", nativeQuery = true)
    int countScoreByQuestionIdAndMarkerIdAndStageAndResult(Long questionId, Long markerId, int stage, String result, Long batchNo);

    @Query(value = "select count(1) from mark_task_score m inner join paper p on m.paper_id = p.id and p.level = ?4 and p.score_batch_no is not null where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is not null", nativeQuery = true)
    int countScoreByQuestionIdAndMarkerIdAndStageAndResultAll(Long questionId, Long markerId, int stage, String result);

    /**
     * 查询评卷员的评卷任务
     *
     * @param markerId 评卷员ID
     * @param stage    阶段
     */
    @Query(value = "select mt from MarkTaskScore mt left outer join fetch mt.paper where mt.markerId = ? and mt.stage = ?")
    List<MarkTaskScore> findByMarkerIdAndStage(Long markerId, MarkStage stage);

    /**
     * 删除评卷员所有任务
     *
     * @param subject 科目
     * @param stage   阶段
     */
    Long deleteByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage);

    /**
     * 考区已评数及总数（分档阶段）
     *
     * @param workId 工作ID
     * @param name   科目
     */
    @Query(value = "SELECT q.area_code areaCode, q.area_name areaName, q.`name`,SUM(IF(p.`score` IS NULL,1,0)) leftCount,COUNT(p.question_id) totalCount " +
            "FROM exam_question q LEFT  JOIN paper p ON q.id = p.question_id AND q.subject = p.subject WHERE " +
            "p.work_id = ? and p.subject = ? and p.is_missing = false  GROUP BY q.area_code, q.area_name ORDER BY q.area_code", nativeQuery = true)
    List<Object[]> listGroupByAreaName(Long workId, String name);

    /**
     * 统计评卷员各试题已评数及总数（分档阶段）
     *
     * @param markerId 评卷员ID
     * @param stageId  阶段
     */
    @Query(value = "SELECT q.`id`,q.`name`,SUM(IF(t.`result` IS NULL,1,0)),COUNT(t.question_id), q.area_code \n" +
            "FROM mark_task_score t LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.`marker_id` = ? and t.stage = ? and p.is_missing = false GROUP BY t.`question_id` ORDER BY q.`id`", nativeQuery = true)
    List<Object[]> countGroupByQuestion(Long markerId, Integer stageId);

    /**
     * 统计各评卷员各试题已评数及总数（分档阶段）
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param stageId 阶段
     * @param markers 评卷员ID集合
     */
    @Query(value = "SELECT q.`id`,t.`marker_id` markerId,mu.login_name loginName,t.`marker_name` markerName,q.`name`,SUM(IF(t.`result` IS NULL,1,0)) leftCount,COUNT(t.question_id) totalCount,SUM(t.is_rejected) rejectedCount, q.area_code areaCode " +
            "FROM mark_task_score t LEFT JOIN mark_user mu on t.marker_id = mu.id LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.work_id = :workId and t.subject = :subject and t.stage = :stageId and p.score_batch_no is not null and t.`marker_id` in (:markers) and p.is_missing = false GROUP BY t.marker_id,mu.login_name, t.`marker_name` ORDER BY q.`id`, t.`marker_id`", nativeQuery = true)
    List<Object[]> listGroupByQuestionAndMakerId(@Param("workId") Long workId, @Param("subject") String subject, @Param("stageId") Integer stageId, @Param("markers") List<Long> markers);

    /**
     * 统计各评卷员各试题已评数及总数（分档阶段）
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param stageId 阶段
     * @param batchNo 批次号
     * @param markers 评卷员ID集合
     */
    @Query(value = "SELECT q.`id`,t.`marker_id` markerId,mu.login_name loginName,t.`marker_name` markerName,q.`name`,SUM(IF(t.`result` IS NULL,1,0)) leftCount,COUNT(t.question_id) totalCount,SUM(t.is_rejected) rejectedCount, q.area_code areaCode " +
            "FROM mark_task_score t LEFT JOIN mark_user mu on t.marker_id = mu.id LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.work_id = :workId and t.subject = :subject and t.stage = :stageId and p.score_batch_no = :batchNo and t.`marker_id` in (:markers) and p.is_missing = false GROUP BY t.marker_id,mu.login_name, t.`marker_name` ORDER BY q.`id`, t.`marker_id`", nativeQuery = true)
    List<Object[]> listGroupByQuestionAndScoreBatchNoAndMakerId(@Param("workId") Long workId, @Param("subject") String subject, @Param("stageId") Integer stageId, @Param("batchNo") Long batchNo, @Param("markers") List<Long> markers);

    /**
     * 统计指定试卷的所有任务数量
     *
     * @param id 试卷ID
     */
    Long countByPaperId(Long id);

    List<MarkTaskScore> findByWorkIdAndSecretNumber(Long workId, String secretNumber);

    List<MarkTaskScore> findByWorkIdAndRandomSeqNew(Long workId, Long randomSeqNew);

    @Query(value = "SELECT count(1) from mark_task_score m inner join paper p on m.paper_id = p.id and p.is_shift = 0 and p.is_shift_score = 0 and p.score_batch_no = ?4 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null", nativeQuery = true)
    int countScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(Long questionId, Long markerId, int stage, Long batchNo);

    List<MarkTaskScore> findByWorkId(Long workId);

    @Query(value = "SELECT count(1) from mark_task_score m inner join paper p on m.paper_id = p.id and p.is_shift = 0 and p.is_shift_score = 1 where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null", nativeQuery = true)
    int countShiftScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(Long questionId, Long id, int ordinal);

    @Query(value = "select * from mark_task_score where work_id = ?1 and subject = ?2 and marker_id = ?3 and stage = ?4 limit 1", nativeQuery = true)
    List<MarkTaskScore> findByWorkIdAndSubjectAndMarkerIdAndStageLimit(Long workId, String subject, Long valueOf, int stage);

    int countByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage);

    @Query(value = "select count(1) from mark_task_score m inner join paper p on m.paper_id = p.id where m.work_id = ?1 and m.subject = ?2 and m.marker_id = ?3 and m.stage = ?4 and m.result is null and p.is_shift_score = 1", nativeQuery = true)
    int countByWorkIdAndSubjectAndMarkerIdAndStageAndShiftScoreAndResult(Long workId, String subject, Long valueOf, int stage);

    @Query(value = "select count(1) from mark_task_score m where m.work_id = ?1", nativeQuery = true)
    int countByWorkId(Long workId);

    @Query(value = "select id, random_seq_new randomSeqNew from mark_task_score m where m.work_id = ?1", nativeQuery = true)
    List<Object[]> findAllByWorkId(Long workId);

    @Query("select s.paper from MarkTaskScore s where s.workId = :workId and s.subject = :subject and s.stage = :stage and s.markerId in (:markers)")
    List<Paper> findByWorkIdAndSubjectAndStageAndMarkerIdIn(@Param("workId") Long workId, @Param("subject") Subject subject, @Param("stage") MarkStage stage, @Param("markers") List<Long> markers);

    @Query(value = "select s.paper_id, s.marker_id from mark_task_score s where s.work_id = :workId and s.subject = :subject and s.stage = :stage", nativeQuery = true)
    List<Object[]> findByWorkIdAndSubjectAndStage(@Param("workId") Long workId, @Param("subject") String subject, @Param("stage") int stage);

    List<MarkTaskScore> findByWorkIdAndSubjectAndSecretNumberAndTest(Long workId, Subject subject, String secretNumber, int id);

    @Modifying
    @Query("update MarkTaskScore m set m.isMark = ?2 where m.id = ?1")
    void updateMarkTaskMarkById(Long markTaskId, Boolean isMark);

    void deleteByWorkIdAndSubject(Long workId, Subject subject);

    List<MarkTaskScore> findByPaperId(Long paperId);

    @Query(value = "select max(s.randomSeq) from MarkTaskScore s where s.workId = ?1 and s.subject = ?2 and s.markerId = ?3 and s.questionId = ?4")
    Integer findRandomSeqByWorkIdAndSubjectAndMarkerIdAndQuestionId(Long workId, Subject subject, Long markerId, Long questionId);

    @Query("select m.paper.id from MarkTaskScore m where m.workId = ?1 and m.questionId = ?2 and m.markerId = ?3 and m.stage = ?4 and m.isMark = ?5")
    List<Long> findByWorkIdAndQuestionIdAndMarkerIdAndStageAndIsMark(Long workId, Long questionId, Long id, MarkStage stage, boolean b);

    @Query("select m.paper.id from MarkTaskScore m where m.workId = ?1 and m.questionId = ?2 and m.markerId = ?3 and m.stage = ?4 and m.batchNo = ?5 and m.isMark = true")
    List<Long> findByWorkIdAndQuestionIdAndMarkerIdAndStageAndBatchNoAndPaperIsMark(Long workId, Long questionId, Long id, MarkStage stage, Long batchNo);

    @Query("select m.randomSeqNew from MarkTaskScore m where m.workId = ?1")
    Set<Long> listRandomByWorkId(Long workId);
}
