package cn.com.qmth.stmms.ms.core.repository;

import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import java.util.List;
import java.util.Set;

/**
 * Created by zhengmin on 2016/9/23.
 */
public interface PaperRepo extends JpaRepository<Paper, Long>, JpaSpecificationExecutor<Paper> {

    List<Paper> findByRandomSeq(Long randomSeq);

    int countByWorkIdAndExamNumberAndRandomSeq(Long workId, String examNumber, Long randomSeq);

    List<Paper> findByWorkIdAndSubject(Long workId, Subject subject);

    List<Paper> findByWorkIdAndSubjectAndIsMissingAndTest(Long workId, Subject subject, boolean isMissing, int test);

    List<Paper> findByWorkIdAndSubject(Long workId, Subject subject, Sort sort);

    @Query(value = "select * from paper p where p.`level` = ?1 and p.is_sample = ?2 and p.is_missing = ?3 and p.subject = ?4 and p.is_test = ?5 and p.area_code = ?7 and p.work_id = ?8 order by p.updated_on desc", nativeQuery = true)
    List<Paper> findSample(String level, boolean isSample, boolean isMissing, String subject, int test, String areaCode, Long workId);

    /**
     * 统计各省各科试卷数量[areaCode,areaName,questionName,count]
     *
     * @param workId 工作ID
     */
    @Query(value = "SELECT q.`area_code`,q.`area_name`,p.`question_name`,COUNT(*) FROM paper p LEFT OUTER JOIN exam_question q ON p.`question_id` = q.`id` WHERE p.`work_id` = ? GROUP BY p.`question_id`,p.`question_name` ORDER BY p.`question_id` ", nativeQuery = true)
    List<Object[]> countGroupByQuestion(Long workId);

    /**
     * 统计科目各试题已评数及总数（分档阶段）
     *
     * @param workId  工作ID
     * @param subject 科目
     */
    @Query(value = "SELECT p.`question_id`,p.question_name,SUM(IF(p.level IS NULL,1,0)),COUNT(p.`question_id`), MAX(p.area_code) \n" +
            "FROM paper p WHERE p.`work_id` = ? AND p.`subject` = ? AND p.`is_missing` = false AND p.`is_test` = ? " +
            "GROUP BY p.`question_id` ,p.question_name ORDER BY p.`question_id`", nativeQuery = true)
    List<Object[]> countGroupByQuestionForLevel(Long workId, String subject, int test);

    /**
     * 统计科目指定试题的各档位数量
     *
     * @param questionId 考区ID
     */
    @Query(value = "SELECT p.`level`,COUNT(*)," +
            "SUM(p.`is_rejected`)," +
            "SUM(p.`is_arbitrated`)" +
            "FROM paper p " +
            "WHERE p.`question_id` = ? and p.batch_no is not null " +
            "GROUP BY p.`level`", nativeQuery = true)
    List<Object[]> countGroupByLevel(Long questionId);

    @Query(value = "SELECT p.`rough_level`,COUNT(*)," +
            "SUM(p.`is_rejected`)," +
            "SUM(p.`is_arbitrated`)" +
            "FROM paper p " +
            "WHERE p.`question_id` = ? and p.rough_batch_no is not null " +
            "GROUP BY p.`rough_level`", nativeQuery = true)
    List<Object[]> countGroupByRoughLevel(Long questionId);

    /**
     * 管理员-统计科目指定试题的各档位数量
     *
     * @param questionId 考区ID
     * @param test       试评状态
     */
    @Query(value = "SELECT p.`rough_level`,COUNT(*)," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift_score` = 1, 1, 0))" +
            "FROM paper p " +
            "WHERE p.`question_id` = ? and p.is_missing = false and p.is_test = ? and p.rough_batch_no is not null " +
            "GROUP BY p.`rough_level` ORDER BY p.`rough_level`", nativeQuery = true)
    List<Object[]> countGroupByRoughLevelAll(Long questionId, int test);

    /**
     * 管理员-统计科目指定试题的各档位数量
     *
     * @param questionId 考区ID
     * @param test       试评状态
     */
    @Query(value = "SELECT p.`level`,COUNT(*)," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift_score` = 1, 1, 0))" +
            "FROM paper p " +
            "WHERE p.`question_id` = ? and p.is_missing = false and p.is_test = ? and p.batch_no is not null " +
            "GROUP BY p.`level` ORDER BY p.`level`", nativeQuery = true)
    List<Object[]> countGroupByLevelAll(Long questionId, int test);

    /**
     * 科组长-统计科目指定试题的各档位数量
     *
     * @param questionId 考区ID
     * @param test       试评状态
     * @param paperIds   试卷ID集合
     */
    @Query(value = "SELECT p.`rough_level`,COUNT(*)," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift_score` = 1, 1, 0))" +
            "FROM paper p " +
            "WHERE p.`question_id` = :questionId and p.is_missing = false and p.is_test = :test and p.rough_batch_no is not null and p.id in (:paperIds) " +
            "GROUP BY p.`rough_level` ORDER BY p.`rough_level`", nativeQuery = true)
    List<Object[]> countGroupByRoughLevelAllAndIdIn(@Param("questionId") Long questionId, @Param("test") int test, @Param("paperIds") List<Long> paperIds);


    /**
     * 科组长-统计科目指定试题的各档位数量
     *
     * @param questionId 考区ID
     * @param test       试评状态
     * @param paperIds   试卷ID集合
     */
    @Query(value = "SELECT p.`level`,COUNT(*)," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift_score` = 1, 1, 0))" +
            "FROM paper p " +
            "WHERE p.`question_id` = :questionId and p.is_missing = false and p.is_test = :test and p.batch_no is not null and p.id in (:paperIds) " +
            "GROUP BY p.`level` ORDER BY p.`level`", nativeQuery = true)
    List<Object[]> countGroupByLevelAllAndIdIn(@Param("questionId") Long questionId, @Param("test") int test, @Param("paperIds") List<Long> paperIds);

    /**
     * 统计科目各试题已评数及总数（打分阶段）
     *
     * @param workId 工作Id
     */
    @Query(value = "SELECT p.`question_id`,p.question_name,SUM(IF(p.score IS NULL,1,0)),COUNT(p.`question_id`) ,MAX(p.area_code) \n" +
            "FROM paper p WHERE p.`work_id` = ? AND p.`subject` = ? AND p.is_missing = false " +
            "GROUP BY p.`question_id` ,p.question_name ORDER BY p.`question_id`", nativeQuery = true)
    List<Object[]> countGroupByQuestionForScore(Long workId, String string);

    long countByWorkIdAndScoreIsNull(Long workId);

    long countByWorkIdAndQuestionId(Long workId, Long questionId);

    long countByWorkIdAndQuestionIdAndIdIn(Long workId, Long questionId, List<Long> paperIds);

    long countByQuestionId(Long questionId);

    /**
     * 统计同一个考生的paper数量
     *
     * @param workId     工作ID
     * @param examNumber 准考证号
     */
    Long countByWorkIdAndExamNumber(Long workId, String examNumber);

    /**
     * 查询指定试卷
     *
     * @param workId     工作ID
     * @param subject    科目
     * @param examNumber 准考证号
     */
    Paper findByWorkIdAndSubjectAndExamNumber(Long workId, Subject subject, String examNumber);

    /**
     * 查询同一个考生的paper
     *
     * @param workId     工作ID
     * @param examNumber 准考证号
     */
    List<Paper> findByWorkIdAndExamNumber(Long workId, String examNumber);

    /**
     * @param workId       工作ID
     * @param secretNumber 密号
     */
    List<Paper> findByWorkIdAndSecretNumber(Long workId, String secretNumber);

    @Query(value = "select p from Paper p where p.score >= ?1 or p.score <= ?2")
    List<Paper> findByScoreRange(Double highScore, Double lowScore);

    @Query(value = "SELECT p.`exam_number`,p.`student_name`,p.question_id, p.source_name, GROUP_CONCAT(if(isnull(p.inspect_score),p.score,p.inspect_score)  ORDER BY p.`subject`) FROM paper p WHERE p.`work_id` = ? GROUP BY p.exam_number,p.`student_name` , p.source_name", nativeQuery = true)
    List<Object[]> findScores(Long workId);

    List<Paper> findByWorkId(Long workId);

    /**
     * 根据条件查找paper
     *
     * @param workId  工作ID
     * @param subject 科目
     * @param test    试评状态
     */
    List<Paper> findByWorkIdAndSubjectAndTest(Long workId, Subject subject, int test);

    @Modifying
    @Query(value = "update Paper p set p.test = ?4 where p.workId = ?1 and p.subject = ?2 and p.test = ?3")
    void updateByWorkIdAndSubjectAndTest(Long workId, Subject subject, int oldTest, int newTest);

    long countByWorkIdAndSubjectAndLevel(Long workId, Subject subject, String code);

    long countByWorkIdAndSubjectAndLevelAndIdIn(Long workId, Subject subject, String code, List<Long> finalPaperIds);

    long countByWorkIdAndSubject(Long workId, Subject subject);

    long countByWorkIdAndSubjectAndQuestionIdAndLevel(Long workId, Subject subject, Long id, String code);

    long countByQuestionIdAndLevelAndSubjectAndWorkId(Long questionId, String code, Subject subject, Long workId);

    long countByWorkIdAndSubjectAndQuestionId(Long workId, Subject subject, Long id);

    int countByWorkIdAndSubjectAndIsMissingFalseAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndRoughLevelIsNullAndIsMissingFalseAndRoughBatchNoNotNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndLevelIsNullAndIsMissingFalseAndBatchNoNotNullAndTest(Long workId, Subject subject, int test);


    @Query("select max(p.batchNo) from Paper p where p.questionId = ?1")
    Long findByQuestionId(Long questionId);

    List<Paper> findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTest(Long workId, Subject subject, Long questionId, boolean missing, int test);

    int countByWorkIdAndSubjectAndIsArbitratedTrue(Long workId, Subject subject);

    int countByWorkIdAndSubjectAndScoreBatchNoIsNotNullAndScoreIsNull(Long workId, Subject subject);

    List<Paper> findByWorkIdAndSubjectAndIsRoughSampleTrue(Long workId, Subject subject);

    List<Paper> findByWorkIdAndSubjectAndIsSampleTrue(Long workId, Subject subject);

    @Query("select max(p.scoreBatchNo) from Paper p where p.questionId = ?1")
    Long findScoreBatchNoByQuestionId(Long questionId);

    List<Paper> findByWorkIdAndSubjectAndIsRoughOneClickTrue(Long workId, Subject subject);

    List<Paper> findByWorkIdAndSubjectAndIsOneClickTrue(Long workId, Subject subject);

    @Query(value = "SELECT p.`score`,SUM(IF(p.`is_shift` = 0, 1, 0))," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift_score` = 1, 1, 0)) " +
            "FROM paper p " +
            "WHERE p.`question_id` = ? and p.is_missing = false and p.is_test = ? and p.score is null and p.score_batch_no is not null " +
            "GROUP BY p.`score`", nativeQuery = true)
    List<Object[]> countScoreGroupByLevelAll(Long questionId, int test);

    int countByWorkIdAndSubjectAndLevelAndScoreNotNull(Long workId, Subject subject, Object id);

    int countByWorkIdAndSubjectAndLevelAndScoreNotNullAndIdIn(Long workId, Subject subject, Object id, List<Long> paperIds);

    int countByWorkIdAndSubjectAndQuestionIdAndLevelAndScoreNotNull(Long workId, Subject subject, Long questionId, String valueOf);

    int countByWorkIdAndSubjectAndQuestionIdAndLevelAndScoreNotNullAndIdIn(Long workId, Subject subject, Long questionId, String valueOf, List<Long> paperIds);

    @Query("select max(p.roughBatchNo) from Paper p where p.workId = ?1 and p.subject = ?2")
    Long findMaxRoughBatchNoByWorkIdAndSubject(Long workId, Subject subject);

    @Query("select max(p.batchNo) from Paper p where p.workId = ?1 and p.subject = ?2")
    Long findMaxBatchNoByWorkIdAndSubject(Long workId, Subject subject);

    @Query(value = "select cast(p.rough_batch_no as char) from paper p where p.work_id = ?1 and p.subject = ?2 order by p.rough_batch_no desc limit 1", nativeQuery = true)
    List<Object> findRoughBatchNoByWorkIdAndSubject(Long workId, String subject);

    @Query(value = "select cast(p.batch_no as char) from paper p where p.work_id = ?1 and p.subject = ?2 order by p.batch_no desc limit 1", nativeQuery = true)
    List<Object> findBatchNoByWorkIdAndSubject(Long workId, String subject);

    @Query("select max(p.scoreBatchNo) from Paper p where p.workId = ?1 and p.subject = ?2")
    Long findMaxScoreBatchNoByWorkIdAndSubject(Long workId, Subject subject);

    @Query(value = "select cast(p.score_batch_no as char) from paper p where p.work_id = ?1 and p.subject = ?2 order by score_batch_no desc limit 1", nativeQuery = true)
    List<Object> findScoreBatchNoByWorkIdAndSubject(Long workId, String subject);

    List<Paper> findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTestAndLevelIsNull(Long workId, Subject valueOf, Long questionId, boolean b, int ordinal);

    List<Paper> findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTestAndRoughLevelIsNull(Long workId, Subject valueOf, Long questionId, boolean b, int ordinal);

    long countByWorkIdAndQuestionIdAndIsMissingIsFalse(Long workId, Long questionId);

    @Modifying
    @Query(value = "update paper p set p.is_rejected = false, p.is_tagged = false, p.score = null, p.score_batch_no = null, p.is_shift = false, p.is_shift_score = false, p.is_rejected_by_leader = false, p.is_mark = false where p.work_id = ?1 and p.subject = ?2", nativeQuery = true)
    void updateScoreByWorkIdAndSubject(Long workId, String name);

    @Modifying
    @Query(value = "update paper p set p.rough_level = case p.is_rough_sample when true then p.rough_level else null end, p.redo_rough_level = null,p.sort_num = case p.is_rough_sample when true then p.sort_num else 0 end, p.rough_batch_no = null, p.is_arbitrated = false, p.is_rejected = false, p.is_tagged = false, p.mark_by_leader = false, p.is_rough_one_click = false, p.is_mark = false where p.work_id = ?1 and p.subject = ?2", nativeQuery = true)
    void updateRoughLevelByWorkIdAndSubject(Long workId, String name);

    @Modifying
    @Query(value = "update paper p set p.level = case p.is_sample when true then p.level else null end, p.redo_level = null,p.sort_num = case p.is_sample when true then p.sort_num else 0 end, p.batch_no = null, p.is_arbitrated = false, p.is_rejected = false, p.is_tagged = false, p.mark_by_leader = false, p.is_one_click = false, p.is_mark = false where p.work_id = ?1 and p.subject = ?2", nativeQuery = true)
    void updateLevelByWorkIdAndSubject(Long workId, String name);

    @Query(value = "SELECT p.`score`,SUM(IF(p.`is_shift` = 0, 1, 0))," +
            "SUM(IF(p.`is_rejected` = 1, 1, 0))," +
            "SUM(IF(p.`is_arbitrated` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift` = 1, 1, 0))," +
            "SUM(IF(p.`is_shift_score` = 1, 1, 0)) " +
            "FROM paper p " +
            "WHERE p.`question_id` = :questionId and p.is_missing = false and p.is_test = :test and p.score is null and p.score_batch_no is not null and p.id in (:paperIds) " +
            "GROUP BY p.`score`", nativeQuery = true)
    List<Object[]> countScoreGroupByLevelAllAndIdIn(@Param("questionId") Long questionId, @Param("test") int test, @Param("paperIds") List<Long> paperIds);

    long countByWorkIdAndSubjectAndIdIn(Long workId, Subject subject, List<Long> paperIds);

    List<Paper> findByWorkIdAndSubjectAndIsArbitratedTrue(Long workId, Subject subject);

    int countByWorkIdAndSubjectAndIsMissingFalse(Long workId, Subject subject);

    @Modifying
    @Query("update Paper s set s.roughLevel = ?2, s.redoLevel = null, s.isShift = false , s.isShiftScore = false where s.id = ?1")
    void updateRoughLevelById(Long paperId, String originalLevel);

    @Modifying
    @Query("update Paper s set s.level = ?2, s.redoLevel = null, s.isShift = false , s.isShiftScore = false where s.id = ?1")
    void updateLevelById(Long paperId, String originalLevel);

    long countByWorkIdAndSubjectAndLevelAndTest(Long workId, Subject subject, String valueOf, int test);

    long countByWorkIdAndQuestionIdAndIsMissingIsFalseAndTest(Long workId, Long questionId, int test);

    long countByWorkIdAndSubjectAndLevelAndTestAndBatchNoNotNull(Long workId, Subject subject, String valueOf, int test);

    List<Paper> findByWorkIdAndSubjectAndSecretNumberAndTest(Long workId, Subject subject, String secretNumber, int id);

    @Modifying
    @Query("update Paper s set s.isMark = ?2 where s.id = ?1")
    void updatePaperMarkById(Long paperId, boolean isMark);

    @Query(value = "select * from paper p where p.scan_user_id = ?1 order by p.uploaded_on desc limit 1", nativeQuery = true)
    Paper findByScanUserIdOrderByUploadedOnDescLimit(Long id);

    List<Paper> findByWorkIdAndIsMissing(Long workId, boolean missing);

    List<Paper> findByWorkIdAndIsAdminMarkTrue(Long workId);

    @Modifying
    @Query("update Paper p set p.test = ?3, p.batchNo = null where p.workId = ?1 and p.test = ?2")
    void updateTestByWorkId(Long workId, int source, int dest);

    List<Paper> findByWorkIdAndSubjectOrderByLevelAscSortNumDesc(Long workId, Subject subject);

    long countByWorkId(Long workId);

    int countByWorkIdAndSubjectAndIsMissingFalseAndBatchNoIsNullAndTestAndQuestionIdIn(Long workId, Subject subject, int test, List<Long> questionIds);

    int countByWorkIdAndSubjectAndRoughLevelNotNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndRoughLevelNotNullAndRoughBatchNoNotNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndLevelNotNullAndBatchNoNotNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndScoreNotNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndIsMissingFalseAndRoughBatchNoIsNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndIsMissingFalseAndRoughBatchNoIsNullAndTestAndQuestionIdIn(Long workId, Subject subject, int test, List<Long> questionIdsLong);

    int countByWorkIdAndSubjectAndIsMissingFalseAndBatchNoIsNullAndTest(Long workId, Subject subject, int test);

    long countByQuestionIdAndRoughLevelAndSubjectAndWorkId(Long questionId, String valueOf, Subject subject, Long workId);

    long countByWorkIdAndSubjectAndRoughLevelAndTestAndRoughBatchNoNotNull(Long workId, Subject subject, String valueOf, int test);

    long countByWorkIdAndSubjectAndRoughLevelAndIdIn(Long workId, Subject subject, String valueOf, List<Long> finalPaperIds);

    List<Paper> findByWorkIdAndQuestionIdAndSubjectAndLevelAndScoreBatchNoIsNull(Long workId, Long questionId0, Subject subject, String code, Sort sort);

    List<Paper> findByWorkIdAndSubjectAndLevelNotNullAndScoreBatchNoIsNull(Long workId, Subject subject, Sort sort);

    int countByWorkIdAndSubjectAndRoughLevelIsNullAndIsMissingFalseAndRoughBatchNoIsNullAndTest(Long workId, Subject subject, int test);

    int countByWorkIdAndSubjectAndLevelIsNullAndIsMissingFalseAndBatchNoIsNullAndTest(Long workId, Subject subject, int test);

    @Modifying
    @Query("update Paper s set s.isAdminMark = ?2 where s.id = ?1")
    void updatePaperAdminMarkById(Long paperId, boolean isMark);

    long countByWorkIdAndSubjectAndRoughLevelAndTest(Long workId, Subject subject, String valueOf, int test);

    @Query("select p.randomSeq from Paper p where p.workId = ?1")
    Set<Long> listRandomByWorkId(Long workId);
}
