package cn.com.qmth.stmms.ms.log.aop;

import cn.com.qmth.stmms.ms.commons.constant.ArbitrateCallback;
import cn.com.qmth.stmms.ms.commons.constant.ArbitrateResult;
import cn.com.qmth.stmms.ms.commons.utils.EncrypAES;
import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.config.MarkingConfig;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.service.DetermineRoughLevelService;
import cn.com.qmth.stmms.ms.marking.service.MarkingRoughLevelService;
import cn.com.qmth.stmms.ms.marking.service.MarkingService;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.net.URLDecoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 纪检审查aop
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/10/19
 */
@Aspect
@Component
public class MarkLogAop {

    private static Logger LOGGER = LoggerFactory.getLogger(MarkLogAop.class);

    private final String TOKEN = "token",
            MARK_USER = "markUser",//登录用户
            PARAM_ARGS = "args",//方法参数
    //            PARAM_ARG_NAMES = "argNames",//方法参数名
    ACTION = "action",//动作
            SAMPLING = "sampling",//标准卷设置
            LEVELING = "leveling",//定档
            REJECT = "reject",//档位打回
            PAPER = "paper",//paper
            STAGE = "stage",//stage
            LEVEL = "level",//level
            BODY = "body",//body
            RESULT = "result",//result
            DEFAULT_RESULT = "-";//默认值

    @Autowired
    private MarkLogRepo markLogRepo;

    @Autowired
    MarkUserRepo markUserRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Autowired
    MarkTaskLevelRepo markTaskLevelRepo;

    @Autowired
    MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    MarkingConfig markingConfig;

    @Autowired
    WorkRepo workRepo;

    @Autowired
    ArbitrateCallback arbitrateCallback;

    @Resource
    MarkTaskJobRepo markTaskJobRepo;

    @Autowired
    LevelRepo levelRepo;

    /**
     * 标准卷、定档、档位打回切入点
     */
    @Pointcut("execution(* cn.com.qmth.stmms.ms.marking.api.PaperApi.marking(..))")
    public void paperAspect() {
    }

    /**
     * 分档、打分、回评档位、回评分数、档位打回回评切入点
     */
    @Pointcut("execution(* cn.com.qmth.stmms.ms.marking.api.MarkTaskApi.marking(..))")
    public void markAspect() {
    }

    /**
     * 采集端采集试卷切入点
     */
    @Pointcut("execution(* cn.com.qmth.stmms.ms.collect.api.CollectApi.imageUploadSheet(..))")
    public void collectAspect() {
    }

    /**
     * 标准卷设置、定档设置、档位打回切入
     *
     * @param joinPoint
     * @param result
     * @throws Exception
     */
//    @AfterReturning(value = "paperAspect()", returning = RESULT)
    @Transactional
    public void afterReturnPaperPoint(JoinPoint joinPoint, Object result) throws Exception {
        Map<String, Object> map = getAspectCommon(joinPoint);
        JSONArray jsonArgsArray = (JSONArray) map.get(PARAM_ARGS);
        MarkUser markUser = (MarkUser) map.get(MARK_USER);
//        Paper paper = JSONObject.parseObject(((JSONObject) jsonArgsArray.get(0)).toJSONString(), Paper.class);
        JSONObject jsonObjectResult = (JSONObject) jsonArgsArray.get(0);
        String paperIds = (String) jsonObjectResult.get("paperIds");
        MarkStage stage = MarkStage.valueOf((String) jsonObjectResult.get("stage"));
        if (paperIds.length() > 0) {
            String[] ids = paperIds.split(",");
            Date date = new Date();
            for (String id : ids) {
                Paper paper = paperRepo.findOne(Long.valueOf(id));

                String action = null;
                Integer operType = null;
                String operName = null;
                String originLevel = null;
                if (Objects.nonNull(jsonObjectResult) && Objects.nonNull(jsonObjectResult.get(ACTION))) {
                    action = (String) jsonObjectResult.get(ACTION);
                    if (Objects.equals(SAMPLING, action)) {
                        //标准卷设置
                        operType = MarkLogOperType.CRITERION_PAPER_SET.getId();
                        operName = MarkLogOperType.CRITERION_PAPER_SET.getName();
                        paper.setOneClick(false);
                    } else if (Objects.equals(LEVELING, action)) {
                        //定档设置
                        operType = MarkLogOperType.ONE_CLICK_LEVEl.getId();
                        operName = MarkLogOperType.ONE_CLICK_LEVEl.getName();
                        //原始档位
                        originLevel = Objects.nonNull(jsonObjectResult.get("originLevel")) ? (String) jsonObjectResult.get("originLevel") : null;
                        paper.setOneClick(true);
                    } else if (Objects.equals(REJECT, action)) {
                        //档位打回设置
                        operType = MarkLogOperType.CALLBACK_LEVEl.getId();
                        operName = MarkLogOperType.CALLBACK_LEVEl.getName();
                        paper.setOneClick(false);
                    }
                }
                //标准卷设置、定档设置、档位打回start
                MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), stage, operType, markUser.getSubject().toString(), paper.getExamNumber());
                String operResult = DEFAULT_RESULT;
                if (Objects.nonNull(markLogPrev)) {
                    operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                }
                if (Objects.equals(LEVELING, action)) {
                    //定档设置
                    operResult = originLevel;
                }
                Work work = workRepo.findOne(paper.getWorkId());
                MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), markUser.getSubject(), MarkStage.LEVEL, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), markUser.getRole(), operType, operName, operResult, String.valueOf(jsonObjectResult.get(LEVEL)), null, date);
                LOGGER.info("markLog:{}", JSONObject.toJSONString(markLog));
                //标准卷设置、定档设置、档位打回end
                markLogRepo.save(markLog);
                //更新
                paperRepo.save(paper);
            }
        }
    }

    /**
     * 分档、打分、回评档位、回评分数、档位打回回评切入
     *
     * @param joinPoint
     * @param result
     * @throws Exception
     */
//    @AfterReturning(value = "markAspect()", returning = RESULT)
    @Transactional
    public void afterReturnMarkPoint(JoinPoint joinPoint, Object result) throws Exception {
        Map<String, Object> map = getAspectCommon(joinPoint);
        JSONArray jsonArgsArray = (JSONArray) map.get(PARAM_ARGS);
        MarkUser markUser = (MarkUser) map.get(MARK_USER);
        JSONObject aopResult = JSONObject.parseObject(JSONObject.toJSONString(result));
        boolean isRejected = false, oldRejected = false, oldShift = false, oldShiftScore = false;
        String daterMineResult = null;
        if (Objects.nonNull(aopResult)) {
            MarkTaskDTO resultBody = JSONObject.parseObject(((JSONObject) aopResult.get(BODY)).toJSONString(), MarkTaskDTO.class);
            isRejected = resultBody.isRejected();//新的是否撤档
            oldRejected = resultBody.isOldRejected();//旧的是否撤档
            oldShift = resultBody.isShift();//旧的改档
            oldShiftScore = resultBody.isShiftScore();//旧的改档打分
            daterMineResult = resultBody.getDateMineResult();
        }
        JSONObject jsonObjectResult = (JSONObject) jsonArgsArray.get(1);
        MarkStage markStage = MarkStage.valueOf(String.valueOf(jsonObjectResult.get(STAGE)));

        long markTaskId = Long.parseLong(jsonArgsArray.get(0).toString());
        Paper paper = null;
        if (markStage.equals(MarkStage.ROUGH_LEVEL)) {
            MarkTaskRoughLevel markTaskRoughLevel = markTaskRoughLevelRepo.findOne(markTaskId);
            paper = markTaskRoughLevel.getPaper();
        } else if (markStage.equals(MarkStage.LEVEL)) {
            MarkTaskLevel markTaskLevel = markTaskLevelRepo.findOne(markTaskId);
            paper = markTaskLevel.getPaper();
        } else if (markStage.equals(MarkStage.SCORE)) {
            MarkTaskScore markTaskScore = markTaskScoreRepo.findOne(markTaskId);
            paper = markTaskScore.getPaper();
        }

        LOGGER.info("markTaskId:{},paper:{},jsonObjectResult:{}", markTaskId, paper, jsonObjectResult);
        Integer operType = MarkLogOperType.LEVEL.getId();//分档,第一次为分档，之后为回评档位,档位打回后为档位打回回评
        String operName = MarkLogOperType.LEVEL.getName();
        //分档、打分、回评档位、回评分数、档位打回回评start

        if (markStage.ordinal() == 1 && paper.isMarkByLeader()) {//分档,如果已经有科组长定档，则不允许评卷员再分档
            return;
        } else if ((markStage.ordinal() == 1 || markStage.ordinal() == 2) && oldRejected && !isRejected) {//档位打回回评
            if (oldShift) {
                operType = MarkLogOperType.CHANGE_LEVEL.getId();
                operName = MarkLogOperType.CHANGE_LEVEL.getName();
            } else {
                operType = MarkLogOperType.CALLBACK_ONCE_LEVEL.getId();
                operName = MarkLogOperType.CALLBACK_ONCE_LEVEL.getName();
            }
        } else if (markStage.ordinal() == 3) {//打分
            if (oldShiftScore) {
                operType = MarkLogOperType.CHANGE_SCORE.getId();
                operName = MarkLogOperType.CHANGE_SCORE.getName();
            } else {
                operType = MarkLogOperType.SCORE.getId();
                operName = MarkLogOperType.SCORE.getName();
            }
        }
        //先查是否有档位或分数打回
        String operResult = DEFAULT_RESULT;
        if (markStage.ordinal() == 1) {//分档
            //先查是否有档位打回回评的数据
            MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), markStage, MarkLogOperType.CALLBACK_ONCE_LEVEL.getId(), markUser.getSubject().toString(), paper.getExamNumber());
            if (Objects.isNull(markLogPrev)) {//如果为null，则再查分档和回评档位
                if (oldRejected && !isRejected) {//档位打回
                    if (oldShift) {
                        operType = MarkLogOperType.CHANGE_LEVEL.getId();
                        operName = MarkLogOperType.CHANGE_LEVEL.getName();
                    } else {
                        operType = MarkLogOperType.CALLBACK_ONCE_LEVEL.getId();
                        operName = MarkLogOperType.CALLBACK_ONCE_LEVEL.getName();
                    }
                } else {
                    operType = selectOperType(markUser.getId(), MarkLogOperType.LEVEL.getId(), MarkLogOperType.ONCE_LEVEl.getId(), markUser.getSubject().toString(), paper.getExamNumber());
                    operName = MarkLogOperType.convertToName(operType);
                    markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), markStage, operType, markUser.getSubject().toString(), paper.getExamNumber());
                    if (Objects.nonNull(markLogPrev)) {
                        operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                    } else {
                        if (operType != MarkLogOperType.LEVEL.getId()) {
                            markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), markStage, MarkLogOperType.LEVEL.getId(), markUser.getSubject().toString(), paper.getExamNumber());
                            operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                        }
                    }
                }
            } else {
                operType = MarkLogOperType.CALLBACK_ONCE_LEVEL.getId();//档位打回
                operName = MarkLogOperType.CALLBACK_ONCE_LEVEL.getName();//档位打回
                operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
            }
        } else if (markStage.ordinal() == 2 && !oldShiftScore) {//打分
            operType = selectOperType(markUser.getId(), MarkLogOperType.SCORE.getId(), MarkLogOperType.ONCE_SCORE.getId(), markUser.getSubject().toString(), paper.getExamNumber());
            operName = MarkLogOperType.convertToName(operType);
            MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), markStage, operType, markUser.getSubject().toString(), paper.getExamNumber());
            if (Objects.nonNull(markLogPrev)) {
                operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
            } else {
                if (operType != MarkLogOperType.SCORE.getId()) {
                    markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markUser.getId(), markStage, MarkLogOperType.SCORE.getId(), markUser.getSubject().toString(), paper.getExamNumber());
                    operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse(DEFAULT_RESULT);
                }
            }
        }
        Work work = workRepo.findOne(paper.getWorkId());
        MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), markUser.getSubject(), markStage, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), markUser.getRole(), operType, operName, operResult, String.valueOf(jsonObjectResult.get(RESULT)), null, new Date());
        LOGGER.info("markLog:{}", JSONObject.toJSONString(markLog));
        //分档、打分、回评档位、回评分数、档位打回、档位打回回评end
        markLogRepo.save(markLog);
        //20191107wangliang加入仲裁自动打回算法 start
        if (markStage.ordinal() == 1 && paper.isArbitrated()) {
            if (ParamCache.levelConfigMap.get(String.valueOf(work.getId())).getAutoCallback() == 1) {//为true则自动打回
                JSONObject js = new JSONObject();
                js.put("paperId", paper.getId());
                js.put("type", "autoCallback");
                Integer deviation = Optional.ofNullable(ParamCache.levelConfigMap.get(String.valueOf(work.getId())).getDeviation()).orElse(3);
                js.put("deviation", deviation);
                MarkTaskJob markTaskJob = new MarkTaskJob(js.toJSONString(), false, paper.getWorkId());
                markTaskJobRepo.save(markTaskJob);
                if (markStage.equals(MarkStage.ROUGH_LEVEL)) {
                    List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperIdAndStage(paper.getId(), markStage);
                    autoCallbackRoughLevel(markTasks, deviation, paper, markTaskJob, work.getName(), daterMineResult);
                } else if (markStage.equals(MarkStage.LEVEL)) {
                    List<MarkTaskLevel> markTasks = markTaskLevelRepo.findByPaperIdAndStage(paper.getId(), markStage);
                    autoCallback(markTasks, deviation, paper, markTaskJob, work.getName(), daterMineResult);
                }
            }
        }
        //20191107wangliang加入仲裁自动打回算法 end
    }

    /**
     * 自动打回异步回调
     *
     * @param markTasks
     * @param deviation
     * @param paper
     * @param markTaskJob
     * @param workName
     * @param daterMineResult
     */
    public void autoCallback(List<MarkTaskLevel> markTasks, int deviation, Paper paper, MarkTaskJob markTaskJob, String workName, String daterMineResult) {
        List<MarkTaskLevel> result = markTasks.stream().filter(x -> Objects.isNull(x.getResult())).collect(Collectors.toList());
        if (Objects.nonNull(result) && result.size() > 0) {
            LOGGER.info("数据有误，未完成评卷任务");
            markTaskJobRepo.updateMarkTaskJobByVersion(markTaskJob.getId(), markTaskJob.getVersion());
            return;
        }

        List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());
        Map<String, Integer> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, Level::getLevelValue));

        Map<Long, String> levelsMap = markTasks.stream().collect(Collectors.toMap(MarkTaskLevel::getMarkerId, o -> o.getResult().toUpperCase()));
        LOGGER.info("this:{} markLogAop异步回调进来了", this);
        arbitrateCallback.judge(levelsMap, deviation, new ArbitrateResult() {
            @Override
            public void callback(List<ArbitrateCallback.Distance> list) {
                LOGGER.info("this:{} callback markLogAop异步回调进来了", this);
                String operResult = DEFAULT_RESULT;
                //全部打回则不触发自动仲裁
                if (Objects.nonNull(list) && list.size() > 0 && markTasks.size() != list.size()) {
                    paper.reject(null);
                    List<MarkLog> markLogList = new ArrayList<>();
                    Date date = new Date();
                    for (ArbitrateCallback.Distance d : list) {
                        for (MarkTaskLevel m : markTasks) {
                            if (Objects.equals(String.valueOf(d.getC()), m.getResult()) && d.getMarkId().longValue() == m.getMarkerId().longValue()) {
                                m.setDeviationDirection(MarkingService.calcDeviationDirection(levelMap, m.getResult(), daterMineResult));
                                m.setRejected(true);
                                m.setOriginLevel(m.getResult());
                                m.setResult(null);
                                m.setLevel(null);
                                MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(m.getMarkerId(), m.getStage(), MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getId(), m.getSubject().toString(), paper.getExamNumber());
                                if (Objects.nonNull(markLogPrev)) {
                                    operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse("-");
                                }
                                //加入打回日志
                                MarkLog markLog = new MarkLog(paper.getWorkId(), workName, paper.getId(), m.getSubject(), m.getStage(), m.getTest(), paper.getExamNumber(), paper.getStudentName(), m.getMarkerId(), m.getMarkerName(), Role.MARKER, MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getId(), MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getName(), operResult, String.valueOf(d.getC()), null, date);
                                markLogList.add(markLog);
                            }
                        }
                    }
                    int count = markTaskJobRepo.updateMarkTaskJobByVersion(markTaskJob.getId(), markTaskJob.getVersion());
                    if (count > 0) {
                        markTaskLevelRepo.save(markTasks);
                        paperRepo.save(paper);
                        markLogRepo.save(markLogList);
                    }
                }
                LOGGER.info("this:{} callback markLogAop异步回调结束了", this);
            }
        });
        LOGGER.info("this:{} markLogAop异步回调结束了", this);
    }

    /**
     * 粗分档自动打回异步回调
     *
     * @param markTasks
     * @param deviation
     * @param paper
     * @param markTaskJob
     * @param workName
     * @param daterMineResult
     */
    public void autoCallbackRoughLevel(List<MarkTaskRoughLevel> markTasks, int deviation, Paper paper, MarkTaskJob markTaskJob, String workName, String daterMineResult) {
        List<MarkTaskRoughLevel> result = markTasks.stream().filter(x -> Objects.isNull(x.getResult())).collect(Collectors.toList());
        if (Objects.nonNull(result) && !result.isEmpty()) {
            LOGGER.info("数据有误，未完成评卷任务");
            markTaskJobRepo.updateMarkTaskJobByVersion(markTaskJob.getId(), markTaskJob.getVersion());
            return;
        }

        List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());
        Map<Long, String> levelsMap = markTasks.stream().collect(Collectors.toMap(MarkTaskRoughLevel::getMarkerId, o -> o.getResult().toUpperCase()));
        LOGGER.info("this:{} markLogAop异步回调进来了", this);
        arbitrateCallback.judge(levelsMap, deviation, new ArbitrateResult() {
            @Override
            public void callback(List<ArbitrateCallback.Distance> list) {
                LOGGER.info("this:{} callback markLogAop异步回调进来了", this);
                String operResult = DEFAULT_RESULT;
                //全部打回则不触发自动仲裁
                if (Objects.nonNull(list) && !list.isEmpty() && markTasks.size() != list.size()) {
                    paper.reject(null);
                    List<MarkLog> markLogList = new ArrayList<>();
                    Date date = new Date();
                    for (ArbitrateCallback.Distance d : list) {
                        for (MarkTaskRoughLevel m : markTasks) {
                            if (Objects.equals(String.valueOf(d.getC()), m.getResult()) && d.getMarkId().longValue() == m.getMarkerId().longValue()) {
                                m.setDeviationDirection(MarkingRoughLevelService.calcDeviationDirection(m.getResult(), daterMineResult));
                                m.setRejected(true);
                                m.setOriginLevel(m.getResult());
                                m.setResult(null);
                                m.setLevel(null);
                                MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(m.getMarkerId(), m.getStage(), MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getId(), m.getSubject().toString(), paper.getExamNumber());
                                if (Objects.nonNull(markLogPrev)) {
                                    operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse("-");
                                }
                                //加入打回日志
                                MarkLog markLog = new MarkLog(paper.getWorkId(), workName, paper.getId(), m.getSubject(), m.getStage(), paper.getTest(), paper.getExamNumber(), paper.getStudentName(), m.getMarkerId(), m.getMarkerName(), Role.MARKER, MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getId(), MarkLogOperType.SYSTEM_CALLBACK_LEVEl.getName(), operResult, String.valueOf(d.getC()), null, date);
                                markLogList.add(markLog);
                            }
                        }
                    }
                    int count = markTaskJobRepo.updateMarkTaskJobByVersion(markTaskJob.getId(), markTaskJob.getVersion());
                    if (count > 0) {
                        markTaskRoughLevelRepo.save(markTasks);
                        paperRepo.save(paper);
                        markLogRepo.save(markLogList);
                    }
                }
                LOGGER.info("this:{} callback markLogAop异步回调结束了", this);
            }
        });
        LOGGER.info("this:{} markLogAop异步回调结束了", this);
    }


    /**
     * 查询处于哪个档位公用
     *
     * @param createUserId
     * @param operTypeStart
     * @param operTypeEnd
     * @param subject
     * @param examNumber
     * @return
     */
    public Integer selectOperType(Long createUserId, Integer operTypeStart, Integer operTypeEnd, String subject, String examNumber) {
        String sql = "select t.oper_type as operType, count(*) as result from mark_log as t where t.create_user_id = ? and (t.oper_type = ? or t.oper_type = ?) and t.subject = ? and t.exam_number = ? group by t.oper_type ";
        List list = sqlUtil.execSqlForMapNative(sql, createUserId, operTypeStart, operTypeEnd, subject, examNumber);
        Integer operType = operTypeStart;
        if (Objects.nonNull(list) && list.size() > 0) {
            for (int i = 0; i < list.size(); i++) {
                Map m = (Map) list.get(i);
                Integer type = Integer.parseInt(String.valueOf(m.get("operType")));
                if (type.intValue() == operTypeStart.intValue()) {
                    operType = operTypeEnd;
                    break;
                } else {
                    operType = type;
                    break;
                }
            }
        }
        return operType;
    }

    /**
     * 切面公用方法
     *
     * @param joinPoint
     * @return
     * @throws Exception
     */
    public Map getAspectCommon(JoinPoint joinPoint) throws Exception {
        String className = joinPoint.getSignature().getDeclaringTypeName();
        String methodName = joinPoint.getSignature().getName();
        LOGGER.info("{} aspect path:{}", this.getClass().getMethods()[0], className + "." + methodName);
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        String token = request.getHeader("Authorization");
        if (Objects.isNull(token)) {
            throw new Exception("请先登录");
        }
        token = URLDecoder.decode(token, "UTF-8");
        token = EncrypAES.decrypt(token);
        Object[] args = joinPoint.getArgs(); // 参数值
        String userId = request.getHeader("userId");
        // markLeaderId是批量打回小工具中的参数
        userId = StringUtils.isBlank(userId) || (StringUtils.isNotBlank(userId) && "0".equals(userId)) ? request.getHeader("markLeaderId") : userId;
        MarkUser markUser = markUserRepo.findOne(Long.valueOf(userId));
        if (Objects.isNull(markUser)) {
            String[] strs = token.split("#");
            markUser = strs == null || strs[2] == null || strs[2].equals("null") ? null : markUserRepo.findOne(Long.parseLong(strs[2]));
        }
        LOGGER.info("markUser:{}", JSONObject.toJSONString(markUser));
//        String[] argNames = ((MethodSignature) joinPoint.getSignature()).getParameterNames();//参数名
        JSONArray jsonArgsArray = JSONObject.parseArray(JSONObject.toJSONString(args));
//        JSONArray jsonArgNamesArray = JSONObject.parseArray(JSONObject.toJSONString(argNames));
        LOGGER.info("jsonArgsArray:{},jsonArgNamesArray:{}", jsonArgsArray);
//        LOGGER.info("jsonArgNamesArray:{}", jsonArgNamesArray);
        Map<String, Object> map = new HashMap<>();
        map.put(PARAM_ARGS, jsonArgsArray);
//        map.put(PARAM_ARG_NAMES, jsonArgNamesArray);
        map.put(MARK_USER, markUser);
        return map;
    }
}
