package cn.com.qmth.stmms.ms.log.controller;

import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.commons.utils.specification.PagingAndSortingDTO;
import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkLogRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;


/**
 * 纪检审计接口controller
 */
@RestController
@RequestMapping("api/marklog")
public class MarkLogController {

    @Autowired
    MarkLogRepo markLogRepo;

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    WorkRepo workRepo;

    @Resource
    MarkUserRepo markUserRepo;

    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    private PaperAssembler paperAssembler;

    @GetMapping("/selectMarkLog")
    public PagingAndSortingDTO selectMarkLog(
            @RequestParam(required = false) String subject,
            @RequestParam(required = false) Long examNumber,
            @RequestParam(required = false) String studentName,
            @RequestParam(required = false) Integer operType,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime,
            @RequestParam(required = false) Long workId,
            @RequestParam Integer curPage,
            @RequestParam Integer pageSize) {
        Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "createTime");
        Specification<MarkLog> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(subject)) {
                predicates.add(cb.equal(root.get("subject").as(String.class), subject));
            }
            if (Objects.nonNull(examNumber)) {
                predicates.add(cb.equal(root.get("examNumber").as(String.class), examNumber));
            }
            if (Objects.nonNull(studentName) && !studentName.equals("")) {
                predicates.add(cb.like(root.get("studentName").as(String.class), "%" + studentName + "%"));
            }
            if (Objects.nonNull(operType)) {
                predicates.add(cb.equal(root.get("operType").as(Integer.class), operType));
            }
            if (Objects.nonNull(startTime)) {
                //大于或等于传入时间
                predicates.add(cb.greaterThanOrEqualTo(root.get("createTime").as(String.class), startTime));
            }
            if (Objects.nonNull(endTime)) {
                //小于或等于传入时间
                predicates.add(cb.lessThanOrEqualTo(root.get("createTime").as(String.class), endTime));
            }
            if (Objects.nonNull(workId)) {
                predicates.add(cb.equal(root.get("workId").as(Long.class), workId));
            }
            List<Map> mapList = MarkLogOperType.listTypes();
            if (!mapList.isEmpty()) {
                CriteriaBuilder.In<Object> in = cb.in(root.get("operType"));
                for (Map map : mapList) {
                    in.value(map.get("id"));
                }
                predicates.add(cb.and(cb.and(in)));
            }
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Page<MarkLog> result = markLogRepo.findAll(specification, pageable);
        boolean roughLevelEnable = ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1;
        for (MarkLog markLog : result.getContent()) {
            MarkStage stage = markLog.getStage();
            if(MarkStage.ROUGH_LEVEL.equals(stage)){
                markLog.setStageName("粗分档阶段");
            } else if(MarkStage.LEVEL.equals(stage)){
                markLog.setStageName(roughLevelEnable ? "细分档阶段" : "分档阶段");
            } else if(MarkStage.SCORE.equals(stage)){
                markLog.setStageName("打分阶段");
            } else {
                markLog.setStageName("无");
            }
        }
        return new PagingAndSortingDTO(result.getNumber(), result.getSize(), result.getTotalPages(),
                result.getTotalElements(), result.getContent());
    }

    /**
     * 采集日志
     *
     * @param subject
     * @param examNumber
     * @param studentName
     * @param createUserId
     * @param startTime
     * @param endTime
     * @param curPage
     * @param pageSize
     * @return
     */
    @GetMapping("/selectCollectMarkLog")
    public PagingAndSortingDTO selectCollectMarkLog(
            @RequestParam(required = false) String subject,
            @RequestParam(required = false) Long examNumber,
            @RequestParam(required = false) String studentName,
            @RequestParam(required = false) String createUserId,
            @RequestParam(required = false) String startTime,
            @RequestParam(required = false) String endTime,
            @RequestParam(required = false) Long workId,
            @RequestParam Integer curPage,
            @RequestParam Integer pageSize) {
        Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "createTime");
        Specification<MarkLog> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(subject)) {
                predicates.add(cb.equal(root.get("subject").as(String.class), subject));
            }
            if (Objects.nonNull(examNumber)) {
                predicates.add(cb.equal(root.get("examNumber").as(String.class), examNumber));
            }
            if (Objects.nonNull(studentName) && studentName != "") {
                predicates.add(cb.like(root.get("studentName").as(String.class), "%" + studentName + "%"));
            }
            if (Objects.nonNull(createUserId)) {
                predicates.add(cb.equal(root.get("createUserId").as(String.class), createUserId));
            }
            if (Objects.nonNull(startTime)) {
                //大于或等于传入时间
                predicates.add(cb.greaterThanOrEqualTo(root.get("createTime").as(String.class), startTime));
            }
            if (Objects.nonNull(endTime)) {
                //小于或等于传入时间
                predicates.add(cb.lessThanOrEqualTo(root.get("createTime").as(String.class), endTime));
            }
            if (Objects.nonNull(workId)) {
                predicates.add(cb.equal(root.get("workId").as(Long.class), workId));
            }
            //采集端
            predicates.add(cb.equal(root.get("operType").as(Integer.class), MarkLogOperType.COLLECT.getId()));
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Page<MarkLog> result = markLogRepo.findAll(specification, pageable);
        return new PagingAndSortingDTO(result.getNumber(), result.getSize(), result.getTotalPages(),
                result.getTotalElements(), result.getContent());
    }

    /**
     * 采集成功记录日志
     *
     * @param workId
     * @param request
     * @return
     * @throws ParseException
     */
    @RequestMapping(value = "/saveCollectLog", method = RequestMethod.POST)
    public Object saveCollectLog(Long workId, HttpServletRequest request) throws ParseException {
        //取采集用户id
        Long clientUserId = Long.valueOf(request.getParameter("clientUserId"));
        //取采集用户名称
        String clientUsername = request.getParameter("clientUsername");
        //学生信息
        String name = request.getParameter("name");
        //科目信息
        Integer subjectId = Integer.valueOf(request.getParameter("subjectId"));
        //准考证号
        String examNumber = request.getParameter("examNumber");
        Subject subject = Subject.values()[subjectId - 1];
        //试卷信息
        Paper paper = paperRepo.findByWorkIdAndSubjectAndExamNumber(workId, subject, examNumber);

        Integer operType = MarkLogOperType.COLLECT.getId();

        Work work = workRepo.findOne(workId);

        MarkLog markLog = new MarkLog();
        markLog.setCreateUserId(clientUserId);
        markLog.setCreateUserName(clientUsername);
        markLog.setCreateRole(Role.COLLECTOR.getName());
        markLog.setSubject(subject.toString());
        markLog.setExamNumber(examNumber);
        markLog.setStudentName(name);
        markLog.setOperType(operType);
        markLog.setWorkId(workId);
        markLog.setWorkName(work.getName());
        if (paper != null) {
            markLog.setPaperId(paper.getId());
        }
        markLog.setStage(MarkStage.INIT);
        //识别方式
        String manual = request.getParameter("manual");
        //增加考生不存在备注
        markLog.setRemark(paper == null ? "考生不存在" : "0".equals(manual) ? "正常扫描" : "1".equals(manual) ? "手工绑定" : null);
        //操作时间
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String time = request.getParameter("time");
        markLog.setCreateTime(formatter.parse(time));
        //登录时间
        String clientUserLoginTime = request.getParameter("clientUserLoginTime");
        markLog.setLoginTime(formatter.parse(clientUserLoginTime));
        //退出时间(没有退出时间)
        return markLogRepo.saveAndFlush(markLog);
    }

    /**
     * 设备监控（列表）
     *
     * @param workId   工作ID
     * @param pageable 分页参数
     */
    @RequestMapping(value = "/equipmentMonitor", method = RequestMethod.GET)
    public Object saveCollectLog(Long workId, Pageable pageable) {
        Specification<MarkUser> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("workId"), workId));
            predicates.add(builder.equal(root.get("role"), Role.COLLECTOR));
            return builder.and(predicates.toArray(new Predicate[0]));
        };
        Sort sort = new Sort(Sort.Direction.ASC, "loginName");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Page<MarkUser> markUserPage = markUserRepo.findAll(specification, pageable1);
        List<MarkUser> markUserList = markUserPage.getContent();
        List<JSONObject> arrayList = new ArrayList<>();
        for (MarkUser markUser : markUserList) {
            JSONObject object = new JSONObject();
            object.put("createUserName", markUser.getName());
            Paper paper = paperRepo.findByScanUserIdOrderByUploadedOnDescLimit(markUser.getId());
            PaperDTO paperDTO = paperAssembler.toDTOForEquipmentMonitor(paper, false);
            object.put("imgSrc", paper == null ? null : paperDTO.getImgSrc());
            object.put("thumbSrc", paper == null ? null : paperDTO.getThumbSrc());
            object.put("sheetSrc", paper == null ? null : paperDTO.getSheetSrc());
            arrayList.add(object);
        }
        return new PageableDTO(arrayList, markUserPage.getTotalElements(), markUserPage.getTotalPages(), pageable.getPageNumber());
    }

    /**
     * 设备监控
     */
    @RequestMapping(value = "/listTypes", method = RequestMethod.GET)
    public Object listTypes() {
        return MarkLogOperType.listTypes();
    }
}
