package cn.com.qmth.stmms.ms.quartz;

import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkTaskJob;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.service.MarkingLevelService;
import cn.com.qmth.stmms.ms.marking.service.MarkingRoughLevelService;
import cn.com.qmth.stmms.ms.marking.utils.RandomUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description: quartz定时任务 task
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/1/16
 */
@Component
@Async
public class ScheduledTask implements InitializingBean {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(ScheduledTask.class);

    @Resource
    SqlUtil sqlUtil;

    @Resource
    RandomUtil randomUtil;

    @Resource
    PaperRepo paperRepo;

    @Resource
    WorkRepo workRepo;

    @Resource
    MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    MarkTaskJobRepo markTaskJobRepo;

    @Resource
    MarkingRoughLevelService markingRoughLevelService;

    @Resource
    MarkingLevelService markingLevelService;

    boolean repairPaper = false, repairTask = false, markTaskJob = false;

    /**
     * 修复密号重复 30秒一次
     */
//    @Scheduled(cron = "0 0/5 * * * ?")
    public void repairSecretNumberTask() {
        List<Work> works = workRepo.findAll();
        if (Objects.nonNull(works)) {
            if (!this.repairPaper) {
                LOGGER.info("repairRepeatSecretNumberTask 修复试卷密号进来了");
                for (Work work : works) {
                    repairPaper(work.getId());
                }
                LOGGER.info("repairRepeatSecretNumberTask 修复试卷密号结束了");
            }
            if (!this.repairTask) {
                LOGGER.info("repairRepeatSecretNumberTask 修复任务密号进来了");
                for (Work work : works) {
                    repairTask(work.getId());
                }
                LOGGER.info("repairRepeatSecretNumberTask 修复任务密号结束了");
            }
        }
    }

    /**
     * 修复异步任务日志遗漏 1分钟一次
     */
    @Scheduled(cron = "0 0/1 * * * ?")
    public void markTaskJob() {
        List<Work> works = workRepo.findAll();
        if (Objects.nonNull(works)) {
            if (!markTaskJob) {
                LOGGER.info("markTaskJob 进来了");
                for (Work work : works) {
                    repairMarkTaskJob(work);
                }
                LOGGER.info("markTaskJob 结束了");
            }
        }
    }

    /**
     * 修复异步任务日志遗漏
     *
     * @param work 工作
     */
    public void repairMarkTaskJob(Work work) {
        try {
            List<MarkTaskJob> markTaskJobList = markTaskJobRepo.findAllByStatusIsFalseAndWorkId(work.getId());
            if (Objects.nonNull(markTaskJobList) && !markTaskJobList.isEmpty()) {
                long startTime = System.currentTimeMillis();
                this.markTaskJob = true;
                for (MarkTaskJob m : markTaskJobList) {
                    long createTime = m.getCreateTime().getTime();
                    if ((startTime - createTime) / 1000 > 60) {
                        JSONObject jsonObject = JSON.parseObject(m.getDataObject());
                        String type = String.valueOf(jsonObject.get("type"));
                        Long paperId = Long.parseLong(String.valueOf(jsonObject.get("paperId")));
                        MarkStage stage = MarkStage.valueOf(jsonObject.getString("stage"));
                        Paper paper = paperRepo.findOne(paperId);
                        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                            List<MarkTaskRoughLevel> markTaskList = markTaskRoughLevelRepo.findByWorkIdAndPaperIdAndStage(work.getId(), paperId, stage);
                            if (Objects.nonNull(markTaskList) && !markTaskList.isEmpty()) {
                                if (Objects.equals("levelsLog", type)) {
                                    markingRoughLevelService.levelsLog(markTaskList, paper, stage, m);//档位差日志记录
                                }
                            }
                        } else if (MarkStage.LEVEL.equals(stage)) {
                            List<MarkTaskLevel> markTaskList = markTaskLevelRepo.findByWorkIdAndPaperIdAndStage(work.getId(), paperId, stage);
                            if (Objects.nonNull(markTaskList) && !markTaskList.isEmpty()) {
                                if (Objects.equals("levelsLog", type)) {
                                    markingLevelService.levelsLog(markTaskList, paper, stage, m);//档位差日志记录
                                }
                            }
                        }

                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        } finally {
            this.markTaskJob = false;
        }
    }

    /**
     * 修复试卷密号
     *
     * @param workId 工作ID
     */
    @Transactional
    public void repairPaper(Long workId) {
        try {
            String sql = "select * from (select p.random_seq, count(p.random_seq) as seq from paper p where p.work_id = ? group by p.random_seq) temp where temp.seq > 1 LIMIT 500";
            List list = sqlUtil.execSqlForMapNative(sql, workId);
            Set papers = new HashSet();
            if (Objects.nonNull(list) && list.size() > 0) {
                this.repairPaper = true;
                for (int i = 0; i < list.size(); i++) {
                    Map map = (Map) list.get(i);
                    Long randomSeq = Long.parseLong(String.valueOf(map.get("random_seq")));
                    List<Paper> paperList = paperRepo.findByRandomSeq(randomSeq);
                    paperList.remove(0);//删除一条
                    for (Paper paper : paperList) {
                        Long randomNew = randomUtil.randomPaper(workId);
                        String secretNumberNew = paper.getSubject().ordinal() + String.valueOf(randomSeq);
                        paper.setRandomSeq(randomNew);
                        paper.setSecretNumber(secretNumberNew);
                        papers.add(paper);
                    }
                }
            }
            if (Objects.nonNull(papers) && papers.size() > 0) {
                LOGGER.info("需要修改的paper数为:{}", papers.size());
                paperRepo.save(papers);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        } finally {
            this.repairPaper = false;
        }
    }

    /**
     * 修复任务密号
     *
     * @param workId 工作ID
     */
    @Transactional
    public void repairTask(Long workId) {
        try {
            repairMarkTaskRoughLevel(workId);
            repairMarkTaskLevel(workId);
            repairMarkTaskScore(workId);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        } finally {
            this.repairTask = false;
        }
    }

    private void repairMarkTaskRoughLevel(Long workId) {
        String sql = "select mt.random_seq_new, count(1) as seq from mark_task_rough_level mt where mt.work_id = ? group by mt.random_seq_new having count(1) > 1 LIMIT 100";
        List list = sqlUtil.execSqlForMapNative(sql, workId);
        Set tasks = new HashSet();
        if (Objects.nonNull(list) && !list.isEmpty()) {
            this.repairTask = true;
            for (int i = 0; i < list.size(); i++) {
                Map map = (Map) list.get(i);
                Long randomSeq = Long.valueOf(map.get("random_seq_new").toString());
                List<MarkTaskRoughLevel> markTaskList = markTaskRoughLevelRepo.findByWorkIdAndRandomSeqNew(workId, randomSeq);
                markTaskList.remove(0);//删除一条
                for (MarkTaskRoughLevel markTask : markTaskList) {
                    Long randomNew = randomUtil.randomRoughLevel(workId);
                    String secretNumberNew = MarkStage.ROUGH_LEVEL.ordinal() + String.valueOf(randomNew);
                    markTask.setRandomSeqNew(randomNew);
                    markTask.setSecretNumber(secretNumberNew);
                    tasks.add(markTask);
                }
            }
        }
        if (!tasks.isEmpty()) {
            markTaskRoughLevelRepo.save(tasks);
        }
    }

    private void repairMarkTaskLevel(Long workId) {
        String sql = "select mt.random_seq_new, count(1) as seq from mark_task_level mt where mt.work_id = ? group by mt.secret_number having count(1) > 1 LIMIT 100";
        List list = sqlUtil.execSqlForMapNative(sql, workId);
        Set tasks = new HashSet();
        if (Objects.nonNull(list) && !list.isEmpty()) {
            this.repairTask = true;
            for (int i = 0; i < list.size(); i++) {
                Map map = (Map) list.get(i);
                Long randomSeq = Long.valueOf(map.get("random_seq_new").toString());
                List<MarkTaskLevel> markTaskList = markTaskLevelRepo.findByWorkIdAndRandomSeqNew(workId, randomSeq);
                markTaskList.remove(0);//删除一条
                for (MarkTaskLevel markTask : markTaskList) {
                    Long randomNew = randomUtil.randomLevel(workId);
                    String secretNumberNew = MarkStage.LEVEL.ordinal() + String.valueOf(randomNew);
                    markTask.setRandomSeqNew(randomNew);
                    markTask.setSecretNumber(secretNumberNew);
                    tasks.add(markTask);
                }
            }
        }
        if (!tasks.isEmpty()) {
            markTaskLevelRepo.save(tasks);
        }
    }

    private void repairMarkTaskScore(Long workId) {
        String sql = "select mt.random_seq_new, count(1) as seq from mark_task_score mt where mt.work_id = ? group by mt.secret_number having count(1) > 1 LIMIT 100";
        List list = sqlUtil.execSqlForMapNative(sql, workId);
        Set tasks = new HashSet();
        if (Objects.nonNull(list) && !list.isEmpty()) {
            this.repairTask = true;
            for (int i = 0; i < list.size(); i++) {
                Map map = (Map) list.get(i);
                Long randomSeq = Long.valueOf(map.get("random_seq_new").toString());
                List<MarkTaskScore> markTaskList = markTaskScoreRepo.findByWorkIdAndRandomSeqNew(workId, randomSeq);
                markTaskList.remove(0);//删除一条
                for (MarkTaskScore markTask : markTaskList) {
                    Long randomNew = randomUtil.randomScore(workId);
                    String secretNumberNew = MarkStage.SCORE + String.valueOf(randomNew);
                    markTask.setRandomSeqNew(randomNew);
                    markTask.setSecretNumber(secretNumberNew);
                    tasks.add(markTask);
                }
            }
        }
        if (!tasks.isEmpty()) {
            markTaskScoreRepo.save(tasks);
        }
    }

    @Override
    public void afterPropertiesSet() {
        this.repairSecretNumberTask();
        this.markTaskJob();
    }
}
