package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.ChangeLevelAssembler;
import cn.com.qmth.stmms.ms.marking.dto.ChangeLevelDTO;
import cn.com.qmth.stmms.ms.marking.service.MarkingLevelService;
import cn.com.qmth.stmms.ms.marking.service.MarkingRoughLevelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api/changelevel")
public class ChangeLevelApi {

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private ChangeLevelRepo changeLevelRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    private ChangeLevelAssembler changeLevelAssembler;

    @Resource
    private MarkingRoughLevelService markingRoughLevelService;

    @Resource
    private MarkingLevelService markingLevelService;

    @Autowired
    private MarkLogRepo markLogRepo;

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    MarkUserRepo markUserRepo;

    @Resource
    MarkSubjectRepo markSubjectRepo;

    /**
     * 查询改档列表
     *
     * @param workId     工作ID
     * @param questionId 考区ID
     * @param subject    科目
     * @param status     状态
     */
    @GetMapping("/list")
    public PageableDTO changeLevel(@RequestParam(required = false) Long workId,
                                   @RequestParam(required = false) Long questionId,
                                   @RequestParam(required = false) String subject,
                                   @RequestParam(required = false) String status,
                                   @RequestParam(defaultValue = "0") String type,
                                   Pageable pageable) {
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject);
        List<ChangeLevelDTO> levelDTOs = new ArrayList<>();
        Specification<ChangeLevel> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (!StringUtils.isEmpty(workId)) {
                predicates.add(builder.equal(root.get("workId"), workId));
            }
            if (!StringUtils.isEmpty(questionId)) {
                List<Paper> papers = null;
                if ("1".equals(type)) {
                    papers = paperRepo.findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTest(workId, Subject.valueOf(subject), questionId, false, markSubject.getTest());
                    if (papers != null && !papers.isEmpty()) {
                        CriteriaBuilder.In<Object> in = builder.in(root.get("paperId"));
                        for (Paper paper : papers) {
                            in.value(paper.getId());
                        }
                        predicates.add(builder.and(builder.and(in)));
                    }
                } else {
                    if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                        papers = paperRepo.findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTestAndRoughLevelIsNull(workId, Subject.valueOf(subject), questionId, false, markSubject.getTest());
                    } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
                        papers = paperRepo.findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTestAndLevelIsNull(workId, Subject.valueOf(subject), questionId, false, markSubject.getTest());
                    }
                    if (papers != null && !papers.isEmpty()) {
                        CriteriaBuilder.In<Object> in = builder.in(root.get("paperId"));
                        for (Paper paper : papers) {
                            in.value(paper.getId());
                        }
                        predicates.add(builder.and(builder.and(in)));
                    } else {
                        CriteriaBuilder.In<Object> in = builder.in(root.get("paperId"));
                        in.value(-1);
                        predicates.add(builder.and(builder.and(in)));
                    }
                }

            }
            if (!StringUtils.isEmpty(subject)) {
                predicates.add(builder.equal(root.get("subject"), subject));
            }
            //改档处理
            if ("1".equals(type)) {
                if (!StringUtils.isEmpty(status)) {
                    if (Objects.equals(status, "0")) {
                        predicates.add(builder.equal(root.get("auditStatus"), status));
                    } else {
                        predicates.add(builder.notEqual(root.get("auditStatus"), "0"));
                    }
                }
            } else {
                if (!StringUtils.isEmpty(status)) {
                    if (Objects.equals(status, "0")) {
                        predicates.add(builder.equal(root.get("auditStatus"), status));
                    } else {
                        predicates.add(builder.notEqual(root.get("auditStatus"), "2"));
                    }

                }
                predicates.add(builder.equal(root.get("isCurr"), "1"));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Page<ChangeLevel> levels = changeLevelRepo.findAll(specification, pageable);

        levels.getContent().forEach(m -> {
            levelDTOs.add(changeLevelAssembler.toDTO(m));
        });
        return new PageableDTO(levelDTOs, levels.getTotalElements(), levels.getTotalPages(), pageable.getPageNumber());
    }

    /**
     * 科组长改档申请
     *
     * @param markSubject 科目
     * @param paperId     试卷
     * @param level       档位
     */
    @PostMapping("{markSubject}/changeLevel")
    @Transactional
    public ResponseEntity changeLevel(@PathVariable MarkSubject markSubject,
                                      @RequestParam Long paperId,
                                      @RequestParam String level,
                                      @RequestParam Long userId) {
        boolean roughLevelEnable = ParamCache.levelConfigMap.get(String.valueOf(markSubject.getWorkId())).getRoughLevelEnable() == 1;
        if (!roughLevelEnable) {
            //只有打分阶段才能改档
            if (!Objects.equals(markSubject.getStage(), MarkStage.SCORE)) {
                throw new RuntimeException("未开启粗分档，只有打分阶段才允许改档");
            }
        } else {
            //只有打分阶段才能改档
            if (!Objects.equals(markSubject.getStage(), MarkStage.LEVEL)
                    && !Objects.equals(markSubject.getStage(), MarkStage.SCORE)) {
                throw new RuntimeException("已开启粗分档，只有细分档阶段或者打分阶段才允许改档");
            }
        }

        Paper paper = paperRepo.findOne(paperId);
        MarkUser markUser = markUserRepo.findOne(userId);

        //
        ChangeLevel changeLevelCurr = changeLevelRepo.findByWorkIdAndSubjectAndStageAndPaperIdAndAuditStatusAndIsCurr(markSubject.getWorkId(), markSubject.getSubject().name(), markSubject.getStage(), paperId);
        if (changeLevelCurr != null) {
            if (MarkStage.LEVEL.equals(changeLevelCurr.getStage())) {
                if (Objects.isNull(paper.getRoughLevel())) {
                    throw new RuntimeException("纪检员已同意，请评卷员先完成改大档");
                }
                if (Objects.isNull(paper.getLevel())) {
                    throw new RuntimeException("改档已完成，请评卷员完成改小档");
                }
            } else if (MarkStage.SCORE.equals(changeLevelCurr.getStage())) {
                if (Objects.isNull(paper.getLevel())) {
                    throw new RuntimeException("纪检员已同意，请评卷员先完成改档");
                }
                if (Objects.isNull(paper.getScore())) {
                    throw new RuntimeException("改档已完成，请评卷员完成改档打分");
                }
            }
        }

        List<ChangeLevel> list = changeLevelRepo.findAllByWorkIdAndSubjectAndStageAndPaperId(markSubject.getWorkId(), markSubject.getSubject().name(), markSubject.getStage(), paperId);
        list = list.stream().map(m -> {
            m.setIsCurr(0);
            return m;
        }).collect(Collectors.toList());
        changeLevelRepo.save(list);

        ChangeLevel changeLevel = changeLevelRepo.findByWorkIdAndSubjectAndStageAndPaperId(markSubject.getWorkId(), markSubject.getSubject().name(), markSubject.getStage(), paperId);
        if (changeLevel == null) {
            changeLevel = new ChangeLevel();
            changeLevel.setWorkId(markSubject.getWorkId());
            changeLevel.setSubject(markSubject.getSubject().name());
            changeLevel.setStage(markSubject.getStage());
            changeLevel.setPaperId(paperId);
            if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                changeLevel.setOriginalLevel(paper.getRoughLevel());
            } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
                changeLevel.setOriginalLevel(paper.getLevel());
            }
            changeLevel.setSuggestLevel(level);
            changeLevel.setAuditStatus(0);
            changeLevel.setCreateId(markUser.getId());
            changeLevel.setCreateDate(new Date());
            changeLevel.setIsCurr(1);
        } else {
            if (MarkStage.LEVEL.equals(changeLevel.getStage())) {
                changeLevel.setOriginalLevel(paper.getRoughLevel());
            } else if (MarkStage.SCORE.equals(changeLevel.getStage())) {
                changeLevel.setOriginalLevel(paper.getLevel());
            }
            changeLevel.setSuggestLevel(level);
            changeLevel.setCreateId(markUser.getId());
            changeLevel.setCreateDate(new Date());
            changeLevel.setIsCurr(1);
        }
        changeLevelRepo.save(changeLevel);

        //记录日志
        Work work = workRepo.findOne(paper.getWorkId());
        MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), markUser.getSubject(), changeLevel.getStage(), paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), markUser.getRole(), MarkLogOperType.CHANGE_LEVEL_APPLY.getId(),MarkLogOperType.CHANGE_LEVEL_APPLY.getName(), changeLevel.getOriginalLevel(), changeLevel.getSuggestLevel(), null, new Date());
        markLogRepo.save(markLog);

        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 科组长改档审核
     *
     * @param id          id
     * @param auditStatus 审核状态
     */
    @PostMapping("/changeLevelAudit")
    @Transactional
    public ResponseEntity changeLevelAudit(@RequestParam Long id,
                                           @RequestParam Integer auditStatus,
                                           @RequestParam Long userId) {
        ChangeLevel changeLevel = changeLevelRepo.findOne(id);
        changeLevel.setAuditStatus(auditStatus);
        changeLevelRepo.save(changeLevel);
        //同意
        if (auditStatus == 1) {
            Paper paper = paperRepo.findOne(changeLevel.getPaperId());

            //记录日志
            MarkUser markUser = markUserRepo.findOne(userId);
            Work work = workRepo.findOne(paper.getWorkId());
            MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), paper.getSubject(), changeLevel.getStage(), paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), markUser.getRole(), MarkLogOperType.CHANGE_LEVEL_AGREE.getId(),MarkLogOperType.CHANGE_LEVEL_AGREE.getName(), changeLevel.getOriginalLevel(), changeLevel.getSuggestLevel(), null, new Date());
            markLogRepo.save(markLog);

            if (MarkStage.LEVEL.equals(changeLevel.getStage())) {
                //重置细分档档位
                paper.setLevel(null);

                List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());
                List<String> ranges = markTasks.stream().map(m -> m.getMarkerId().toString()).collect(Collectors.toList());
                markingRoughLevelService.reject(paper, changeLevel.getSuggestLevel(), String.join(",", ranges), true, false, changeLevel.getCreateId());

                //重置已经打分的数据
                List<MarkTaskLevel> markTasksLevels = markTaskLevelRepo.findByPaperId(paper.getId());
                if (markTasksLevels != null && !markTasksLevels.isEmpty()) {
                    markTasksLevels.forEach(o -> {
                        if (o.getResult() != null) {
                            o.setOriginLevel(o.getResult());
                        }
                        o.setResult(null);
                        o.setLevel(null);
                    });
                    markTaskLevelRepo.save(markTasksLevels);
                }
            } else if (MarkStage.SCORE.equals(changeLevel.getStage())) {
                //重置分数
                paper.setScore(null);

                List<MarkTaskLevel> markTasks = markTaskLevelRepo.findByPaperId(paper.getId());
                List<String> ranges = markTasks.stream().map(m -> m.getMarkerId().toString()).collect(Collectors.toList());
                markingLevelService.reject(paper, changeLevel.getSuggestLevel(), String.join(",", ranges), true, false, changeLevel.getCreateId());

                //重置已经打分的数据
                List<MarkTaskScore> markTasksScore = markTaskScoreRepo.findByPaperId(paper.getId());
                if (markTasksScore != null && !markTasksScore.isEmpty()) {
                    markTasksScore.forEach(o -> {
                        if (o.getResult() != null) {
                            o.setOriginLevel(o.getResult());
                        }
                        o.setResult(null);
                        o.setLevel(null);
                    });
                    markTaskScoreRepo.save(markTasksScore);
                }
            }
            //状态设置
            paper.setShift(true);
            paper.setShiftScore(true);

            paperRepo.save(paper);
        }
        return new ResponseEntity(HttpStatus.OK);
    }

}
