package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.accesscontrol.config.LoginConfig;
import cn.com.qmth.stmms.ms.commons.threadPool.MyThreadPool;
import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroupLeader;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.LevelStatAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkerAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.LevleProgressDTO;
import cn.com.qmth.stmms.ms.marking.dto.MarkerDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import cn.com.qmth.stmms.ms.marking.service.GroupingService;
import cn.com.qmth.stmms.ms.marking.service.MarkerGroupLeaderService;
import cn.com.qmth.stmms.ms.marking.service.MarkingService;
import org.apache.commons.lang.StringUtils;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * 评卷中，评卷员的相关数据api
 * Created by zhengmin on 2016/10/14.
 */
@RestController
@RequestMapping("api/markers")
public class MakrerApi {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(MakrerApi.class);

    @Autowired
    private MarkUserRepo markUserRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    private MarkerAssembler markerAssembler;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private LevelStatAssembler levelStatAssembler;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    private LoginConfig loginConfig;

    @Autowired
    MarkingService markingService;

    @Autowired
    MarkerGroupLeaderService markerGroupLeaderService;

    @Autowired
    GroupingService groupingService;

    /**
     * 评卷员信息
     *
     * @param markerId 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{markerId}", method = RequestMethod.GET)
    public MarkerDTO getMarker(@PathVariable Long markerId) {
        MarkUser marker = markUserRepo.findOne(markerId);
        return markerAssembler.toDTO(marker);
    }

    /**
     * 更新默认显示试卷数量
     *
     * @param markerId 评卷员用户id
     * @return
     */
    @RequestMapping(value = "updateDisplayCount", method = RequestMethod.POST)
    public ResponseEntity getMarker(Long markerId, Integer displayCount) {
        MarkUser marker = markUserRepo.findOne(markerId);
        marker.setDisplayCount(displayCount);
        markUserRepo.saveAndFlush(marker);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 评卷员列表
     *
     * @param workId  评卷工作id
     * @param subject 科目
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public List<MarkUser> list(@RequestParam Long workId,
                               @RequestParam Subject subject) {
        List<MarkUser> markUsers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        String markSubjectId = workId + "-" + subject.toString();
        MarkSubject markSubject = markSubjectRepo.findOne(markSubjectId);
        if (MarkStage.INIT == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_LEVELING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        } else if (MarkStage.LEVEL == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_SCORING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        }

        // 加入科组长
        List<MarkUser> leaderMarkUsers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARK_LEADER);
        if (!CollectionUtils.isEmpty(leaderMarkUsers)) {
            markUsers.addAll(leaderMarkUsers);
        }
        return markUsers;
    }

    @Autowired
    MyThreadPool myThreadPool;

    /**
     * 评卷员分档数量及占比数据
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/levels", method = RequestMethod.GET)
    public List<LevelStatDTO> levelStats(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));

        Integer levelShowAllPaper = ParamCache.levelConfigMap.get(String.valueOf(marker.getWorkId())).getLevelShowAllPaper();
        Integer propDenominator = ParamCache.levelConfigMap.get(String.valueOf(marker.getWorkId())).getPropDenominator();
        Long batchNo = null;
        if (levelShowAllPaper == 0) {
            batchNo = paperRepo.findMaxBatchNoByWorkIdAndSubject(marker.getWorkId(), marker.getSubject());
        }
        Long finalBatchNo = batchNo;
        Future future = myThreadPool.arbitratePoolTaskExecutor.submit(new Callable<Map<String, Long>>() {

            @Override
            public Map<String, Long> call() throws Exception {
                Map<String, Long> map = new HashMap<>();
                //当前老师当前批次试卷总数
                long kdtotal;
                if (levelShowAllPaper == 1) {
                    kdtotal = markTaskLevelRepo.countByQuestionIdAll(questionId, marker.getId());
                } else {
                    kdtotal = markTaskLevelRepo.countByQuestionIdAndBatchNo(questionId, finalBatchNo, marker.getId());
                }
                //当前考区内试卷总数
                long total = 0;
                if (propDenominator == 1) {
                    // 全部考生
                    total = paperRepo.countByWorkIdAndQuestionId(markSubject.getWorkId(), questionId);
                } else if (propDenominator == 2) {
                    // 去除缺考考生
                    total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
                }
                map.put("kdtotal", kdtotal);
                map.put("total", total);
                return map;
            }
        });
        //当前考区各档位定档数量
        List<Object[]> list = paperRepo.countGroupByLevel(questionId);
        list.forEach(o -> {
            LevelStatDTO levelStatDTO = levelStatAssembler.toDTO(o);
            levelStatDTO.setGcount(levelStatDTO.getCount());
            if (Objects.isNull(levelStatDTO.getId())) {
                //求任务数为null的条数
                int count;
                if (levelShowAllPaper == 1) {
                    count = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultIsNullAll(questionId, marker.getId(), MarkStage.LEVEL.ordinal());
                } else {
                    count = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultNull(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), finalBatchNo);
                }
                //查询当前老师打回个数
                int rejectCount = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndIsRejectedTrue(questionId, marker.getId(), MarkStage.LEVEL.ordinal());
                levelStatDTO.setCount(count);
                levelStatDTO.setRejected(rejectCount);
            }
            levelStatDTOs.add(levelStatDTO);
        });
        for (Level level : levels) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);
                levelStatDTOs.add(dto);
            }
        }
        long kdtotal = 0L, total = 0L;
        if (Objects.nonNull(future) && Objects.nonNull(future.get())) {
            Map<String, Long> map = (Map<String, Long>) future.get();
            kdtotal = map.get("kdtotal");
            total = map.get("total");
        }

        long finalKdtotal = kdtotal;
        long finalTotal = total;
        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                //当前老师各档位数量
                int countNew;
                if (levelShowAllPaper == 1) {
                    countNew = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultAll(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), String.valueOf(o.getId()));
                } else {
                    countNew = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResult(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), String.valueOf(o.getId()), finalBatchNo);
                }
                o.setCount(countNew);
                //该档位考区内已定档数量
                long gcountNew = paperRepo.countByQuestionIdAndLevelAndSubjectAndWorkId(questionId, String.valueOf(o.getId()), markSubject.getSubject(), marker.getWorkId());
                o.setGcount((int) gcountNew);

                o.setPt(levelMap.get(o.getId()).getPt());
                o.setKdpt(levelMap.get(o.getId()).getKdpt());
                double p = finalKdtotal == 0 ? 0 : (double) o.getCount() / finalKdtotal * 100;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                o.setPercent(bd.doubleValue());

                o.setGcount(Objects.isNull(o.getGcount()) ? 0 : o.getGcount());
                double gp = finalTotal == 0 ? 0 : (double) o.getGcount() / finalTotal * 100;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());
            }
        });
        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 评卷员分档数量及占比数据
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     */
    @GetMapping("{marker}/stat/totalRoughLevels")
    public List<LevelStatDTO> totalRoughLevels(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Integer levelShowAllPaper = ParamCache.roughLevelConfigMap.get(String.valueOf(marker.getWorkId())).getLevelShowAllPaper();
        Integer propDenominator = ParamCache.roughLevelConfigMap.get(String.valueOf(marker.getWorkId())).getPropDenominator();
        Long batchNo = null;
        if (levelShowAllPaper == 0) {
            batchNo = paperRepo.findMaxRoughBatchNoByWorkIdAndSubject(marker.getWorkId(), marker.getSubject());
        }
        Long finalBatchNo = batchNo;
        Future future = myThreadPool.arbitratePoolTaskExecutor.submit(new Callable<Map<String, Long>>() {
            @Override
            public Map<String, Long> call() throws Exception {
                Map<String, Long> map = new HashMap<>();
                //当前老师当前批次试卷总数
                long kdtotal = 0;
                if (levelShowAllPaper == 1) {
                    kdtotal = markTaskRoughLevelRepo.countByQuestionIdAll(questionId, marker.getId());
                } else {
                    kdtotal = markTaskRoughLevelRepo.countByQuestionIdAndBatchNo(questionId, finalBatchNo, marker.getId());
                }

                //当前考区内试卷总数
                long total = 0;
                if (propDenominator == 1) {
                    // 全部考生
                    total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
                } else if (propDenominator == 2) {
                    // 去除缺考考生
                    total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
                }
                map.put("kdtotal", kdtotal);
                map.put("total", total);
                return map;
            }
        });
        //当前考区各档位定档数量
        List<Object[]> list = paperRepo.countGroupByRoughLevel(questionId);
        list.forEach(o -> {
            LevelStatDTO levelStatDTO = levelStatAssembler.toDTO(o);
            levelStatDTO.setGcount(levelStatDTO.getCount());
            if (Objects.isNull(levelStatDTO.getId())) {
                //求任务数为null的条数
                int count = 0;
                if (levelShowAllPaper == 1) {
                    count = markTaskRoughLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultIsNullAll(questionId, marker.getId(), markSubject.getStage().ordinal());
                } else {
                    count = markTaskRoughLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultNull(questionId, marker.getId(), markSubject.getStage().ordinal(), finalBatchNo);
                }
                //查询当前老师打回个数
                int rejectCount = markTaskRoughLevelRepo.countByQuestionIdAndMarkerIdAndStageAndIsRejectedTrue(questionId, marker.getId(), markSubject.getStage().ordinal());
                levelStatDTO.setCount(count);
                levelStatDTO.setRejected(rejectCount);
            }
            levelStatDTOs.add(levelStatDTO);
        });

        List<String> roughCodes = levels.stream().map(Level::getRoughCode).distinct().collect(Collectors.toList());
        for (String roughCode : roughCodes) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(roughCode)).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(roughCode);
                dto.setCount(0);
                dto.setPercent(0.0);
                levelStatDTOs.add(dto);
            }
        }
        long kdtotal = 0L, total = 0L;
        if (Objects.nonNull(future) && Objects.nonNull(future.get())) {
            Map<String, Long> map = (Map<String, Long>) future.get();
            kdtotal = map.get("kdtotal");
            total = map.get("total");
        }

        long finalKdtotal = kdtotal;
        long finalTotal = total;
        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                //当前老师各档位数量
                int countNew;
                if (levelShowAllPaper == 1) {
                    countNew = markTaskRoughLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultAll(questionId, marker.getId(), markSubject.getStage().ordinal(), String.valueOf(o.getId()));
                } else {
                    countNew = markTaskRoughLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResult(questionId, marker.getId(), markSubject.getStage().ordinal(), String.valueOf(o.getId()), finalBatchNo);
                }
                o.setCount(countNew);
                //该档位考区内已定档数量
                long gcountNew = paperRepo.countByQuestionIdAndRoughLevelAndSubjectAndWorkId(questionId, String.valueOf(o.getId()), markSubject.getSubject(), marker.getWorkId());
                o.setGcount((int) gcountNew);

//                o.setPt(levelMap.get(o.getId()).getPt());
//                o.setKdpt(levelMap.get(o.getId()).getKdpt());
                o.setPt(0);
                o.setKdpt(0);
                double p = finalKdtotal == 0 ? 0 : (double) o.getCount() / finalKdtotal * 100;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                o.setPercent(bd.doubleValue());
                o.setFinalKdTotal(Math.toIntExact(finalKdtotal));

                o.setGcount(Objects.isNull(o.getGcount()) ? 0 : o.getGcount());
                double gp = finalTotal == 0 ? 0 : (double) o.getGcount() / finalTotal * 100;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());
                o.setFinalTotal(Math.toIntExact(finalTotal));
            }
        });

        // 标记试卷（评卷员标记的和科组长标记的都统计）
        LevelStatDTO markPaperDTO = new LevelStatDTO();
        markPaperDTO.setId("markPaper");
        int markPaperCount;
        if (ParamCache.roughLevelConfigMap.get(String.valueOf(marker.getWorkId())).getLevelShowAllPaper() == 1) {
            // 评卷员和科组长标记的试卷
            List<Long> markTaskPaperId = markTaskRoughLevelRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndIsMark(markSubject.getWorkId(), questionId,marker.getId(), markSubject.getStage(), true);
            markPaperCount = Math.toIntExact(markTaskPaperId.stream().distinct().count());
        } else {
            // 评卷员标记的试卷
            List<Long> markTaskPaperIdAll = markTaskRoughLevelRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndBatchNoAndPaperIsMark(markSubject.getWorkId(), questionId, marker.getId(), markSubject.getStage(), batchNo);
            markPaperCount = Math.toIntExact(markTaskPaperIdAll.stream().distinct().count());
        }
        markPaperDTO.setCount(markPaperCount);
        levelStatDTOs.add(markPaperDTO);
        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 评卷员分档数量及占比数据
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/totalLevels", method = RequestMethod.GET)
    public List<LevelStatDTO> totalLevels(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));
        Integer levelShowAllPaper = ParamCache.levelConfigMap.get(String.valueOf(marker.getWorkId())).getLevelShowAllPaper();
        Integer propDenominator = ParamCache.levelConfigMap.get(String.valueOf(marker.getWorkId())).getPropDenominator();
        Long batchNo = null;
        if (levelShowAllPaper == 0) {
            batchNo = paperRepo.findMaxBatchNoByWorkIdAndSubject(marker.getWorkId(), marker.getSubject());
        }
        Long finalBatchNo = batchNo;
        Future future = myThreadPool.arbitratePoolTaskExecutor.submit(new Callable<Map<String, Long>>() {
            @Override
            public Map<String, Long> call() throws Exception {
                Map<String, Long> map = new HashMap<>();
                //当前老师当前批次试卷总数
                long kdtotal = 0;
                if (levelShowAllPaper == 1) {
                    kdtotal = markTaskLevelRepo.countByQuestionIdAll(questionId, marker.getId());
                } else {
                    kdtotal = markTaskLevelRepo.countByQuestionIdAndBatchNo(questionId, finalBatchNo, marker.getId());
                }

                //当前考区内试卷总数
                long total = 0;
                if (propDenominator == 1) {
                    // 全部考生
                    total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
                } else if (propDenominator == 2) {
                    // 去除缺考考生
                    total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
                }
                map.put("kdtotal", kdtotal);
                map.put("total", total);
                return map;
            }
        });
        //当前考区各档位定档数量
        List<Object[]> list = paperRepo.countGroupByLevel(questionId);
        list.forEach(o -> {
            LevelStatDTO levelStatDTO = levelStatAssembler.toDTO(o);
            levelStatDTO.setGcount(levelStatDTO.getCount());
            if (Objects.isNull(levelStatDTO.getId())) {
                //求任务数为null的条数
                int count = 0;
                if (levelShowAllPaper == 1) {
                    count = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultIsNullAll(questionId, marker.getId(), MarkStage.LEVEL.ordinal());
                } else {
                    count = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultNull(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), finalBatchNo);
                }
                //查询当前老师打回个数
                int rejectCount = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndIsRejectedTrue(questionId, marker.getId(), MarkStage.LEVEL.ordinal());
                levelStatDTO.setCount(count);
                levelStatDTO.setRejected(rejectCount);
            }
            levelStatDTOs.add(levelStatDTO);
        });
        for (Level level : levels) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);
                levelStatDTOs.add(dto);
            }
        }
        long kdtotal = 0L, total = 0L;
        if (Objects.nonNull(future) && Objects.nonNull(future.get())) {
            Map<String, Long> map = (Map<String, Long>) future.get();
            kdtotal = map.get("kdtotal");
            total = map.get("total");
        }

        long finalKdtotal = kdtotal;
        long finalTotal = total;

        Optional<LevelStatDTO> optionalLevelStatDTO = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals("null")).findFirst();
        LevelStatDTO nullLevelStatDto = optionalLevelStatDTO.isPresent() ? optionalLevelStatDTO.get() : null;

        if (nullLevelStatDto == null) {
            LevelStatDTO levelStatDTO = new LevelStatDTO();
            //求任务数为null的条数
//            int totalCount = markTaskLevelRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), batchNo);
//            levelStatDTO.setCount(totalCount);
            levelStatDTO.setCount(0);
            levelStatDTO.setRejected(0);
            //查询改档
            int shiftCount = markTaskRoughLevelRepo.countShiftByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(questionId, marker.getId(), MarkStage.ROUGH_LEVEL.ordinal(), false);
            levelStatDTO.setShift(shiftCount);
            //查询改档打分
            int shiftScoreCount = markTaskLevelRepo.countShiftScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.LEVEL.ordinal());
            levelStatDTO.setShiftScore(shiftScoreCount);
            levelStatDTOs.add(levelStatDTO);
        } else {
//            int totalCount = markTaskLevelRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), batchNo);
//            nullLevelStatDto.setCount(totalCount);
            //查询改档
            int shiftCount = markTaskRoughLevelRepo.countShiftByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(questionId, marker.getId(), MarkStage.ROUGH_LEVEL.ordinal(), false);
            nullLevelStatDto.setShift(shiftCount);
            //查询改档打分
            int shiftScoreCount = markTaskLevelRepo.countShiftScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.LEVEL.ordinal());
            nullLevelStatDto.setShiftScore(shiftScoreCount);
        }

        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                //当前老师各档位数量
                int countNew;
                if (levelShowAllPaper == 1) {
                    countNew = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResultAll(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), String.valueOf(o.getId()));
                } else {
                    countNew = markTaskLevelRepo.countByQuestionIdAndMarkerIdAndStageAndResult(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), String.valueOf(o.getId()), finalBatchNo);
                }
                o.setCount(countNew);
                //该档位考区内已定档数量
                long gcountNew = paperRepo.countByQuestionIdAndLevelAndSubjectAndWorkId(questionId, String.valueOf(o.getId()), markSubject.getSubject(), marker.getWorkId());
                o.setGcount((int) gcountNew);

                o.setPt(levelMap.get(o.getId()).getPt());
                o.setKdpt(levelMap.get(o.getId()).getKdpt());
                double p = finalKdtotal == 0 ? 0 : (double) o.getCount() / finalKdtotal * 100;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                o.setPercent(bd.doubleValue());
                o.setFinalKdTotal(Math.toIntExact(finalKdtotal));

                o.setGcount(Objects.isNull(o.getGcount()) ? 0 : o.getGcount());
                double gp = finalTotal == 0 ? 0 : (double) o.getGcount() / finalTotal * 100;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());
                o.setFinalTotal(Math.toIntExact(finalTotal));
            }
        });

        // 标记试卷（评卷员标记的和科组长标记的都统计）
        LevelStatDTO markPaperDTO = new LevelStatDTO();
        markPaperDTO.setId("markPaper");
        int markPaperCount;
        if (ParamCache.levelConfigMap.get(String.valueOf(marker.getWorkId())).getLevelShowAllPaper() == 1) {
            // 评卷员和科组长标记的试卷
            List<Long> markTaskPaperId = markTaskLevelRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndIsMark(markSubject.getWorkId(), questionId,marker.getId(), markSubject.getStage(), true);
            markPaperCount = Math.toIntExact(markTaskPaperId.stream().distinct().count());
        } else {
            // 评卷员标记的试卷
            List<Long> markTaskPaperIdAll = markTaskLevelRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndBatchNoAndPaperIsMark(markSubject.getWorkId(), questionId, marker.getId(), markSubject.getStage(), batchNo);
            markPaperCount = Math.toIntExact(markTaskPaperIdAll.stream().distinct().count());
        }
        markPaperDTO.setCount(markPaperCount);
        levelStatDTOs.add(markPaperDTO);

        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 评卷员打分数量
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/scores", method = RequestMethod.GET)
    public List<LevelStatDTO> scoreStats(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));
        Long batchNo = paperRepo.findMaxScoreBatchNoByWorkIdAndSubject(marker.getWorkId(), marker.getSubject());
        Future future = myThreadPool.arbitratePoolTaskExecutor.submit(new Callable<Map<String, Long>>() {

            @Override
            public Map<String, Long> call() {
                Map<String, Long> map = new HashMap<>();
                //当前老师所有的评档次数(不分档位)
                long total = paperRepo.countByQuestionId(questionId);
                map.put("total", total);
                return map;
            }
        });
        LevelStatDTO levelStatDTO = new LevelStatDTO();
        //求任务数为null的条数
        int totalCount = markTaskScoreRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.SCORE.ordinal(), batchNo);
        levelStatDTO.setCount(totalCount);
        //查询改档
        int shiftCount = markTaskLevelRepo.countShiftByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), false);
        levelStatDTO.setShift(shiftCount);
        //查询改档打分
        int shiftScoreCount = markTaskScoreRepo.countShiftScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.SCORE.ordinal());
        levelStatDTO.setShiftScore(shiftScoreCount);
        levelStatDTOs.add(levelStatDTO);

        for (Level level : levels) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);

                //当前老师当前档位评档次数(所有考试)
                dto.setCount(0);
                levelStatDTOs.add(dto);
            }
        }
        long total = 0L;
        if (Objects.nonNull(future) && Objects.nonNull(future.get())) {
            Map<String, Long> map = (Map<String, Long>) future.get();
            total = map.get("total");
        }

        long finalTotal = total;
        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                o.setPt(levelMap.get(o.getId()).getPt());
                o.setKdpt(levelMap.get(o.getId()).getKdpt());
                o.setCount(Objects.isNull(o.getCount()) ? 0 : o.getCount());
                o.setPercent(0D);

                o.setGcount(Objects.isNull(o.getGcount()) ? 0 : o.getGcount());
                double gp = (double) o.getGcount() / finalTotal;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());

                int count;
                if (ParamCache.scoreConfigMap.get(String.valueOf(marker.getWorkId())).getScoreShowAllPaper() == 1) {
                    count = markTaskScoreRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResultAll(questionId, marker.getId(), MarkStage.SCORE.ordinal(), o.getId().toString());
                } else {
                    count = markTaskScoreRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResult(questionId, marker.getId(), MarkStage.SCORE.ordinal(), o.getId().toString(), batchNo);
                }
                o.setPercent(count == 0 ? 0 : o.getPercent());
                o.setCount(count);
            }
        });

        // 标记试卷（评卷员标记的和科组长标记的都统计）
        LevelStatDTO markPaperDTO = new LevelStatDTO();
        markPaperDTO.setId("markPaper");
        int markPaperCount;
        if (ParamCache.scoreConfigMap.get(String.valueOf(marker.getWorkId())).getScoreShowAllPaper() == 1) {
            // 评卷员和科组长标记的试卷
            List<Long> markTaskPaperId = markTaskScoreRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndIsMark(markSubject.getWorkId(), questionId,marker.getId(), markSubject.getStage(), true);
            markPaperCount = Math.toIntExact(markTaskPaperId.stream().distinct().count());
        } else {
            // 评卷员标记的试卷
            List<Long> markTaskPaperIdAll = markTaskScoreRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndBatchNoAndPaperIsMark(markSubject.getWorkId(), questionId, marker.getId(), markSubject.getStage(), batchNo);
            markPaperCount = Math.toIntExact(markTaskPaperIdAll.stream().distinct().count());
        }
        markPaperDTO.setCount(markPaperCount);
        levelStatDTOs.add(markPaperDTO);
        return levelStatDTOs;
    }

    /**
     * 评卷员打分数量
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/totalScores", method = RequestMethod.GET)
    public List<LevelStatDTO> totalScores(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));
        Long batchNo = paperRepo.findMaxScoreBatchNoByWorkIdAndSubject(marker.getWorkId(), marker.getSubject());
        Future future = myThreadPool.arbitratePoolTaskExecutor.submit(new Callable<Map<String, Long>>() {
            @Override
            public Map<String, Long> call() {
                Map<String, Long> map = new HashMap<>();
                //当前老师所有的评档次数(不分档位)
//                long total = paperRepo.countByWorkIdAndQuestionId(markSubject.getWorkId(), questionId);
                long total = paperRepo.countByQuestionId(questionId);
                map.put("total", total);
                return map;
            }
        });
        LevelStatDTO levelStatDTO = new LevelStatDTO();
        //求任务数为null的条数
        int totalCount = markTaskScoreRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.SCORE.ordinal(), batchNo);
        levelStatDTO.setCount(totalCount);
        //查询改档
        int shiftCount = markTaskLevelRepo.countShiftByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), false);
        levelStatDTO.setShift(shiftCount);
        //查询改档打分
        int shiftScoreCount = markTaskScoreRepo.countShiftScoreByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.SCORE.ordinal());
        levelStatDTO.setShiftScore(shiftScoreCount);
        levelStatDTOs.add(levelStatDTO);

        for (Level level : levels) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);

                //当前老师当前档位评档次数(所有考试)
                dto.setCount(0);
                levelStatDTOs.add(dto);
            }
        }
        long total = 0L;
        if (Objects.nonNull(future) && Objects.nonNull(future.get())) {
            Map<String, Long> map = (Map<String, Long>) future.get();
            total = map.get("total");
        }

        long finalTotal = total;
        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                o.setPt(levelMap.get(o.getId()).getPt());
                o.setKdpt(levelMap.get(o.getId()).getKdpt());
                o.setCount(Objects.isNull(o.getCount()) ? 0 : o.getCount());
                o.setPercent(0D);

                o.setGcount(Objects.isNull(o.getGcount()) ? 0 : o.getGcount());
                double gp = (double) o.getGcount() / finalTotal;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());
                o.setFinalTotal(Math.toIntExact(finalTotal));

                int count;
                if (ParamCache.scoreConfigMap.get(String.valueOf(marker.getWorkId())).getScoreShowAllPaper() == 1) {
                    count = markTaskScoreRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResultAll(questionId, marker.getId(), MarkStage.SCORE.ordinal(), o.getId().toString());
                } else {
                    count = markTaskScoreRepo.countScoreByQuestionIdAndMarkerIdAndStageAndResult(questionId, marker.getId(), MarkStage.SCORE.ordinal(), o.getId().toString(), batchNo);
                }
                o.setPercent(count == 0 ? 0 : o.getPercent());
                o.setCount(count);
            }
        });

        // 标记试卷（评卷员标记的和科组长标记的都统计）
        LevelStatDTO markPaperDTO = new LevelStatDTO();
        markPaperDTO.setId("markPaper");
        int markPaperCount;
        if (ParamCache.scoreConfigMap.get(String.valueOf(marker.getWorkId())).getScoreShowAllPaper() == 1) {
            // 评卷员和科组长标记的试卷
            List<Long> markTaskPaperId = markTaskScoreRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndIsMark(markSubject.getWorkId(), questionId,marker.getId(), markSubject.getStage(), true);
            markPaperCount = Math.toIntExact(markTaskPaperId.stream().distinct().count());
        } else {
            // 评卷员标记的试卷
            List<Long> markTaskPaperIdAll = markTaskScoreRepo.findByWorkIdAndQuestionIdAndMarkerIdAndStageAndBatchNoAndPaperIsMark(markSubject.getWorkId(), questionId, marker.getId(), markSubject.getStage(), batchNo);
            markPaperCount = Math.toIntExact(markTaskPaperIdAll.stream().distinct().count());
        }
        markPaperDTO.setCount(markPaperCount);
        levelStatDTOs.add(markPaperDTO);
        return levelStatDTOs;
    }

    /**
     * 各试题统计数据
     *
     * @param marker 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/questions", method = RequestMethod.GET)
    public List<QuestionStatDTO> questions(@PathVariable MarkUser marker) {
        List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
        List<Object[]> qStats = null;
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            qStats = markTaskLevelRepo.countGroupByQuestion(marker.getId(), markSubject.getStage().ordinal(), markSubject.getTest());
        } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
            qStats = markTaskScoreRepo.countGroupByQuestion(marker.getId(), markSubject.getStage().ordinal());
        }
        if (qStats != null) {
            for (Object[] objects : qStats) {
                QuestionStatDTO dto = questionStatAssembler.toDTO(objects);
                questionStatDTOs.add(dto);
            }
        }
        return questionStatDTOs;
    }

    /**
     * 管理员-各评卷员评卷进度、考区评卷进度
     *
     * @param workId  工作Id
     * @param subject 科目
     */
    @GetMapping("/stat/progress")
    public Map markProgress(@RequestParam Long workId, @RequestParam Subject subject) {
        Map<String, Object> objectMap = new HashMap<>();
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.toString());

        //总体进度
        if (markSubject.getStage().name().equals(MarkStage.INIT.name())) {
            List<Student> students = studentRepo.findByWorkId(workId);
            long stuTotalCount = students.size();
            long c = 0L;
            for (Student student : students) {
                String[] uploadStatus = student.getUploadStatus().split(",");
                for (String s : uploadStatus) {
                    if ("1".equals(s.split(":")[1])) {
                        if (!subject.name().equals(s.split(":")[0])) {
                            continue;
                        }
                        ++c;
                    }
                }
            }

            LevleProgressDTO progressDTO = new LevleProgressDTO();
            progressDTO.setSuccessCount((int) c);
            progressDTO.setTotalCount((int) stuTotalCount);
            progressDTO.setWaitCount((int) (stuTotalCount - c));
            BigDecimal progress = stuTotalCount == 0 ? new BigDecimal("0") : new BigDecimal(c * 100).divide(new BigDecimal(stuTotalCount), 2, BigDecimal.ROUND_HALF_UP);
            progressDTO.setProgress(progress.doubleValue());
            objectMap.put("totalProgress", progressDTO);
        } else {
            //已评任务数
            groupingService.getTotalProgress(workId, markSubject, objectMap);
        }


        //考区进度
        groupingService.getAreaProgress(workId, markSubject, objectMap);

        // 查询分组内科组长和评卷员
        List<Long> userMarkLeaders = new ArrayList<>();
        groupingService.getGroupMarkersProgress(workId, markSubject, userMarkLeaders, null, true, objectMap);

        //科组长
        groupingService.getMakerLeaderProgress(workId, markSubject, userMarkLeaders, objectMap);

        return objectMap;
    }

    /**
     * 科组长-各评卷员评卷进度、考区评卷进度
     *
     * @return
     */
    @RequestMapping(value = "/mark_leader/stat/progress", method = RequestMethod.GET)
    public Map markLeaderProgress(@RequestParam Long workId, @RequestParam Subject subject) {
        Map<String, Object> objectMap = new HashMap<>();
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.toString());

        Long markerId = ServletUtil.getUserId();
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndMarkLeaderId(workId, markSubject, markerId);
        boolean isTotalMarkLeader = CollectionUtils.isEmpty(markerGroupLeaders);

        //已评任务数
        groupingService.getTotalProgress(workId, markSubject, objectMap);

        //考区进度
        groupingService.getAreaProgress(workId, markSubject, objectMap);

        // 查询分组内科组长和评卷员
        List<Long> userMarkLeaders = new ArrayList<>();
        groupingService.getGroupMarkersProgress(workId, markSubject, userMarkLeaders, markerId, isTotalMarkLeader, objectMap);

        if (isTotalMarkLeader) {
            //科组长
            groupingService.getMakerLeaderProgress(workId, markSubject, userMarkLeaders, objectMap);
        }

        return objectMap;
    }

    /**
     * 清空当前阶段数据
     *
     * @param workId
     * @param subject
     * @param stage
     * @param loginName
     * @param password
     * @return
     */
    @RequestMapping(value = "/subject/reset", method = RequestMethod.POST)
    public Object subjectReset(@RequestParam Long workId,
                               @RequestParam Subject subject,
                               @RequestParam MarkStage stage,
                               @RequestParam String loginName,
                               @RequestParam String password) {
        if (Objects.isNull(workId)) {
            throw new RuntimeException("工作ID不能为空");
        }
        if (Objects.isNull(subject)) {
            throw new RuntimeException("科目不能为空");
        }
        if (Objects.isNull(password)) {
            throw new RuntimeException("密码不能为空");
        }

        String loginNameStr = loginConfig.adminLoginConfig().getLoginName();
        if(StringUtils.isBlank(loginNameStr)){
            throw new RuntimeException("没有设置管理员账号");
        }
        List<String> loginNames = Arrays.asList(loginNameStr.split(","));
        if (!loginNames.contains(loginName)) {
            throw new RuntimeException("只有管理员才能操作");
        }
        if (!loginConfig.adminLoginConfig().getPassword().equals(password)) {
            throw new RuntimeException("管理员密码错误");
        }

        // 如果是试评，必须结束试评后再清除
        MarkSubject markSubject = markSubjectRepo.findOne(workId+"-"+subject);
        if(markSubject.getTest() == TrialEnum.START_TRIAL.getId()){
            throw new RuntimeException("请先结束试评");
        }

        // 清空当前阶段数据
        markingService.subjectReset(workId, subject, stage);

        return true;
    }

}
