package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.user.*;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.LevelStatAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.LevleProgressDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import cn.com.qmth.stmms.ms.marking.service.DetermineRoughLevelService;
import cn.com.qmth.stmms.ms.marking.service.MarkerGroupLeaderService;
import cn.com.qmth.stmms.ms.marking.service.StageControlService;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 评卷科目数据api
 * Created by zhengmin on 2016/10/10.
 */
@RestController
@RequestMapping("api/marksubjects")
public class MarkSubjectApi {

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private LevelStatAssembler levelStatAssembler;

    @Autowired
    private MarkerGroupRepo markerGroupRepo;

    @Autowired
    MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private StageControlService stageControlService;

    @Autowired
    private TaskPublishSettingRepo taskPublishSettingRepo;

    @Resource
    MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    MarkerGroupStudentRepo markerGroupStudentRepo;

    @Resource
    ChangeLevelRepo changeLevelRepo;

    @Autowired
    MarkerGroupLeaderService markerGroupLeaderService;

    /**
     * 查询所有科目状态
     *
     * @param workId 工作ID
     * @return
     */
    @RequestMapping(value = "getAllSubjectStage/{workId}", method = RequestMethod.GET)
    public List<MarkSubject> getAllSubjectStage(@PathVariable Long workId) {
        return markSubjectRepo.findAllByWorkIdAndEnableTrue(workId);
    }

    /**
     * 查询所有科目状态
     *
     * @param workId 工作ID
     */
    @Transactional
    @PostMapping("/go_next_stage")
    public void goNextStage(@RequestParam Long workId,
                            @RequestParam Subject subject) {
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        MarkStage initStage = markSubject.getStage();

        // 所有改档未审核的数据。默认审核不通过
        List<ChangeLevel> changeLevelList = changeLevelRepo.findByWorkIdAndSubjectAndAuditStatusAndStage(workId, subject.name(), 0, initStage);
        if (!changeLevelList.isEmpty()) {
            List<ChangeLevel> list = new ArrayList<>();
            for (ChangeLevel m : changeLevelList) {
                m.setAuditStatus(2);
                list.add(m);
            }
            changeLevelRepo.save(list);
        }

        if (MarkStage.INIT.equals(markSubject.getStage())) {
            if (ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1) {
                markSubject.setStage(MarkStage.ROUGH_LEVEL);
            } else {
                markSubject.setStage(MarkStage.LEVEL);
            }
        } else if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            if (markSubject.getTest() == TrialEnum.START_TRIAL.getId()) {
                throw new RuntimeException("当前为试评阶段，请结束试评后再试");
            }
            markSubject.setStage(MarkStage.LEVEL);
            //是否有未结束的分档任务
            int count = paperRepo.countByWorkIdAndSubjectAndRoughLevelIsNullAndIsMissingFalseAndRoughBatchNoNotNullAndTest(workId, subject, markSubject.getTest());
            if (count > 0) {
                throw new RuntimeException("评卷任务没有分档完成");
            }
            count = paperRepo.countByWorkIdAndSubjectAndRoughLevelIsNullAndIsMissingFalseAndRoughBatchNoIsNullAndTest(workId, subject, markSubject.getTest());
            if (count > 0) {
                throw new RuntimeException("有未发布的评卷任务");
            }
            count = paperRepo.countByWorkIdAndSubjectAndIsArbitratedTrue(workId, subject);
            if (count > 0) {
                throw new RuntimeException("有待仲裁的评卷任务");
            }
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            if (markSubject.getTest() == TrialEnum.START_TRIAL.getId()) {
                throw new RuntimeException("当前为试评阶段，请结束试评后再试");
            }
            markSubject.setStage(MarkStage.SCORE);
            stageControlService.enterToScoreStage(markSubject);
        }
        markSubjectRepo.save(markSubject);

        // 清除用户表中分组ID
        if (!MarkStage.INIT.equals(initStage)) {
            markUserRepo.updateGroupIdByWorkIdAndSubject(workId, subject);
            markerGroupRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());
            markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());

            stageControlService.logoutAndBackupDb(markSubject);
        }
    }

    /**
     * 该评卷科目进入下一阶段
     *
     * @param markSubject 评卷科目id
     */
    @PatchMapping("{markSubject}")
    public void createTask(@PathVariable MarkSubject markSubject, @RequestBody Map map) throws Exception {
        stageControlService.createTask(markSubject, map);
    }

    /**
     * 打分任务发布
     *
     * @param markSubject 评卷科目id
     */
    @PostMapping("{markSubject}/publishScore")
    public void publishScore(@PathVariable MarkSubject markSubject, @RequestBody Map map) throws Exception {
        Object questionId = map.get("questionId");
        List<TaskPublishSetting> taskList = JSONObject.parseArray(JSONObject.toJSONString(map.get("taskList")), TaskPublishSetting.class);
        stageControlService.enterScoreStage(markSubject, taskList, questionId);
    }

    /**
     * 单个评卷科目信息
     *
     * @param markSubjectId 评卷科目id
     */
    @GetMapping("/{markSubjectId}")
    public MarkSubject get(@PathVariable String markSubjectId) {
        MarkSubject markSubject = markSubjectRepo.findOne(markSubjectId);
        int count = 0;
        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            count = markTaskRoughLevelRepo.countByWorkIdAndSubjectAndStageAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), TrialEnum.DEFAULT.getId());
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            count = markTaskLevelRepo.countByWorkIdAndSubjectAndStageAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), TrialEnum.DEFAULT.getId());
        }
        markSubject.setFormal(count > 0);
        markSubject.setRoughLevelEnable(ParamCache.levelConfigMap.get(String.valueOf(markSubject.getWorkId())).getRoughLevelEnable() == 1);
        return markSubject;
    }

    /**
     * 该评卷科目下的评卷员列表
     *
     * @param markSubject 评卷科目id
     */
    @RequestMapping(value = "{markSubject}/markers", method = RequestMethod.GET)
    public List<MarkUser> markUsers(@PathVariable MarkSubject markSubject) {
        List<MarkUser> markUsers = markUserRepo.findByWorkIdAndSubjectAndRole(markSubject.getWorkId(), markSubject.getSubject(), Role.MARKER);
        if (MarkStage.LEVEL == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_LEVELING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        } else if (MarkStage.SCORE == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_SCORING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        }
        return markUsers;
    }

    /**
     * 该评卷科目的试题统计信息
     *
     * @param markSubject 评卷科目id
     * @return
     */
    @RequestMapping(value = "{markSubject}/stat/questions", method = RequestMethod.GET)
    public List<QuestionStatDTO> questions(@PathVariable MarkSubject markSubject) {
        List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
        List<Object[]> qStats = null;
        if (markSubject.getStage() == MarkStage.LEVEL) {
            qStats = paperRepo.countGroupByQuestionForLevel(markSubject.getWorkId(), markSubject.getSubject().toString(), markSubject.getTest());
        } else {
            qStats = paperRepo.countGroupByQuestionForScore(markSubject.getWorkId(), markSubject.getSubject().toString());
        }
        for (Object[] objects : qStats) {
            QuestionStatDTO dto = questionStatAssembler.toDTO(objects);
            questionStatDTOs.add(dto);
        }
        return questionStatDTOs;
    }

    /**
     * 管理员-该评卷科目的分档统计信息
     *
     * @param markSubject 评卷科目id
     * @param questionId  试题id
     * @return
     */
    @RequestMapping(value = "{markSubject}/stat/levels", method = RequestMethod.GET)
    public List<LevelStatDTO> levelStat(@PathVariable MarkSubject markSubject, @RequestParam Long questionId) {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            paperRepo.countGroupByRoughLevelAll(questionId, markSubject.getTest())
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
            List<RoughLevel> roughLevels = DetermineRoughLevelService.parseRoughLevels(levels);
            for (RoughLevel level : roughLevels) {
                long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getRoughCode())).count();
                //没有该档位，新增
                if (count == 0) {
                    LevelStatDTO dto = new LevelStatDTO();
                    dto.setId(level.getRoughCode());
                    dto.setCount(0);
                    dto.setPercent(0.0);
                    levelStatDTOs.add(dto);
                }
            }
            //当前考区总数
            long total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalseAndTest(markSubject.getWorkId(), questionId, markSubject.getTest());
            //所有考区总数
            long gtotal = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getTest());
            levelStatDTOs.forEach(o -> {
                if (o.getId() != null) {
                    //当前考区档位占比
                    double p = total == 0 ? 0 : (double) o.getCount() / total * 100;
                    BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                    o.setPercent(bd.doubleValue());
                    //考区阈值
                    o.setKdpt(0);
                    //所有考区档位数量
                    long gcount = paperRepo.countByWorkIdAndSubjectAndRoughLevelAndTestAndRoughBatchNoNotNull(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(o.getId()), markSubject.getTest());
                    o.setGcount((int) gcount);
                    //所有考区档位占比
                    double gp = gtotal == 0 ? 0 : (double) o.getGcount() / gtotal * 100;
                    BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                    o.setGpercent(gbd.doubleValue());
                    o.setPt(0);
                }
            });
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            paperRepo.countGroupByLevelAll(questionId, markSubject.getTest())
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
            for (Level level : levels) {
                long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
                //没有该档位，新增
                if (count == 0) {
                    LevelStatDTO dto = new LevelStatDTO();
                    dto.setId(level.getCode());
                    dto.setCount(0);
                    dto.setPercent(0.0);
                    levelStatDTOs.add(dto);
                }
            }
            //当前考区总数
            long total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalseAndTest(markSubject.getWorkId(), questionId, markSubject.getTest());
            //所有考区总数
            long gtotal = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getTest());
            levelStatDTOs.forEach(o -> {
                if (o.getId() != null) {
                    //当前考区档位占比
                    double p = total == 0 ? 0 : (double) o.getCount() / total * 100;
                    BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                    o.setPercent(bd.doubleValue());
                    //考区阈值
                    Level level = levels.stream().filter(l -> l.getCode().equals(o.getId())).collect(Collectors.toList()).get(0);
                    o.setKdpt(level.getKdpt());
                    //所有考区档位数量
                    long gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndTestAndBatchNoNotNull(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(o.getId()), markSubject.getTest());
                    o.setGcount((int) gcount);
                    //所有考区档位占比
                    double gp = gtotal == 0 ? 0 : (double) o.getGcount() / gtotal * 100;
                    BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                    o.setGpercent(gbd.doubleValue());
                    o.setPt(level.getPt());
                }
            });
        }

        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 科组长-该评卷科目的分档统计信息
     *
     * @param markSubject 评卷科目id
     * @param questionId  试题id
     * @return
     */
    @GetMapping("{markSubject}/mark_leader/stat/roughLevels")
    public List<LevelStatDTO> markLeaderRoughLevelStat(@PathVariable MarkSubject markSubject, @RequestParam Long questionId) {
        Long markerId = ServletUtil.getUserId();
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndMarkLeaderId(markSubject.getWorkId(), markSubject, markerId);
        List<Long> paperIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            List<Long> markers = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
            paperIds = markerGroupLeaderService.listPaperIdsByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), markers);
        }
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        List<String> roughCodes = levels.stream().map(Level::getRoughCode).distinct().collect(Collectors.toList());
        if (CollectionUtils.isEmpty(paperIds)) {
            paperRepo.countGroupByRoughLevelAll(questionId, markSubject.getTest())
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            for (String roughCode : roughCodes) {
                long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(roughCode)).count();
                //没有该档位，新增
                if (count == 0) {
                    LevelStatDTO dto = new LevelStatDTO();
                    dto.setId(roughCode);
                    dto.setCount(0);
                    dto.setPercent(0.0);
                    levelStatDTOs.add(dto);
                }
            }
            //当前考区总数
            long total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
            //所有考区总数
            long gtotal = paperRepo.countByWorkIdAndSubjectAndIsMissingFalse(markSubject.getWorkId(), markSubject.getSubject());
            levelStatDTOs.forEach(o -> {
                if (o.getId() != null) {
                    //当前考区档位占比
                    double p = total == 0 ? 0 : (double) o.getCount() / total * 100;
                    BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                    o.setPercent(bd.doubleValue());
                    //考区阈值
//                    Level level = levels.stream().filter(l -> l.getCode().equals(o.getId())).collect(Collectors.toList()).get(0);
                    o.setKdpt(0);
                    //所有考区档位数量
                    long gcount = paperRepo.countByWorkIdAndSubjectAndRoughLevelAndTestAndRoughBatchNoNotNull(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(o.getId()), markSubject.getTest());
                    o.setGcount((int) gcount);
                    //所有考区档位占比
                    double gp = gtotal == 0 ? 0 : (double) o.getGcount() / gtotal * 100;
                    BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                    o.setGpercent(gbd.doubleValue());
                    o.setPt(0);
                }
            });
        } else {
            paperRepo.countGroupByRoughLevelAllAndIdIn(questionId, markSubject.getTest(), paperIds)
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            for (String roughCode : roughCodes) {
                long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(roughCode)).count();
                //没有该档位，新增
                if (count == 0) {
                    LevelStatDTO dto = new LevelStatDTO();
                    dto.setId(roughCode);
                    dto.setCount(0);
                    dto.setPercent(0.0);
                    levelStatDTOs.add(dto);
                }
            }
            //当前考区总数
            long total = paperRepo.countByWorkIdAndQuestionIdAndIdIn(markSubject.getWorkId(), questionId, paperIds);
            //所有考区总数
            long gtotal = paperRepo.countByWorkIdAndSubjectAndIdIn(markSubject.getWorkId(), markSubject.getSubject(), paperIds);
            List<Long> finalPaperIds = paperIds;
            levelStatDTOs.forEach(o -> {
                if (o.getId() != null) {
                    //当前考区档位占比
                    double p = total == 0 ? 0 : (double) o.getCount() / total * 100;
                    BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                    o.setPercent(bd.doubleValue());
                    //考区阈值
//                    Level level = levels.stream().filter(l -> l.getCode().equals(o.getId())).collect(Collectors.toList()).get(0);
                    o.setKdpt(0);
                    //所有考区档位数量
                    long gcount = paperRepo.countByWorkIdAndSubjectAndRoughLevelAndIdIn(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(o.getId()), finalPaperIds);
                    o.setGcount((int) gcount);
                    //所有考区档位占比
                    double gp = gtotal == 0 ? 0 : (double) o.getGcount() / gtotal * 100;
                    BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                    o.setGpercent(gbd.doubleValue());
                    o.setPt(0);
                } else {
                    int count = o.getCount() - o.getArbitrated() - o.getRejected();
                    o.setCount(count);
                }
            });
        }
        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 科组长-该评卷科目的分档统计信息
     *
     * @param markSubject 评卷科目id
     * @param questionId  试题id
     * @return
     */
    @RequestMapping(value = "{markSubject}/mark_leader/stat/levels", method = RequestMethod.GET)
    public List<LevelStatDTO> markLeaderLevelStat(@PathVariable MarkSubject markSubject, @RequestParam Long questionId) {
        Long markerId = ServletUtil.getUserId();
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndMarkLeaderId(markSubject.getWorkId(), markSubject, markerId);
        List<Long> paperIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            List<Long> markers = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
            paperIds = markerGroupLeaderService.listPaperIdsByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), markers);
        }
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        if (CollectionUtils.isEmpty(paperIds)) {
            paperRepo.countGroupByLevelAll(questionId, markSubject.getTest())
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            for (Level level : levels) {
                long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
                //没有该档位，新增
                if (count == 0) {
                    LevelStatDTO dto = new LevelStatDTO();
                    dto.setId(level.getCode());
                    dto.setCount(0);
                    dto.setPercent(0.0);
                    levelStatDTOs.add(dto);
                }
            }
            //当前考区总数
            long total = paperRepo.countByWorkIdAndQuestionIdAndIsMissingIsFalse(markSubject.getWorkId(), questionId);
            //所有考区总数
            long gtotal = paperRepo.countByWorkIdAndSubjectAndIsMissingFalse(markSubject.getWorkId(), markSubject.getSubject());
            levelStatDTOs.forEach(o -> {
                if (o.getId() != null) {
                    //当前考区档位占比
                    double p = total == 0 ? 0 : (double) o.getCount() / total * 100;
                    BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                    o.setPercent(bd.doubleValue());
                    //考区阈值
                    Level level = levels.stream().filter(l -> l.getCode().equals(o.getId())).collect(Collectors.toList()).get(0);
                    o.setKdpt(level.getKdpt());
                    //所有考区档位数量
                    long gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndTestAndBatchNoNotNull(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(o.getId()), markSubject.getTest());
                    o.setGcount((int) gcount);
                    //所有考区档位占比
                    double gp = gtotal == 0 ? 0 : (double) o.getGcount() / gtotal * 100;
                    BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                    o.setGpercent(gbd.doubleValue());
                    o.setPt(level.getPt());
                } else {
                    int count = o.getCount() - o.getArbitrated() - o.getRejected();
                    o.setCount(count);
                }
            });
        } else {
            paperRepo.countGroupByLevelAllAndIdIn(questionId, markSubject.getTest(), paperIds)
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            for (Level level : levels) {
                long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
                //没有该档位，新增
                if (count == 0) {
                    LevelStatDTO dto = new LevelStatDTO();
                    dto.setId(level.getCode());
                    dto.setCount(0);
                    dto.setPercent(0.0);
                    levelStatDTOs.add(dto);
                }
            }
            //当前考区总数
            long total = paperRepo.countByWorkIdAndQuestionIdAndIdIn(markSubject.getWorkId(), questionId, paperIds);
            //所有考区总数
            long gtotal = paperRepo.countByWorkIdAndSubjectAndIdIn(markSubject.getWorkId(), markSubject.getSubject(), paperIds);
            List<Long> finalPaperIds = paperIds;
            levelStatDTOs.forEach(o -> {
                if (o.getId() != null) {
                    //当前考区档位占比
                    double p = total == 0 ? 0 : (double) o.getCount() / total * 100;
                    BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                    o.setPercent(bd.doubleValue());
                    //考区阈值
                    Level level = levels.stream().filter(l -> l.getCode().equals(o.getId())).collect(Collectors.toList()).get(0);
                    o.setKdpt(level.getKdpt());
                    //所有考区档位数量
                    long gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndIdIn(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(o.getId()), finalPaperIds);
                    o.setGcount((int) gcount);
                    //所有考区档位占比
                    double gp = gtotal == 0 ? 0 : (double) o.getGcount() / gtotal * 100;
                    BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                    o.setGpercent(gbd.doubleValue());
                    o.setPt(level.getPt());
                } else {
                    int count = o.getCount() - o.getArbitrated() - o.getRejected();
                    o.setCount(count);
                }
            });
        }
        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }


    /**
     * 该评卷科目的分档统计信息
     *
     * @param markSubject 评卷科目id
     * @param questionId  试题id
     * @return
     */
    @RequestMapping(value = "{markSubject}/stat/scores", method = RequestMethod.GET)
    public List<LevelStatDTO> scoreStat(HttpServletRequest request, @PathVariable MarkSubject markSubject, @RequestParam Long questionId) {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        paperRepo.countScoreGroupByLevelAll(questionId, markSubject.getTest())
                .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        for (Level level : levels) {
            //没有该档位，新增
            LevelStatDTO dto = new LevelStatDTO();
            dto.setId(level.getCode());
            dto.setCount(0);
            dto.setPercent(0.0);
            levelStatDTOs.add(dto);
        }
        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                int count = paperRepo.countByWorkIdAndSubjectAndQuestionIdAndLevelAndScoreNotNull(markSubject.getWorkId(), markSubject.getSubject(), questionId, String.valueOf(o.getId()));
                o.setCount(count);
                //所有考区已打分数量
                int gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndScoreNotNull(markSubject.getWorkId(), markSubject.getSubject(), o.getId());
                o.setGcount(gcount);
            }
        });
        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 该评卷科目的分档统计信息
     *
     * @param markSubject 评卷科目id
     * @param questionId  试题id
     */
    @RequestMapping(value = "{markSubject}/mark_leader/stat/scores", method = RequestMethod.GET)
    public List<LevelStatDTO> markLeaderScoreStat(@PathVariable MarkSubject markSubject,
                                                  @RequestParam Long questionId) {
        Long markerId = ServletUtil.getUserId();
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndMarkLeaderId(markSubject.getWorkId(), markSubject, markerId);
        List<Long> paperIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            List<Long> markers = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
            paperIds = markerGroupLeaderService.listPaperIdsByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), markers);
        }

        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        if (CollectionUtils.isEmpty(paperIds)) {
            paperRepo.countScoreGroupByLevelAll(questionId, markSubject.getTest())
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            for (Level level : levels) {
                //没有该档位，新增
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setPercent(0.0);
                int count = paperRepo.countByWorkIdAndSubjectAndQuestionIdAndLevelAndScoreNotNull(markSubject.getWorkId(), markSubject.getSubject(), questionId, level.getCode());
                dto.setCount(count);
                int gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndScoreNotNull(markSubject.getWorkId(), markSubject.getSubject(), level.getCode());
                dto.setGcount(gcount);
                levelStatDTOs.add(dto);
            }
        } else {
            paperRepo.countScoreGroupByLevelAllAndIdIn(questionId, markSubject.getTest(), paperIds)
                    .forEach(o -> levelStatDTOs.add(levelStatAssembler.toDTO(o)));
            for (Level level : levels) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setPercent(0.0);
                int count = paperRepo.countByWorkIdAndSubjectAndQuestionIdAndLevelAndScoreNotNullAndIdIn(markSubject.getWorkId(), markSubject.getSubject(), questionId, level.getCode(), paperIds);
                dto.setCount(count);
                int gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndScoreNotNullAndIdIn(markSubject.getWorkId(), markSubject.getSubject(), level.getCode(), paperIds);
                dto.setGcount(gcount);
                levelStatDTOs.add(dto);
            }
        }
        Collections.sort(levelStatDTOs, (o1, o2) -> {
            if (o1.getId() == null || o2.getId() == null) {
                return 1;
            }
            return o1.getId().toString().compareTo(o2.getId().toString());
        });
        return levelStatDTOs;
    }

    /**
     * 该评卷科目下的评卷员分组列表
     *
     * @param markSubject 评卷科目id
     */
    @GetMapping("{markSubject}/markergroups")
    public List<MarkerGroup> getMarkerGroups(@PathVariable MarkSubject markSubject) {
        MarkStage stage = markSubject.getStage();
        if (MarkStage.INIT.equals(markSubject.getStage())) {
            if (ParamCache.levelConfigMap.get(String.valueOf(markSubject.getWorkId())).getRoughLevelEnable() == 1) {
                stage = MarkStage.ROUGH_LEVEL;
            } else {
                stage = MarkStage.LEVEL;
            }
        }
        List<MarkerGroup> markerGroups = markerGroupRepo.findByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), stage);
        markerGroups.forEach(m -> {
            long count = markerGroupStudentRepo.countByGroupIdAndUsed(m.getId(), false);
            m.setHasStudent(count > 0);
        });
        return markerGroups;
    }

    /**
     * 创建评卷组
     *
     * @param markSubject 评卷科目id
     * @param markerGroup 评卷组
     */
    @PostMapping("{markSubject}/markergroups")
    public MarkerGroup createMarkerGroup(@PathVariable MarkSubject markSubject, @RequestBody MarkerGroup markerGroup) {
        if (markerGroup.getMarkers().isEmpty()) {
            throw new RuntimeException("不可创建空分组");
        }
        int count;
        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            count = markTaskRoughLevelRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage());
            if (count > 0) {
                throw new RuntimeException("当前阶段已发布任务，不能编辑分组");
            }
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            count = markTaskLevelRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage());
            if (count > 0) {
                throw new RuntimeException("当前阶段已发布任务，不能编辑分组");
            }
        } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
            count = markTaskScoreRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage());
            if (count > 0) {
                throw new RuntimeException("当前阶段已发布任务，不能编辑分组");
            }
        }

        markerGroup.setWorkId(markSubject.getWorkId());
        markerGroup.setSubject(markSubject.getSubject());
        MarkStage stage = markSubject.getStage();
        if (markSubject.getStage().equals(MarkStage.INIT)) {
            if (ParamCache.levelConfigMap.get(String.valueOf(markSubject.getWorkId())).getRoughLevelEnable() == 1) {
                stage = MarkStage.ROUGH_LEVEL;
            } else {
                stage = MarkStage.LEVEL;
            }
        }
        markerGroup.setStage(stage);
        return markerGroupLeaderService.save(markerGroup);
    }

    /**
     * 删除评卷组考生名单
     *
     * @param markSubject 评卷科目ID
     * @param groupId     评卷组ID
     */
    @DeleteMapping("/marker_groups_student/{markSubject}/{groupId}")
    public void deleteMarkerGroupStudent(@PathVariable MarkSubject markSubject, @PathVariable Long groupId) {
        markerGroupStudentRepo.deleteByGroupIdAndUsed(groupId, false);
    }

    /**
     * 修改评卷组名
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     * @param markerGroup 评卷组
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}", method = RequestMethod.PATCH)
    public void updateMarkerGroup(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain, @RequestBody MarkerGroup markerGroup) {
        domain.setName(markerGroup.getName());
        markerGroupRepo.save(markerGroup);
    }

    /**
     * 删除评卷组
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}", method = RequestMethod.DELETE)
    public void removeMarkerGroup(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain) {
        markerGroupLeaderService.delete(markSubject, domain);
    }

    /**
     * 单个评卷组信息
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     * @return
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}", method = RequestMethod.GET)
    public MarkerGroup getMarkerGroup(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain) {
        return domain;
    }

    /**
     * 为评卷组设置评卷员
     *
     * @param markSubject 评卷科目id
     * @param domain      评卷组id
     * @param markerIds   评卷员用户id数组
     */
    @RequestMapping(value = "{markSubject}/markergroups/{domain}/setmaker", method = RequestMethod.PATCH)
    public void setMaker(@PathVariable MarkSubject markSubject, @PathVariable MarkerGroup domain, @RequestParam Long... markerIds) {
        domain.setMarkers(new HashSet<MarkUser>());
        for (Long markerId : markerIds) {
            domain.addMarker(markUserRepo.findOne(markerId));
        }
        markerGroupRepo.save(domain);
    }

    /**
     * 是否可以进行打分任务投放
     *
     * @param markSubject
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "{markSubject}/canScore", method = RequestMethod.GET)
    public Boolean canScore(@PathVariable MarkSubject markSubject) {
        int count = paperRepo.countByWorkIdAndSubjectAndScoreBatchNoIsNotNullAndScoreIsNull(markSubject.getWorkId(), markSubject.getSubject());
        return count <= 0;
    }

    /**
     * 是否可以进行打分任务全部考区发布
     *
     * @param markSubject
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "{markSubject}/canScoreAll", method = RequestMethod.GET)
    public Boolean canScoreAll(@PathVariable MarkSubject markSubject) {
        //查询pulish_count为1的，如果总数不等于成功数，则为单个考区发过任务的
        List<TaskPublishSetting> taskPublishSettings = taskPublishSettingRepo.findByWorkIdAndSubjectAndPublishCount(markSubject.getWorkId(), markSubject.getSubject().name(), 1);
        int totalTaskCount = taskPublishSettings.stream().mapToInt(TaskPublishSetting::getTotalCount).sum();
        int successTaskCount = taskPublishSettings.stream().mapToInt(TaskPublishSetting::getSuccessCount).sum();

        return totalTaskCount - successTaskCount <= 0;
    }

    /**
     * 检查是否已分档完成
     */
    @GetMapping("{markSubject}/canLevel")
    public Boolean canLevel(@PathVariable MarkSubject markSubject) {
        int count = 0;
        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            count = paperRepo.countByWorkIdAndSubjectAndRoughLevelIsNullAndIsMissingFalseAndRoughBatchNoNotNullAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getTest());
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            count = paperRepo.countByWorkIdAndSubjectAndLevelIsNullAndIsMissingFalseAndBatchNoNotNullAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getTest());
        }
        return count <= 0;
    }

    /**
     * 查询分档进度
     *
     * @param markSubject 评卷科目id
     */
    @RequestMapping(value = "{markSubject}/progress", method = RequestMethod.GET)
    public LevleProgressDTO progress(@PathVariable MarkSubject markSubject) {
        return stageControlService.progress(markSubject);
    }

    /**
     * 查询打分进度
     *
     * @param markSubject 评卷科目id
     */
    @RequestMapping(value = "{markSubject}/scoreProgress", method = RequestMethod.GET)
    public List scoreProgress(@PathVariable MarkSubject markSubject, @RequestParam(defaultValue = "") Long questionId) {
        return stageControlService.scoreProgress(markSubject, questionId);
    }

    /**
     * 分考区查询分档进度
     *
     * @param markSubject 评卷科目id
     */
    @GetMapping("{markSubject}/areaProgress")
    public List<LevleProgressDTO> areaProgress(@PathVariable MarkSubject markSubject) {
        return stageControlService.areaProgress(markSubject);
    }

    /**
     * 查询所有科目
     *
     * @param workId
     * @return
     */
    @RequestMapping(value = "{workId}/subjects", method = RequestMethod.GET)
    public List<MarkSubject> subjects(@PathVariable Long workId) {
        return markSubjectRepo.findAllByWorkId(workId);
    }

    /**
     * 更新科目状态
     *
     * @param markSubject 评卷科目id
     */
    @RequestMapping(value = "{markSubject}/updateEnable", method = RequestMethod.POST)
    public ResponseEntity updateEnable(@PathVariable MarkSubject markSubject) {
        //有采集数据，不能禁用
        List<Paper> papers = paperRepo.findByWorkIdAndSubject(markSubject.getWorkId(), markSubject.getSubject());
        if (papers != null && papers.size() > 0) {
            throw new RuntimeException("已有采集数据，不可" + (markSubject.isEnable() ? "禁用" : "启用"));
        }
        markSubject.setEnable(!markSubject.isEnable());
        markSubjectRepo.saveAndFlush(markSubject);
        return new ResponseEntity(HttpStatus.OK);
    }

    /**
     * 更新科目名称
     *
     * @param markSubject 评卷科目id
     */
    @RequestMapping(value = "{subjectId}/updateName", method = RequestMethod.POST)
    public ResponseEntity updateName(@PathVariable String subjectId, MarkSubject markSubject) {
        MarkSubject markSubjectT = markSubjectRepo.findOne(subjectId);
        //有采集数据，不能禁用
        List<Paper> papers = paperRepo.findByWorkIdAndSubject(markSubjectT.getWorkId(), markSubjectT.getSubject());
        if (papers != null && papers.size() > 0) {
            throw new RuntimeException("已有采集数据，不能修改科目名称");
        }
        markSubjectT.setName(markSubject.getName());
        markSubjectRepo.saveAndFlush(markSubjectT);
        return new ResponseEntity(HttpStatus.OK);
    }

}
