package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.utils.DigitalToChineseUtil;
import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroupLeader;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelDetailDTO;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import cn.com.qmth.stmms.ms.marking.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 评卷任务数据api
 * Created by zhengmin on 2016/10/12.
 */
@RestController
@RequestMapping("api/marktasks")
public class MarkTaskApi {
    private static Logger LOGGER = LoggerFactory.getLogger(MarkTaskApi.class);

    @Autowired
    private PaperAssembler paperAssembler;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Resource
    private MarkerGroupLeaderService markerGroupLeaderService;

    @Resource
    private MarkTaskRoughLevelService markTaskRoughLevelService;

    @Resource
    private MarkTaskLevelService markTaskLevelService;

    @Resource
    private MarkTaskScoreService markTaskScoreService;

    /**
     * 评卷员的评卷任务
     *
     * @param markerId   评卷员用户id
     * @param stage      评卷阶段
     * @param level      档位
     * @param sn         密号
     * @param questionId 试题id
     */
    @RequestMapping(method = RequestMethod.GET)
    public PageableDTO list(@RequestParam Long markerId,
                            @RequestParam Long workId,
                            @RequestParam MarkStage stage,
                            @RequestParam(required = false) Boolean isSample,
                            @RequestParam(defaultValue = "false") Boolean reject,
                            @RequestParam(required = false) String level,
                            @RequestParam(required = false) String sn,
                            @RequestParam Long questionId,
                            @RequestParam(required = false) String areaCode,
                            Pageable pageable) throws Exception {
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            return markTaskRoughLevelService.listMarkTaskLevel(markerId, isSample, reject, level, questionId, areaCode, pageable);
        } else if (MarkStage.LEVEL.equals(stage)) {
            return markTaskLevelService.listMarkTaskLevel(markerId, isSample, reject, level, questionId, areaCode, pageable);
        } else if (MarkStage.SCORE.equals(stage)) {
            return markTaskScoreService.listMarkTaskScore(markerId, isSample, reject, level, questionId, areaCode, pageable);
        }
        return null;
    }

    /**
     * 评卷员的评卷任务
     *
     * @param markerId   评卷员用户id
     * @param questionId 试题id
     */
    @GetMapping("/shift")
    public PageableDTO list(@RequestParam Long markerId,
                            @RequestParam(required = false) Long workId,
                            @RequestParam MarkStage stage,
                            @RequestParam(required = false) Boolean isShift,
                            @RequestParam(required = false) Boolean isShiftScore,
                            @RequestParam Long questionId,
                            Pageable pageable) {
        if (MarkStage.LEVEL.equals(stage)) {
            return markTaskLevelService.shiftLevel(markerId, workId, isShift, isShiftScore, questionId, pageable);
        } else if (MarkStage.SCORE.equals(stage)) {
            return markTaskScoreService.shiftScore(markerId, workId, isShift, isShiftScore, questionId, pageable);
        }
        return null;

    }

    /**
     * 提交评卷任务
     *
     * @param markTask 评卷你任务id
     * @param body     评卷内容
     */
    @PatchMapping("{markTask}")
    public ResponseEntity marking(@PathVariable Long markTask, @RequestBody HashMap<String, String> body) {
        MarkStage stage = MarkStage.valueOf(body.get("stage"));
        MarkTaskDTO markTaskDTO = null;
        if (stage.equals(MarkStage.ROUGH_LEVEL)) {
            markTaskDTO = markTaskRoughLevelService.markingLevel(markTask, body);
        } else if (stage.equals(MarkStage.LEVEL)) {
            markTaskDTO = markTaskLevelService.markingLevel(markTask, body);
        } else if (stage.equals(MarkStage.SCORE)) {
            markTaskDTO = markTaskScoreService.markingScore(markTask, body);
        }
        return new ResponseEntity(markTaskDTO, HttpStatus.OK);
    }

    /**
     * 批量提交评卷任务
     *
     * @param body 评卷内容yoc
     */
    @RequestMapping(value = "/batch", method = RequestMethod.PATCH)
    public ResponseEntity batch(@RequestBody HashMap<String, String> body) {
        MarkStage stage = MarkStage.valueOf(body.get("stage"));
        String markIds = body.get("taskIds");
        if (Objects.isNull(markIds)) {
            throw new RuntimeException("请选择待评试卷");
        }
        String[] ids = markIds.split(",");
        List<MarkTaskDTO> list = new ArrayList<>();
        for (String id : ids) {
            MarkTaskDTO markTaskDTO = null;
            if (stage.equals(MarkStage.ROUGH_LEVEL)) {
                markTaskDTO = markTaskRoughLevelService.markingLevel(Long.valueOf(id), body);
            } else if (stage.equals(MarkStage.LEVEL)) {
                markTaskDTO = markTaskLevelService.markingLevel(Long.valueOf(id), body);
            } else if (stage.equals(MarkStage.SCORE)) {
                markTaskDTO = markTaskScoreService.markingScore(Long.valueOf(id), body);
            }
            list.add(markTaskDTO);
        }

        return new ResponseEntity(list, HttpStatus.OK);
    }

    /**
     * 跳过评卷任务
     *
     * @param markTaskId 评卷你任务id
     * @return
     */
    @RequestMapping(value = "{stage}/{markTaskId}/skip", method = RequestMethod.POST)
    public ResponseEntity skip(@PathVariable MarkStage stage, @PathVariable Long markTaskId) {
        if (stage.equals(MarkStage.ROUGH_LEVEL)) {
            return markTaskRoughLevelService.skipLevel(markTaskId);
        } else if (stage.equals(MarkStage.LEVEL)) {
            return markTaskLevelService.skipLevel(markTaskId);
        } else if (stage.equals(MarkStage.SCORE)) {
            return markTaskScoreService.skipScore(markTaskId);
        }
        return null;
    }

    /**
     * 查询标记试卷
     *
     * @param markTaskId 评卷你任务id
     */
    @RequestMapping(value = "/{stage}/mark_task", method = RequestMethod.POST)
    @Transactional
    public Object markTask(@PathVariable MarkStage stage,
                           @RequestParam Long markTaskId,
                           @RequestParam Boolean isMark) {
        if (stage.equals(MarkStage.ROUGH_LEVEL)) {
            return markTaskRoughLevelService.updateMarkTaskMarkById(markTaskId, isMark);
        } else if (stage.equals(MarkStage.LEVEL)) {
            return markTaskLevelService.updateMarkTaskMarkById(markTaskId, isMark);
        } else if (stage.equals(MarkStage.SCORE)) {
            return markTaskScoreService.updateMarkTaskMarkById(markTaskId, isMark);
        }
        return true;
    }

    /**
     * 已评试卷回显
     *
     * @param markerId   工用ID
     * @param stage      阶段
     * @param questionId 考区ID
     * @param areaCode   考区
     */
    @GetMapping("/reviewPaper")
    public List<MarkTaskDTO> reviewPaper(@RequestParam Long markerId,
                                         @RequestParam MarkStage stage,
                                         @RequestParam Long questionId,
                                         @RequestParam(required = false) String areaCode) {
        if (stage.equals(MarkStage.ROUGH_LEVEL)) {
            return markTaskRoughLevelService.reviewPaperLevel(markerId, questionId, areaCode);
        } else if (stage.equals(MarkStage.LEVEL)) {
            return markTaskLevelService.reviewPaperLevel(markerId, questionId, areaCode);
        } else if (stage.equals(MarkStage.SCORE)) {
            return markTaskScoreService.reviewPaperScore(markerId, questionId, areaCode);
        }
        return new ArrayList<>();
    }


    @GetMapping("/kzzReviewPaper")
    public List<PaperDTO> reviewPaper(@RequestParam Long workId,
                                      @RequestParam Subject subject,
                                      @RequestParam Long questionId) {
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        Long markerId = ServletUtil.getUserId();
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndMarkLeaderId(markSubject.getWorkId(), markSubject, markerId);
        List<Long> paperIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            List<Long> markers = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
            paperIds = markerGroupLeaderService.listPaperIdsByWorkIdAndSubjectAndStage(workId, markSubject.getSubject(), markSubject.getStage(), markers);
        }
        List<Long> finalPaperIds = paperIds;
        Specification<Paper> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("workId"), workId));
            predicates.add(builder.equal(root.get("subject"), subject));
            if (Objects.nonNull(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }

            if (!CollectionUtils.isEmpty(finalPaperIds)) {
                CriteriaBuilder.In<Object> in = builder.in(root.get("id"));
                for (Long id : finalPaperIds) {
                    in.value(id);
                }
                predicates.add(builder.and(builder.and(in)));
            }

            predicates.add(builder.equal(root.get("markByLeader"), true));
            List<Predicate> list = new ArrayList<>();
            if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                list.add(builder.equal(root.get("isRoughSample"), true));
                list.add(builder.equal(root.get("isRoughOneClick"), true));
            } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                list.add(builder.equal(root.get("isSample"), true));
                list.add(builder.equal(root.get("isOneClick"), true));
            }

            list.add(builder.equal(root.get("isRejectedByLeader"), true));
            Predicate Pre_And = builder.and(predicates.toArray(new Predicate[predicates.size()]));
            Predicate Pre_Or = builder.or(list.toArray(new Predicate[list.size()]));
            return query.where(Pre_And, Pre_Or).getRestriction();
        };

        Sort sort = new Sort(Sort.Direction.DESC, "updatedOn");
        Pageable pageable = new PageRequest(0, 5, sort);
        Page<Paper> papers = paperRepo.findAll(specification, pageable);

        List<PaperDTO> paperList = new ArrayList<>();
        papers.getContent().forEach(p -> {
            paperList.add(paperAssembler.toDTO(p));
        });
        return paperList;
    }


    /**
     * 分档详情
     *
     * @param workId
     * @param subject
     * @param questionId
     * @return
     */
    @RequestMapping(value = "/levelStatDetail", method = RequestMethod.GET)
    public List<LevelDetailDTO> levelStatDetail(@RequestParam Long workId,
                                                @RequestParam Subject subject,
                                                @RequestParam(required = false) Long questionId) {
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());

        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            List<Level> levels = levelRepo.findByWorkIdOrderByCode(workId);
            List<RoughLevel> roughLevels = DetermineRoughLevelService.parseRoughLevels(levels);
            List<Paper> papers;
            if (questionId == null) {
                papers = paperRepo.findByWorkIdAndSubjectAndIsMissingAndTest(workId, subject, false, markSubject.getTest());
            } else {
                papers = paperRepo.findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTest(workId, subject, questionId, false, markSubject.getTest());
            }


            long gtotal = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getTest());
            List<LevelDetailDTO> list = new ArrayList<>();
            for (RoughLevel level : roughLevels) {
                LevelDetailDTO levelDetailDTO = new LevelDetailDTO();
                levelDetailDTO.setWorkId(workId);
                levelDetailDTO.setSubjectName(markSubject.getName());
                //档位
                levelDetailDTO.setCode(DigitalToChineseUtil.int2chineseNum(Integer.parseInt(level.getRoughCode())));
                levelDetailDTO.setMinScore(level.getMinScore());
                levelDetailDTO.setMaxScore(level.getMaxScore());
                //数量
                long count = papers.stream().filter(m -> Objects.nonNull(m.getRoughLevel()) && Objects.equals(level.getRoughCode(), m.getRoughLevel())).count();
                levelDetailDTO.setLevelCount((int) count);
                BigDecimal prop = papers.isEmpty() ? new BigDecimal("0") : new BigDecimal(String.valueOf(count * 100)).divide(new BigDecimal(String.valueOf(papers.size())), 2, BigDecimal.ROUND_HALF_UP);
                //占比
                levelDetailDTO.setLevelProp(prop.doubleValue());
                //预设占比
                levelDetailDTO.setExamLevelProp(0);
                //差值
//                BigDecimal diff = prop.subtract(new BigDecimal(String.valueOf(level.getPt())));
                levelDetailDTO.setDiffProp(0);
                //累计数量
                int sumCount = (int) (list.stream().mapToInt(LevelDetailDTO::getLevelCount).sum() + count);
                levelDetailDTO.setCumulateCount(sumCount);
                //累计占比
                BigDecimal cumulateProp = papers.isEmpty() ? new BigDecimal("0") : new BigDecimal(String.valueOf(sumCount * 100)).divide(new BigDecimal(String.valueOf(papers.size())), 2, BigDecimal.ROUND_HALF_UP);
                levelDetailDTO.setCumulateProp(cumulateProp.doubleValue());

                //调整
//                int adjustment = new BigDecimal(String.valueOf(papers.size())).multiply(diff.divide(new BigDecimal("100"))).intValue();
                levelDetailDTO.setAdjustmentCount(0);

                //考区阈值
                levelDetailDTO.setKdpt(0);
                levelDetailDTO.setPt(0);
                //所有考区档位数量
                long gcount = paperRepo.countByWorkIdAndSubjectAndRoughLevelAndTest(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(level.getRoughCode()), markSubject.getTest());
                levelDetailDTO.setGcount((int) gcount);
                //所有考区档位占比
                double gp = gtotal == 0 ? 0 : (double) levelDetailDTO.getGcount() / gtotal * 100;
                BigDecimal gbd = new BigDecimal(gp).setScale(2, RoundingMode.HALF_EVEN);
                levelDetailDTO.setGpercent(gbd.doubleValue());

                list.add(levelDetailDTO);
            }
            return list;
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            List<Level> levels = levelRepo.findByWorkIdOrderByCode(workId);
            List<Paper> papers;
            if (questionId == null) {
                papers = paperRepo.findByWorkIdAndSubjectAndIsMissingAndTest(workId, subject, false, markSubject.getTest());
            } else {
                papers = paperRepo.findByWorkIdAndSubjectAndQuestionIdAndIsMissingAndTest(workId, subject, questionId, false, markSubject.getTest());
            }


            long gtotal = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndTest(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getTest());
            List<LevelDetailDTO> list = new ArrayList<>();
            for (Level level : levels) {
                LevelDetailDTO levelDetailDTO = new LevelDetailDTO();
                levelDetailDTO.setWorkId(workId);
                levelDetailDTO.setSubjectName(markSubject.getName());
                //档位
                levelDetailDTO.setCode(level.getCode());
                levelDetailDTO.setMinScore(level.getMinScore());
                levelDetailDTO.setMaxScore(level.getMaxScore());
                //数量
                long count = papers.stream().filter(m -> Objects.nonNull(m.getLevel()) && Objects.equals(level.getCode(), m.getLevel())).count();
                levelDetailDTO.setLevelCount((int) count);
                BigDecimal prop = papers.isEmpty() ? new BigDecimal("0") : new BigDecimal(String.valueOf(count * 100)).divide(new BigDecimal(String.valueOf(papers.size())), 2, BigDecimal.ROUND_HALF_UP);
                //占比
                levelDetailDTO.setLevelProp(prop.doubleValue());
                //预设占比
                levelDetailDTO.setExamLevelProp(level.getPt());
                //差值
                BigDecimal diff = prop.subtract(new BigDecimal(String.valueOf(level.getPt())));
                levelDetailDTO.setDiffProp(diff.doubleValue());
                //累计数量
                int sumCount = (int) (list.stream().mapToInt(LevelDetailDTO::getLevelCount).sum() + count);
                levelDetailDTO.setCumulateCount(sumCount);
                //累计占比
                BigDecimal cumulateProp = papers.isEmpty() ? new BigDecimal("0") : new BigDecimal(String.valueOf(sumCount * 100)).divide(new BigDecimal(String.valueOf(papers.size())), 2, BigDecimal.ROUND_HALF_UP);
                levelDetailDTO.setCumulateProp(cumulateProp.doubleValue());

                //调整
                int adjustment = new BigDecimal(String.valueOf(papers.size())).multiply(diff.divide(new BigDecimal("100"))).intValue();
                levelDetailDTO.setAdjustmentCount(adjustment);

                //考区阈值
                levelDetailDTO.setKdpt(level.getKdpt());
                levelDetailDTO.setPt(level.getPt());
                //所有考区档位数量
                long gcount = paperRepo.countByWorkIdAndSubjectAndLevelAndTest(markSubject.getWorkId(), markSubject.getSubject(), String.valueOf(level.getCode()), markSubject.getTest());
                levelDetailDTO.setGcount((int) gcount);
                //所有考区档位占比
                double gp = gtotal == 0 ? 0 : (double) levelDetailDTO.getGcount() / gtotal * 100;
                BigDecimal gbd = new BigDecimal(gp).setScale(2, RoundingMode.HALF_EVEN);
                levelDetailDTO.setGpercent(gbd.doubleValue());

                list.add(levelDetailDTO);
            }
            return list;
        }
        return null;
    }

    /**
     * 根据任务密号查询试卷
     *
     * @param workId       工作ID
     * @param subject      科目
     * @param secretNumber 密号
     */
    @RequestMapping(value = "/get_by_secret_number", method = RequestMethod.GET)
    public List getBySecretNumber(@RequestParam Long workId,
                                  @RequestParam Subject subject,
                                  @RequestParam String secretNumber) {
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject);
        if (markSubject.getStage().equals(MarkStage.LEVEL)) {
            return markTaskLevelService.getBySecretNumberLevel(workId, subject, secretNumber, markSubject.getTest());
        } else if (markSubject.getStage().equals(MarkStage.SCORE)) {
            return markTaskScoreService.getBySecretNumberScore(workId, subject, secretNumber, markSubject.getTest());
        }
        return null;
    }

    /**
     * 查询标记任务（科组长）
     *
     * @param workId   工作ID
     * @param subject  科目
     * @param stage    阶段
     * @param markerId 评卷员ID
     */
    @GetMapping("/list_mark_task")
    public PageableDTO listMarkTask(@RequestParam Long workId,
                                    @RequestParam Subject subject,
                                    @RequestParam MarkStage stage,
                                    @RequestParam Long markerId,
                                    Pageable pageable) {
        if (stage.equals(MarkStage.ROUGH_LEVEL)) {
            return markTaskRoughLevelService.listMarkedTaskLevel(workId, subject, markerId, pageable);
        } else if (stage.equals(MarkStage.LEVEL)) {
            return markTaskLevelService.listMarkedTaskLevel(workId, subject, markerId, pageable);
        } else if (stage.equals(MarkStage.SCORE)) {
            return markTaskScoreService.listMarkedTaskScore(workId, subject, markerId, pageable);
        }
        return null;
    }

    /**
     * 评卷员的标记试卷查询
     *
     * @param markerId   评卷员ID
     * @param workId     工作ID
     * @param subject    科目
     * @param questionId 考区ID
     * @param pageable   分页
     */
    @GetMapping("/markedPapers")
    public PageableDTO markedPapers(@RequestParam(required = false) Long workId,
                                    @RequestParam Long markerId,
                                    @RequestParam Subject subject,
                                    @RequestParam MarkStage stage,
                                    @RequestParam Long questionId,
                                    Pageable pageable) {

        if (stage.equals(MarkStage.ROUGH_LEVEL)) {
            return markTaskRoughLevelService.markedPapers(workId, subject, markerId, questionId, pageable);
        } else if (stage.equals(MarkStage.LEVEL)) {
            return markTaskLevelService.markedPapers(workId, subject, markerId, questionId, pageable);
        } else if (stage.equals(MarkStage.SCORE)) {
            return markTaskScoreService.markedPapers(workId, subject, markerId, questionId, pageable);
        }
        return null;
    }
}
