package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroupLeader;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskLevelAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskRoughLevelAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskScoreAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import cn.com.qmth.stmms.ms.marking.service.*;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 试卷相关api
 * Created by zhengmin on 2016/10/10.
 */
@RestController
@RequestMapping("api/papers")
public class PaperApi {

    @Autowired
    private PaperAssembler paperAssembler;

    @Resource
    private MarkUserRepo markUserRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Resource
    private WorkRepo workRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private MarkingLevelService markingLevelService;

    @Resource
    private MarkingRoughLevelService markingRoughLevelService;

    @Resource
    private MarkTaskLevelService markTaskLevelService;

    @Resource
    private MarkTaskScoreService markTaskScoreService;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    private MarkLogRepo markLogRepo;

    @Autowired
    SystemConfig systemConfig;

    @Autowired
    MarkerGroupLeaderService markerGroupLeaderService;

    @Resource
    MarkTaskRoughLevelAssembler markTaskRoughLevelAssembler;

    @Resource
    MarkTaskLevelAssembler markTaskLevelAssembler;

    @Resource
    MarkTaskScoreAssembler markTaskScoreAssembler;

    /**
     * 单个试卷信息
     *
     * @param paperId 试卷id
     */
    @GetMapping("{paperId}")
    public PaperDTO get(@PathVariable Long paperId) {
        return paperAssembler.toDTO(paperRepo.findOne(paperId));
    }

    /**
     * 单个试卷信息
     *
     * @param questionId 试题id
     * @param sn         密号
     * @param examNumber 准考证号
     */
    @GetMapping(value = "/one")
    public PaperDTO getOne(@RequestParam Long questionId,
                           @RequestParam(required = false) String sn,
                           @RequestParam(required = false) String examNumber) {
        PaperDTO paperDTO = null;
        Specification<Paper> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("questionId"), questionId));
            if (sn != null) {
                predicates.add(builder.equal(root.get("secretNumber"), sn));
            }
            if (examNumber != null) {
                predicates.add(builder.equal(root.get("examNumber"), examNumber));
            }

            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Optional<Paper> paperOptional = paperRepo.findAll(specification, new PageRequest(0, 1))
                .getContent().stream().findFirst();
        if (paperOptional.isPresent()) {
            paperDTO = paperAssembler.toDTO(paperOptional.get());
        }
        return paperDTO;
    }

    @GetMapping("search/byTaskSecretNumber")
    public PaperDTO getByTaskSecretNumber(@RequestParam Long questionId,
                                          @RequestParam MarkStage stage,
                                          @RequestParam String sn) {
        if (MarkStage.LEVEL.equals(stage)) {
            return markTaskLevelService.getByTaskSecretNumberLevel(questionId, sn);
        } else if (MarkStage.SCORE.equals(stage)) {
            return markTaskScoreService.getByTaskSecretNumberLevel(questionId, sn);
        }
        return null;
    }

    /**
     * 试卷管理-查询
     *
     * @param workId      工作ID
     * @param areaCode    考区
     * @param subject     科目
     * @param startNumber 准考证号（开始）
     * @param endNumber   准考证号（结束）
     * @param isManual    是否手工绑定
     * @param missing     是否缺考
     * @param isRelate
     * @param studentName 考生姓名
     * @param scanUserId  采集员ID
     * @param sortBy      排序字段
     * @param pageable    分页参数
     */
    @RequestMapping(value = "listByQuestion", method = RequestMethod.GET)
    public PageableDTO listByQuestion(@RequestParam Long workId,
                                      @RequestParam(required = false) String areaCode,
                                      @RequestParam Subject subject,
                                      @RequestParam(required = false) String startNumber,
                                      @RequestParam(required = false) String endNumber,
                                      @RequestParam(required = false) Boolean isManual,
                                      @RequestParam(required = false) Boolean missing,
                                      @RequestParam(required = false) Boolean isRelate,
                                      @RequestParam(required = false) String studentName,
                                      @RequestParam(required = false) Long scanUserId,
                                      @RequestParam(required = false) Boolean isMark,
                                      @RequestParam(required = false) String sortBy,
                                      Pageable pageable) {
        Sort sort = null;
        if (Objects.equals("1", sortBy)) {
            sort = new Sort(Sort.Direction.DESC, "uploadedOn");
        }
        if (Objects.equals("2", sortBy)) {
            sort = new Sort(Sort.Direction.ASC, "examNumber");
        }
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Specification<Paper> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("workId"), workId));
            if (scanUserId != null) {
                predicates.add(builder.equal(root.get("scanUserId"), scanUserId));
            }
            if (areaCode != null && !"".equals(areaCode)) {
                predicates.add(builder.equal(root.get("areaCode"), areaCode));
            }
            if (subject != null) {
                predicates.add(builder.equal(root.get("subject"), subject));
            }
            if (!StringUtils.isEmpty(startNumber) && !StringUtils.isEmpty(endNumber)) {
                predicates.add(builder.between(root.get("examNumber"), startNumber, endNumber));
            } else if (!StringUtils.isEmpty(startNumber) && StringUtils.isEmpty(endNumber)) {
                predicates.add(builder.equal(root.get("examNumber"), startNumber));
            } else if (StringUtils.isEmpty(startNumber) && !StringUtils.isEmpty(endNumber)) {
                predicates.add(builder.equal(root.get("examNumber"), endNumber));
            }
            if (isManual != null) {
                predicates.add(builder.equal(root.get("isManual"), isManual));
            }
            if (missing != null) {
                predicates.add(builder.equal(root.get("isMissing"), missing));
            }
            if (isRelate != null) {
                predicates.add(builder.equal(root.get("isRelate"), isRelate));
            }
            if (isMark != null) {
                predicates.add(builder.equal(root.get("isAdminMark"), isMark));
            }
            //考生姓名
            if (studentName != null && !"".equals(studentName)) {
                predicates.add(builder.equal(root.get("studentName"), studentName));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        List<PaperDTO> paperDTOs = new ArrayList<>();
        Page<Paper> paperPage = paperRepo.findAll(specification, pageable1);
        paperPage.getContent().forEach(p -> {
            paperDTOs.add(paperAssembler.toDTOForPaperManager(p, isManual));
        });
        return new PageableDTO(paperDTOs, paperPage.getTotalElements(), paperPage.getTotalPages(), pageable1.getPageNumber());
    }

    /**
     * 试卷列表
     *
     * @param questionId 试题id
     * @param level      档位，如果为空，则测序待评
     * @param arbi       是否仲裁
     * @param reject     是否打回
     * @param pageable   分页参数
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public PageableDTO list(@RequestParam(required = false) Long questionId,
                            @RequestParam(required = false) String level,
                            @RequestParam(required = false) Boolean isSample,
                            @RequestParam(required = false) Boolean markedLogic,
                            @RequestParam(required = false) Boolean arbi,
                            @RequestParam(required = false) Boolean reject,
                            @RequestParam(defaultValue = "false") Boolean isScore,//打分阶段该档位已打分数
                            Pageable pageable) {
        ExamQuestion examQuestion = examQuestionRepo.findOne(questionId);
        Long workId = examQuestion.getWorkId();
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + examQuestion.getSubject().name());
        Long markerId = ServletUtil.getUserId();
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderService.listByWorkIdAndSubjectAndStageAndMarkLeaderId(workId, markSubject, markerId);
        List<Long> paperIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            List<Long> markers = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
            paperIds = markerGroupLeaderService.listPaperIdsByWorkIdAndSubjectAndStage(workId, markSubject.getSubject(), markSubject.getStage(), markers);
        }

        // 处理试评（大科组长能查询的试卷范围）
        if (paperIds == null && markSubject.getTest() == TrialEnum.START_TRIAL.getId()) {
            List<Paper> papers = paperRepo.findByWorkIdAndSubjectAndTest(workId, markSubject.getSubject(), TrialEnum.START_TRIAL.getId());
            paperIds = papers.stream().map(Paper::getId).collect(Collectors.toList());
        }

        List<Long> finalPaperIds = paperIds;
        Specification<Paper> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }
            //isScore为true时，为科组长打分详情页面数据查询条件
            if (isSample != null && isSample) {
                if (isScore) {
                    //level为null时，查询待评数量
                    if (!StringUtils.isEmpty(level)) {
                        predicates.add(builder.equal(root.get("level"), level));
                        predicates.add(builder.isNotNull(root.get("score")));
                    }
                } else {
                    if (!StringUtils.isEmpty(level)) {
                        predicates.add(builder.equal(root.get("level"), level));
                        predicates.add(builder.isNotNull(root.get("batchNo")));
                    }
//                    if (!CollectionUtils.isEmpty(finalPaperIds)) {
//                        CriteriaBuilder.In<Object> in = builder.in(root.get("id"));
//                        for (Long id : finalPaperIds) {
//                            in.value(id);
//                        }
//                        predicates.add(builder.and(builder.and(in)));
//                    }
                }
            } else {
                if (isScore) {
                    //level为null时，查询待评数量
                    if (StringUtils.isEmpty(level)) {
                        predicates.add(builder.isNotNull(root.get("level")));
                        predicates.add(builder.isNull(root.get("score")));
                        predicates.add(builder.isNotNull(root.get("scoreBatchNo")));
                    } else {
                        predicates.add(builder.equal(root.get("level"), level));
                        predicates.add(builder.isNotNull(root.get("score")));
                    }
                } else {
                    if (StringUtils.isEmpty(level)) {
                        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                            predicates.add(builder.isNull(root.get("roughLevel")));
                        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                            predicates.add(builder.isNull(root.get("level")));
                        }
                        //查询当前批次号数据
//                predicates.add(builder.equal(root.get("batchNo"), batchNo));
                    } else {
                        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                            predicates.add(builder.equal(root.get("roughLevel"), level));
                            predicates.add(builder.isNotNull(root.get("roughBatchNo")));
                        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                            predicates.add(builder.equal(root.get("level"), level));
                            predicates.add(builder.isNotNull(root.get("batchNo")));
                        }
                    }
                }
                if (!CollectionUtils.isEmpty(finalPaperIds)) {
                    CriteriaBuilder.In<Object> in = builder.in(root.get("id"));
                    for (Long id : finalPaperIds) {
                        in.value(id);
                    }
                    predicates.add(builder.and(builder.and(in)));
                }
            }
            if (arbi != null) {
                predicates.add(builder.equal(root.get("isArbitrated"), arbi));
            } else {
                predicates.add(builder.equal(root.get("isArbitrated"), false));
            }
            if (reject != null) {
                predicates.add(builder.equal(root.get("isRejected"), reject));
            } else {
                predicates.add(builder.equal(root.get("isRejected"), false));
            }
            if (isSample != null) {
                if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                    predicates.add(builder.equal(root.get("isRoughSample"), isSample));
                } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                    predicates.add(builder.equal(root.get("isSample"), isSample));
                }
            }
            if (markedLogic != null) {
                predicates.add(builder.equal(root.get("markedLogic"), markedLogic));
            }
            //空白卷不展示
            predicates.add(builder.equal(root.get("isMissing"), false));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        String[] sortProperties = new String[]{"isSample", "sortNum"};
        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            sortProperties = new String[]{"isRoughSample", "sortNum"};
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            sortProperties = new String[]{"isSample", "sortNum"};
        }
        Sort sort = new Sort(Sort.Direction.DESC, sortProperties);
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Page<Paper> papers = paperRepo.findAll(specification, pageable1);
        List<PaperDTO> paperDTOs = new ArrayList<>();
        Long userId = ServletUtil.getUserId();
        MarkUser markUser = markUserRepo.findOne(userId);
        if (markUser != null && markUser.getRole().equals(Role.MARK_LEADER)) {
            userId = null;
        }
        Long finalUserId = userId;
        papers.forEach(p -> paperDTOs.add(paperAssembler.toDTO(p, finalUserId)));
        return new PageableDTO(paperDTOs, papers.getTotalElements(), papers.getTotalPages(), pageable.getPageNumber());
    }

    /**
     * 科组长试卷处理。提交评卷
     *
     * @param paper 试卷id
     * @param body  试卷信息
     */
    @RequestMapping(value = "{paper}", method = RequestMethod.PATCH)
    @Transactional
    public ResponseEntity marking(@PathVariable Paper paper, @RequestBody HashMap<String, String> body) {
        String action = body.get("action");
        String level = body.get("level");
        String tagged = body.get("tagged");
        MarkStage stage = MarkStage.valueOf(body.get("stage"));
        if (action != null && level != null) {
            if (action.equals("leveling")) {
                paper.setRejectedByLeader(false);
                if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                    paper.setRoughOneClick(true);
                    markingRoughLevelService.levelMarkPaper(paper, level, false);
                } else if (MarkStage.LEVEL.equals(stage)) {
                    paper.setOneClick(true);
                    markingLevelService.levelMarkPaper(paper, level, false);
                }
            } else if (action.equals("sampling")) {
                paper.setRejectedByLeader(false);
                if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                    paper.setRoughOneClick(false);
                    markingRoughLevelService.levelMarkPaper(paper, level, true);
                } else if (MarkStage.LEVEL.equals(stage)) {
                    paper.setOneClick(false);
                    markingLevelService.levelMarkPaper(paper, level, true);
                }
            } else if (action.equals("reject")) {
                paper.setRejectedByLeader(true);
                //需要打回的评卷员
                String ranges = body.get("range");
                String leaderConfirmStr = body.get("leaderConfirm");
                boolean leaderConfirm = leaderConfirmStr == null || "".equals(leaderConfirmStr) || Boolean.parseBoolean(leaderConfirmStr);
                if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                    paper.setRoughOneClick(false);
                    markingRoughLevelService.reject(paper, level, ranges, leaderConfirm, false, null);
                } else if (MarkStage.LEVEL.equals(stage)) {
                    paper.setOneClick(false);
                    markingLevelService.reject(paper, level, ranges, leaderConfirm, false, null);
                }
            }
        }
        if (tagged != null) {
            boolean isTagged = paper.isTagged();
            paper.setTagged(!isTagged);
            paperRepo.save(paper);
        }
        return new ResponseEntity(paperAssembler.toDTO(paper), HttpStatus.OK);
    }

    /**
     * 批量试卷处理。提交评卷
     *
     * @param body 试卷信息
     * @return
     */
    @RequestMapping(value = "batch", method = RequestMethod.PATCH)
    @Transactional
    public ResponseEntity marking(@RequestBody HashMap<String, String> body) {
        String action = body.get("action");
        String level = body.get("level");
        String tagged = body.get("tagged");
        String paperIds = body.get("paperIds");
        MarkStage stage = MarkStage.valueOf(body.get("stage"));
        if (paperIds.length() > 0) {
            List<PaperDTO> list = new ArrayList<>();
            String[] ids = paperIds.split(",");
            for (String id : ids) {
                Paper paper = paperRepo.findOne(Long.valueOf(id));
                if (action != null && level != null) {
                    if (action.equals("leveling")) {
                        paper.setRejectedByLeader(false);
                        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                            paper.setRoughOneClick(true);
                            markingRoughLevelService.levelMarkPaper(paper, level, false);
                        } else if (MarkStage.LEVEL.equals(stage)) {
                            paper.setOneClick(true);
                            markingLevelService.levelMarkPaper(paper, level, false);
                        }
                    } else if (action.equals("sampling")) {
                        paper.setRejectedByLeader(false);
                        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                            paper.setRoughOneClick(false);
                            markingRoughLevelService.levelMarkPaper(paper, level, true);
                        } else if (MarkStage.LEVEL.equals(stage)) {
                            paper.setOneClick(false);
                            markingLevelService.levelMarkPaper(paper, level, true);
                        }
                    } else if (action.equals("reject")) {
                        paper.setRejectedByLeader(true);
                        //需要打回的评卷员
                        String ranges = body.get("range");
                        String leaderConfirmStr = body.get("leaderConfirm");
                        boolean leaderConfirm = leaderConfirmStr == null || "".equals(leaderConfirmStr) || Boolean.parseBoolean(leaderConfirmStr);
                        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                            paper.setRoughOneClick(false);
                            markingRoughLevelService.reject(paper, level, ranges, leaderConfirm, false, null);
                        } else if (MarkStage.LEVEL.equals(stage)) {
                            paper.setOneClick(false);
                            markingLevelService.reject(paper, level, ranges, leaderConfirm, false, null);
                        }
                    }
                }
                if (tagged != null) {
                    boolean isTagged = paper.isTagged();
                    paper.setTagged(!isTagged);
                    paperRepo.save(paper);
                }
                PaperDTO paperDTO = paperAssembler.toDTO(paper);
                list.add(paperDTO);
            }
            return new ResponseEntity(list, HttpStatus.OK);
        }
        return null;
    }

    /**
     * 批量取消标准卷
     *
     * @param body 试卷信息
     * @return
     */
    @RequestMapping(value = "batch/cancelSample", method = RequestMethod.PATCH)
    @Transactional
    public ResponseEntity cancelSample(@RequestBody HashMap<String, String> body) {
        String paperIds = body.get("paperIds");
        String stage = body.get("stage");
        if (paperIds.length() > 0) {
            List<MarkLog> markLogs = new ArrayList<>();
            String[] ids = paperIds.split(",");
            if (MarkStage.ROUGH_LEVEL.name().equals(stage)) {
                for (String id : ids) {
                    Paper paper = paperRepo.findOne(Long.valueOf(id));
                    Work work = workRepo.findOne(paper.getWorkId());
                    paper.determineRoughLevel(null);
                    paper.setMarkByLeader(false);
                    paper.setRoughSample(false);
                    paperRepo.save(paper);

                    List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperIdAndStage(paper.getId(), MarkStage.ROUGH_LEVEL);
                    for (MarkTaskRoughLevel markTask : markTasks) {
                        markTask.setResult(null);
                        markTask.setLevel(null);
                        markTask.setSample(false);
                        markTaskRoughLevelRepo.save(markTask);
                    }

                    // 记录日志
                    Long userId = ServletUtil.getUserId();
                    MarkUser markUser = markUserRepo.findOne(userId);
                    MarkLog markLog = new MarkLog(work.getId(), work.getName(), paper.getId(), paper.getSubject(), MarkStage.ROUGH_LEVEL, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), markUser.getRole(), MarkLogOperType.CANCEL_SAMPLE.getId(), MarkLogOperType.CANCEL_SAMPLE.getName(), paper.getRoughLevel(), null, null, new Date());
                    markLogs.add(markLog);
                }
            } else if (MarkStage.LEVEL.name().equals(stage)) {
                for (String id : ids) {
                    Paper paper = paperRepo.findOne(Long.valueOf(id));
                    Work work = workRepo.findOne(paper.getWorkId());
                    paper.determineLevel(null);
                    paper.setMarkByLeader(false);
                    paper.setSample(false);
                    paperRepo.save(paper);

                    List<MarkTaskLevel> markTasks = markTaskLevelRepo.findByPaperIdAndStage(paper.getId(), MarkStage.LEVEL);
                    for (MarkTaskLevel markTask : markTasks) {
                        markTask.setResult(null);
                        markTask.setLevel(null);
                        markTask.setSample(false);
                        markTaskLevelRepo.save(markTask);
                    }

                    // 记录日志
                    Long userId = ServletUtil.getUserId();
                    MarkUser markUser = markUserRepo.findOne(userId);
                    MarkLog markLog = new MarkLog(work.getId(), work.getName(), paper.getId(), paper.getSubject(), MarkStage.LEVEL, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), markUser.getRole(), MarkLogOperType.CANCEL_SAMPLE.getId(), MarkLogOperType.CANCEL_SAMPLE.getName(), paper.getLevel(), null, null, new Date());
                    markLogs.add(markLog);
                }
            }

            if (!markLogs.isEmpty()) {
                markLogRepo.save(markLogs);
            }

            return new ResponseEntity(true, HttpStatus.OK);
        }
        return null;
    }

    /**
     * 标记试卷
     *
     * @param paper 试卷id
     * @return
     */
    @RequestMapping(value = "{paper}/tag", method = RequestMethod.PATCH)
    @Transactional
    public Paper tagging(@PathVariable Paper paper) {
        boolean isTagged = paper.isTagged();
        paper.setTagged(!isTagged);
        paperRepo.save(paper);
        return paper;
    }

    /**
     * 该试卷的评卷记录
     *
     * @param paperId 试卷id
     * @param stage   评卷阶段
     * @return
     */
    @RequestMapping(value = "{paperId}/marktasks", method = RequestMethod.GET)
    public List<MarkTaskDTO> markTasks(@PathVariable Long paperId, @RequestParam MarkStage stage) {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            markTaskRoughLevelRepo.findByPaperId(paperId).forEach(o -> {
                markTaskDTOs.add(markTaskRoughLevelAssembler.toDTO(o));
            });
        } else if (MarkStage.LEVEL.equals(stage)) {
            markTaskLevelRepo.findByPaperId(paperId).forEach(o -> {
                markTaskDTOs.add(markTaskLevelAssembler.toDTO(o));
            });
        } else if (MarkStage.SCORE.equals(stage)) {
            markTaskScoreRepo.findByPaperId(paperId).forEach(o -> {
                markTaskDTOs.add(markTaskScoreAssembler.toDTO(o));
            });
        }
        markTaskDTOs.sort(Comparator.comparing(MarkTaskDTO::getMarker));
        return markTaskDTOs;
    }

    /**
     * 考生成绩查询
     *
     * @param workId      工作ID
     * @param questionId  考区ID
     * @param subject     科目
     * @param type        类型
     * @param number      号码
     * @param studentName 学生姓名
     */
    @GetMapping("/studentScore")
    public List<Map> markTasks(@RequestParam Long workId,
                               @RequestParam(required = false) Long questionId,
                               @RequestParam(required = false) Subject subject,
                               @RequestParam(required = false) String type,
                               @RequestParam(required = false) String number,
                               @RequestParam(required = false) String studentName) {
        Specification<Paper> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (!StringUtils.isEmpty(workId)) {
                predicates.add(builder.equal(root.get("workId"), workId));
            }
            if (!StringUtils.isEmpty(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }
            if (!StringUtils.isEmpty(subject)) {
                predicates.add(builder.equal(root.get("subject"), subject));
            }
            if (!StringUtils.isEmpty(type) && !StringUtils.isEmpty(number)) {
                if (Objects.equals(type, "examNumber")) {
                    predicates.add(builder.equal(root.get("examNumber"), number));
                } else if (Objects.equals(type, "paper")) {
                    predicates.add(builder.equal(root.get("secretNumber"), number));
                } else if (Objects.equals(type, "task")) {
                    List<MarkTaskRoughLevel> markRoughTasks = markTaskRoughLevelRepo.findByWorkIdAndSecretNumber(workId, number);
                    if (markRoughTasks.isEmpty()) {
                        List<MarkTaskLevel> markTasks = markTaskLevelRepo.findByWorkIdAndSecretNumber(workId, number);
                        if (markTasks == null || markTasks.isEmpty()) {
                            List<MarkTaskScore> markScoreTasks = markTaskScoreRepo.findByWorkIdAndSecretNumber(workId, number);
                            if (markScoreTasks == null || markScoreTasks.isEmpty()) {
                                predicates.add(builder.isNull(root.get("id")));
                            } else {
                                predicates.add(builder.equal(root.get("id"), markScoreTasks.get(0).getPaper().getId()));
                            }
                            predicates.add(builder.isNull(root.get("id")));
                        } else {
                            predicates.add(builder.equal(root.get("id"), markTasks.get(0).getPaper().getId()));
                        }
                    } else {
                        predicates.add(builder.equal(root.get("id"), markRoughTasks.get(0).getPaper().getId()));
                    }

                }
            }
            if (!StringUtils.isEmpty(studentName)) {
                predicates.add(builder.equal(root.get("studentName"), studentName));
            }
            return builder.and(predicates.toArray(new Predicate[0]));
        };

        List<Paper> papers = paperRepo.findAll(specification);
        //根据考生分组
        Map<String, List<Paper>> groupPaper = papers.stream().collect(Collectors.groupingBy(p -> p.getExamNumber() + ":" + p.getStudentName()));
        List<Map> mapList = new ArrayList<>();
        for (Map.Entry<String, List<Paper>> entry : groupPaper.entrySet()) {
            JSONObject object = new JSONObject();
            String key = entry.getKey();
            object.put("roughLevelEnable", ParamCache.levelConfigMap.get(String.valueOf(workId)).getRoughLevelEnable() == 1);
            object.put("examNumber", key.split(":")[0]);
            object.put("name", key.split(":")[1]);

            List<Paper> paperList = entry.getValue();
            double sumScore = paperList.stream().filter(m -> Objects.nonNull(m.getScore())).mapToDouble(Paper::getScore).sum();
            object.put("sumScore", Optional.of(sumScore).orElse(0D).intValue());
            List<JSONObject> paperDTOs = new ArrayList<>();
            for (Paper paper : paperList) {
                PaperDTO paperDTO = paperAssembler.toDTO(paper);
                JSONObject subObject = new JSONObject();
                subObject.put("secretNumber", paperDTO.getSecretNumber());
                subObject.put("level", paperDTO.getLevel());
                subObject.put("score", paperDTO.getScore());
                subObject.put("imgSrc", paperDTO.getImgSrc());
                subObject.put("thumbSrc", paperDTO.getImgSrc());
                subObject.put("sheetSrc", paperDTO.getSheetSrc());
                // 粗分档明细
                List<MarkTaskRoughLevel> taskRoughLevels = markTaskRoughLevelRepo.findByPaperIdAndStageNative(paper.getId(), MarkStage.ROUGH_LEVEL);
                subObject.put("taskRoughLevels", taskRoughLevels.isEmpty() ? null : taskRoughLevels);
                // 分档明细
                List<MarkTaskLevel> taskLevels = markTaskLevelRepo.findByPaperIdAndStageNative(paper.getId(), MarkStage.LEVEL);
                subObject.put("taskLevels", taskLevels.isEmpty() ? null : taskLevels);
                // 打分明细
                List<MarkTaskScore> taskScores = markTaskScoreRepo.findByPaperIdAndStageNative(paper.getId(), MarkStage.SCORE);
                subObject.put("taskScores", taskScores.isEmpty() ? null : taskScores);
                paperDTOs.add(subObject);
            }
            object.put("papers", paperDTOs);
            mapList.add(object);
        }
        return mapList;
    }

    /**
     * 上传试题列表
     *
     * @param workId 工作ID
     */
    @RequestMapping(value = "/{workId}/listUploadPaper", method = {RequestMethod.GET})
    public List<Map> uploadPaperFile(@PathVariable Long workId) {
        List<Object[]> questions = examQuestionRepo.findByWorkId(workId);
        List<MarkSubject> subjects = markSubjectRepo.findAllByWorkId(workId);
        List<Map> mapList = new ArrayList<>();
        for (Object[] question : questions) {
            Map map = new HashMap();
            map.put("questionId", question[0]);
            map.put("questionName", question[1]);
            List<Map> subjectList = new ArrayList<>();
            for (MarkSubject subject : subjects) {
                //科目信息
                Map subjectMap = new HashMap();
                subjectMap.put("subject", subject.getSubject().name());
                subjectMap.put("subjectName", subject.getName());
                subjectMap.put("enable", subject.isEnable());
                //上传路径，已有上传，取文件路径，没有为null
                String subjectUrl = null;
                String path = systemConfig.getLocalhostPath() + File.separator + "files" + File.separator + workId + File.separator + question[0] + File.separator + subject.getSubject().name();
                File pathFile = new File(path);
                if (pathFile.exists()) {
                    File[] file = pathFile.listFiles();
                    if (file.length > 0) {
                        subjectUrl = file[0].getPath();
                    }
                }
                subjectMap.put("url", Objects.isNull(subjectUrl) ? null : systemConfig.getFilesUrl() + File.separator + subjectUrl);
                subjectList.add(subjectMap);
            }
            map.put("subjects", subjectList);
            mapList.add(map);
        }
        return mapList;
    }


    /**
     * 上传试卷
     *
     * @param workId
     * @param questionId
     * @param subject
     * @param file
     * @param request
     * @param response
     * @throws Exception
     */
    @RequestMapping(value = "/{workId}/{questionId}/{subject}/uploadPaperFile", method = {RequestMethod.POST})
    public void uploadPaperFile(@PathVariable Long workId,
                                @PathVariable Long questionId,
                                @PathVariable Subject subject,
                                @RequestParam MultipartFile file,
                                HttpServletRequest request, HttpServletResponse response) throws Exception {
        if (Objects.isNull(file)) {
            throw new Exception("file文件不能为空");
        }
        int temp = file.getOriginalFilename().indexOf(".");
        String format = file.getOriginalFilename().substring(temp);
        if (Objects.nonNull(format) && !(format.equalsIgnoreCase(".jpg")
                || format.equalsIgnoreCase(".jpeg")
                || format.equalsIgnoreCase(".png")
                || format.equalsIgnoreCase(".pdf"))) {
            throw new Exception("file文件格式只能为jpg,jpeg,png,pdf");
        }
        InputStream inputStream = file.getInputStream();
        //保存
        String savePath = systemConfig.getLocalhostPath() + File.separator + "files" + File.separator + workId + File.separator + questionId + File.separator + subject.name();
        File fileSave = new File(savePath);
        if (!fileSave.exists()) {
            fileSave.mkdirs();
        }
        //删除原文件
        File[] oldFiles = fileSave.listFiles();
        if (oldFiles.length > 0) {
            for (File oldFile : oldFiles) {
                if (oldFile.exists()) {
                    oldFile.delete();
                }
            }
        }
        File outSheetFile = new File(savePath + File.separator + DigestUtils.md5Hex(workId + "-" + questionId + "-" + subject.name()) + format);
        OutputStream outputSheetStream = new FileOutputStream(outSheetFile);
        int index = 0;
        byte[] bytes = new byte[1024 * 8];
        while ((index = inputStream.read(bytes)) != -1) {
            outputSheetStream.write(bytes, 0, index);
        }
        inputStream.close();
        outputSheetStream.flush();
        outputSheetStream.close();

        String md5 = request.getHeader("md5");
        if (Objects.isNull(md5)) {
            throw new Exception("request的md5为空");
        }
        FileInputStream in = new FileInputStream(outSheetFile);
        String sliceMD5 = DigestUtils.md5Hex(in);
        if (!md5.equalsIgnoreCase(sliceMD5)) {
            throw new Exception("图片md5值不一致");
        }
        in.close();
    }

    /**
     * 查看试题
     *
     * @param workId
     * @throws Exception
     */
    @RequestMapping(value = "/{workId}/{subject}/viewUploadPaper", method = {RequestMethod.GET})
    public List<Map> viewPaperFile(@PathVariable Long workId, @PathVariable Subject subject) {
        List<ExamQuestion> questions = examQuestionRepo.findByWorkIdAndSubject(workId, subject);
        List<Map> mapList = new ArrayList<>();
        for (ExamQuestion question : questions) {
            Map map = new HashMap();
            map.put("questionId", question.getAreaCode());
            map.put("questionName", question.getAreaName());
            //科目信息
            MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
            map.put("subject", markSubject.getSubject().name());
            map.put("subjectName", markSubject.getName());
            //上传路径，已有上传，取文件路径，没有为null
            String subjectUrl = null;
            String path = systemConfig.getLocalhostPath() + File.separator + "files" + File.separator + workId + File.separator + question.getAreaCode() + File.separator + markSubject.getSubject().name();
            File pathFile = new File(path);
            if (pathFile.exists()) {
                File[] file = pathFile.listFiles();
                if (file.length > 0) {
                    subjectUrl = file[0].getPath();
                }
            }
            map.put("url", systemConfig.getFilesUrl() + File.separator + subjectUrl);
            mapList.add(map);
        }
        return mapList;
    }

    /**
     * 根据试卷密号查询试卷
     *
     * @param workId       工作ID
     * @param subject      科目
     * @param secretNumber 试卷密号
     */
    @RequestMapping(value = "/get_by_secret_number", method = RequestMethod.GET)
    public List<Paper> getBySecretNumber(@RequestParam Long workId,
                                         @RequestParam Subject subject,
                                         @RequestParam String secretNumber) {
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        return paperRepo.findByWorkIdAndSubjectAndSecretNumberAndTest(workId, subject, secretNumber, markSubject.getTest());
    }

    /**
     * 试卷管理-标记试卷
     *
     * @param paperId 试卷ID
     * @param isMark  标记状态
     */
    @PostMapping("/mark_paper")
    @Transactional
    public Object markPaper(@RequestParam Long paperId, @RequestParam boolean isMark, @RequestParam Role role) {
        if (Role.ADMIN.equals(role)) {
            paperRepo.updatePaperAdminMarkById(paperId, isMark);
        } else if (Role.MARK_LEADER.equals(role)) {
            paperRepo.updatePaperMarkById(paperId, isMark);
        }
        return true;
    }

    /**
     * 根据工作、科目、考号查询原卷
     *
     * @param workId     工作id
     * @param subject    科目
     * @param examNumber 考号
     */
    @RequestMapping(value = "{workId}/{subject}/{examNumber}", method = RequestMethod.GET)
    public PaperDTO get(@PathVariable Long workId, @PathVariable Subject subject, @PathVariable String examNumber) {
        Paper paper = paperRepo.findByWorkIdAndSubjectAndExamNumber(workId, subject, examNumber);
        return paperAssembler.toDTO(paper);
    }
}
