package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.utils.RandomUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 评卷任务分配
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class AssignTaskService {

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private MarkerGroupStudentRepo markerGroupStudentRepo;

    @Autowired
    private TaskPublishSettingRepo taskPublishSettingRepo;

    @Autowired
    RandomUtil randomUtil;

    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    MarkUserRepo markUserRepo;

    /**
     * 细分档任务发布
     *
     * @param groupPaperMap 各分组待发试卷集合
     * @param markerGroups  所有分组
     * @param markSubject   科目
     */
    @Transactional
    public void assignForGroupingRoughLevel(Map<Long, List<Paper>> groupPaperMap, List<MarkerGroup> markerGroups, MarkSubject markSubject) throws Exception {
        markerGroups = markerGroups.stream().filter(m -> m.getMarkers() != null && !m.getMarkers().isEmpty()).collect(Collectors.toList());
        long currentTime = System.currentTimeMillis();
        List<MarkTaskRoughLevel> markTaskLevelList = new ArrayList<>();
        List<Paper> paperList = new ArrayList<>();
        // 解析粗档位
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        List<RoughLevel> roughLevels = DetermineRoughLevelService.parseRoughLevels(levels);

        Map<Long, MarkerGroup> groupMap = markerGroups.stream().collect(Collectors.toMap(MarkerGroup::getId, Function.identity()));
        for (Map.Entry<Long, List<Paper>> entry : groupPaperMap.entrySet()) {
            Long groupId = entry.getKey();
            List<Paper> papers = entry.getValue();
            papers.forEach(paper -> {
                int markerCount = 0;
                Set<MarkUser> markUsers = groupMap.get(groupId).getMarkers();
                markUsers = markUsers.stream().filter(m -> Role.MARKER.equals(m.getRole())).collect(Collectors.toSet());
                for (MarkUser marker : markUsers) {
                    markerCount++;
                    Long random = null;
                    try {
                        random = randomUtil.randomRoughLevel(marker.getWorkId());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    MarkTaskRoughLevel markTask;
                    if (paper.isRoughSample() && paper.getRoughLevel() != null) {
                        Optional<RoughLevel> optional = roughLevels.stream().filter(m -> m.getRoughCode().equals(paper.getRoughLevel())).findFirst();
                        if (!optional.isPresent()) {
                            throw new RuntimeException("未找到档位【" + paper.getRoughLevel() + "】");
                        }
                        RoughLevel level = optional.get();
                        markTask = new MarkTaskRoughLevel(marker, paper, level, markSubject.getStage(), random, markSubject.getTest());

                        // 初始化排序值
                        if (markerCount == markUsers.size()) {
                            paper.setSortNum(level.getWeight() * markerCount);
                        }
                    } else {
                        markTask = new MarkTaskRoughLevel(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                    }
                    markTask.setBatchNo(currentTime);

                    markTaskLevelList.add(markTask);
                }
                paper.setRoughBatchNo(currentTime);
                paperList.add(paper);
            });
        }

        List<MarkTaskRoughLevel> data = new ArrayList<>();
        //5000条提交一次
        for (MarkTaskRoughLevel markTask : markTaskLevelList) {
            if (data.size() == 2000) {
                markTaskRoughLevelRepo.save(data);
                data.clear();
            }
            data.add(markTask);
        }
        //将剩下的数据也导入
        if (!data.isEmpty()) {
            markTaskRoughLevelRepo.save(data);
        }

        paperRepo.save(paperList);
    }

    /**
     * 细分档任务发布
     *
     * @param groupPaperMap 各分组待发试卷集合
     * @param markerGroups  所有分组
     * @param markSubject   科目
     */
    @Transactional
    public void assignForGroupingLevel(Map<Long, List<Paper>> groupPaperMap, List<MarkerGroup> markerGroups, MarkSubject markSubject) throws Exception {
        markerGroups = markerGroups.stream().filter(m -> m.getMarkers() != null && !m.getMarkers().isEmpty()).collect(Collectors.toList());
        long currentTime = System.currentTimeMillis();
        List<MarkTaskLevel> markTaskLevelList = new ArrayList<>();
        List<Paper> paperList = new ArrayList<>();
        List<Object[]> markTasks = markTaskLevelRepo.findAllByWorkId(markSubject.getWorkId());
        Map<Long, Object> randomMap = new HashMap<>();
        markTasks.forEach(m -> randomMap.put(Long.valueOf(m[1].toString()), m[0]));
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());

        Map<Long, MarkerGroup> groupMap = markerGroups.stream().collect(Collectors.toMap(MarkerGroup::getId, Function.identity()));
        for (Map.Entry<Long, List<Paper>> entry : groupPaperMap.entrySet()) {
            Long groupId = entry.getKey();
            List<Paper> papers = entry.getValue();

            for (Paper paper : papers) {
                int markerCount = 0;
                Set<MarkUser> markUsers = groupMap.get(groupId).getMarkers();
                markUsers = markUsers.stream().filter(m -> Role.MARKER.equals(m.getRole())).collect(Collectors.toSet());
                for (MarkUser marker : markUsers) {
                    markerCount++;
                    Long random = randomUtil.randomLevel(marker.getWorkId());
                    MarkTaskLevel markTask;
                    if (paper.isSample() && paper.getLevel() != null) {
                        Level level = levels.stream().filter(m -> m.getCode().equals(paper.getLevel())).findFirst().orElseGet(null);
                        markTask = new MarkTaskLevel(marker, paper, level, markSubject.getStage(), random, markSubject.getTest());

                        // 初始化排序值
                        if (markerCount == markUsers.size()) {
                            paper.setSortNum(level.getWeight() * markerCount);
                        }
                    } else {
                        markTask = new MarkTaskLevel(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                    }
                    markTask.setBatchNo(currentTime);

                    markTaskLevelList.add(markTask);
                    randomMap.put(random, markTask);
                }
                paper.setBatchNo(currentTime);
                paperList.add(paper);
            }
        }


        List<MarkTaskLevel> data = new ArrayList<>();
        //5000条提交一次
        for (MarkTaskLevel markTask : markTaskLevelList) {
            if (data.size() == 2000) {
                markTaskLevelRepo.save(data);
                data.clear();
            }
            data.add(markTask);
        }
        //将剩下的数据也导入
        if (!data.isEmpty()) {
            markTaskLevelRepo.save(data);
        }
        paperRepo.save(paperList);
    }

    /**
     * 打分任务发布
     * <p>
     * 分组任务模式，将该科目的评卷员分成N组，将该科目试卷平均分给这些组，每组内的评卷员每人将这些试卷打分
     * 比如：6个评卷员分2组，有1000份试卷，每组评500份，每个评卷员评500份
     *
     * @param papers       试卷
     * @param markerGroups 分组
     * @param markSubject  科目
     * @param taskList     打分任务
     */
    @Transactional
    public void assignForGroupingScore(List<Paper> papers, List<MarkerGroup> markerGroups, MarkSubject markSubject, List<TaskPublishSetting> taskList) throws Exception {
        markerGroups = markerGroups.stream().filter(m -> m.getMarkers() != null && !m.getMarkers().isEmpty()).collect(Collectors.toList());
        int sum = markerGroups.stream().mapToInt(m -> m.getMarkers().size()).sum();
        int groupSize = markerGroups.size();
        int idx = 0;
        long currentTime = System.currentTimeMillis();
        Iterator<Paper> iterator = papers.iterator();
        List<MarkTaskScore> markTaskScoreList = new ArrayList<>(papers.size() * sum);
        List<Paper> paperList = new ArrayList<>(papers.size());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));
        //定序序号记录
        Map<String, Integer> map = new HashMap();
        Map<String, Integer> seqNewMap = new HashMap();
        while (iterator.hasNext()) {
            if (idx >= groupSize) {
                idx = 0;
            }
            Paper paper = iterator.next();
            String serialNumber = null;
            Integer displayNumber = null;
            Integer seqNewNumber = null;
            //是否定序
            int seq = 0;
            long sortRule = taskList.stream().filter(m -> Objects.equals(m.getCode(), paper.getLevel()) && m.getSortRule() == 1).count();
            if (sortRule != 0) {
                if (map.containsKey(paper.getLevel() + idx)) {
                    seq = map.get(paper.getLevel() + idx);
                }
                ++seq;
                serialNumber = paper.getLevel() + seq;
                //是否显示序号
                long display = taskList.stream().filter(m -> Objects.equals(m.getCode(), paper.getLevel()) && m.getDisplayNumber() == 1).count();
                displayNumber = (int) display;
                //设置新值
                map.put(paper.getLevel() + idx, seq);

                int seqNew = 0;
                if (seqNewMap.containsKey(paper.getLevel())) {
                    seqNew = seqNewMap.get(paper.getLevel());
                }
                seqNewNumber = ++seqNew;
                seqNewMap.put(paper.getLevel(), seqNew);
            }
            Set<MarkUser> markUsers = markerGroups.get(idx).getMarkers();
            markUsers = markUsers.stream().filter(m -> Role.MARKER.equals(m.getRole())).collect(Collectors.toSet());
            for (MarkUser marker : markUsers) {
                Long random = randomUtil.randomScore(marker.getWorkId());
                MarkTaskScore markTask = new MarkTaskScore(marker, paper, markSubject.getStage(), random, markSubject.getTest());
                String levelValue = String.valueOf(levelMap.get(paper.getLevel()).getLevelValue() + 1);
                if (Objects.nonNull(serialNumber)) {
                    //和随机数位数保持一致（8位）
                    String randomSeqNew = String.format("%08d", seqNewNumber);
                    markTask.setRandomSeqNew(Long.valueOf(levelValue + randomSeqNew));
                } else {
                    markTask.setRandomSeqNew(Long.valueOf(levelValue + markTask.getRandomSeqNew()));
                }
                markTask.setBatchNo(currentTime);
                markTask.setSerialNumber(serialNumber);
                markTask.setDisplayNumber(displayNumber);
                markTask.setLevel(paper.getLevel());

                markTaskScoreList.add(markTask);
            }
            paper.setScoreBatchNo(currentTime);
            paperList.add(paper);
            idx++;
        }

        List<MarkTaskScore> data = new ArrayList<>();
        //5000条提交一次
        for (MarkTaskScore markTask : markTaskScoreList) {
            if (data.size() == 2000) {
                markTaskScoreRepo.save(data);
                data.clear();
            }
            data.add(markTask);
        }
        //将剩下的数据也导入
        if (!data.isEmpty()) {
            markTaskScoreRepo.save(data);
        }
        paperRepo.save(paperList);
    }

    /**
     * 生成打分汇总数据
     *
     * @param workId  工作ID
     * @param subject 科目
     */
    public void initTaskPublishData(Long workId, String subject) {
        StringBuffer sql = new StringBuffer();
        sql.append("  SELECT                                                                    ");
        sql.append("          p.work_id workId,                                                  ");
        sql.append("          p.question_id questionId,                                          ");
        sql.append("          p.subject,                                                         ");
        sql.append("          p.level,                                                           ");
        sql.append("              COUNT(1) AS totalCount                                         ");
        sql.append("      FROM                                                                   ");
        sql.append("          paper p                                                            ");
        sql.append("      WHERE                                                                  ");
        sql.append("          p.work_id = '" + workId + "' AND p.subject = '" + subject + "'      ");
        sql.append("              AND p.is_test = 0                                              ");
        sql.append("              AND p.level IS NOT NULL                                        ");
        sql.append("              AND p.is_missing = FALSE                                       ");
        sql.append("              AND p.batch_no is not null                                     ");
        sql.append("      GROUP BY p.work_id , p.question_id, p.subject , p.level                ");
        sql.append("  ORDER BY p.level                                                            ");
        List list = sqlUtil.execSqlForMap(sql.toString());
        List<TaskPublishSetting> taskPublishSettings = new ArrayList<>(list.size());
        for (Object o : list) {
            Map map = (Map) o;
            TaskPublishSetting taskPublishSetting = new TaskPublishSetting();
            taskPublishSetting.setWorkId(Long.valueOf(map.get("workId").toString()));
            taskPublishSetting.setQuestionId(Long.valueOf(map.get("questionId").toString()));
            taskPublishSetting.setSubject((String) map.get("subject"));
            taskPublishSetting.setCode((String) map.get("level"));
            taskPublishSetting.setTotalCount(Integer.valueOf(map.get("totalCount").toString()));
            taskPublishSetting.setSuccessCount(0);
            taskPublishSetting.setWaitCount(Integer.valueOf(map.get("totalCount").toString()));
            taskPublishSetting.setSortRule(0);
            taskPublishSetting.setPublishCount(0);
            taskPublishSettings.add(taskPublishSetting);
        }
        taskPublishSettingRepo.save(taskPublishSettings);
    }

    /**
     * 根据分组,分配任务
     *
     * @param allPapers    所有试卷
     * @param markSubject  科目
     * @param markerGroups 分组
     * @param questionIds  考区
     * @param taskCount    分档数量
     */
    @Transactional
    public Map<Long, List<Paper>> assignGroupTasks(List<Paper> allPapers, MarkSubject markSubject, List<MarkerGroup> markerGroups, List<Long> questionIds, int taskCount) {
        // 按分组循环
        Map<Long, List<Paper>> groupPaperMap = new HashMap<>();
        if (markSubject.getTest() != 1) {
            // 所有未分档任务
            List<Paper> papersList = new ArrayList<>();
            // 所有分组导入的考生名单
            List<MarkerGroupStudent> markerGroupStudentList = markerGroupStudentRepo.findByWorkIdAndSubjectAndStageAndUsed(markSubject.getWorkId(), markSubject.getSubject(), markSubject.getStage(), false);
            if (questionIds.isEmpty()) {
                if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                    papersList = allPapers.stream().filter(m -> m.getRoughBatchNo() == null).collect(Collectors.toList());
                } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                    papersList = allPapers.stream().filter(m -> m.getBatchNo() == null).collect(Collectors.toList());
                }
            } else {
                // 按考区过滤数据
                if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                    papersList = allPapers.stream().filter(m -> questionIds.contains(m.getQuestionId()) && m.getRoughBatchNo() == null).collect(Collectors.toList());
                } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                    papersList = allPapers.stream().filter(m -> questionIds.contains(m.getQuestionId()) && m.getBatchNo() == null).collect(Collectors.toList());
                }

            }

            Map<Long, List<MarkerGroupStudent>> markerGroupStudentMap = markerGroupStudentList.stream().collect(Collectors.groupingBy(MarkerGroupStudent::getGroupId));

            for (MarkerGroup markerGroup : markerGroups) {
                List<Paper> groupPaperList = new ArrayList<>();
                // 默认为false
                if (markerGroupStudentMap != null && markerGroupStudentMap.containsKey(markerGroup.getId())) {
                    List<MarkerGroupStudent> groupStudents = markerGroupStudentMap.get(markerGroup.getId());
                    if (groupStudents != null && !groupStudents.isEmpty()) {
                        List<Long> paperIds = groupStudents.stream().map(MarkerGroupStudent::getPaperId).collect(Collectors.toList());
                        groupPaperList = papersList.stream().filter(m -> paperIds.contains(m.getId())).collect(Collectors.toList());
                        if (paperIds.size() != groupPaperList.size()) {
                            throw new RuntimeException("任务数据有误");
                        }
                    }
                }
                groupPaperMap.put(markerGroup.getId(), groupPaperList);
            }
            // 根据分组导入名单进行任务分配
            randomTaskList(papersList, groupPaperMap, taskCount, markSubject);

            //分组考生名单更新为已使用
            markerGroupStudentList.forEach(m -> m.setUsed(true));
            markerGroupStudentRepo.save(markerGroupStudentList);
        } else {
            for (MarkerGroup markerGroup : markerGroups) {
                List<Paper> groupPaperList = new ArrayList<>();
                groupPaperMap.put(markerGroup.getId(), groupPaperList);
            }
            randomTaskList(allPapers, groupPaperMap, allPapers.size(), markSubject);
        }

        return groupPaperMap;
    }

    /**
     * 随机发任务
     *
     * @param papersList    试卷集合
     * @param groupPaperMap 分组考生名单
     * @param taskCount     任务数量
     */
    private void randomTaskList(List<Paper> papersList, Map<Long, List<Paper>> groupPaperMap, int taskCount, MarkSubject markSubject) {

        List<Paper> usedPaperList = new ArrayList<>();
        List<Long> nullValueKeys = new ArrayList<>();
        for (Map.Entry<Long, List<Paper>> entry : groupPaperMap.entrySet()) {
            List<Paper> value = entry.getValue();
            if (!value.isEmpty()) {
                usedPaperList.addAll(value);
                continue;
            }
            // 未导入考生名单的分组
            nullValueKeys.add(entry.getKey());
        }

        //
        if (!nullValueKeys.isEmpty()) {
            // 剔除已导入的数据
            papersList.removeAll(usedPaperList);
            int leftCount = taskCount - usedPaperList.size();
            if (leftCount < 0) {
                throw new RuntimeException("任务数量有误");
            }
            // 随机取
            List<Paper> list = new ArrayList<>();
            if (leftCount == papersList.size()) {
                list.addAll(papersList);
            } else {
                // 开启粗分档，并且当前为细分档时，系统默认按档位从高到低取任务，评委端显示也是从高到低
                if (MarkStage.LEVEL.equals(markSubject.getStage()) && ParamCache.levelConfigMap.get(String.valueOf(markSubject.getWorkId())).getRoughLevelEnable() == 1) {
                    papersList.sort(Comparator.comparing(Paper::getRoughLevel));
                    list = papersList.subList(0, leftCount);
                } else {
                    Random random = new Random();
                    for (int i = 0; i < leftCount; i++) {
                        int intRandom = random.nextInt(papersList.size() - 1);
                        list.add(papersList.get(intRandom));
                        papersList.remove(intRandom);
                    }
                }
            }

            // 给每个组发任务
            int idx = 0;
            for (Paper paper : list) {
                if (idx >= nullValueKeys.size()) {
                    idx = 0;
                }
                groupPaperMap.get(nullValueKeys.get(idx)).add(paper);
                idx++;
            }
        }
    }
}
