package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevleProgressDTO;
import cn.com.qmth.stmms.ms.marking.dto.MarkQuestionStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 分组服务
 * Created by zhengmin on 2016/9/22.
 */
@Service
public class GroupingService {

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    PaperRepo paperRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    private MarkerGroupRepo markerGroupRepo;

    @Autowired
    private MarkerGroupLeaderService markerGroupLeaderService;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;

    /**
     * 自动分组
     *
     * @param count   组数
     * @param markers 评卷员列表
     * @return
     */
    public List<MarkerGroup> autoGrouping(int count, Iterable<MarkUser> markers) {
        List<MarkerGroup> groups = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            groups.add(new MarkerGroup());
        }
        Iterator<MarkUser> iterator = markers.iterator();
        int idx = 0;
        while (iterator.hasNext()) {
            if (idx >= count) {
                idx = 0;
            }
            MarkUser marker = iterator.next();
            groups.get(idx).setWorkId(marker.getWorkId());
            groups.get(idx).setSubject(marker.getSubject());
            groups.get(idx).addMarker(marker);
            idx++;
        }
        markerGroupRepo.save(groups);
        return groups;
    }

    /**
     * 总体进度
     *
     * @param workId
     * @param objectMap
     */
    public void getTotalProgress(Long workId, MarkSubject markSubject, Map<String, Object> objectMap) {
        int successCount = 0;
        if (markSubject.getStage().equals(MarkStage.ROUGH_LEVEL)) {
            successCount = paperRepo.countByWorkIdAndSubjectAndRoughLevelNotNullAndRoughBatchNoNotNullAndTest(workId, markSubject.getSubject(), markSubject.getTest());
        } else if (markSubject.getStage().equals(MarkStage.LEVEL)) {
            successCount = paperRepo.countByWorkIdAndSubjectAndLevelNotNullAndBatchNoNotNullAndTest(workId, markSubject.getSubject(), markSubject.getTest());
        } else if (markSubject.getStage().equals(MarkStage.SCORE)) {
            successCount = paperRepo.countByWorkIdAndSubjectAndScoreNotNullAndTest(workId, markSubject.getSubject(), markSubject.getTest());
        }
        //所有任务数(查询所有有效试卷)
        int totalCount = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndTest(workId, markSubject.getSubject(), markSubject.getTest());

        BigDecimal waitTotal = BigDecimal.valueOf(totalCount).subtract(BigDecimal.valueOf(successCount));
        //进度
        BigDecimal progress = BigDecimal.ZERO.compareTo(BigDecimal.valueOf(totalCount)) == 0 ? BigDecimal.ZERO : BigDecimal.valueOf(successCount).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(totalCount), 2, BigDecimal.ROUND_HALF_UP);

        LevleProgressDTO progressDTO = new LevleProgressDTO();
        progressDTO.setSuccessCount(successCount);
        progressDTO.setTotalCount(totalCount);
        progressDTO.setWaitCount(waitTotal.intValue());
        progressDTO.setProgress(progress.doubleValue());
        objectMap.put("totalProgress", progressDTO);
    }

    /**
     * 考区进度
     *
     * @param workId      工作ID
     * @param markSubject 科目
     * @param objectMap   结果map
     */
    public void getAreaProgress(Long workId, MarkSubject markSubject, Map<String, Object> objectMap) {
        List<Object[]> areas = null;
        if (markSubject.getStage().equals(MarkStage.ROUGH_LEVEL)) {
            areas = markTaskRoughLevelRepo.listGroupByAreaName(workId, markSubject.getSubject().name(), markSubject.getTest());
        } else if (markSubject.getStage().equals(MarkStage.LEVEL)) {
            areas = markTaskLevelRepo.listGroupByAreaName(workId, markSubject.getSubject().name(), markSubject.getTest());
        } else if (markSubject.getStage().name().equals(MarkStage.SCORE.name())) {
            areas = markTaskScoreRepo.listGroupByAreaName(workId, markSubject.getSubject().name());
        }
        if (areas != null) {
            List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
            for (Object[] objects : areas) {
                QuestionStatDTO dto = questionStatAssembler.toProgressDTO(objects);
                questionStatDTOs.add(dto);
            }
            objectMap.put("areaProgress", questionStatDTOs);
        }
    }

    /**
     * 各评卷员进度
     *
     * @param workId            工作ID
     * @param markSubject       科目
     * @param userMarkLeaders   科组长
     * @param markerId          评卷员ID
     * @param isTotalMarkLeader 是否大科组长
     * @param objectMap         返回对象
     */
    public void getGroupMarkersProgress(Long workId, MarkSubject markSubject, List<Long> userMarkLeaders, Long markerId, boolean isTotalMarkLeader, Map<String, Object> objectMap) {
        List<Map<String, Object>> groupMarkers = new ArrayList<>();
        List<Map<String, List<Long>>> groupLeaderAndMarkers = markerGroupLeaderService.listGroupLeaderAndMarkersByWorkIdAndSubjectAndStage(workId, markSubject.getSubject(), markSubject.getStage());
        if (!isTotalMarkLeader && Objects.nonNull(markerId)) {
            groupLeaderAndMarkers = groupLeaderAndMarkers.stream().filter(m -> m.get("markLeaders").contains(markerId)).collect(Collectors.toList());
        }
        if (!CollectionUtils.isEmpty(groupLeaderAndMarkers)) {
            for (Map<String, List<Long>> groupLeaderAndMarker : groupLeaderAndMarkers) {
                Map<String, Object> tempMap = new HashMap<>();
                List<Long> markerLeaders = groupLeaderAndMarker.get("markLeaders");
                List<Long> markers = groupLeaderAndMarker.get("markers");

                List<Long> paperIds = markerGroupLeaderService.listPaperIdsByWorkIdAndSubjectAndStage(workId, markSubject.getSubject(), markSubject.getStage(), markers);
                //仲裁数量
                List<Paper> arbitratedPaperList = paperRepo.findByWorkIdAndSubjectAndIsArbitratedTrue(workId, markSubject.getSubject());
                long arbitrate = CollectionUtils.isEmpty(arbitratedPaperList) || CollectionUtils.isEmpty(paperIds) ? 0L : arbitratedPaperList.stream().filter(m -> paperIds.contains(m.getId())).count();
                if (!CollectionUtils.isEmpty(markerLeaders)) {
                    userMarkLeaders.addAll(markerLeaders);
                    List<Map<String, Object>> markerLeaderList = new ArrayList<>();
                    for (Long markerLeaderId : markerLeaders) {
                        Map<String, Object> map = new HashMap();
                        MarkUser markUser = markUserRepo.findOne(markerLeaderId);
                        map.put("loginName", markUser.getLoginName());
                        map.put("name", markUser.getName());
                        map.put("arbitrated", arbitrate);
                        markerLeaderList.add(map);
                    }
                    tempMap.put("kzzProgress", markerLeaderList);
                }

                // 评卷员
                //各评卷员评卷进度
                List<Object[]> qStats = null;
                Long batchNo;
                if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                    if (ParamCache.roughLevelConfigMap.get(String.valueOf(workId)).getLevelShowAllPaper() == 1) {
                        qStats = markTaskRoughLevelRepo.listGroupByQuestionAndMarkerId(workId, markSubject.getSubject().name(), markSubject.getStage().ordinal(), markSubject.getTest(), markers);
                    } else {
                        batchNo = paperRepo.findMaxRoughBatchNoByWorkIdAndSubject(workId, markSubject.getSubject());
                        qStats = markTaskRoughLevelRepo.listGroupByQuestionAndBatchNoAndMarkerId(workId, markSubject.getSubject().name(), markSubject.getStage().ordinal(), markSubject.getTest(), batchNo, markers);
                    }
                } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                    if (ParamCache.levelConfigMap.get(String.valueOf(workId)).getLevelShowAllPaper() == 1) {
                        qStats = markTaskLevelRepo.listGroupByQuestionAndMarkerId(workId, markSubject.getSubject().name(), markSubject.getStage().ordinal(), markSubject.getTest(), markers);
                    } else {
                        batchNo = paperRepo.findMaxBatchNoByWorkIdAndSubject(workId, markSubject.getSubject());
                        qStats = markTaskLevelRepo.listGroupByQuestionAndBatchNoAndMarkerId(workId, markSubject.getSubject().name(), markSubject.getStage().ordinal(), markSubject.getTest(), batchNo, markers);
                    }
                } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
                    if (ParamCache.scoreConfigMap.get(String.valueOf(workId)).getScoreShowAllPaper() == 1) {
                        qStats = markTaskScoreRepo.listGroupByQuestionAndMakerId(workId, markSubject.getSubject().name(), markSubject.getStage().ordinal(), markers);
                    } else {
                        batchNo = paperRepo.findMaxScoreBatchNoByWorkIdAndSubject(workId, markSubject.getSubject());
                        qStats = markTaskScoreRepo.listGroupByQuestionAndScoreBatchNoAndMakerId(workId, markSubject.getSubject().name(), markSubject.getStage().ordinal(), batchNo, markers);
                    }
                }
                if (qStats != null) {
                    List<MarkQuestionStatDTO> questionStatDTOs = new ArrayList<>();
                    for (Object[] objects : qStats) {
                        MarkQuestionStatDTO dto = questionStatAssembler.toMarkProgressDTO(objects, workId, markSubject.getSubject(), markSubject.getStage());
                        questionStatDTOs.add(dto);
                    }
                    tempMap.put("markerProgress", questionStatDTOs);
                }
                groupMarkers.add(tempMap);
            }
        }
        objectMap.put("groupMarkers", groupMarkers);
    }

    /**
     * 总科组长进度
     *
     * @param workId          工作ID
     * @param markSubject     科目
     * @param userMarkLeaders 乘组长
     * @param objectMap       返回对象
     */
    public void getMakerLeaderProgress(Long workId, MarkSubject markSubject, List<Long> userMarkLeaders, Map<String, Object> objectMap) {
        List<MarkUser> markUsers = markUserRepo.findByWorkIdAndSubjectAndRoleAndIdNotIn(workId, markSubject.getSubject(), Role.MARK_LEADER, userMarkLeaders);
        //仲裁数量
        int arbitrate = paperRepo.countByWorkIdAndSubjectAndIsArbitratedTrue(workId, markSubject.getSubject());
        if (markUsers != null && !markUsers.isEmpty()) {
            List<Map<String, Object>> mapList = new ArrayList<>();
            for (MarkUser markUser : markUsers) {
                Map<String, Object> map = new HashMap();
                map.put("loginName", markUser.getLoginName());
                map.put("name", markUser.getName());
                map.put("arbitrated", arbitrate);
                mapList.add(map);
            }
            objectMap.put("kzz", mapList);
        }
    }
}
