package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkLogRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Optional;

/**
 * 日志Service
 */
@Service
public class MarkLogService {

    @Resource
    private MarkUserRepo markUserRepo;

    @Resource
    private WorkRepo workRepo;

    @Resource
    private MarkLogRepo markLogRepo;

    /**
     * 粗分档正评、回评、档位打回回评日志
     *
     * @param markTask  任务
     * @param levelCode 档位
     */
    public MarkLog markingRoughLevelLog(MarkTaskRoughLevel markTask, String levelCode) {
        Work work = workRepo.findOne(markTask.getWorkId());
        Paper paper = markTask.getPaper();
        MarkLogOperType markLogOperType;
        String operDateBefore = "-";
        String operDateAfter;
        MarkStage stage = MarkStage.ROUGH_LEVEL;
        // 正评或者打回重评
        if (markTask.getResult() == null) {
            // 打回重评
            if (markTask.isRejected()) {
                // 细分档阶段改档打档
                if (paper.isShift()) {
                    stage = MarkStage.LEVEL;
                    markLogOperType = MarkLogOperType.CHANGE_LEVEL;
                } else {
                    markLogOperType = MarkLogOperType.CALLBACK_ONCE_LEVEL;
                }
                operDateBefore = markTask.getOriginLevel();
                operDateAfter = levelCode;
            }
            // 正评
            else {
                markLogOperType = MarkLogOperType.LEVEL;
                operDateAfter = levelCode;
            }
        }
        // 回评
        else {
            markLogOperType = MarkLogOperType.ONCE_LEVEl;
            operDateBefore = markTask.getResult();
            operDateAfter = levelCode;
        }
        return new MarkLog(markTask.getWorkId(), work.getName(), paper.getId(), markTask.getSubject(), stage, markTask.getTest(), paper.getExamNumber(), paper.getStudentName(), markTask.getMarkerId(), markTask.getMarkerName(), Role.MARKER, markLogOperType.getId(),markLogOperType.getName(), Optional.ofNullable(operDateBefore).orElse("-"), Optional.ofNullable(operDateAfter).orElse("-"), null, new Date());
    }

    /**
     * 细分档正评、回评、档位打回回评日志
     *
     * @param markTask  任务
     * @param levelCode 档位
     */
    public MarkLog markingLevelLog(MarkTaskLevel markTask, String levelCode) {
        Work work = workRepo.findOne(markTask.getWorkId());
        Paper paper = markTask.getPaper();
        MarkLogOperType markLogOperType;
        String operDateBefore;
        String operDateAfter;
        MarkStage stage = MarkStage.LEVEL;
        if (markTask.getResult() == null) {
            // 打回重评
            if (markTask.isRejected()) {
                // 打分阶段的改档打档操作
                if (paper.isShift()) {
                    stage = MarkStage.SCORE;
                    markLogOperType = MarkLogOperType.CHANGE_LEVEL;
                } else {
                    markLogOperType = MarkLogOperType.CALLBACK_ONCE_LEVEL;
                }
            }
            // 正评
            else {
                // 细分档阶段的改档打分操作
                if (!paper.isShift() && paper.isShiftScore()) {
                    markLogOperType = MarkLogOperType.CHANGE_SCORE;
                } else {
                    markLogOperType = MarkLogOperType.LEVEL;
                }
            }
        } else {
            markLogOperType = MarkLogOperType.ONCE_LEVEl;
        }
        operDateBefore = markTask.getOriginLevel();
        operDateAfter = levelCode;
        return new MarkLog(markTask.getWorkId(), work.getName(), paper.getId(), markTask.getSubject(), stage, markTask.getTest(), paper.getExamNumber(), paper.getStudentName(), markTask.getMarkerId(), markTask.getMarkerName(), Role.MARKER, markLogOperType.getId(),markLogOperType.getName(), Optional.ofNullable(operDateBefore).orElse("-"), Optional.ofNullable(operDateAfter).orElse("-"), null, new Date());
    }

    /**
     * 打分、回评打分、档位打回回评日志
     *
     * @param markTask 任务
     * @param score    分数
     */
    public MarkLog markingScoreLog(MarkTaskScore markTask, int score) {
        Work work = workRepo.findOne(markTask.getWorkId());
        Paper paper = markTask.getPaper();
        MarkLogOperType markLogOperType;
        String operDateBefore = "-";
        String operDateAfter;
        if (markTask.getResult() == null) {
            // 改档打分
            if (!paper.isShift() && paper.isShiftScore()) {
                markLogOperType = MarkLogOperType.CHANGE_SCORE;
                operDateBefore = markTask.getOriginLevel();
                operDateAfter = String.valueOf(score);
            }
            // 正评
            else {
                markLogOperType = MarkLogOperType.SCORE;
                operDateAfter = String.valueOf(score);
            }
        } else {
            markLogOperType = MarkLogOperType.ONCE_SCORE;
            operDateBefore = markTask.getResult();
            operDateAfter = String.valueOf(score);
        }
        return new MarkLog(markTask.getWorkId(), work.getName(), paper.getId(), markTask.getSubject(), MarkStage.SCORE, markTask.getTest(), paper.getExamNumber(), paper.getStudentName(), markTask.getMarkerId(), markTask.getMarkerName(), Role.MARKER, markLogOperType.getId(),markLogOperType.getName(), Optional.ofNullable(operDateBefore).orElse("-"), Optional.ofNullable(operDateAfter).orElse("-"), null, new Date());
    }

    /**
     * 科组长设置标准卷、一键定档
     *
     * @param stage    阶段
     * @param paper    试卷
     * @param level    档位
     * @param isSample 是否标准卷
     */
    public MarkLog markingPaper(MarkStage stage, Paper paper, String level, boolean isSample) {
        Work work = workRepo.findOne(paper.getWorkId());
        Long userId = ServletUtil.getUserId();
        MarkUser markUser = markUserRepo.findOne(userId);
        MarkLogOperType markLogOperType;
        String operDateBefore;
        String operDateAfter;
        // 标准卷
        if (isSample) {
            markLogOperType = MarkLogOperType.CRITERION_PAPER_SET;
        }
        // 一键定档
        else {
            markLogOperType = MarkLogOperType.ONE_CLICK_LEVEl;
        }
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            operDateBefore = paper.getRoughLevel();
            operDateAfter = level;
        } else if (MarkStage.LEVEL.equals(stage)) {
            operDateBefore = paper.getLevel();
            operDateAfter = level;
        } else {
            throw new RuntimeException("当前阶段不支持科组长操作");
        }
        return new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), paper.getSubject(), stage, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markUser.getId(), markUser.getName(), Role.MARK_LEADER, markLogOperType.getId(),markLogOperType.getName(), Optional.ofNullable(operDateBefore).orElse("-"), Optional.ofNullable(operDateAfter).orElse("-"), null, new Date());
    }

    /**
     * 分档打回
     *
     * @param autoReject 是否自动打回
     * @param markerId   科组长id
     */
    public String markingPaperReject(boolean autoReject, Long markerId) {
        Long userId = markerId != null ? markerId : autoReject ? -1 : ServletUtil.getUserId();
        String userName = autoReject ? "system" : markUserRepo.findOne(userId).getLoginName();
        Role role = autoReject ? Role.SYSTEM : Role.MARK_LEADER;
        return String.format("操作角色：%s，操作员ID：%s，操作员姓名：%s", role.name(), userId, userName);
    }

}
