package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.lock.LockService;
import cn.com.qmth.stmms.ms.commons.lock.LockType;
import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.commons.web.PageableDTO;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.ExamQuestion;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskLevelAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskScoreAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import java.util.*;

/**
 * 评卷工作服务
 */
@Service
public class MarkTaskScoreService {

    @Resource
    private MarkUserRepo markUserRepo;

    @Resource
    private PaperRepo paperRepo;

    @Resource
    private MarkSubjectRepo markSubjectRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    private ExamQuestionRepo examQuestionRepo;

    @Resource
    private MarkingScoreService markingScoreService;

    @Resource
    private LockService lockService;

    @Resource
    private PaperAssembler paperAssembler;

    @Resource
    private MarkTaskLevelAssembler markTaskLevelAssembler;

    @Resource
    private MarkTaskScoreAssembler markTaskScoreAssembler;

    public PageableDTO listMarkTaskScore(Long markerId, Boolean isSample, Boolean reject, String level, Long questionId, String areaCode, Pageable pageable) {
        Long workId = ServletUtil.getWordId();
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        MarkUser markUser = markUserRepo.findOne(markerId);
        Long scoreBatchNo = null;
        List<Object> batchNos = paperRepo.findScoreBatchNoByWorkIdAndSubject(workId, markUser.getSubject().name());
        if (batchNos != null && !batchNos.isEmpty()) {
            Object object = batchNos.get(0);
            scoreBatchNo = Long.valueOf(object.toString());
        }
        Long finalScoreBatchNo = scoreBatchNo;
        Integer scoreShowAllPaper = ParamCache.scoreConfigMap.get(String.valueOf(workId)).getScoreShowAllPaper();
        Specification<MarkTaskScore> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            List<Predicate> onPredicates = new ArrayList<>();
            Join<MarkTaskScore, Paper> join = root.join("paper", JoinType.INNER);
            if (Objects.nonNull(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }
            predicates.add(builder.equal(root.get("markerId"), markerId));
            if (level == null) {
                predicates.add(builder.isNull(root.get("result")));
                onPredicates.add(builder.equal(join.get("isShift"), false));
                onPredicates.add(builder.equal(join.get("isShiftScore"), false));
            } else {
                onPredicates.add(builder.equal(join.get("level"), level));
                predicates.add(builder.isNotNull(root.get("result")));
                onPredicates.add(builder.equal(join.get("isShift"), false));
                if (scoreShowAllPaper == 1) {
                    onPredicates.add(builder.isNotNull(join.get("scoreBatchNo")));
                } else {
                    onPredicates.add(builder.equal(join.get("scoreBatchNo"), finalScoreBatchNo));
                }
            }
            if (isSample != null) {
                onPredicates.add(builder.equal(join.get("isSample"), isSample));
            }
            if (reject != null && reject) {
                onPredicates.add(builder.equal(join.get("isRejected"), reject));
            } else {
                predicates.add(builder.equal(root.get("isRejected"), reject));
            }
            onPredicates.add(builder.isNotNull(join.get("batchNo")));
            //过滤考区
            if (Objects.nonNull(areaCode)) {
                onPredicates.add(builder.equal(join.get("areaCode"), areaCode));
            }
            join.on(onPredicates.toArray(new Predicate[onPredicates.size()]));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Sort sort = new Sort("randomSeq", "randomSeqNew");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Page<MarkTaskScore> markTasks = markTaskScoreRepo.findAll(specification, pageable1);

        if (Objects.isNull(markTasks) || markTasks.getContent().isEmpty()) {
            if (Objects.nonNull(isSample) && isSample) {
                ExamQuestion examQuestion = examQuestionRepo.findOne(questionId);
                MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + examQuestion.getSubject());
                if (markSubject.getTest() == TrialEnum.DEFAULT.getId()) {
                    List<Paper> paperList = paperRepo.findSample(level, isSample, false, examQuestion.getSubject().name(), TrialEnum.DEFAULT.getId(), examQuestion.getAreaCode(), examQuestion.getWorkId());
                    if (Objects.nonNull(paperList) && !paperList.isEmpty()) {
                        for (Paper p : paperList) {
                            MarkTaskScore markTask = new MarkTaskScore(markUser, p, MarkStage.SCORE, 1L);
                            markTaskDTOs.add(markTaskScoreAssembler.toDTO(markTask));
                        }
                    }
                }
            }
        }
        markTasks.getContent().forEach(m -> {
            markTaskDTOs.add(markTaskScoreAssembler.toDTO(m));
        });
        return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
    }

    public MarkTaskDTO markingScore(Long markTaskId, HashMap<String, String> body) {
        MarkTaskScore markTask = markTaskScoreRepo.getOne(markTaskId);
        String result = body.get("result");
        boolean oldRejected = markTask.isRejected();
        boolean oldShift = markTask.getPaper().isShift();
        boolean oldShiftScore = markTask.getPaper().isShiftScore();
        if (result == null) {
            return null;
        }
        int score = Integer.parseInt(result);
        try {
            lockService.waitlock(LockType.SCORE, markTask.getPaper().getId());
            markingScoreService.scoring(markTask, score);
        } catch (Exception e) {
            throw new RuntimeException("打分异常");
        } finally {
            lockService.unlock(LockType.SCORE, markTask.getPaper().getId());
        }
        return markTaskScoreAssembler.toDTO(markTask, oldRejected, oldShift, oldShiftScore);
    }

    public ResponseEntity skipScore(Long markTaskId) {
        MarkTaskScore markTask = markingScoreService.skip(markTaskId);
        return new ResponseEntity(markTaskScoreAssembler.toDTO(markTask), HttpStatus.OK);
    }

    public boolean updateMarkTaskMarkById(Long markTaskId, Boolean isMark) {
        markTaskScoreRepo.updateMarkTaskMarkById(markTaskId, isMark);
        return true;
    }

    public List<MarkTaskDTO> reviewPaperScore(Long markerId, Long questionId, String areaCode) {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        Sort sort = new Sort(Sort.Direction.DESC, "updatedOn");
        Pageable pageable = new PageRequest(0, 5, sort);
        Long scoreBatchNo = paperRepo.findScoreBatchNoByQuestionId(questionId);

        Long finalScoreBatchNo = scoreBatchNo;
        Specification<MarkTaskScore> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (Objects.nonNull(questionId)) {
                predicates.add(builder.equal(root.get("questionId"), questionId));
            }
            predicates.add(builder.equal(root.get("markerId"), markerId));
            predicates.add(builder.isNotNull(root.get("result")));
            if (!Objects.isNull(finalScoreBatchNo)) {
                predicates.add(builder.equal(root.get("paper").get("scoreBatchNo"), finalScoreBatchNo));
            }
            //过滤考区
            if (Objects.nonNull(areaCode)) {
                predicates.add(builder.equal(root.get("paper").get("areaCode"), areaCode));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Page<MarkTaskScore> markTasks = markTaskScoreRepo.findAll(specification, pageable);
        markTasks.getContent().forEach(m -> {
            markTaskDTOs.add(markTaskScoreAssembler.toDTO(m));
        });
        return markTaskDTOs;
    }

    public PageableDTO listMarkedTaskScore(Long workId, Subject subject, Long markerId, Pageable pageable) {
        MarkUser markUser = markUserRepo.findOne(markerId);
        if (Role.MARKER.equals(markUser.getRole())) {
            List<PaperDTO> paperDTOS = new ArrayList<>();
            Specification<MarkTaskScore> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                predicates.add(builder.equal(root.get("subject"), subject));
                predicates.add(builder.equal(root.get("stage"), MarkStage.SCORE));
                predicates.add(builder.equal(root.get("markerId"), markerId));
                predicates.add(builder.equal(root.get("isMark"), true));
                return builder.and(predicates.toArray(new Predicate[0]));
            };
            Page<MarkTaskScore> markTasks = markTaskScoreRepo.findAll(specification, pageable);

            markTasks.getContent().forEach(m -> paperDTOS.add(paperAssembler.toDTO(m.getPaper())));
            return new PageableDTO(paperDTOS, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
        } else if (Role.MARK_LEADER.equals(markUser.getRole())) {
            List<PaperDTO> paperDTOS = new ArrayList<>();
            Specification<Paper> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                predicates.add(builder.equal(root.get("subject"), subject));
                predicates.add(builder.equal(root.get("isMark"), true));
                return builder.and(predicates.toArray(new Predicate[0]));
            };
            Page<Paper> papers = paperRepo.findAll(specification, pageable);

            papers.getContent().forEach(m -> paperDTOS.add(paperAssembler.toDTO(m)));
            return new PageableDTO(paperDTOS, papers.getTotalElements(), papers.getTotalPages(), pageable.getPageNumber());
        } else {
            throw new RuntimeException("当前角色无法查询标记试卷");
        }
    }

    public List<MarkTaskScore> getBySecretNumberScore(Long workId, Subject subject, String secretNumber, int test) {
        return markTaskScoreRepo.findByWorkIdAndSubjectAndSecretNumberAndTest(workId, subject, secretNumber, test);
    }

    public PageableDTO shiftScore(Long markerId, Long workId, Boolean isShift, Boolean isShiftScore, Long questionId, Pageable pageable) {
        if (isShift) {
            List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
            Specification<MarkTaskLevel> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("questionId"), questionId));
                predicates.add(builder.equal(root.get("markerId"), markerId));
                //查询
                predicates.add(builder.equal(root.get("paper").get("isShift"), isShift));
                predicates.add(builder.equal(root.get("stage"), MarkStage.LEVEL));
                predicates.add(builder.isNull(root.get("result")));
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            Sort sort = new Sort("paper.level", "serialNumber", "randomSeq");
            Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
            Page<MarkTaskLevel> markTasks = markTaskLevelRepo.findAll(specification, pageable1);

            markTasks.getContent().forEach(m -> markTaskDTOs.add(markTaskLevelAssembler.toShiftDTO(m)));
            return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
        } else if (isShiftScore) {
            List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
            Specification<MarkTaskScore> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("questionId"), questionId));
                predicates.add(builder.equal(root.get("markerId"), markerId));
                predicates.add(builder.equal(root.get("paper").get("isShift"), false));
                predicates.add(builder.equal(root.get("paper").get("isShiftScore"), isShiftScore));
                predicates.add(builder.equal(root.get("stage"), MarkStage.SCORE));
                predicates.add(builder.isNull(root.get("result")));
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            Sort sort = new Sort("paper.level", "serialNumber", "randomSeq");
            Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
            Page<MarkTaskScore> markTasks = markTaskScoreRepo.findAll(specification, pageable1);

            markTasks.getContent().forEach(m -> markTaskDTOs.add(markTaskScoreAssembler.toShiftDTO(m)));
            return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
        }
        return null;
    }

    public PaperDTO getByTaskSecretNumberLevel(Long questionId, String sn) {
        PaperDTO paperDTO = null;
        Specification<MarkTaskScore> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("paper").get("questionId"), questionId));
            if (sn != null) {
                predicates.add(builder.equal(root.get("secretNumber"), sn));
            }
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Optional<MarkTaskScore> taskOptional = markTaskScoreRepo.findAll(specification, new PageRequest(0, 1)).getContent().stream().findFirst();
        if (taskOptional.isPresent()) {
            paperDTO = paperAssembler.toDTO(taskOptional.get().getPaper());
        }
        return paperDTO;
    }

    public PageableDTO markedPapers(Long workId, Subject subject, Long markerId, Long questionId, Pageable pageable) {
        List<MarkTaskDTO> markTaskDTOs = new ArrayList<>();
        Specification<MarkTaskScore> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();

            if (ParamCache.scoreConfigMap.get(String.valueOf(workId)).getScoreShowAllPaper() == 0) {
                Long batchNo = paperRepo.findMaxScoreBatchNoByWorkIdAndSubject(workId, subject);
                predicates.add(builder.equal(root.get("paper").get("scoreBatchNo"), batchNo));
            }

            predicates.add(builder.equal(root.get("workId"), workId));
            predicates.add(builder.equal(root.get("questionId"), questionId));
            predicates.add(builder.equal(root.get("markerId"), markerId));
            predicates.add(builder.equal(root.get("isMark"), true));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        Sort sort = new Sort("paper.level", "serialNumber", "randomSeq");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        Page<MarkTaskScore> markTasks = markTaskScoreRepo.findAll(specification, pageable1);

        markTasks.getContent().forEach(m -> markTaskDTOs.add(markTaskScoreAssembler.toDTO(m)));
        return new PageableDTO(markTaskDTOs, markTasks.getTotalElements(), markTasks.getTotalPages(), pageable.getPageNumber());
    }
}
