package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroupLeader;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Date: 2021/9/22.
 */
@Service
public class MarkerGroupLeaderService {

    @Resource
    private MarkUserRepo markUserRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    private MarkerGroupRepo markerGroupRepo;

    @Resource
    private MarkerGroupLeaderRepo markerGroupLeaderRepo;

    /**
     * 创建分组
     *
     * @param markerGroup
     * @return
     */
    @Transactional
    public MarkerGroup save(MarkerGroup markerGroup) {
        if (Objects.nonNull(markerGroup.getId())) {
            markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStageAndGroupId(markerGroup.getWorkId(), markerGroup.getSubject(), markerGroup.getStage(), markerGroup.getId());
        }
        Set<MarkUser> groupMarkers = markerGroup.getMarkers();
        List<MarkUser> markLeaders = groupMarkers.stream().filter(m -> Role.MARK_LEADER.equals(m.getRole())).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(markLeaders)) {
            throw new RuntimeException("每个分组至少有一个科组长");
        }
        MarkerGroup group = markerGroupRepo.save(markerGroup);
        List<MarkUser> markers = groupMarkers.stream().filter(m -> Role.MARKER.equals(m.getRole())).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(markers)) {
            List<MarkerGroupLeader> leaderList = new ArrayList<>();
            for (MarkUser markLeader : markLeaders) {
                for (MarkUser marker : markers) {
                    MarkerGroupLeader markerGroupLeader = new MarkerGroupLeader();
                    markerGroupLeader.setWorkId(markerGroup.getWorkId());
                    markerGroupLeader.setSubject(markerGroup.getSubject());
                    markerGroupLeader.setStage(markerGroup.getStage());
                    markerGroupLeader.setGroupId(group.getId());
                    markerGroupLeader.setGroupName(group.getName());
                    markerGroupLeader.setMarkLeaderId(markLeader.getId());
                    markerGroupLeader.setMarkerId(marker.getId());
                    leaderList.add(markerGroupLeader);
                }
            }
            markerGroupLeaderRepo.save(leaderList);
        }

        return group;
    }

    /**
     * 删除分组
     *
     * @param markSubject
     * @param domain
     */
    @Transactional
    public void delete(MarkSubject markSubject, MarkerGroup domain) {
        int count = 0;
        MarkStage stage = markSubject.getStage();
        if (MarkStage.INIT.equals(markSubject.getStage())) {
            if (ParamCache.levelConfigMap.get(String.valueOf(markSubject.getWorkId())).getRoughLevelEnable() == 1) {
                stage = MarkStage.ROUGH_LEVEL;
                count = markTaskRoughLevelRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), stage);
            } else {
                stage = MarkStage.LEVEL;
                count = markTaskLevelRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), stage);
            }
        } else if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            count = markTaskRoughLevelRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), stage);
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            count = markTaskLevelRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), stage);
        } else if (MarkStage.SCORE.equals(markSubject.getStage())) {
            count = markTaskScoreRepo.countByWorkIdAndSubjectAndStage(markSubject.getWorkId(), markSubject.getSubject(), stage);
        }
        if (count > 0) {
            throw new RuntimeException("当前阶段已发布任务，不能删除分组");
        }
        markerGroupRepo.delete(domain);

        markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStageAndGroupId(markSubject.getWorkId(), markSubject.getSubject(), stage, domain.getId());
    }

    /**
     * @param workId
     * @param subject
     * @param stage
     * @return
     */
    public List<Map<String, List<Long>>> listGroupLeaderAndMarkersByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage) {
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStage(workId, subject, stage);
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            Map<Long, List<MarkerGroupLeader>> maps = markerGroupLeaders.stream().collect(Collectors.groupingBy(MarkerGroupLeader::getGroupId));
            List<Map<String, List<Long>>> mapList = new ArrayList<>();
            for (Map.Entry<Long, List<MarkerGroupLeader>> listEntry : maps.entrySet()) {
                Map<String, List<Long>> map = new HashMap<>();
                // 科组长
                List<Long> markLeaderList = listEntry.getValue().stream().map(MarkerGroupLeader::getMarkLeaderId).distinct().collect(Collectors.toList());
                map.put("markLeaders", markLeaderList);
                // 评卷员
                List<Long> markList = listEntry.getValue().stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
                map.put("markers", markList);
                mapList.add(map);
            }
            return mapList;
        }
        return new ArrayList<>();
    }

    /**
     * 根据评卷员，查询试卷
     *
     * @param workId
     * @param subject
     * @param stage
     * @param markers
     * @return
     */
    public List<Long> listPaperIdsByWorkIdAndSubjectAndStageAndMarkerIds(Long workId, Subject subject, MarkStage stage, List<Long> markers) {
        List<Paper> papers = null;
        if (MarkStage.LEVEL.equals(stage)) {
            papers = markTaskLevelRepo.findByWorkIdAndSubjectAndStageAndMarkerIdIn(workId, subject, stage, markers);
        } else if (MarkStage.SCORE.equals(stage)) {
            papers = markTaskScoreRepo.findByWorkIdAndSubjectAndStageAndMarkerIdIn(workId, subject, stage, markers);
        }

        if (!CollectionUtils.isEmpty(papers)) {
            return papers.stream().map(Paper::getId).distinct().collect(Collectors.toList());
        }
        return null;
    }

    public List<Long> listPaperIdsByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage, List<Long> markers) {
        List<Object[]> papers = null;
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            papers = markTaskRoughLevelRepo.findByWorkIdAndSubjectAndStage(workId, subject.name(), stage.ordinal());
        }  else if (MarkStage.LEVEL.equals(stage)) {
            papers = markTaskLevelRepo.findByWorkIdAndSubjectAndStage(workId, subject.name(), stage.ordinal());
        } else if (MarkStage.SCORE.equals(stage)) {
            papers = markTaskScoreRepo.findByWorkIdAndSubjectAndStage(workId, subject.name(), stage.ordinal());
        }
        if (!CollectionUtils.isEmpty(papers)) {
            return papers.stream().filter(m -> markers.contains(Long.valueOf(m[1].toString()))).map(m -> Long.valueOf(m[0].toString())).distinct().collect(Collectors.toList());
        }
        return null;
    }

    /**
     * 根据科组长，查询评卷员
     *
     * @param workId
     * @param markSubject
     * @param markLeaderId
     * @return
     */
    public List<MarkerGroupLeader> listByWorkIdAndSubjectAndStageAndMarkLeaderId(Long workId, MarkSubject markSubject, Long markLeaderId) {
        return markerGroupLeaderRepo.findByWorkIdAndSubjectAndStageAndMarkLeaderId(workId, markSubject.getSubject(), markSubject.getStage(), markLeaderId);
    }

    /**
     * 查询分组
     *
     * @param workId
     * @param subject
     * @param stage
     * @return
     */
    public Set<Map<String, Object>> listGroupByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage) {
        stage = Objects.isNull(stage) ? MarkStage.LEVEL : stage;
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStage(workId, subject, stage);
        Set<Map<String, Object>> list = new HashSet<>();
        for (MarkerGroupLeader markerGroupLeader : markerGroupLeaders) {
            Map<String, Object> stringMap = new HashMap<>();
            stringMap.computeIfAbsent("groupId", v -> markerGroupLeader.getGroupId());
            stringMap.computeIfAbsent("groupName", v -> markerGroupLeader.getGroupName());
            list.add(stringMap);
        }
        return list;
    }

    /**
     * 根据分组查询组内评卷员
     *
     * @param workId
     * @param subject
     * @param stage
     * @param groupId
     * @return
     */
    public List<Long> listByWorkIdAndSubjectAndStageAndGroupId(Long workId, Subject subject, MarkStage stage, Long groupId) {
        if (Objects.isNull(groupId)) {
            return null;
        }
        List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStageAndGroupId(workId, subject, stage, groupId);
        List<Long> markerIds = null;
        if (!CollectionUtils.isEmpty(markerGroupLeaders)) {
            markerIds = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
        }
        return markerIds;
    }

    /**
     * 查询科组长下所有用户
     *
     * @param workId   工作ID
     * @param subject  科目
     * @param stage    阶段
     * @param markerId 科组长ID
     */
    public List<MarkUser> listGroupUsersByWorkIdAndSubjectAndStageAndMarkerId(Long workId, Subject subject, MarkStage stage, Long markerId) {
        MarkUser markUser = markUserRepo.getOne(markerId);
        if (markUser == null) {
            return null;
        }

        if (markUser.getGroupId() == null) {
            return markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        } else {
            List<MarkerGroupLeader> markerGroupLeaders = markerGroupLeaderRepo.findByWorkIdAndSubjectAndStageAndGroupId(workId, subject, stage, markUser.getGroupId());
            if (markerGroupLeaders == null) {
                return null;
            } else {
                List<Long> markerIds = markerGroupLeaders.stream().map(MarkerGroupLeader::getMarkerId).distinct().collect(Collectors.toList());
                return markUserRepo.findAll(markerIds);
            }
        }
    }
}
