package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.constant.ArbitrateCallback;
import cn.com.qmth.stmms.ms.commons.constant.ArbitrateResult;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 评卷，分档仲裁及打分服务
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class MarkingRoughLevelService {

    private static final Logger LOG = LoggerFactory.getLogger(MarkingRoughLevelService.class);

    @Autowired
    private DetermineRoughLevelService determineRoughLevelService;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    MarkingRoughLevelService markingService;

    @Autowired
    MarkLogRepo markLogRepo;

    @Autowired
    WorkRepo workRepo;

    @Autowired
    ArbitrateCallback arbitrateCallback;

    @Resource
    MarkTaskJobRepo markTaskJobRepo;

    @Autowired
    MarkerGroupRepo markerGroupRepo;

    @Autowired
    MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Autowired
    TaskPublishSettingRepo taskPublishSettingRepo;

    @Autowired
    ChangeLevelRepo changeLevelRepo;

    @Autowired
    AssignTaskService assignTaskService;

    @Resource
    MarkingRoughLevelService markingRoughLevelService;

    @Resource
    MarkLogService markLogService;

    /**
     * 提交分档。如果每个评卷都提交了，则进行仲裁判定，如果没有触发仲裁就定档
     *
     * @param markTask
     * @param levelCode
     * @return
     */
    @Transactional
    public MarkTaskRoughLevel levelMark(MarkTaskRoughLevel markTask, String levelCode) {
        long start = System.currentTimeMillis();
        LOG.info("提交分档:{}", start);

        //如果已经由科组长定档，则不做处理
        Paper paper = markTask.getPaper();
        if (paper.isMarkByLeader() && Objects.nonNull(paper.getRoughLevel())) {
            return markTask;
        }

        MarkLog markLog = markLogService.markingRoughLevelLog(markTask, levelCode);

        markTask.setResult(levelCode);
        markTask.setOriginLevel(null);
        markTask.setLevelValue(Integer.valueOf(levelCode));
        markTask.setRejected(false);
        markTask.setDeviationDirection(null);
        markTask.setUpdatedOn(new Date());

        markTaskRoughLevelRepo.save(markTask);
        markLogRepo.save(markLog);

        //判断该任务的试卷是否所有评卷员都评完，要进行仲裁判定或定档
        List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());
        long leftCount = markTasks.stream().filter(i -> i.getResult() == null).count();
        if (leftCount == 0) {
            List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());
            //定档
            RoughDeterResult determine = determineRoughLevelService.determine(markTask.getWorkId(), levels, markTasks);
            // 过半定档或者权重定档
            if (determine.getDeterType() == DetermineRoughLevelService.DeterType.MAJORITY || determine.getDeterType() == DetermineRoughLevelService.DeterType.WEIGHT) {
                paper.determineRoughLevel(determine.getResult());
                paper.setMarkedLogic(false);
                //档位落差值日志
                levelsLog(markTasks, paper);
            }
            //仲裁
            else if (determine.getDeterType() == DetermineRoughLevelService.DeterType.ARBITRATE) {
                paper.roughArbitrate();
            }
            // 打回
            else if (determine.getDeterType() == DetermineRoughLevelService.DeterType.REJECT) {
                paper.setRoughOneClick(false);
                paper.setRejectedByLeader(true);
                //需要打回的评卷员
                String ranges = String.join(",", determine.getRejectTasks());
                markingRoughLevelService.reject(paper, null, ranges, true, true, null);
            }
            //改档后重新分档，状态改为false
            paper.setShift(false);
            //重新分档，打回科组长打回状态改为false
            paper.setRejectedByLeader(false);
            paper.setSortNum(calcSortNum(paper, markTasks, levels));
            paperRepo.save(paper);

            //同步mark_task表中level
            markTaskRoughLevelRepo.updateLevelByPaperId(paper.getRoughLevel(), paper.getId());
        }
        long end = System.currentTimeMillis();
        LOG.info("提交分档耗时：{}", (end - start) / 1000 + "s");
        return markTask;
    }

    /**
     * 档位落差值计算日志
     *
     * @param markTasks
     * @param paper
     */
    @Transactional
    public void levelsLog(List<MarkTaskRoughLevel> markTasks, Paper paper) {
        //20191108wangliang加入档位落差值计算
        LOG.info("this:{} levelsLog异步回调进来了", this);
        JSONObject js = new JSONObject();
        js.put("paperId", paper.getId());
        js.put("type", "levelsLog");
        js.put("stage", MarkStage.ROUGH_LEVEL.name());
        MarkTaskJob markTaskJob = new MarkTaskJob(js.toJSONString(), false, paper.getWorkId());
        levelsLog(markTasks, paper, MarkStage.ROUGH_LEVEL, markTaskJob);
        markTaskJobRepo.save(markTaskJob);
        LOG.info("this:{} levelsLog异步回调结束了", this);
    }

    /**
     * 档位落差值计算日志
     *
     * @param markTasks
     * @param paper
     * @param markTaskJob
     */
    public void levelsLog(List<MarkTaskRoughLevel> markTasks, Paper paper, MarkStage stage, MarkTaskJob markTaskJob) {
        Map<Long, String> levelsMap = markTasks.stream().collect(Collectors.toMap(MarkTaskRoughLevel::getMarkerId, o -> o.getResult().toUpperCase()));
        arbitrateCallback.judge(levelsMap, paper.getRoughLevel(), new ArbitrateResult() {
            @Override
            public void callback(List<ArbitrateCallback.Distance> list) {
                LOG.info("this:{} callback levelsLog异步回调进来了", this);
                String operResult = "-";
                Date date = new Date();
                for (ArbitrateCallback.Distance d : list) {
                    for (MarkTaskRoughLevel m : markTasks) {
                        if (Objects.equals(String.valueOf(d.getC()), m.getResult()) && d.getMarkId().longValue() == m.getMarkerId().longValue()) {
                            MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(m.getMarkerId(), stage, MarkLogOperType.LEVEl_DIFFERENCE.getId(), m.getSubject().toString(), paper.getExamNumber());
                            JSONObject jsonObject = new JSONObject();
                            jsonObject.put("task", m.getResult());
                            jsonObject.put("paper", paper.getLevel());
                            if (Objects.isNull(markLogPrev)) {//新建日志
                                //加入档位落差值日志
                                Work work = workRepo.findOne(paper.getWorkId());
                                MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), m.getSubject(), stage, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), m.getMarkerId(), m.getMarkerName(), Role.MARKER, MarkLogOperType.LEVEl_DIFFERENCE.getId(), MarkLogOperType.LEVEl_DIFFERENCE.getName(), operResult, String.valueOf(d.getDistance()), jsonObject.toJSONString(), date);
                                markLogRepo.save(markLog);
                            } else {//更新日志
                                operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse("-");
                                markLogPrev.setOperDataBefore(operResult);
                                markLogPrev.setOperDataAfter(String.valueOf(d.getDistance()));
                                markLogPrev.setRemark(jsonObject.toJSONString());
                                markLogRepo.save(markLogPrev);
                            }
                            //档位偏差
                            MarkLog markLogDevi = markLogRepo.findTopByCreateUserIdAndStageAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(m.getMarkerId(), stage, MarkLogOperType.LEVEl_DEVIATION.getId(), m.getSubject().toString(), paper.getExamNumber());
                            if (Objects.isNull(markLogDevi)) {//新建日志
                                //加入档位偏差值日志
                                Work work = workRepo.findOne(paper.getWorkId());
                                MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), m.getSubject(), stage, paper.getTest(), paper.getExamNumber(), paper.getStudentName(), m.getMarkerId(), m.getMarkerName(), Role.MARKER, MarkLogOperType.LEVEl_DEVIATION.getId(), MarkLogOperType.LEVEl_DEVIATION.getName(), operResult, String.valueOf(d.getDeviation()), jsonObject.toJSONString(), date);
                                markLogRepo.save(markLog);
                            } else {//更新日志
                                operResult = Optional.ofNullable(markLogDevi.getOperDataAfter()).orElse("-");
                                markLogDevi.setOperDataBefore(operResult);
                                markLogDevi.setOperDataAfter(String.valueOf(d.getDeviation()));
                                markLogDevi.setRemark(jsonObject.toJSONString());
                                markLogRepo.save(markLogDevi);
                            }
                        }
                    }
                }
                markTaskJobRepo.updateMarkTaskJobByVersion(markTaskJob.getId(), markTaskJob.getVersion());
                LOG.info("this:{},callback levelsLog异步回调结束了", this);
            }
        });
    }

    /**
     * 直接给试卷分档
     *
     * @param paper
     * @param level
     */
    @Transactional
    public Paper levelMarkPaper(Paper paper, String level, boolean isSample) {
        List<Level> levels = levelRepo.findByWorkId(paper.getWorkId());

        MarkLog markLog = markLogService.markingPaper(MarkStage.ROUGH_LEVEL, paper, level, isSample);

        paper.determineRoughLevel(level);
        paper.setMarkByLeader(true);
        paper.setRoughSample(isSample);

        List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());
        for (MarkTaskRoughLevel markTask : markTasks) {
            markTask.setRejected(false);
            markTask.setResult(level);
            markTask.setLevel(level);
            markTask.setSample(paper.isRoughSample());
        }
        markTaskRoughLevelRepo.save(markTasks);

        paper.setSortNum(calcSortNum(paper, markTasks, levels));
        paperRepo.save(paper);
        levelsLog(markTasks, paper);
        markLogRepo.save(markLog);
        return paper;
    }

    /**
     * 打回重评
     * 更新试卷状态
     * 更新评卷任务状态为打回
     * 如果评卷任务的档位和建议档位一致，不打回该任务
     *
     * @param paper     打回试卷
     * @param redoLevel 建议重评档位
     * @param ranges    打回评卷员ID集合
     */
    @Transactional
    public Paper reject(Paper paper, String redoLevel, String ranges, boolean leaderConfirm, boolean autoReject, Long rejectKzzId) {
        paper.roughReject(redoLevel);
        //科组长打回标记
        paper.setRejectedByLeader(true);
        paper.setMarkByLeader(true);
        //打回后撤销标准卷
        paper.setRoughSample(false);
        paper.setArbitrated(false);
        List<MarkTaskRoughLevel> markTasks = markTaskRoughLevelRepo.findByPaperId(paper.getId());
        Work work = workRepo.findOne(paper.getWorkId());
        MarkLogOperType markLogOperType = autoReject ? MarkLogOperType.SYSTEM_CALLBACK_LEVEl : MarkLogOperType.CALLBACK_LEVEl;

        List<MarkLog> markLogList = new ArrayList<>();
        for (MarkTaskRoughLevel markTask : markTasks) {
            String[] range = ranges.split(",");
            if (range.length > 0) {
                //加入手动打回日志
                for (String r : range) {
                    // 建议的档位与其中评委所评档位相同时就不打回给此评委
                    if (Objects.equals(String.valueOf(markTask.getMarkerId()), r) && (StringUtils.isBlank(redoLevel) || leaderConfirm || !redoLevel.equals(markTask.getResult()))) {
                        markTask.setRejected(true);
                        if (markTask.getResult() != null) {
                            markTask.setOriginLevel(markTask.getResult());
                        }
                        markTask.setResult(null);
                        markTask.setLevel(null);
                        markTask.setDeviationDirection(calcDeviationDirection(markTask.getOriginLevel(), redoLevel));
                        String remark = markLogService.markingPaperReject(autoReject, rejectKzzId);
                        MarkLog markLog = new MarkLog(paper.getWorkId(), work.getName(), paper.getId(), markTask.getSubject(), markTask.getStage(), paper.getTest(), paper.getExamNumber(), paper.getStudentName(), markTask.getMarkerId(), markTask.getMarkerName(), Role.MARKER, markLogOperType.getId(), markLogOperType.getName(), markTask.getOriginLevel(), redoLevel, remark, new Date());
                        markLogList.add(markLog);
                    }
                }
            }
        }
        markTaskRoughLevelRepo.save(markTasks);
        paperRepo.save(paper);
        markLogRepo.save(markLogList);
        return paper;
    }

    /**
     * 跳过该任务，最后处理
     *
     * @param markTaskId
     */
    @Transactional
    public MarkTaskRoughLevel skip(Long markTaskId) {
        try {
            MarkTaskRoughLevel m = markTaskRoughLevelRepo.findOne(markTaskId);
            Integer randomSeq = markTaskRoughLevelRepo.findRandomSeqByWorkIdAndSubjectAndMarkerIdAndQuestionId(m.getWorkId(), m.getSubject(), m.getMarkerId(), m.getQuestionId());
            if (randomSeq == null) {
                randomSeq = 0;
            }
            m.setRandomSeq(randomSeq + 1);
            markTaskRoughLevelRepo.save(m);
            return m;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("该任务可能被占用，跳过失败");
        }

    }

    /**
     * 计算偏差方向
     *
     * @param result 定档档位
     * @param level  评卷员打的档位
     * @return 正数（偏高） 负数（偏低）
     */
    public static String calcDeviationDirection(String result, String level) {
        if (Objects.nonNull(result) && Objects.nonNull(level)) {
            Integer resultInt = Integer.parseInt(result);
            Integer levelInt = Integer.parseInt(level);
            return String.valueOf(levelInt - resultInt);
        }
        return null;
    }

    public static Integer calcSortNum(Paper paper, List<MarkTaskRoughLevel> markTaskList, List<Level> levels) {
        //计算levelValue和（试评阶段不用）
        List<RoughLevel> roughLevels = DetermineRoughLevelService.parseRoughLevels(levels);
        if (paper.getTest() == 0 && Objects.nonNull(paper.getRoughLevel())) {
            if (!roughLevels.isEmpty()) {
                Map<String, Integer> map = roughLevels.stream().collect(Collectors.toMap(RoughLevel::getRoughCode, RoughLevel::getWeight));
                return markTaskList.stream().mapToInt(m -> map.get(m.getResult()) == null ? 0 : map.get(m.getResult())).sum();
            }
        }
        return 0;
    }

}
