package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.constant.ArbitrateCallback;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.MarkLog;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskScore;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

/**
 * 评卷，分档仲裁及打分服务
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class MarkingScoreService {

    private static final Logger LOG = LoggerFactory.getLogger(MarkingScoreService.class);

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    MarkingScoreService markingService;

    @Autowired
    MarkLogRepo markLogRepo;

    @Autowired
    WorkRepo workRepo;

    @Autowired
    ArbitrateCallback arbitrateCallback;

    @Autowired
    MarkerGroupRepo markerGroupRepo;

    @Autowired
    MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Autowired
    TaskPublishSettingRepo taskPublishSettingRepo;

    @Autowired
    ChangeLevelRepo changeLevelRepo;

    @Autowired
    AssignTaskService assignTaskService;

    @Resource
    MarkLogService markLogService;


    /**
     * 打分。取出组内所有评分，如果都给了分，取平均分更新试卷成绩
     *
     * @param markTask
     * @param score
     */
    @Transactional
    public MarkTaskScore scoring(MarkTaskScore markTask, int score) {
        MarkLog markLog = markLogService.markingScoreLog(markTask, score);

        markTask.setOriginLevel(null);
        markTask.setResult(String.valueOf(score));
        markTask.setUpdatedOn(new Date());
        MarkUser maker = markUserRepo.findOne(markTask.getMarkerId());
        markTask.setMarkerName(maker.getName());
        markTaskScoreRepo.save(markTask);
        markLogRepo.save(markLog);

        Paper paper = markTask.getPaper();
        List<MarkTaskScore> markTasks = markTaskScoreRepo.findByPaperId(paper.getId());
        long leftCount = markTasks.stream().filter(i -> i.getResult() == null).count();
        if (leftCount == 0) {
            // 分数计算方式 （1：去高去低再加权评卷， 0：全部加权平均，默认0）
            boolean removeHighAndLow = Optional.ofNullable(ParamCache.scoreConfigMap.get(String.valueOf(String.valueOf(markTask.getWorkId()))).getRemoveHighAndLow()).orElse(0) == 1;
            if (removeHighAndLow) {
                markTasks.sort(Comparator.comparingInt(x -> Integer.parseInt(x.getResult())));
                markTasks.remove(0);//去掉最大值
                markTasks.remove(markTasks.size() - 1);//去掉最小值
            }

            OptionalDouble finalScore = markTasks.stream().map(MarkTaskScore::getResult).mapToInt(Integer::valueOf).average();
            //湖北省，平均分非零进一
            Integer roundUp = Optional.ofNullable(ParamCache.scoreConfigMap.get(String.valueOf(markTask.getWorkId())).getRoundUp()).orElse(1);
            double fs = roundUp == 0 ? Math.ceil(finalScore.orElse(0)) : Math.round(finalScore.orElse(0));
            paper.setScore(fs);
            //改档打分完成，更新状态为false
            paper.setShiftScore(false);
            paperRepo.save(paper);
        }
        return markTask;
    }

    /**
     * 跳过该任务，最后处理
     *
     * @param markTaskId
     */
    @Transactional
    public MarkTaskScore skip(Long markTaskId) {
        try {
            MarkTaskScore m = markTaskScoreRepo.findOne(markTaskId);
            Integer randomSeq = markTaskScoreRepo.findRandomSeqByWorkIdAndSubjectAndMarkerIdAndQuestionId(m.getWorkId(), m.getSubject(), m.getMarkerId(), m.getQuestionId());
            if (randomSeq == null) {
                randomSeq = 0;
            }
            m.setRandomSeq(randomSeq + 1);
            markTaskScoreRepo.save(m);
            return m;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("该任务可能被占用，跳过失败");
        }

    }

    /**
     * 计算偏差方向
     *
     * @param levelMap
     * @param result   定档档位
     * @param level    评卷员打的档位
     * @return
     */
    public static String calcDeviationDirection(Map<String, Integer> levelMap, String result, String level) {
        if (Objects.nonNull(result) && Objects.nonNull(level)) {
            Integer resultInt = levelMap.get(result);
            Integer levelInt = levelMap.get(level);
            if (resultInt != null && levelInt != null) {
                return String.valueOf(levelInt - resultInt);
            }
        }
        return null;
    }

}
