package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.constant.ArbitrateCallback;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.domain.ChangeLevel;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 评卷，分档仲裁及打分服务
 * Created by zhengmin on 2016/9/23.
 */
@Service
public class MarkingService {

    private static final Logger LOG = LoggerFactory.getLogger(MarkingService.class);

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    MarkLogRepo markLogRepo;

    @Autowired
    WorkRepo workRepo;

    @Autowired
    ArbitrateCallback arbitrateCallback;

    @Autowired
    MarkerGroupRepo markerGroupRepo;

    @Autowired
    MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Autowired
    TaskPublishSettingRepo taskPublishSettingRepo;

    @Autowired
    ChangeLevelRepo changeLevelRepo;

    @Autowired
    private CacheService cacheService;

    @Autowired
    AssignTaskService assignTaskService;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Resource
    MarkerGroupStudentRepo markerGroupStudentRepo;

    @Transactional
    public void subjectReset(Long workId, Subject subject, MarkStage stage) {
        // 清除mark_user分组
        markUserRepo.updateByWorkIdAndSubject(workId, subject);
        // 删除mark_group分组
        markerGroupRepo.deleteBySubjectAndWorkId(subject, workId);
        // 删除各组成员
        markerGroupLeaderRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, stage);
        // 分档阶段
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            // 删除mark_task阶段数据
            markTaskRoughLevelRepo.deleteByWorkIdAndSubject(workId, subject);
            // 清除paper中的档位，各种状态
            paperRepo.updateRoughLevelByWorkIdAndSubject(workId, subject.name());
        } else if (MarkStage.LEVEL.equals(stage)) {
            // 删除mark_task阶段数据
            markTaskLevelRepo.deleteByWorkIdAndSubject(workId, subject);
            // 清除paper中的档位，各种状态
            paperRepo.updateLevelByWorkIdAndSubject(workId, subject.name());

            // 还原改档审核同意前的档位（第一步）
            List<ChangeLevel> changeLevels = changeLevelRepo.findByWorkIdAndSubjectAndAuditStatusAndStage(workId, subject.name(), stage);
            Map<Long, List<ChangeLevel>> changeLevelCollects = changeLevels.stream().collect(Collectors.groupingBy(ChangeLevel::getPaperId));
            for (Map.Entry<Long, List<ChangeLevel>> entry : changeLevelCollects.entrySet()) {
                List<ChangeLevel> changeLevelsList = entry.getValue();
                if(!changeLevelsList.isEmpty()){
                    changeLevelsList.sort(Comparator.comparing(ChangeLevel::getCreateDate).reversed());
                    ChangeLevel changeLevel =changeLevelsList.get(0);
                    paperRepo.updateRoughLevelById(changeLevel.getPaperId(), changeLevel.getOriginalLevel());
                }
            }
            // 清除改档打分记录（第二步）
            changeLevelRepo.deleteByWorkIdAndSubjectAndStage(workId, subject.name(), stage);
        }
        // 打分阶段
        else if (MarkStage.SCORE.equals(stage)) {
            // 删除mark_task阶段数据
            markTaskScoreRepo.deleteByWorkIdAndSubject(workId, subject);
            // 清除paper中的分数
            paperRepo.updateScoreByWorkIdAndSubject(workId, subject.name());
            // 删除task_publish_setting数据
            taskPublishSettingRepo.deleteByWorkIdAndSubject(workId, subject.name());
            //初始化打分任务数据
            assignTaskService.initTaskPublishData(workId, subject.name());
            // 还原改档审核同意前的档位（第一步）
            List<ChangeLevel> changeLevels = changeLevelRepo.findByWorkIdAndSubjectAndAuditStatusAndStage(workId, subject.name(), stage);
            Map<Long, List<ChangeLevel>> changeLevelCollects = changeLevels.stream().collect(Collectors.groupingBy(ChangeLevel::getPaperId));
            for (Map.Entry<Long, List<ChangeLevel>> entry : changeLevelCollects.entrySet()) {
                List<ChangeLevel> changeLevelsList = entry.getValue();
                if(!changeLevelsList.isEmpty()){
                    changeLevelsList.sort(Comparator.comparing(ChangeLevel::getCreateDate).reversed());
                    ChangeLevel changeLevel =changeLevelsList.get(0);
                    paperRepo.updateLevelById(changeLevel.getPaperId(), changeLevel.getOriginalLevel());
                }
            }
            // 清除改档打分记录（第二步）
            changeLevelRepo.deleteByWorkIdAndSubjectAndStage(workId, subject.name(), stage);
        } else {
            throw new RuntimeException("当前阶段无法重置数据");
        }
        // 删除日志表
        markLogRepo.deleteByWorkIdAndSubjectAndStage(workId, subject.name(), stage);
        //清除分组导入的考生名单数据
        markerGroupStudentRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, stage);

        // 所有用户强制退出
        List<MarkUser> userList = markUserRepo.findByWorkIdAndSubject(workId, subject);
        userList = userList.stream().filter(m -> Role.MARKER.equals(m.getRole()) || Role.MARK_LEADER.equals(m.getRole())).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(userList)) {
            for (MarkUser user : userList) {
                cacheService.deleteTokenCache(user.getId().toString());
            }
        }
    }

    /**
     * 计算偏差方向
     *
     * @param levelMap
     * @param result   定档档位
     * @param level    评卷员打的档位
     * @return
     */
    public static String calcDeviationDirection(Map<String, Integer> levelMap, String result, String level) {
        if (Objects.nonNull(result) && Objects.nonNull(level)) {
            Integer resultInt = levelMap.get(result);
            Integer levelInt = levelMap.get(level);
            if (resultInt != null && levelInt != null) {
                return String.valueOf(levelInt - resultInt);
            }
        }
        return null;
    }

}
