package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.utils.ServletUtil;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Message;
import cn.com.qmth.stmms.ms.core.domain.MessageReceive;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.MessageReceiveRepo;
import cn.com.qmth.stmms.ms.core.repository.MessageRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 消息服务类型Created by zhengmin on 2016/9/23.
 */
@Service
public class MessageService {

    private static final Logger LOG = LoggerFactory.getLogger(MessageService.class);

    @Resource
    private MessageRepo messageRepo;

    @Resource
    private MessageReceiveRepo messageReceiveRepo;

    @Transactional
    public Message sendMessage(String data) {
        Long workId = ServletUtil.getWordId();
        JSONObject object = JSON.parseObject(data, JSONObject.class);
        String markers = object.getString("markers");
        List<MarkUser> markUsers = JSON.parseArray(markers, MarkUser.class);
        if (markUsers.isEmpty()) {
            throw new RuntimeException("没有指定接收用户");
        }
        // 保存消息
        Message message = new Message();
        message.setWorkId(workId);
        message.setSubject(Subject.valueOf(object.getString("subject")));
        message.setStage(MarkStage.valueOf(object.getString("stage")));
        message.setSendUserId(object.getLong("sendUserId"));
        message.setSendUserName(object.getString("sendUserName"));
        message.setContent(object.getString("content"));
        message.setReceiveUser(markers);
        message.setCreateTime(new Date());
        messageRepo.saveAndFlush(message);

        List<MessageReceive> messageReceives = new ArrayList<>();
        for (MarkUser markUser : markUsers) {
            MessageReceive messageReceive = new MessageReceive();
            messageReceive.setMessage(message);
            messageReceive.setReceiveUserId(markUser.getId());
            messageReceive.setRead(false);
            messageReceives.add(messageReceive);
        }
        messageReceiveRepo.save(messageReceives);
        return message;
    }


    public Page<MessageReceive> listByReceiveUserIdAll(Subject subject, MarkStage stage, Long receiveUserId, Long lastId, Pageable pageable) {
        Long workId = ServletUtil.getWordId();
        Specification<MessageReceive> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (lastId != null) {
                predicates.add(builder.gt(root.get("id"), lastId));
            }
            predicates.add(builder.equal(root.get("message").get("workId"), workId));
            predicates.add(builder.equal(root.get("message").get("subject"), subject));
            predicates.add(builder.equal(root.get("message").get("stage"), stage));
            predicates.add(builder.equal(root.get("receiveUserId"), receiveUserId));

            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = new Sort(Sort.Direction.DESC, "id");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        return messageReceiveRepo.findAll(specification, pageable1);
    }

    public List<MessageReceive> listByReceiveUserIdRead(Subject subject, MarkStage stage, Long receiveUserId) {
        Long workId = ServletUtil.getWordId();
        Specification<MessageReceive> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("message").get("workId"), workId));
            predicates.add(builder.equal(root.get("message").get("subject"), subject));
            predicates.add(builder.equal(root.get("message").get("stage"), stage));
            predicates.add(builder.equal(root.get("receiveUserId"), receiveUserId));
            predicates.add(builder.equal(root.get("isRead"), false));

            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = new Sort(Sort.Direction.ASC, "id");
        return messageReceiveRepo.findAll(specification, sort);
    }

    public Page<Message> listBySendUserIdAll(Subject subject, MarkStage stage, Long sendUserId, Pageable pageable) {
        Long workId = ServletUtil.getWordId();
        Specification<Message> specification = (root, query, builder) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(builder.equal(root.get("workId"), workId));
            predicates.add(builder.equal(root.get("subject"), subject));
            predicates.add(builder.equal(root.get("stage"), stage));
            predicates.add(builder.equal(root.get("sendUserId"), sendUserId));
            return builder.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        Sort sort = new Sort(Sort.Direction.DESC, "id");
        Pageable pageable1 = new PageRequest(pageable.getPageNumber(), pageable.getPageSize(), sort);
        return messageRepo.findAll(specification, pageable1);
    }

    /**
     * 标记已读
     *
     * @param ids ids
     */
    public void markRead(List<Long> ids) {
        List<MessageReceive> messageReceives = new ArrayList<>();
        for (Long id : ids) {
            MessageReceive messageReceive = messageReceiveRepo.findOne(id);
            messageReceive.setRead(true);
            messageReceives.add(messageReceive);
        }
        messageReceiveRepo.save(messageReceives);
    }
}
