package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.utils.DbBackupUtils;
import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.cache.CacheService;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.MarkerGroup;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.dto.LevleProgressDTO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 评卷阶段控制服务
 * Created by zhengmin on 2016/9/24.
 */
@Service
public class StageControlService {

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Autowired
    private MarkTaskScoreRepo markTaskScoreRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private AssignTaskService assignTaskService;

    @Autowired
    private ExamQuestionRepo examQuestionRepo;

    @Autowired
    private MarkerGroupRepo markerGroupRepo;

    @Resource
    private MarkerGroupStudentRepo markerGroupStudentRepo;

    @Autowired
    private TaskPublishSettingRepo taskPublishSettingRepo;

    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    DbBackupUtils dbBackupUtils;

    @Autowired
    CacheService cacheService;

    public static volatile Map<String, Boolean> statusMap = new Hashtable<>();

    /**
     * 进入下一阶段
     *
     * @param markSubject 科目
     */
    @Transactional
    public void createTask(MarkSubject markSubject, Map<String, Integer> map) throws Exception {
        if (statusMap.get(markSubject.getId()) != null && statusMap.get(markSubject.getId())) {
            return;
        }
        statusMap.put(markSubject.getId(), true);
        MarkStage markStage = markSubject.getStage();
        if (MarkStage.ROUGH_LEVEL.equals(markStage)) {
            int taskCount = map.get("taskCount");
            Object questionId = map.get("questionId");
            enterRoughLevelStage(markSubject, taskCount, questionId);
        } else if (MarkStage.LEVEL.equals(markStage)) {
            int taskCount = map.get("taskCount");
            Object questionId = map.get("questionId");
            enterLevelStage(markSubject, taskCount, questionId);
        } else if (MarkStage.SCORE.equals(markStage)) {
            enterToScoreStage(markSubject);
            // 进入抽查阶段
            /*if (!markSubject.isAllLevel()) {
                markSubject.setStage(MarkStage.SCORE);
                enterScoreStage(markSubject);
            } else {
                markSubject.setStage(MarkStage.INSPECT);
                enterInspectStage(markSubject);
            }*/
        } else if (MarkStage.INSPECT.equals(markStage)) {
            // 结束评卷
            markSubject.setStage(MarkStage.END);
        }
        markSubjectRepo.save(markSubject);
        statusMap.put(markSubject.getId(), false);
    }

    /**
     * 进入粗分档阶段
     * 1.检查是否有设定档位，如果有，将所有level设置连续的levelValue值，用于仲裁误差判定
     * 2.检查有无可分档的评卷员，没有则终止
     * 3.给所有能够分档的评卷员分配任务
     */
    private void enterRoughLevelStage(MarkSubject markSubject, int taskCount, Object questionId) throws Exception {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();
        List<Level> levels = levelRepo.findByWorkId(workId);
        if (levels.isEmpty()) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设定分档档位");
        }

        List<MarkUser> markers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        List<MarkUser> levelMarkers = markers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_ALL || m.getMarkRight() == MarkRight.ALLOW_LEVELING).collect(Collectors.toList());
        if (levelMarkers.isEmpty()) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设定评卷员");
        }

        List<MarkUser> markerLeader = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARK_LEADER);
        if (markerLeader.isEmpty()) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设定科组长");
        }

        List<MarkerGroup> markerGroups = markerGroupRepo.findByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());
        if (markerGroups.isEmpty()) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("评卷员没有进行分组");
        }
        List<Long> questionIdsLong = new ArrayList<>();
        if (Objects.nonNull(questionId)) {
            List<Integer> questionIds = (List<Integer>) questionId;
            questionIdsLong = questionIds.stream().map(m -> Long.parseLong(m.toString())).collect(Collectors.toList());
        }
        List<Paper> allPapers = paperRepo.findByWorkIdAndSubjectAndIsMissingAndTest(workId, subject, false, markSubject.getTest());
        if (markSubject.getTest() != 1) {
            //是否有未结束的分档任务
            long count = allPapers.stream().filter(m -> m.getRoughLevel() == null && m.getRoughBatchNo() != null).count();
            if (count > 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("没有分档完成");
            }

            //任务数量大于0，小于等于待评数量
            if (taskCount == 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("请输入正确的分配任务数量");
            }
            int waitCount;
            if (questionIdsLong.isEmpty()) {
                waitCount = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndRoughBatchNoIsNullAndTest(workId, subject, markSubject.getTest());
            } else {
                waitCount = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndRoughBatchNoIsNullAndTestAndQuestionIdIn(workId, subject, markSubject.getTest(), questionIdsLong);
            }

            if (waitCount == 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("没有可分档任务");
            }

            if (taskCount - waitCount > 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("分配任务数量不能大于待评数量");
            }
        }
        Map<Long, List<Paper>> groupPaperMap = assignTaskService.assignGroupTasks(allPapers, markSubject, markerGroups, questionIdsLong, taskCount);
        assignTaskService.assignForGroupingRoughLevel(groupPaperMap, markerGroups, markSubject);
    }

    /**
     * 进入分档阶段
     * 1.检查是否有设定档位，如果有，将所有level设置连续的levelValue值，用于仲裁误差判定
     * 2.检查有无可分档的评卷员，没有则终止
     * 3.给所有能够分档的评卷员分配任务
     *
     * @param markSubject
     */
    private void enterLevelStage(MarkSubject markSubject, int taskCount, Object questionId) throws Exception {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();
        List<Level> levels = levelRepo.findByWorkId(workId);
        if (levels.size() == 0) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设定档位");
        }

        List<MarkUser> markers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        List<MarkUser> levelMarkers = markers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_ALL || m.getMarkRight() == MarkRight.ALLOW_LEVELING).collect(Collectors.toList());
        if (levelMarkers.size() == 0) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设定评卷员");
        }

        List<MarkUser> markerLeader = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARK_LEADER);
        if (markerLeader.size() == 0) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设定科组长");
        }

        for (int i = 0; i < levels.size(); i++) {
            levels.get(i).setLevelValue(i);
        }
        levelRepo.save(levels);
        //assignTaskService.assignForAll(papers, levelMarkers, markSubject);
        List<MarkerGroup> markerGroups = markerGroupRepo.findByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());
        if (markerGroups.size() == 0) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("评卷员没有进行分组");
        }

        List<Long> questionIdsLong = new ArrayList<>();
        if (Objects.nonNull(questionId)) {
            List<Integer> questionIds = (List<Integer>) questionId;
            questionIdsLong = questionIds.stream().map(m -> Long.parseLong(m.toString())).collect(Collectors.toList());
        }

        if (markSubject.getTest() != 1) {
            //是否有未结束的分档任务
            int count = paperRepo.countByWorkIdAndSubjectAndLevelIsNullAndIsMissingFalseAndBatchNoNotNullAndTest(workId, subject, markSubject.getTest());
            if (count > 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("没有分档完成");
            }

            //任务数量大于0，小于等于待评数量
            if (taskCount == 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("请输入正确的分配任务数量");
            }
            int waitCount;
            if (questionIdsLong.isEmpty()) {
                waitCount = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndBatchNoIsNullAndTest(workId, subject, markSubject.getTest());
            } else {
                waitCount = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndBatchNoIsNullAndTestAndQuestionIdIn(workId, subject, markSubject.getTest(), questionIdsLong);
            }

            if (waitCount == 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("没有可分档任务");
            }

            if (taskCount - waitCount > 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("分配任务数量不能大于待评数量");
            }

            // 分配任务数量必须大于名单总数量
            long markerGroupStudentCount = markerGroupStudentRepo.countByWorkIdAndSubjectAndStageAndUsed(workId, subject, markSubject.getStage(), false);
            if (taskCount - markerGroupStudentCount < 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("分配任务数量必须大于各分组名单总数量");
            }
        }

        List<Paper> allPapers = paperRepo.findByWorkIdAndSubjectAndIsMissingAndTest(workId, subject, false, markSubject.getTest());
        Map<Long, List<Paper>> groupPaperMap = assignTaskService.assignGroupTasks(allPapers, markSubject, markerGroups, questionIdsLong, taskCount);
        assignTaskService.assignForGroupingLevel(groupPaperMap, markerGroups, markSubject);
    }

    /**
     * 随机发任务
     *
     * @param papers    试卷集合
     * @param taskCount 任务数量
     */
    private List<Paper> randomList(List<Paper> papers, int taskCount) {
        if (papers.size() <= taskCount) {
            return papers;
        } else {
            List<Paper> list = new ArrayList<>();
            Random random = new Random();
            for (int i = 0; i < taskCount; i++) {
                int intRandom = random.nextInt(papers.size() - 1);
                list.add(papers.get(intRandom));
                papers.remove(intRandom);
            }
            return list;
        }
    }

    public void enterToScoreStage(MarkSubject markSubject) {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();
        int count = paperRepo.countByWorkIdAndSubjectAndLevelIsNullAndIsMissingFalseAndBatchNoNotNullAndTest(workId, subject, markSubject.getTest());
        if (count > 0) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("评卷任务没有分档完成");
        }
        count = paperRepo.countByWorkIdAndSubjectAndLevelIsNullAndIsMissingFalseAndBatchNoIsNullAndTest(workId, subject, markSubject.getTest());
        if (count > 0) {
            throw new RuntimeException("有未发布的评卷任务");
        }
        count = paperRepo.countByWorkIdAndSubjectAndIsArbitratedTrue(workId, subject);
        if (count > 0) {
            throw new RuntimeException("有待仲裁的评卷任务");
        }
        this.isPtFull(workId, markSubject);
        markSubject.setAllLevel(true);
        assignTaskService.initTaskPublishData(workId, subject.name());
    }

    @Transactional
    public void enterScoreStage(MarkSubject markSubject, List<TaskPublishSetting> taskList, Object questionId) throws Exception {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();

        //已发布任务是否都已完成
        int waitCount = paperRepo.countByWorkIdAndSubjectAndScoreBatchNoIsNotNullAndScoreIsNull(workId, subject);
        if (waitCount > 0) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有打分完成");
        }

        // 校验阈值
        this.isPtFull(workId, markSubject);

        List<MarkUser> markers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        List<MarkUser> levelMarkers = markers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_ALL || m.getMarkRight() == MarkRight.ALLOW_SCORING).collect(Collectors.toList());
        if (levelMarkers.isEmpty()) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设置评卷员");
        }

        List<MarkerGroup> markerGroups = markerGroupRepo.findByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());
        if (markerGroups.isEmpty()) {
            statusMap.put(markSubject.getId(), false);
            throw new RuntimeException("没有设置分组");
        }

        for (MarkerGroup markerGroup : markerGroups) {
            if (markerGroup.getMarkers().isEmpty()) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("分组没有绑定评卷员");
            }
        }

        //校验发布任务数量
        for (TaskPublishSetting taskPublishSetting : taskList) {
            if (Objects.nonNull(taskPublishSetting.getTaskCount()) && taskPublishSetting.getTaskCount() > 0 && taskPublishSetting.getWaitCount() < taskPublishSetting.getTaskCount()) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException(taskPublishSetting.getCode() + "档位本次任务数量必须小于等于未评数量");
            }
        }

        //总的发布任务数
        int totalTaskCount = taskList.stream().map(m -> {
            if (Objects.isNull(m.getTaskCount())) {
                m.setTaskCount(0);
            }
            return m;
        }).mapToInt(TaskPublishSetting::getTaskCount).sum();

        Sort sort = new Sort("questionId", "level");
        List<Paper> papers;
        //questionId为空，为所有考区
        if (Objects.isNull(questionId)) {
            int totalCount = taskList.stream().mapToInt(TaskPublishSetting::getTotalCount).sum();
            int successCount = taskList.stream().mapToInt(TaskPublishSetting::getSuccessCount).sum();
            papers = paperRepo.findByWorkIdAndSubjectAndLevelNotNullAndScoreBatchNoIsNull(workId, subject, sort);
            if (successCount > 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("单考区发布过任务，不能再发布所有考区任务");
            } else if (totalTaskCount - totalCount != 0) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException("选择所有考区时，必须一次发布所有任务");
            }
        } else {
            Long questionId0 = Long.valueOf(questionId.toString());
            papers = new ArrayList<>();
            for (TaskPublishSetting taskPublishSetting : taskList) {
                if (taskPublishSetting.getTaskCount() == 0) {
                    continue;
                }
                List<Paper> paperList = paperRepo.findByWorkIdAndQuestionIdAndSubjectAndLevelAndScoreBatchNoIsNull(workId, questionId0, subject, taskPublishSetting.getCode(), sort);
                List<Paper> subPaperList = paperList.subList(0, taskPublishSetting.getTaskCount());
                papers.addAll(subPaperList);
            }
        }

        assignTaskService.assignForGroupingScore(papers, markerGroups, markSubject, taskList);

        //保存发布数据
        int max = taskPublishSettingRepo.maxPublishCount(workId, subject.name());
        long scoreBatchNo = papers.get(0).getScoreBatchNo();
        List<TaskPublishSetting> taskListAll = taskPublishSettingRepo.findByWorkIdAndSubjectAndPublishCount(workId, subject.name(), max);
        if (Objects.isNull(questionId)) {
            Map<String, TaskPublishSetting> map = taskList.stream().collect(Collectors.toMap(TaskPublishSetting::getCode, p -> p));
            List<TaskPublishSetting> finalList = taskListAll.stream().map(m -> {
                TaskPublishSetting taskPublishSetting = new TaskPublishSetting();
                taskPublishSetting.setWorkId(workId);
                taskPublishSetting.setQuestionId(m.getQuestionId());
                taskPublishSetting.setSubject(subject.name());
                taskPublishSetting.setCode(m.getCode());
                taskPublishSetting.setTotalCount(m.getTotalCount());
                taskPublishSetting.setSuccessCount(m.getTotalCount());
                taskPublishSetting.setWaitCount(0);
                taskPublishSetting.setSortRule(map.get(m.getCode()).getSortRule());
                taskPublishSetting.setDisplayNumber(map.get(m.getCode()).getDisplayNumber());
                taskPublishSetting.setTaskCount(0);
                taskPublishSetting.setBatchNo(scoreBatchNo);
                taskPublishSetting.setPublishCount(max + 1);
                return taskPublishSetting;
            }).collect(Collectors.toList());
            taskPublishSettingRepo.save(finalList);
        } else {
            Long questionId0 = Long.valueOf(questionId.toString());
            List<TaskPublishSetting> finalList = taskListAll.stream().map(m -> {
                TaskPublishSetting taskPublishSetting = new TaskPublishSetting();
                taskPublishSetting.setId(null);
                taskPublishSetting.setWorkId(m.getWorkId());
                taskPublishSetting.setQuestionId(m.getQuestionId());
                taskPublishSetting.setSubject(m.getSubject());
                taskPublishSetting.setCode(m.getCode());
                taskPublishSetting.setSuccessCount(m.getSuccessCount());
                taskPublishSetting.setTotalCount(m.getTotalCount());
                taskPublishSetting.setWaitCount(m.getWaitCount());
                taskPublishSetting.setSortRule(m.getSortRule());
                taskPublishSetting.setTaskCount(0);
                taskPublishSetting.setPublishCount(max + 1);
                taskPublishSetting.setBatchNo(scoreBatchNo);
                for (TaskPublishSetting tps : taskList) {
                    if (m.getQuestionId().equals(questionId0) && m.getCode().equals(tps.getCode())) {
                        taskPublishSetting.setSuccessCount(tps.getSuccessCount() + tps.getTaskCount());
                        taskPublishSetting.setWaitCount(tps.getTotalCount() - taskPublishSetting.getSuccessCount());
                        taskPublishSetting.setSortRule(tps.getSortRule());
                        taskPublishSetting.setDisplayNumber(tps.getDisplayNumber());
                        taskPublishSetting.setTaskCount(0);
                    }
                }
                return taskPublishSetting;
            }).collect(Collectors.toList());
            taskPublishSettingRepo.save(finalList);
        }

    }

    /**
     * @param workId      工作ID
     * @param markSubject 科目
     */
    private void isPtFull(Long workId, MarkSubject markSubject) {
        //控制分档比例
        List<Level> levels = levelRepo.findByWorkId(workId);
        for (Level targetLevel : levels) {
            int pt = targetLevel.getPt();
            long count = paperRepo.countByWorkIdAndSubjectAndLevel(workId, markSubject.getSubject(), targetLevel.getCode());
            long total = paperRepo.countByWorkIdAndSubject(workId, markSubject.getSubject());

            if ((count * 1.0D) / total * 1.0D > pt / 100.D) {
                statusMap.put(markSubject.getId(), false);
                throw new RuntimeException(targetLevel.getCode() + "档已满，占比总阀值已达" + pt + "%");
            }
            //判断考点阀值是否已满
            int kdpt = targetLevel.getKdpt();
            List<ExamQuestion> examQuestions = examQuestionRepo.findByWorkIdAndSubject(workId, markSubject.getSubject());
            for (ExamQuestion examQuestion : examQuestions) {
                long kdcount = paperRepo.countByWorkIdAndSubjectAndQuestionIdAndLevel(workId, markSubject.getSubject(), examQuestion.getId(), targetLevel.getCode());
                long kdtotal = paperRepo.countByWorkIdAndSubjectAndQuestionId(workId, markSubject.getSubject(), examQuestion.getId());
                if ((kdcount * 1.0D) / kdtotal * 1.0D > kdpt / 100.D) {
                    statusMap.put(markSubject.getId(), false);
                    throw new RuntimeException(targetLevel.getCode() + "档已满，考点阀值已达" + kdpt + "%。");
                }
            }
        }
    }

    /**
     * 进入到抽查阶段
     *
     * @param markSubject
     */
    private void enterInspectStage(MarkSubject markSubject) {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();
//        List<MarkUser> checkers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.INSPECTOR);
//        if (checkers.size() == 0) {
//            statusMap.put(markSubject.getId(), false);
//            throw new RuntimeException("没有设定复评专家");
//        }

    }

    /**
     * 重置所有评卷数据，回到INIT状态
     *
     * @param markSubject
     * @param stage       重置的阶段
     * @param force       是否强制重置，如果为false，当有评卷记录时，不可重置，如果为true，不论有无评卷记录，删除所有数据
     */
    @Transactional
    public void reset(MarkSubject markSubject, MarkStage stage, boolean force) {
        Subject subject = markSubject.getSubject();
        Long workId = markSubject.getWorkId();
        if (MarkStage.LEVEL.equals(stage)) {
            markTaskLevelRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());
        } else if (MarkStage.SCORE.equals(stage)) {
            markTaskScoreRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, markSubject.getStage());
        }
        markSubject.setStage(stage);
        markSubjectRepo.save(markSubject);
    }

    /**
     * 分档进度查询
     *
     * @param markSubject
     */
    public LevleProgressDTO progress(MarkSubject markSubject) {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();
        //已评任务数
        int successCount = 0;
        if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
            successCount = paperRepo.countByWorkIdAndSubjectAndRoughLevelNotNullAndRoughBatchNoNotNullAndTest(workId, subject, markSubject.getTest());
        } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
            successCount = paperRepo.countByWorkIdAndSubjectAndLevelNotNullAndBatchNoNotNullAndTest(workId, subject, markSubject.getTest());
        }

        //所有任务数(查询所有有效试卷)
        int totalCount = paperRepo.countByWorkIdAndSubjectAndIsMissingFalseAndTest(workId, subject, markSubject.getTest());

        BigDecimal waitTotal = BigDecimal.valueOf(totalCount).subtract(BigDecimal.valueOf(successCount));
        //进度
        BigDecimal progress = BigDecimal.ZERO.compareTo(BigDecimal.valueOf(totalCount)) == 0 ? BigDecimal.ZERO : BigDecimal.valueOf(successCount).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(totalCount), 2, BigDecimal.ROUND_HALF_UP);

        LevleProgressDTO progressDTO = new LevleProgressDTO();
        progressDTO.setSuccessCount(successCount);
        progressDTO.setTotalCount(totalCount);
        progressDTO.setWaitCount(waitTotal.intValue());
        progressDTO.setProgress(progress.doubleValue());

        return progressDTO;
    }

    /**
     * 分档进度查询
     *
     * @param markSubject
     */
    public List scoreProgress(MarkSubject markSubject, Long questionId) {
        /*List<TaskPublishSetting> publishSettings = taskPublishSettingRepo.findByWorkIdAndSubject(markSubject.getWorkId(), markSubject.getSubject().name());
        if (publishSettings != null && publishSettings.size() > 0) {
            int maxCount = taskPublishSettingRepo.maxPublishCount(markSubject.getWorkId(), markSubject.getSubject().name());
            return taskPublishSettingRepo.findByWorkIdAndSubjectAndPublishCount(markSubject.getWorkId(), markSubject.getSubject().name(), maxCount);
        }*/

        Integer maxCount = taskPublishSettingRepo.maxPublishCount(markSubject.getWorkId(), markSubject.getSubject().name());
        if (maxCount != null && maxCount.intValue() >= 0) {
            List settingList = null;
            //单个考区
            if (questionId != null) {
                settingList = taskPublishSettingRepo.findByWorkIdAndQuestionIdAndSubjectAndPublishCount(markSubject.getWorkId(), questionId, markSubject.getSubject().name(), maxCount);
                return settingList;
            }
            //所有考区
            else {
                List<Object[]> settingList1 = taskPublishSettingRepo.findAllByWorkIdAndSubjectAndPublishCount(markSubject.getWorkId(), markSubject.getSubject().name(), maxCount);
                List<TaskPublishSetting> taskPublishSettings = new ArrayList<>();
                settingList1.forEach(m -> {
                    TaskPublishSetting taskPublishSetting = new TaskPublishSetting();
                    taskPublishSetting.setWorkId(Long.valueOf(m[0].toString()));
                    taskPublishSetting.setSubject(String.valueOf(m[1]));
                    taskPublishSetting.setCode(String.valueOf(m[2]));
                    taskPublishSetting.setTotalCount(Integer.valueOf(m[3].toString()));
                    taskPublishSetting.setSuccessCount(Integer.valueOf(m[4].toString()));
                    taskPublishSetting.setWaitCount(Double.valueOf(m[5].toString()).intValue());
                    taskPublishSettings.add(taskPublishSetting);
                });
                return taskPublishSettings;
            }
        }
        return null;
    }


    /**
     * 分档进度查询
     *
     * @param markSubject
     */
    public List<LevleProgressDTO> areaProgress(MarkSubject markSubject) {
        Long workId = markSubject.getWorkId();
        Subject subject = markSubject.getSubject();

        List<LevleProgressDTO> list = new ArrayList<>();
        List<ExamQuestion> examQuestions = examQuestionRepo.findByWorkIdAndSubject(workId, markSubject.getSubject());
        List<Paper> allPapers = paperRepo.findByWorkIdAndSubjectAndIsMissingAndTest(workId, subject, false, markSubject.getTest());
        MarkStage stage = markSubject.getStage();

        List<MarkerGroupStudent> markerGroupStudents = markerGroupStudentRepo.findByWorkIdAndSubjectAndStageAndUsed(workId, subject, stage, false);

        for (ExamQuestion examQuestion : examQuestions) {
            //已评任务数
            long areaSuccessCount = 0;
            if (MarkStage.ROUGH_LEVEL.equals(stage)) {
                areaSuccessCount = allPapers.stream().filter(m -> m.getRoughLevel() != null && m.getRoughBatchNo() != null && m.getQuestionId().equals(examQuestion.getId())).count();
            } else if (MarkStage.LEVEL.equals(stage)) {
                areaSuccessCount = allPapers.stream().filter(m -> StringUtils.isNotBlank(m.getLevel()) && m.getBatchNo() != null && m.getQuestionId().equals(examQuestion.getId())).count();
            }
            //所有任务数(查询所有有效试卷)
            long areaTotalCount = allPapers.stream().filter(m -> m.getQuestionId().equals(examQuestion.getId())).count();

            BigDecimal areaWaitTotal = BigDecimal.valueOf(areaTotalCount).subtract(BigDecimal.valueOf(areaSuccessCount));
            //进度
            BigDecimal areaProgress = BigDecimal.ZERO.compareTo(BigDecimal.valueOf(areaTotalCount)) == 0 ? BigDecimal.ZERO : BigDecimal.valueOf(areaSuccessCount).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(areaTotalCount), 2, BigDecimal.ROUND_HALF_UP);

            LevleProgressDTO areaProgressDTO = new LevleProgressDTO();
            areaProgressDTO.setSuccessCount(Math.toIntExact(areaSuccessCount));
            areaProgressDTO.setTotalCount(Math.toIntExact(areaTotalCount));
            areaProgressDTO.setWaitCount(areaWaitTotal.intValue());
            areaProgressDTO.setProgress(areaProgress.doubleValue());
            areaProgressDTO.setAreaName(examQuestion.getAreaName());
            areaProgressDTO.setQuestionId(examQuestion.getId());
            // 查询初始值
            long initCount = markerGroupStudents.isEmpty() ? 0 : markerGroupStudents.stream().filter(m -> m.getQuestionId().equals(examQuestion.getId())).count();
            areaProgressDTO.setInitCount(Math.toIntExact(initCount));
            areaProgressDTO.setInitChecked(initCount > 0);
            list.add(areaProgressDTO);
        }

        if (examQuestions.size() > 1) {
            //已评任务数
            long successCount = 0;
            if (MarkStage.ROUGH_LEVEL.equals(markSubject.getStage())) {
                successCount = allPapers.stream().filter(m -> m.getRoughLevel() != null && m.getRoughBatchNo() != null).count();
            } else if (MarkStage.LEVEL.equals(markSubject.getStage())) {
                successCount = allPapers.stream().filter(m -> StringUtils.isNotBlank(m.getLevel()) && m.getBatchNo() != null).count();
            }
            //所有任务数(查询所有有效试卷)
            int totalCount = allPapers.size();

            BigDecimal waitTotal = BigDecimal.valueOf(totalCount).subtract(BigDecimal.valueOf(successCount));
            //进度
            BigDecimal progress = BigDecimal.ZERO.compareTo(BigDecimal.valueOf(totalCount)) == 0 ? BigDecimal.ZERO : BigDecimal.valueOf(successCount).multiply(BigDecimal.valueOf(100)).divide(BigDecimal.valueOf(totalCount), 2, BigDecimal.ROUND_HALF_UP);

            LevleProgressDTO progressDTO = new LevleProgressDTO();
            progressDTO.setSuccessCount(Math.toIntExact(successCount));
            progressDTO.setTotalCount(totalCount);
            progressDTO.setWaitCount(waitTotal.intValue());
            progressDTO.setProgress(progress.doubleValue());
            progressDTO.setAreaName("所有考区");
            progressDTO.setQuestionId(null);
            list.add(progressDTO);
        }
        return list;
    }

    /**
     * 用户强制退出且做数据库备份
     *
     * @param markSubject 科目
     */
    public void logoutAndBackupDb(MarkSubject markSubject) {
        // 所有用户强制退出
        List<MarkUser> userList = markUserRepo.findByWorkIdAndSubject(markSubject.getWorkId(), markSubject.getSubject());
        userList = userList.stream().filter(m -> Role.MARKER.equals(m.getRole()) || Role.MARK_LEADER.equals(m.getRole())).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(userList)) {
            for (MarkUser user : userList) {
                cacheService.deleteTokenCache(user.getId().toString());
            }
        }
        dbBackupUtils.startBackup(markSubject.getWorkId(), markSubject.getStage().name());
    }

}
