package cn.com.qmth.stmms.ms.marking.service.arbitration;


import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;

import java.util.*;
import java.util.stream.IntStream;

/**
 * 仲裁
 * Created by zhengmin on 2016/9/22.
 */
public class ArbitrationService {

    /**
     * 仲裁，如果两两差值大于误差，或不存在相同数大于半数，就判定仲裁，返回true
     *
     * @param deviation 误差
     * @param sources
     */
    public static boolean arbitrate(int deviation, int... sources) {

        int[] values = new int[(int) Math.pow(sources.length, 2)];
        int i = 0;
        for (int source : sources) {
            for (int inner : sources) {
                values[i] = Math.abs(source - inner);
                i++;
            }
        }
        OptionalInt optionalInt = IntStream.of(values).max();
        int max = optionalInt.orElse(0);
        if (max >= deviation) {
            return true;
        }
        return false;
    }


    /**
     * 计算是否超过仲裁档位差
     *
     * @param sources 档位
     * @return boolean true超过
     */
    public static boolean overDeviation(int deviation, Set<Integer> sources) {
        Optional<Integer> maxOptional = sources.stream().max(Integer::compare);
        Optional<Integer> minOptional = sources.stream().min(Integer::compare);
        int max = 0;
        if (maxOptional.isPresent()) {
            max = maxOptional.get();
        }
        int min = 0;
        if (minOptional.isPresent()) {
            min = minOptional.get();
        }
        int diff = Math.abs(max - min);
        return diff >= deviation;
    }

    /**
     * 计算累计档位差
     *
     * @param cumulateError 打回累计误差
     * @param markTasks     评卷任务
     * @return List<MarkTaskLevel>
     */
    public static List<String> calcCumulateRoughLevel(int cumulateError, List<MarkTaskRoughLevel> markTasks) {
        List<String> markerIds = new ArrayList<>();
        for (MarkTaskRoughLevel markTask : markTasks) {
            int cumulate = 0;
            for (MarkTaskRoughLevel task : markTasks) {
                cumulate += Math.abs(markTask.getLevelValue() - task.getLevelValue());
            }
            if (cumulate >= cumulateError) {
                markerIds.add(String.valueOf(markTask.getMarkerId()));
            }
        }
        return markerIds;
    }

    public static List<String> calcCumulateLevel(int cumulateError, List<MarkTaskLevel> markTasks) {
        List<String> markerIds = new ArrayList<>();
        for (MarkTaskLevel markTask : markTasks) {
            int cumulate = 0;
            for (MarkTaskLevel task : markTasks) {
                cumulate += Math.abs(markTask.getLevelValue() - task.getLevelValue());
            }
            if (cumulate >= cumulateError) {
                markerIds.add(String.valueOf(markTask.getMarkerId()));
            }
        }
        return markerIds;
    }
}
