package cn.com.qmth.stmms.ms.marking.utils;

import cn.com.qmth.stmms.ms.core.repository.MarkTaskLevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRoughLevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskScoreRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import org.apache.commons.lang.math.RandomUtils;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * 随机数生成工具类
 */
@Component
public class RandomUtil {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(RandomUtil.class);
    public static final int SECRET_NUMBER_START = 100000000;
    public static final int SECRET_NUMBER_END = 999999999;
    private Map<Long, Set<Long>> randomPaperMap = new HashMap<>();
    private Map<Long, Set<Long>> randomRoughLevelMap = new HashMap<>();
    private Map<Long, Set<Long>> randomLevelMap = new HashMap<>();
    private Map<Long, Set<Long>> randomScoreMap = new HashMap<>();

    @Resource
    private PaperRepo paperRepo;

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskScoreRepo markTaskScoreRepo;

    public Map<Long, Set<Long>> getRandomPaperMap() {
        return randomPaperMap;
    }

    public void setRandomPaperMap(Map<Long, Set<Long>> randomPaperMap) {
        this.randomPaperMap = randomPaperMap;
    }

    public Map<Long, Set<Long>> getRandomRoughLevelMap() {
        return randomRoughLevelMap;
    }

    public void setRandomRoughLevelMap(Map<Long, Set<Long>> randomRoughLevelMap) {
        this.randomRoughLevelMap = randomRoughLevelMap;
    }

    public Map<Long, Set<Long>> getRandomLevelMap() {
        return randomLevelMap;
    }

    public void setRandomLevelMap(Map<Long, Set<Long>> randomLevelMap) {
        this.randomLevelMap = randomLevelMap;
    }

    public Map<Long, Set<Long>> getRandomScoreMap() {
        return randomScoreMap;
    }

    public void setRandomScoreMap(Map<Long, Set<Long>> randomScoreMap) {
        this.randomScoreMap = randomScoreMap;
    }

    private static Long randomLong() {
        return (long) (RandomUtils.nextDouble() * (RandomUtil.SECRET_NUMBER_END - RandomUtil.SECRET_NUMBER_START)) + RandomUtil.SECRET_NUMBER_START;
    }

    public void initRandom(Long workId) {
        Set<Long> paperRandoms = paperRepo.listRandomByWorkId(workId);
        randomPaperMap.put(workId, paperRandoms);

        Set<Long> markTaskRoughLevelRandoms = markTaskRoughLevelRepo.listRandomByWorkId(workId);
        randomRoughLevelMap.put(workId, markTaskRoughLevelRandoms);

        Set<Long> markTaskLevelRandoms = markTaskLevelRepo.listRandomByWorkId(workId);
        randomRoughLevelMap.put(workId, markTaskLevelRandoms);

        Set<Long> markTaskScoreRandoms = markTaskScoreRepo.listRandomByWorkId(workId);
        randomScoreMap.put(workId, markTaskScoreRandoms);
    }

    public Long randomPaper(Long workId) {
        Set<Long> longs = randomPaperMap.computeIfAbsent(workId, k -> new HashSet<>());
        Long random = randomLong();
        if (longs.contains(random)) {
            randomPaper(workId);
        }
        randomPaperMap.get(workId).add(random);
        return random;
    }

    public Long randomRoughLevel(Long workId) {
        Set<Long> longs = randomRoughLevelMap.computeIfAbsent(workId, k -> new HashSet<>());
        Long random = randomLong();
        if (longs.contains(random)) {
            randomRoughLevel(workId);
        }
        randomRoughLevelMap.get(workId).add(random);
        return random;
    }

    public Long randomLevel(Long workId) {
        Set<Long> longs = randomLevelMap.computeIfAbsent(workId, k -> new HashSet<>());
        Long random = randomLong();
        if (longs.contains(random)) {
            randomLevel(workId);
        }
        randomLevelMap.get(workId).add(random);
        return random;
    }

    public Long randomScore(Long workId) {
        Set<Long> longs = randomScoreMap.computeIfAbsent(workId, k -> new HashSet<>());
        Long random = randomLong();
        if (longs.contains(random)) {
            randomScore(workId);
        }
        randomScoreMap.get(workId).add(random);
        return random;
    }
}
